/*********************************************************************/
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2024                                     */
/*********************************************************************/
/*--START OF HISTORY-------------------------------------------------*/
/*  DATE        DESCRIPTION                                          */
/*  20240716    PJ47254 - RTMC CDC Dashboards                        */
/*--END OF HISTORY---------------------------------------------------*/
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     Functions that return JSON for metrics common to    */
/*               all databases.                                      */
/*                                                                   */
/* "extra_cols" rule:                                                */
/* In the following JSON, the rule for what is in the extra_cols is: */
/* If a database table column is not an attribute, metric,           */
/* virtual metric, primary key then it's an extra column.            */
/*********************************************************************/

-- Set the default character set to latin1 for consistency across MariaDB and MySQL
SET NAMES 'latin1';

DROP FUNCTION IF EXISTS `tpf_get_udm_metrics`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_udm_metrics` ()
RETURNS JSON
DETERMINISTIC
BEGIN
   RETURN '[]';
END//
DELIMITER ;


DROP FUNCTION IF EXISTS `tpf_get_zrtmc_communications_metrics`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_zrtmc_communications_metrics` ()
RETURNS JSON
DETERMINISTIC
BEGIN
DECLARE result JSON;

-- This function contains structures that specify "dashboard_key":"communications"

SET result = '
  [
    {"panel_key":"rest_services",
	 "dashboard_key":"communications",
	 "subsystem_applies":"yes",
     "table_name":"tpf_service_app_metrics_table",
     "attribute_columns":[{"attribute_name":"operation","attribute_table":"service_app_operations","table_column":"operation_id"}],
     "metric_type":"All",
     "metrics":["internal_operations_per_sec",
                "external_operations_per_sec",
                "total_operations_per_sec",
                "timeouts_per_sec",
                "iface_errors_per_sec",
                "srvc_errors_per_sec",
                "unrec_errors_per_sec"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"service_jam",
     "dashboard_key":"communications",
     "subsystem_applies":"yes",
     "table_name":"tpf_service_jam_metrics_table",
     "attribute_columns":[{"attribute_name":"jam_name","attribute_table":"","table_column":"jam_name"}],
     "metric_type":"All",
     "metrics":["internal_operations_per_sec",
                "external_operations_per_sec",
                "total_operations_per_sec",
                "threads_in_use",
                "threads_in_use_highwater",
                "threads_config"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"tcpip",
     "dashboard_key":"communications",
     "subsystem_applies":"no",
     "table_name":"tpf_tcpip_metrics_table",
     "attribute_columns":[{"attribute_name":"app_name","attribute_table":"tpf_tcpip_name_table","table_column":"app_name_id"},
                          {"attribute_name":"port","attribute_table":"","table_column":"port"}],
     "metric_type":"All",
     "metrics":["msgs_per_sec",
                "msgs_sent_per_sec",
                "msgs_recvd_per_sec",
                "bytes_per_sec",
                "bytes_sent_per_sec",
                "bytes_recvd_per_sec",
                "packets_per_sec",
                "packets_sent_per_sec",
                "packets_recvd_per_sec"],
     "virtual_metrics":[],
    "extra_cols":["weight"],
     "altered_column":"no"
    },
    {"panel_key":"mq_summary",
     "dashboard_key":"communications",
     "subsystem_applies":"yes",
     "table_name":"tpf_mq_summary_metrics_table",
     "attribute_columns":[{"attribute_name":"queue_manager","attribute_table":"tpf_mq_queue_manager_table","table_column":"queue_manager_id"}],
     "metric_type":"All",
     "metrics":["queues",
                "local_queues",
                "transmit_queues",
                "remote_queues",
                "queue_opens_per_sec",
                "queue_closes_per_sec",
                "queue_sweeps_per_sec",
                "queue_msgs_persistent_per_sec",
                "queue_msg_length_persistent",
                "queue_msgs_nonpersistent_per_sec",
                "queue_msg_length_nonpersistent",
                "channels",
                "receiver_channels",
                "sender_channels",
                "server_channels",
                "channel_msgs_per_sec",
                "channel_bytes_recvd_per_sec",
                "channel_bytes_sent_per_sec",
                "channel_bytes_handled_per_sec",
                "channel_batch_size",
                "channel_active_instances",
                "channel_last_batch_size",
                "channel_instances_highwater"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"mq_queues",
     "dashboard_key":"communications",
     "subsystem_applies":"yes",
     "table_name":"tpf_mq_queue_metrics_table",
     "attribute_columns":[{"attribute_name":"queue_manager","attribute_table":"tpf_mq_queue_manager_table","table_column":"queue_manager_id"},
                          {"attribute_name":"queue_name","attribute_table":"tpf_mq_queue_name_table","table_column":"queue_name_id"},
                          {"attribute_name":"queue_type","attribute_table":"","table_column":"queue_type"}],
     "metric_type":"All",
     "metrics":["opens_per_sec",
                "closes_per_sec",
                "sweeps_per_sec",
                "msgs_persistent_per_sec",
                "msg_length_persistent",
                "msgs_nonpersistent_per_sec",
                "msg_length_nonpersistent"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"mq_channels",
     "dashboard_key":"communications",
     "subsystem_applies":"yes",
     "table_name":"tpf_mq_channel_metrics_table",
     "attribute_columns":[{"attribute_name":"queue_manager","attribute_table":"tpf_mq_queue_manager_table","table_column":"queue_manager_id"},
                          {"attribute_name":"channel_name","attribute_table":"tpf_mq_channel_name_table","table_column":"channel_name_id"},
                          {"attribute_name":"channel_type","attribute_table":"","table_column":"channel_type"}],
     "metric_type":"All",
     "metrics":["msgs_per_sec",
                "bytes_recvd_per_sec",
                "bytes_sent_per_sec",
                "bytes_handled_per_sec",
                "batch_size",
                "active_instances",
                "last_batch_size",
                "instances_highwater"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"channel_utilization",
     "dashboard_key":"communications",
     "subsystem_applies":"no",
     "table_name":"tpf_enh_chl_util_metrics_table",
     "attribute_columns":[{"attribute_name":"channel_id","attribute_table":"","table_column":"channel_id"},
                          {"attribute_name":"active","attribute_table":"","table_column":"active"},
                          {"attribute_name":"type","attribute_table":"tpf_chl_util_type_table","table_column":"type_id"},
                          {"attribute_name":"shared","attribute_table":"","table_column":"shared"},
                          {"attribute_name":"channel_group","attribute_table":"","table_column":"channel_group"}],
     "metric_type":"All",
     "metrics":["part_util_pct",
                "total_util_pct",
                "bus_util_pct",
                "part_mb_read_per_sec",
                "total_mb_read_per_sec",
                "part_mb_write_per_sec",
                "total_mb_write_per_sec"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"mpif",
     "dashboard_key":"communications",
     "subsystem_applies":"no",
     "table_name":"tpf_mpif_metrics_table",
     "attribute_columns":[{"attribute_name":"path_name","attribute_table":"tpf_mpif_path_name_table","table_column":"path_name_id"},
                          {"attribute_name":"device_name","attribute_table":"tpf_mpif_device_name_table","table_column":"device_name_id"},
                          {"attribute_name":"class_name","attribute_table":"","table_column":"class_name"}],
     "metric_type":"All",
     "metrics":["reads_per_sec",
                "writes_per_sec",
                "bytes_sent_per_sec",
                "msgs_sent_per_sec",
                "queue_overruns_per_sec"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    }
  ]';

RETURN result;

END//
DELIMITER ;


DROP FUNCTION IF EXISTS `tpf_get_zrtmc_storage_metrics`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_zrtmc_storage_metrics` ()
RETURNS JSON
DETERMINISTIC
BEGIN
DECLARE result JSON;

-- This function contains structures that specify "dashboard_key":"storage"

SET result = '
  [
    {"panel_key":"dasd_pools",
     "dashboard_key":"storage",
     "subsystem_applies":"yes",
     "table_name":"tpf_dasd_pool_metrics_table",
     "attribute_columns":[{"attribute_name":"logical_device_type","attribute_table":"","table_column":"logical_device_type"},
                          {"attribute_name":"pool_type","attribute_table":"","table_column":"pool_type"}],
     "metric_type":"All",
     "metrics":["set_size",
                "reorder_time",
                "dispensed_per_sec",
                "returned_per_sec"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"dasd_io",
     "dashboard_key":"storage",
     "subsystem_applies":"yes",
     "table_name":"tpf_dasd_io_metrics_table",
     "attribute_columns":[{"attribute_name":"logical_device_type","attribute_table":"","table_column":"logical_device_type"}],
     "metric_type":"All",
     "metrics":["io_per_sec",
                "writes_per_sec",
                "reads_per_sec",
                "queue_length"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"dasd_queue",
	 "dashboard_key":"storage",
	 "subsystem_applies":"yes",
     "table_name":"tpf_dasd_queue_metrics_table",
     "attribute_columns":[{"attribute_name":"dev_address","attribute_table":"","table_column":"dev_address"}],
     "metric_type":"All",
     "metrics":["dev_queue"],
     "virtual_metrics":[],
     "extra_cols":["dev_mod"],
     "altered_column":"no"
    },
    {"panel_key":"dasd_service_time",
     "dashboard_key":"storage",
     "subsystem_applies":"no",
     "table_name":"tpf_dasd_service_time_metrics_table",
     "attribute_columns":[{"attribute_name":"ssid","attribute_table":"","table_column":"ssid"}],
     "metric_type":"All",
     "metrics":["average_service_time"],
     "virtual_metrics":[],
     "extra_cols":["lowest_online_sda_on_lss"],
     "altered_column":"no"
    },
    {"panel_key":"vfa",
     "dashboard_key":"storage",
     "subsystem_applies":"yes",
     "table_name":"tpf_vfa_metrics_table",
     "attribute_columns":[],
     "metric_type":"All",
     "metrics":["data_reads_per_sec",
                "finds_per_sec",
                "candidate_files_per_sec",
                "noncandidate_files_per_sec",
                "force_files_per_sec",
                "file_immediate_per_sec",
                "buffer_unavailable_per_sec",
                "SLock_hits_per_sec",
                "XLock_hits_per_sec",
                "sync_misses_per_sec",
                "sync_content_per_sec",
                "cached_rht_hts_per_sec",
                "sync_age_outs_per_sec",
                "dispense_4K_per_sec",
                "rht_content_per_sec"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"remote_data_store",
     "dashboard_key":"storage",
     "subsystem_applies":"yes",
     "table_name":"tpf_remote_data_store_metrics_table",
     "attribute_columns":[{"attribute_name":"rds_name","attribute_table":"tpf_rds_name_table","table_column":"rds_name_id"},
                          {"attribute_name":"rds_mode","attribute_table":"tpf_rds_mode_table","table_column":"rds_mode_id"}],
     "metric_type":"All",
     "metrics":["reader_active_ecbs",
                "reader_highwater",
                "reader_max",
                "writer_active_ecbs",
                "writer_highwater",
                "writer_max",
                "reads_per_sec",
                "creates_per_sec",
                "updates_per_sec",
                "deletes_per_sec",
                "total_ops_per_sec",
                "read_reject_at_max",
                "timeouts_per_sec",
                "TPF_error",
                "server_error_mongo"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"tape",
     "dashboard_key":"storage",
     "subsystem_applies":"yes",
     "table_name":"tpf_tape_metrics_table",
     "attribute_columns":[{"attribute_name":"tape_name","attribute_table":"","table_column":"tape_name"}],
     "metric_type":"All",
     "metrics":["bytes_per_sec",
             "writes_per_sec",
             "queue_length"],
     "virtual_metrics":[],
     "extra_cols":["device_address"],
     "altered_column":"no"
    },
    {"panel_key":"tpfdf",
     "dashboard_key":"storage",
     "subsystem_applies":"yes",
     "table_name":"tpf_tpfdf_metrics_table",
     "attribute_columns":[{"attribute_name":"file_id","attribute_table":"","table_column":"file_id"},
                          {"attribute_name":"file_id_ebcdic","attribute_table":"","table_column":"file_id_ebcdic"},
                          {"attribute_name":"file_name_alias","attribute_table":"tpf_tpfdf_file_name_alias_table","table_column":"file_name_alias_id"}],
     "metric_type":"All",
     "metrics":["dbadd_per_sec",
             "dbadr_per_sec",
             "dbckp_per_sec",
             "dbcls_per_sec",
             "dbcpy_per_sec",
             "dbcre_per_sec",
             "dbdel_per_sec",
             "dbdix_per_sec",
             "dbdsp_per_sec",
             "dbidx_per_sec",
             "dbkey_per_sec",
             "dbmod_per_sec",
             "dbmrg_per_sec",
             "dbopn_per_sec",
             "dbred_per_sec",
             "dbrep_per_sec",
             "dbret_per_sec",
             "dbrst_per_sec",
             "dbspa_per_sec",
             "dbsrt_per_sec",
             "dbtlg_per_sec",
             "dbtld_per_sec",
             "dbtrd_per_sec",
             "dbuky_per_sec",
             "dbhold_per_sec",
             "dbuhld_per_sec",
             "filnc_per_sec",
             "filec_per_sec",
             "cfile_per_sec",
             "pfile_per_sec",
             "twrtc_per_sec",
             "getfc_per_sec",
             "relfc_per_sec",
             "detac_per_sec",
             "attac_per_sec",
             "fsysw_per_sec",
             "getcc_per_sec",
             "relcc_per_sec",
             "cfind_per_sec",
             "pfind_per_sec",
             "btfil_per_sec",
             "btfnd_per_sec",
             "lrfil_per_sec",
             "lrfnd_per_sec",
             "opn_hd_per_sec",
             "opn_nhd_per_sec",
             "opn_hnd_per_sec",
             "opn_nhnd_per_sec",
             "cls_rel_per_sec",
             "cls_norel_per_sec",
             "cls_reuse_per_sec",
             "cls_opn_per_sec",
             "prefp_per_sec",
             "prefpw_per_sec",
             "cache_prtl_iocnt_per_sec",
             "cache_prtl_ioreq_per_sec",
             "cache_read_hit_per_sec",
             "cache_read_miss_per_sec",
             "blk_read_depth",
             "rsf_create_per_sec",
             "rsf_read_per_sec",
             "rsf_update_per_sec",
             "rsf_delete_per_sec"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"trad_db_encryption",
     "dashboard_key":"storage",
     "subsystem_applies":"yes",
     "table_name":"tpf_trad_db_encryption_metrics_table",
     "attribute_columns":[],
     "metric_type":"All",
     "metrics":["encryptions_per_sec",
                "encryptions_compressions_per_sec",
                "decryptions_per_sec",
                "decryption_expansions_per_sec",
                "encryption_errors_per_sec",
                "compression_errors_per_sec"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    }
  ]';

RETURN result;

END//
DELIMITER ;


DROP FUNCTION IF EXISTS `tpf_get_zrtmc_system_metrics`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_zrtmc_system_metrics` ()
RETURNS JSON
DETERMINISTIC
BEGIN
DECLARE result JSON;

-- This function contains structures that specify "dashboard_key":"system"

SET result = '
  [
    {"panel_key":"system_messages",
     "dashboard_key":"system",
     "subsystem_applies":"no",
     "table_name":"tpf_system_messages_metrics_table",
     "attribute_columns":[],
     "metric_type":"All",
     "metrics":["high_spd_msgs_per_sec",
                "tcpip_msgs_per_sec",
                "routed_entries_per_sec",
                "created_entries_per_sec",
                "commits_per_sec",
                "number_of_istreams",
                "usable_istreams",
                "processor_util_pct",
                "gp_processor_util_pct",
                "gp_max_cap_for_lpar_util_pct",
                "gp_number_of_engines",
                "istream_cap"],
     "virtual_metrics":[],
     "extra_cols":["speed_boost",
                "istream_boost"],
     "altered_column":"no"
    },
    {"panel_key":"system_blocks",
     "dashboard_key":"system",
     "subsystem_applies":"no",
     "table_name":"tpf_system_blocks_metrics_table",
     "attribute_columns":[],
     "metric_type":"All",
     "metrics":["iob_allocated",
                "iob_inuse",
                "iob_inuse_pct",
                "ecb_allocated",
                "ecb_inuse",
                "ecb_inuse_pct",
                "swb_allocated",
                "swb_inuse",
                "swb_inuse_pct",
                "common_allocated",
                "common_inuse",
                "common_inuse_pct",
                "frame_allocated",
                "frame_inuse",
                "frame_inuse_pct",
                "frame1mb_allocated",
                "frame1mb_inuse",
                "frame1mb_inuse_pct",
                "xwb_allocated",
                "xwb_inuse",
                "xwb_inuse_pct",
                "rlog_allocated",
                "rlog_inuse",
                "rlog_inuse_pct",
                "active_ecbs",
                "low_priority_ecbs"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"system_lists",
     "dashboard_key":"system",
     "subsystem_applies":"no",
     "table_name":"tpf_system_lists_metrics_table",
     "attribute_columns":[],
     "metric_type":"All",
     "metrics":["cross_list",
                "ready_list",
                "input_list",
                "deferred_list",
                "suspend_list",
                "vct_list"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"istream",
     "dashboard_key":"system",
     "subsystem_applies":"no",
     "table_name":"tpf_istreams_metrics_table",
     "attribute_columns":[{"attribute_name":"istream","attribute_table":"","table_column":"istream"}],
     "metric_type":"All",
     "metrics":["cross_list",
                "ready_list",
                "input_list",
                "deferred_list",
                "suspend_list",
                "vct_list",
                "active_ecbs",
                "routed_entries_per_sec",
                "proc_util",
                "proc_adj_util",
                "pseudo_util",
                "low_priority_util"],
     "virtual_metrics":[],
     "extra_cols":["status"],
     "altered_column":"no"
    },
    {"panel_key":"lpar_utilization",
     "dashboard_key":"system",
     "subsystem_applies":"no",
     "table_name":"tpf_lpar_util_metrics_table",
     "attribute_columns":[{"attribute_name":"lpar_name","attribute_table":"tpf_lpar_name_table","table_column":"part_name_id"},
                          {"attribute_name":"part_number","attribute_table":"","table_column":"part_number"},
                          {"attribute_name":"type","attribute_table":"","table_column":"type"}],
     "metric_type":"All",
     "metrics":["shared_cpus",
                "recognized_cpus",
                "weight",
                "util",
                "util_vs_cap",
                "machine",
                "entitle"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"shutdown_events",
     "dashboard_key":"system",
     "subsystem_applies":"no",
     "table_name":"tpf_shutdown1_metrics_table",
     "attribute_columns":[{"attribute_name":"shutdown_type","attribute_table":"tpf_shutdown1_shutdown_type_table","table_column":"shutdown_type_id"},
                          {"attribute_name":"reason","attribute_table":"tpf_shutdown1_reason_table","table_column":"reason_id"}],
     "metric_type":"All",
     "metrics":["shutdown_occurred"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"traces",
     "dashboard_key":"system",
     "subsystem_applies":"no",
     "table_name":"tpf_traces_metrics_table",
     "attribute_columns":[],
     "metric_type":"All",
     "metrics":[],
     "virtual_metrics":[],
     "extra_cols":["block_check_mode",
                "branch",
                "branch_relative_target_check_mode",
                "ECB_heap",
                "ECB_Macro",
                "enter_back",
                "extended_C_function",
                "function_trace",
                "heap_check_mode",
                "IO",
                "IBM_DBUGC_log",
                "IBM_DBUGC_test",
                "idle_timer",
                "IO_CCW",
                "lock_trace",
                "reg_this_IPL",
                "socket",
                "system_log",
                "user_DBUGC_log",
                "user_DBUGC_test",
                "VFA"],
     "altered_column":"yes"
    },
    {"panel_key":"LODIC_resource_classes",
     "dashboard_key":"system",
     "subsystem_applies":"no",
     "table_name":"tpf_lodic_resources_metrics_table",
     "attribute_columns":[{"attribute_name":"resource_priority_class","attribute_table":"tpf_lodic_resource_name_table","table_column":"class_name_id"}],
     "metric_type":"All",
     "metrics":["ecbs_using_class"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"LODIC_utilization_classes",
     "dashboard_key":"system",
     "subsystem_applies":"no",
     "table_name":"tpf_lodic_utilization_metrics_table",
     "attribute_columns":[{"attribute_name":"utilization_priority_class","attribute_table":"tpf_lodic_utilization_name_table","table_column":"class_name_id"}],
     "metric_type":"All",
     "metrics":["ecbs_using_class"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"time_slice_ecbs",
     "dashboard_key":"system",
     "subsystem_applies":"no",
     "table_name":"tpf_time_slice_ecb_metrics_table",
     "attribute_columns":[],
     "metric_type":"All",
     "metrics":["time_sliceable_ecbs",
             "time_sliced_ecbs_per_sec"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"common_deployment",
	 "dashboard_key":"system",
	 "subsystem_applies":"yes",
      "table_name":"tpf_common_deployment_metrics_table",
     "attribute_columns":[{"attribute_name":"cd_key","attribute_table":"tpf_common_deployment_keys","table_column":"key_id"},
                          {"attribute_name":"cd_type","attribute_table":"tpf_common_deployment_types","table_column":"type_id"},
                          {"attribute_name":"cd_version","attribute_table":"tpf_common_deployment_versions","table_column":"version_id"}],
     "metric_type":"All",
     "metrics":["usage_per_sec"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"machine",
     "dashboard_key":"system",
     "subsystem_applies":"no",
     "table_name":"tpf_static_metrics_table",
     "attribute_columns":[],
     "metric_type":"All",
     "metrics":["subsystem_count",
                "subsystem_user_count",
                "usable_istreams"],
     "virtual_metrics":[],
     "extra_cols":["cpu_serial",
                "cpu_machine_type",
                "tpf_release",
                "tpf_prod_update_level",
                "tpf_complex_name",
                "tpf_enterprise_name",
                "cpu_Id"],
     "altered_column":"yes"
    }
  ]';

RETURN result;

END//
DELIMITER ;

DROP FUNCTION IF EXISTS `tpf_get_zrtmc_subsystem_metrics`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_zrtmc_subsystem_metrics` ()
RETURNS JSON
DETERMINISTIC
BEGIN
DECLARE result JSON;

-- This function contains structures that specify "dashboard_key":"subsystems"

SET result = '
  [
    {"panel_key":"subsystems",
     "dashboard_key":"subsystems",
     "subsystem_applies":"yes",
     "table_name":"tpf_subsystems_metrics_table",
     "attribute_columns":[],
     "metric_type":"All",
     "metrics":["high_spd_msgs_per_sec",
                "routed_entries_per_sec",
                "created_entries_per_sec",
                "ecb_exits_per_sec"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"subsystem_users",
     "dashboard_key":"subsystems",
     "subsystem_applies":"yes",
     "table_name":"tpf_subsystem_users_metrics_table",
     "attribute_columns":[{"attribute_name":"subsystem_user_name","attribute_table":"","table_column":"subsystem_user_name"}],
     "metric_type":"All",
     "metrics":["high_spd_msgs_per_sec",
                "routed_entries_per_sec",
                "created_entries_per_sec",
                "ecb_exits_per_sec"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"subsystem_utility_switches",
     "dashboard_key":"subsystems",
     "subsystem_applies":"yes",
     "table_name":"tpf_subsystems_utility_switches_metrics_table",
     "attribute_columns":[],
     "metric_type":"All",
     "metrics":[],
     "virtual_metrics":[],
     "extra_cols":["recoup",
                "selective_file_trace",
                "RCS_file",
                "RCS_enable",
                "RCS_implement",
                "data_col",
                "capture",
                "restore",
                "SON_copy",
                "trap",
                "DB_reorg",
                "CCP",
                "link_trace"],
     "altered_column":"yes"
    }
  ]';

RETURN result;

END//
DELIMITER ;


DROP FUNCTION IF EXISTS `tpf_get_metrics_json`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_metrics_json` ()
RETURNS JSON
DETERMINISTIC
BEGIN
DECLARE result JSON;

-- This function contains structures that specify an empty dashboard_key.  e.g. "dashboard_key":""

SET result = '
  [
	{"panel_key":"system",
	 "dashboard_key":"",
	 "subsystem_applies":"yes",
     "table_name":"system_metrics_table",
     "attribute_columns":[{"attribute_name":"","attribute_table":""}],
	 "metric_type":"primary",
     "metrics":["actual_cpu",
                "modeled_cpu",
                "workload_cpu",
                "lp_cpu",
                "sys_pseudo_cpu",
                "fenced_pseudo_cpu",
                "inuse_iob",
                "inuse_ecb",
                "inuse_istreams",
                "istream_cap"],
	 "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
   },
   {"panel_key":"system",
    "dashboard_key":"",
    "subsystem_applies":"yes",
    "table_name":"system_metrics_table",
    "attribute_columns":[{"attribute_name":"","attribute_table":""}],
    "metric_type":"nvp_estimated",
    "metrics":["est_msg_rate",
               "est_msg_exist",
               "est_msg_cpu",
               "est_msg_find_file"],
    "virtual_metrics":[],
    "extra_cols":[],
    "altered_column":"no"
   },
   {"panel_key":"nvp",
	"dashboard_key":"",
	"subsystem_applies":"yes",
    "table_name":"nvp_message_metrics_table",
    "attribute_columns":[{"attribute_name":"","attribute_table":""}],
    "metric_type":"primary",
    "metrics":["cpu_used",
               "exist_time",
               "cret",
               "file",
               "file_dasd",
               "file_vfa",
               "find",
               "find_dasd",
               "find_vfa",
               "find_file",
               "est_msg_rate"],
    "virtual_metrics":["est_msg_rate"],
    "extra_cols":[],
    "altered_column":"no"
   },
   {"panel_key":"nvp",
	"dashboard_key":"",
	"subsystem_applies":"yes",
    "table_name":"nvp_message_metrics_table",
    "attribute_columns":[{"attribute_name":"","attribute_table":""}],
    "metric_type":"storage",
    "metrics":["ecb1mf",
               "ecb4kf",
               "hctl",
               "mal",
               "cow",
               "cmbk",
               "swbk",
               "xwbk",
               "sysh",
               "rlog"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
   },
   {"panel_key":"nvp",
	"dashboard_key":"",
	"subsystem_applies":"yes",
    "table_name":"nvp_message_metrics_table",
    "attribute_columns":[{"attribute_name":"","attribute_table":""}],
    "metric_type":"tpfdf",
    "metrics":["df_attac",
               "df_btfil",
               "df_btfnd",
               "df_cfile",
               "df_cfind",
               "df_cls_no_rel",
               "df_cls_opn",
               "df_cls_rels",
               "df_cls_reuse",
               "df_dbadd",
               "df_dbcls",
               "df_dbdel",
               "df_dbdix",
               "df_dbidx",
               "df_dbkey",
               "df_dbmod",
               "df_dbopn",
               "df_dbred",
               "df_dbrep",
               "df_detac",
               "df_getcc",
               "df_getfc",
               "df_lrfil",
               "df_lrfnd",
               "df_opn_hd",
               "df_opn_hnd",
               "df_opn_nhd",
               "df_opn_nhnd",
               "df_pfile",
               "df_pfind",
               "df_relcc",
               "df_relfc",
               "df_rsf_create",
               "df_rsf_delete",
               "df_rsf_read",
               "df_rsf_update"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"nvp",
	 "dashboard_key":"",
	 "subsystem_applies":"yes",
     "table_name":"nvp_message_metrics_table",
     "attribute_columns":[{"attribute_name":"","attribute_table":""}],
     "metric_type":"other",
     "metrics":["dd_time",
                "gfsl",
                "gfss",
                "grfs",
                "relf",
                "rout",
                "serr",
                "snap",
                "tour",
                "twrt"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    },
    {"panel_key":"dasd_metrics",
     "dashboard_key":"",
     "subsystem_applies":"yes",
     "table_name":"tpf_dasd_metrics_table",
     "attribute_columns":[{"attribute_name":"dev_indicator","attribute_table":"","table_column":"dev_indicator"},
                          {"attribute_name":"symbolic_module","attribute_table":"","table_column":"symbolic_module"},
                          {"attribute_name":"volume_number","attribute_table":"","table_column":"volume_number"},
                          {"attribute_name":"type","attribute_table":"","table_column":"type"},
                          {"attribute_name":"address","attribute_table":"","table_column":"address"},
                          {"attribute_name":"dasd_subsystem_id","attribute_table":"","table_column":"dasd_subsystem_id"}],
     "metric_type":"All",
     "metrics":["module_queue",
                "data_reads_per_sec",
                "data_writes_per_sec",
                "accumulated_response_time",
                "response_time_call_per_sec",
                "sioscs_locks_per_sec",
                "sioscs_unlocks_per_sec",
                "mplf_lock_contention_per_sec",
                "fdctc_per_sec"],
     "virtual_metrics":[],
     "extra_cols":[],
     "altered_column":"no"
    }
  ]';

SET result = JSON_MERGE(result,tpf_get_udm_metrics());
SET result = JSON_MERGE(result,tpf_get_zrtmc_communications_metrics());
SET result = JSON_MERGE(result,tpf_get_zrtmc_storage_metrics());
SET result = JSON_MERGE(result,tpf_get_zrtmc_system_metrics());
SET result = JSON_MERGE(result,tpf_get_zrtmc_subsystem_metrics());

RETURN result;

END//
DELIMITER ;

