/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2019, 2024                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or disclosure restricted by GSA ADP Schedule Contract        */
/*      with IBM Corp.                                               */
/*********************************************************************/
/*--START OF HISTORY-------------------------------------------------*/
/*  DATE        DESCRIPTION                                          */
/*  12182019    Initial implementation of z/TPF Realtime Insights    */
/*              Dashboard Starter Kit.                               */
/*  05202020    Update for PJ45657 support for Real-time Runtime     */
/*              Metrics Collection.                                  */
/*  12152021    Initial Message Analysis Tool support                */
/*  06162022    PJ46737 - complex-wide dashboard and mysql support   */
/*  20240515    PJ48025 - Modify DROP for TEMPORARY tables           */
/*--END OF HISTORY---------------------------------------------------*/
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     sql for various stored procedures for the z/TPF     */
/*               name-value pair collection.                         */
/*                                                                   */
/*********************************************************************/
-- --------------------------------------------------------
-- Server version:               10.2.17-MariaDB-log - MariaDB Server
-- Server OS:                    Linux
-- HeidiSQL Version:             9.4.0.5125
-- --------------------------------------------------------

/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;
/*!40101 SET NAMES utf8 */;
/*!50503 SET NAMES utf8mb4 */;
/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;
/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO' */;

-- These procedures depend on ONLY_FULL_GROUP_BY. MySQL and MariaDB always use the sql_mode
-- That was active when the procedures were created, so this permanently sets it for these procs.
SET sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));

-- Set the default character set to latin1 for consistency across MariaDB and MySQL
SET NAMES 'latin1';

-- Dumping structure for procedure GrafanaDB.createBaselineCollection
DROP PROCEDURE IF EXISTS `createBaselineCollection`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `createBaselineCollection`(
	IN `userGroupId` CHAR(50),
	IN `userCollectionIds` BLOB,
	IN `userBaselineName` CHAR(40)

)
createBaselineProc:BEGIN  


/************************************************************************/
/*  IBM TEMPLATE: createBaselineCollection                              */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure takes in a list of collection IDs and creates a new   */
/* collection ID to be stored in the baseline table with averaged       */
/* values for all metrics.                                              */
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* userCollectionIds blob - The collection id of the collection         */
/*    to query against.                                                 */
/* userBaselineName varchar(40) - The name the user wishes to use to    */
/*    identify the baseline being collection being created.             */
/* userGroupId varchar(40) - The group id of the group being averaged.  */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* None                                                                 */
/* This procedure creates the data_for_group_table_baseline table if it */
/*    does not exists.                                                  */
/* This procedure updates the viewOfAllCollectionGroupIdPairs view to   */
/*    search for the data_for_group_table_baseline table as well.       */
/* This procedure averages the collections specified by the user and    */
/*    inputs them with a collection_id value equal to the               */
/*    userBaselineName into the data_for_group_table_baseline table.    */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* call createBaselineCollection(                                       */
/*   '264b86d1b37d24533618545db6c18eefe1fdd',                           */
/*   '093fbe...2908, 871b...2fea2, b23aa...3bf99',                      */
/*   'new_baseline_12'                                                  */
/* );                                                                   */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/

/* Procedure "Global" variables */
DECLARE allColumnsInDataTable VARCHAR(5000);
DECLARE columnMetricName VARCHAR(1000);
DECLARE queryString VARCHAR(15000);
DECLARE baselineGroupId VARCHAR(12);
/* While loop limits */
DECLARE n INT DEFAULT 0;
DECLARE i INT DEFAULT 0;
/* input read variables */
DECLARE stringLength INT DEFAULT 0;
DECLARE subStringLength INT DEFAULT 0;
DECLARE readCollectionId VARCHAR(50);
DECLARE fromGroupTableX VARCHAR(54); 
DECLARE whereCollectionsAreY VARCHAR(1000);

/***********************************************/
/* Create Baseline group table - if not exists */
/***********************************************/
/*
`baseline_id` INT NOT NULL,
	`group_id` CHAR(40) NULL DEFAULT NULL,
	`dimension_id` VARCHAR(255) NULL DEFAULT NULL,
	`description` VARCHAR(144) NULL DEFAULT NULL,
	`irow_type` VARCHAR(10) NOT NULL,
	`message_count` INT(11) NULL DEFAULT NULL,
	*/


CREATE TABLE IF NOT EXISTS  `data_for_group_table_baseline` (
	`collection_id` CHAR(40) NOT NULL,
   `tape_number` INT(11) NULL DEFAULT NULL,
	`dimension_id` VARCHAR(255) NULL DEFAULT NULL,
	`irow_type` VARCHAR(10) NOT NULL,
	`message_count` INT(11) NULL DEFAULT NULL,
	`FIND` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`FILE` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`SERR` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`SWBK` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`GRFS` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`CMBK` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`SYSH` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`CRET` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`GFSS` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`GFSL` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`RELF` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`ROUT` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`TOUR` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`TWRT` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`SNAP` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`COW` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`FIND_DASD` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`FILE_DASD` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`FIND_VFA` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`FILE_VFA` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`MAL` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`HCTL` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBADD` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBADR` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBCKP` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBCLS` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBCPY` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBCRE` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBDEL` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBDIX` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBDSP` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBIDX` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBKEY` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBMOD` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBMRG` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBOPN` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBRED` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBREP` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBRET` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBRST` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBSPA` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBSRT` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBTLD` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBTLG` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBTRD` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBUKY` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBHOLD` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DBUHLD` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_FILNC` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_FILEC` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`NONE` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_CFILE` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_PFILE` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_TWRTC` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_GETFC` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_RELFC` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_DETAC` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_ATTAC` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_FSYSW` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_GETCC` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_RELCC` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_CFIND` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_PFIND` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_BTFIL` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_BTFND` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_LRFIL` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_LRFND` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_OPN_HD` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_OPN_NHD` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_OPN_HND` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_OPN_NHND` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_CLS_RELS` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_CLS_NO_REL` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_CLS_REUSE` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_CLS_OPN` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_PREFP` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DF_PREFPW` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`NONE1` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`DD_TIME` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`CPU_USED` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`EXIST_TIME` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`ECB1MF` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL,
	`ECB4KF` DOUBLE(22,2) UNSIGNED NULL DEFAULT NULL
)
COLLATE='latin1_swedish_ci'
ENGINE=InnoDB;

/* refresh the views */
call updateViews();

/***********************************************/
/* Parse Input                                 */
/***********************************************/

/* Input error handling */
IF userCollectionIds IS NULL OR userCollectionIds = '' OR userCollectionIds = ' ' THEN
	LEAVE createBaselineProc;
END IF;

IF userGroupId IS NULL OR userGroupId = '' OR userGroupId = ' ' THEN
	LEAVE createBaselineProc;
END IF;

IF userBaselineName IS NULL OR userBaselineName = '' OR userBaselineName = ' ' THEN
	LEAVE createBaselineProc;
END IF;

IF EXISTS (SELECT DISTINCT * FROM data_for_group_table_baseline WHERE collection_id = userBaselineName) THEN
	SELECT "[Error] Name already in use. Please select another name.";
	LEAVE createBaselineProc;
END IF;

/* Initialize variables and read from user inputs */
SET fromGroupTableX = CONCAT("data_for_group_table_", userGroupId);

SET whereCollectionsAreY = " FALSE ";
read_input:
	LOOP
		SET stringLength = LENGTH(userCollectionIds);	
		SET readCollectionId = SUBSTRING_INDEX(userCollectionIds, ',', 1);
		/* Read next collection ID, use that to set the "collection_id=x" value (used in the WHERE segment) */
		SET whereCollectionsAreY = CONCAT(whereCollectionsAreY, " OR userSelectedGroup.collection_id = TRIM('", readCollectionId, "') ");
		SET subStringLength = LENGTH(SUBSTRING_INDEX(userCollectionIds, ',', 1))+2; /* plus 2 to move past the next "," */
		SET userCollectionIds = MID(userCollectionIds, subStringLength, stringLength);
	IF userCollectionIds = NULL OR userCollectionIds = "" THEN
      LEAVE read_input;
   END IF;
END LOOP read_input;

/***********************************************/
/* Create and exicute baseline group content   */
/***********************************************/

/* Zero out */
SET allColumnsInDataTable = "";
SET columnMetricName = "";
SET queryString = "";
SET n = 0;
SET i = 5; /* while loop will start at 4 due to unique columns. */

/* fromGroupTableX Issue, Part1: The only way it would work was to be used within the below EXEC, normal procedural queries didn't work (see Part 1). */
CALL tpf_execute_immediate(CONCAT('SELECT GROUP_CONCAT(COLUMN_NAME) INTO @allColumnsInDataTable_exec from INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = "', fromGroupTableX, '" AND TABLE_SCHEMA = (SELECT DATABASE())'));
set allColumnsInDataTable = @allColumnsInDataTable_exec;

/* fromGroupTableX Issue, Part2a: It works here, just fine (see Part 1). */
SET n = (SELECT COUNT(*) from INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = fromGroupTableX AND TABLE_SCHEMA = (SELECT DATABASE()));
SET queryString = CONCAT("INSERT INTO data_for_group_table_baseline ( ", allColumnsInDataTable);
SET queryString = CONCAT(queryString, ") SELECT '",userBaselineName,"' AS collection_id, NULL as tape_number, dimension_id, irow_type, AVG(message_count)");

WHILE i<n DO 
	/* fromGroupTableX Issue, Part2b: It works here, just fine (see Part 1). */
	SET columnMetricName = (SELECT COLUMN_NAME from INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = fromGroupTableX AND TABLE_SCHEMA = (SELECT DATABASE()) LIMIT i,1);
	/* Build averaged metric select entry. Place in query statement string. */
	SET queryString = CONCAT(queryString, ", (SUM(message_count * ");
	SET queryString = CONCAT(queryString, columnMetricName);
	SET queryString = CONCAT(queryString, " ) / SUM(message_count)) AS ");
	SET queryString = CONCAT(queryString, columnMetricName);
	SET i = i + 1;
END WHILE;

CALL tpf_execute_immediate(CONCAT(queryString, " FROM ", fromGroupTableX," AS userSelectedGroup WHERE ", whereCollectionsAreY, " GROUP BY dimension_id"));

END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.getBaseCollectionList
DROP PROCEDURE IF EXISTS `getBaseCollectionList`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `getBaseCollectionList`()
getBaseCollectionListProc:BEGIN
/************************************************************************/
/* IBM TEMPLATE: getBaseCollectionList                                  */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used to used to call                               */
/* getBaseCollectionList to get a list of collections in the            */
/* data_for_group_table_baseline table.                                 */ 
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* None                                                                 */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (1 column):                                                    */
/* collection_id - List of the collection_ids in the baseline table     */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* CALL getBaseCollectionList();                                        */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/ 
/*  Returns strings for some error checking.                            */
/*                                                                      */
/*                                                                      */
/************************************************************************/
IF EXISTS (SELECT TABLE_NAME from INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = "data_for_group_table_baseline" AND TABLE_SCHEMA = (SELECT DATABASE())) THEN
	IF NOT EXISTS (SELECT * FROM data_for_group_table_baseline) THEN
		SELECT "[Error] data_for_group_table_baseline is empty.";
		LEAVE getBaseCollectionListProc;
	END IF;
	SELECT DISTINCT collection_id 
	FROM data_for_group_table_baseline
	ORDER BY collection_id ASC;
	LEAVE getBaseCollectionListProc;
ELSE
	SELECT "[Error] data_for_group_table_baseline does not exist, run createBaseLineCollection.";
END IF;
END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.getCollectionHorizontals
DROP PROCEDURE IF EXISTS `getCollectionHorizontals`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `getCollectionHorizontals`(
	IN `userGroupId` VARCHAR(50)

)
BEGIN
/************************************************************************/
/* IBM TEMPLATE: getCollectionHorizontals                               */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used to get a list of horizontal names used in the */
/* group specified.                                                     */ 
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* userGroupId varchar(50) - (Required) The group id of the group being */
/*    compared.                                                         */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (1 column):                                                    */
/* name_string - List of the horizontal names in the group requested    */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* CALL getCollectionHorizontals(                                       */
/*    "310987fea03987198c371931237caf"                                  */
/*    );                                                                */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/ 
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
SELECT name_string AS NameTable 
FROM name_table 
WHERE  relationship_type='horizontal'
AND name_id IN
	(SELECT name_id FROM relationship_table  WHERE relationship_type='group' 
	AND relationship_id IN (SELECT relationship_id FROM group_table WHERE group_id = userGroupId));
END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.getCollectionList
DROP PROCEDURE IF EXISTS `getCollectionList`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `getCollectionList`(
	IN `userGroupId` VARCHAR(50)

)
BEGIN
/************************************************************************/
/* IBM TEMPLATE: getCollectionList                                    */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used to to get a list of collections in the        */
/* group specified.                                                     */ 
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* userGroupId varchar(50) - (Required) The group id of the group being */
/*    compared.                                                         */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (4 columns):                                                   */
/* collection_id - the Id hash of the collection.                       */
/* collection_date_time - the date and time, with seconds, of when the  */
/*    collection was collected.                                         */
/* messages_collected - number of messages collected during collection. */
/* messages_skipped - number of messages skipped during collection.     */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* CALL getCollectionList(                                              */
/*    "310987fea03987198c371931237caf"                                  */
/*    );                                                                */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/ 
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
SELECT 
	collection_id,CONCAT(collection_date, " ",collection_time) AS collection_UTCDateTime, ecbs_collected, ecbs_skipped 
FROM 
	collection_table 
WHERE 
	group_id = userGroupId
ORDER BY 
        collection_date DESC,collection_time DESC;
END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.getCollectionVerticals
DROP PROCEDURE IF EXISTS `getCollectionVerticals`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `getCollectionVerticals`(
	IN `userGroupId` VARCHAR(50)

)
BEGIN
/************************************************************************/
/* IBM TEMPLATE: getCollectionVerticals                                 */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used to get a list of vertical names used in the   */
/* group specified.                                                     */ 
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* userGroupId varchar(50) - (Required) The group id of the group being */
/*    compared.                                                         */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (1 column):                                                    */
/* name_string - List of the vertical names in the group requested      */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* CALL getCollectionVerticals(                                         */
/*    "310987fea03987198c371931237caf"                                  */
/*    );                                                                */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/ 
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
SELECT 
	name_string AS NameTable 
FROM 
	name_table 
WHERE  
	relationship_type='vertical'
AND name_id IN(
	SELECT 
		name_id 
	FROM 
		relationship_table  
	WHERE 
		relationship_type='group' AND 
		relationship_id IN (
			SELECT 
				relationship_id 
			FROM 
				group_table 
			WHERE 
				group_id = userGroupId
		)
);
END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.getDimensionInfo
DROP PROCEDURE IF EXISTS `getDimensionInfo`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `getDimensionInfo`(
	IN `userGroupId` VARCHAR(50),
	IN `collectionId` VARCHAR(50)





)
BEGIN

DECLARE metricsStr VARCHAR(300);
SET metricsStr = "SELECT * FROM ";
SET metricsStr = CONCAT(metricsStr,"  data_for_group_", userGroupId," "); 
SET metricsStr = CONCAT(metricsStr," WHERE collection_id='",collectionID, "'");

CALL tpf_execute_immediate(metricsStr);

SELECT metricsStr;

SET metricsStr = "CREATE TEMPORARY TABLE dimensionNames AS SELECT * FROM ";
SET metricsStr = CONCAT(metricsStr,"  data_for_group_", userGroupId," "); 
SET metricsStr = CONCAT(metricsStr," WHERE collection_id='",collectionID, "'");

CALL tpf_execute_immediate(metricsStr);

SELECT metricsStr;


SELECT * FROM dimensionNames;

DROP TEMPORARY TABLE dimensionNames;

END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.getFilterQuery
DROP PROCEDURE IF EXISTS `getFilterQuery`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `getFilterQuery`(
	INOUT `filterQuery` BLOB,
	IN `horizontalInput` BLOB,
	IN `verticalInput` BLOB,
	INOUT `verticalCount` INT






)
BEGIN
/************************************************************************/
/* IBM TEMPLATE: getFilterQuery                                         */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used to generate SQL that will obtain the          */
/* dimension IDs that correspond to this filter. The caller needs to    */
/* complete the WHERE clause.                                           */ 
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* filterQuery blob - (Required) The filter query that is returned to   */
/*    the caller.                                                       */
/* horizontalInput varchar(100) - (Optional) A list of horizontal name  */
/*    value pairs the caller wants to break the data down by.           */
/* verticalInput varchar(100) - (Optional) A list of vertical name      */
/*    value pairs the caller wants to break the data down by.           */
/* verticalCount int - (Required) - The number of vertical filters.     */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* None                                                                 */
/* Returns the filter query string in filterQuery INOUT variable        */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* CALL getFilterQuery(                                                 */
/*    ""                                                                */
/*   '',                                                                */
/*   'IOwnerHi=PackageA',                                               */
/*    1                                                                 */
/*    );                                                                */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/ 
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/

DECLARE filterStr BLOB(800);
DECLARE getRelationStr BLOB(500);
DECLARE getDimensionIdStr BLOB(500);
DECLARE curFilter BLOB(500);
DECLARE verticalCountStr BLOB(500);
DECLARE separatorLoc INT;
DECLARE firstItem BOOLEAN DEFAULT TRUE;
DECLARE lastItem BOOLEAN DEFAULT FALSE;
DECLARE fieldName CHAR(200);
DECLARE fieldValue CHAR(200);
DECLARE curMetric CHAR(200);
DECLARE done INT DEFAULT FALSE;
DECLARE colGroupTotal INT;
DECLARE resultCount INT;

SET @verticalCount = 0;
SET @verticalCountStr = "";

/* Only get the vertical if this was provided by the user. */        
IF verticalInput IS NOT NULL AND verticalInput != "NULL" AND verticalInput != "" THEN 
	CALL getMetricsVerticals(verticalInput,@verticalCountStr,@verticalCount);
END IF;


SET getDimensionIdStr = "(SELECT DISTINCT dimension_id FROM dimension_table ";

IF horizontalInput IS NOT NULL AND horizontalInput != "NULL" AND horizontalInput != "" THEN 
rel_loop: LOOP
	IF firstItem = TRUE  THEN
		SET getRelationStr = "WHERE "; 
	ELSE 
		SET getRelationStr = "AND "; 
	END IF;
	SET getRelationStr = concat(getRelationStr," horizontal_relationship_id IN (SELECT relationship_id "); 
	SET getRelationStr = concat(getRelationStr," FROM viewOfRelationshipTableWithNameAndValueStrings"); 	
	SET separatorLoc = LOCATE(',',horizontalInput);
	IF separatorLoc = 0 THEN
		SET curFilter = horizontalInput;
		SET lastItem = TRUE;
	ELSE
		SET curFilter = SUBSTRING_INDEX(horizontalInput, ',',1);
		SET horizontalInput = SUBSTRING(horizontalInput, (separatorLoc + 1));		
	END IF;
	SET fieldName  = SUBSTRING_INDEX(curFilter, '=',1);
	SET fieldValue  = SUBSTRING_INDEX(curFilter, '=',-1);
	SET fieldName  = TRIM(fieldName);
	SET fieldValue  = TRIM(fieldValue);
	SET getRelationStr = CONCAT(getRelationStr," WHERE name_string='", fieldName,"'");
	SET getRelationStr = CONCAT(getRelationStr," AND value_string='", fieldValue,"') ");		
	SET getDimensionIdStr = CONCAT(getDimensionIdStr," ", getRelationStr);
	SET firstItem = FALSE;
	IF lastItem = TRUE THEN
		LEAVE rel_loop;	
	END IF;
END LOOP rel_loop;
END IF;

IF @verticalCount > 0 THEN
	IF horizontalInput IS NOT NULL AND horizontalInput != "NULL"  AND horizontalInput != "" THEN
		SET getDimensionIdStr = CONCAT(getDimensionIdStr," AND  vertical_relationship_id IN (", @verticalCountStr, ")");
	ELSE 
		SET getDimensionIdStr = CONCAT(getDimensionIdStr," WHERE vertical_relationship_id IN (", @verticalCountStr, ")");
	END IF;	
	SET getDimensionIdStr = CONCAT(getDimensionIdStr,") AS infoTable USING (dimension_id)  WHERE ");
END IF;

IF @verticalCount = 0 THEN
	/* Assumed there is at least one horizontal in this case, since this isn't called when no filters are specified. */
	SET getDimensionIdStr = CONCAT(getDimensionIdStr," AND vertical_relationship_id IS NULL ) AS infoTable USING (dimension_id)  WHERE ");
	SET getDimensionIdStr = CONCAT(getDimensionIdStr," irow_type = 'horizontal' AND ");
END IF;

SET filterQuery = getDimensionIdStr;

END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.getGroupTable
DROP PROCEDURE IF EXISTS `getGroupTable`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `getGroupTable`()
BEGIN
/************************************************************************/
/* IBM TEMPLATE: getGroupTable                                         */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used to find the groups listed in the group table. */ 
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* filterQuery blob - (Required) The filter query that is returned to   */
/*    the caller.                                                       */
/* horizontalInput varchar(100) - (Optional) A list of horizontal name  */
/*    value pairs the caller wants to break the data down by.           */
/* verticalInput varchar(100) - (Optional) A list of vertical name      */
/*    value pairs the caller wants to break the data down by.           */
/* verticalCount int - (Required) - The number of vertical filters.     */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (2 columns):                                                   */
/* group_id - The group id hash.                                        */
/* group_string - The string name of the group.                         */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* CALL getGroupTable();                                                */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/ 
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
SELECT
    group_id,group_string
FROM
    group_table;
END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.getKeyMetrics
DROP PROCEDURE IF EXISTS `getKeyMetrics`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `getKeyMetrics`(
	IN `userHorizontalFilter` varchar(100),
	IN `userVerticalFilter` VARCHAR(200),
	IN `userMetricsList` VARCHAR(100),
	IN `userCollectionId` CHAR(50),
	IN `userBaselineCollectionId` CHAR(50),
	INOUT `inoutFoundResults` INT















)
keyMetricsProc:BEGIN
/************************************************************************/
/* IBM TEMPLATE: getKeyMetrics                                        */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* Call getFilterQuery to get the dimension Ids for the given filters.  */
/* Call this procedure for both the collection ID and the baseline      */
/* collection ID. Two tables are created to contain the data_for_group  */
/* rows that  match the filter criteria. These are filterTable  and     */
/* baselineTable. For each  metric, read the value from the filterTable */
/* and the baselineTable, and calculate the delta. This will be         */
/* displayed to the user.                                               */
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* userCollectionId varchar(50) - (Required) The collection id of the   */
/*    collection to query against.                                      */
/* userMetricsList varchar(100) - (Required) A list of comma seperated  */
/*    metrics who's information is desired by the user.                 */
/* userBaselineCollectionId varchar(50) - (Required) The collection id  */
/*    of the second collection to compared against the first. This is   */
/*    typically a baseline collection.                                  */
/* userHorizontalFilter varchar(100) - (Optional) This is the list of   */
/*    horizontal NVPs the user wants to breakdown percentage by.        */
/* userVerticalFilter varchar(100) - (Optional) This is the list of     */
/*    vertical NVPs the user wants to breakdown percentage by.          */
/* inoutFoundResults int - return code handler                          */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (4 columns):                                                   */
/* metric - The metric being queried                                    */
/* current_value - the metric's value related to the collectionId       */
/* baseline_value - the metric's value related to the baseCollectionId  */
/* delta - baseline_value minus current_value                           */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* CALL getKeyMetrics(                                                  */
/*    "Msgtype=Shopping,MsgOriginType=Mobile",                          */
/*    "IownerMid=Phase2",                                               */
/*    "FIND,SWBK",                                                      */
/*    "310987fea03987198c371931237caf",                                 */
/*    "09861df8739f36eac8790b87fde08b",                                 */
/*    "");                                                              */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/ 
/* inoutFoundResults will return a return code that lets the caller     */
/* know if results were successful or not. This is used to help         */
/* metabase display useful data.                                        */
/*                                                                      */
/*                                                                      */
/************************************************************************/
/*


*/
DECLARE filterGroupDataStr BLOB(800);
DECLARE filterBaselineDataStr BLOB(800);
DECLARE filterStr BLOB(800);
DECLARE filterTotalsDataStr BLOB(800);
DECLARE filterQueryStr BLOB(800);
DECLARE baseFilterStr BLOB(800);
DECLARE separatorLoc INT;
DECLARE curMetric CHAR(200);
DECLARE done INT DEFAULT FALSE;
DECLARE colTotal INT;
DECLARE baseTotal INT;
DECLARE metricsCopy CHAR(100);
DECLARE deltaQueryString BLOB(800);
DECLARE compareColIDString BLOB(500);
DECLARE baseColIDString BLOB(500);
DECLARE filterCount INT;
DECLARE tempTotal INT;
DECLARE curGroupId  CHAR(40);
DECLARE baseGroupId CHAR(40);
DECLARE baseTableName CHAR(100);
DECLARE dbName CHAR (100);
DECLARE baselineGroupString CHAR(100);
DECLARE groupIdString VARCHAR(100);
DECLARE baseCollection CHAR (40);
DECLARE foundInColTable INT DEFAULT FALSE;
DECLARE colTableEntry CHAR(40);
DECLARE hasFilters BOOLEAN DEFAULT FALSE;

/*
DECLARE hasHorizontals BOOLEAN DEFAULT FALSE;
DECLARE hasVerticals BOOLEAN DEFAULT FALSE;
*/
DECLARE getGroupStr CHAR(150);
DECLARE cur1 CURSOR FOR SELECT SUM(message_count) FROM filterTable;
DECLARE cur2 CURSOR FOR SELECT SUM(message_count) FROM baselineTable;

/* Error handling for input parameters so that we fail 
   gracefully if we do not have the necessary input.
   If the required input parameters are not specified,
   return an empty table so that the display looks nice.
   If the collection ID is the same as the base collection ID, then report an error.
*/

DROP TEMPORARY TABLE IF EXISTS filterTable;
DROP TEMPORARY TABLE IF EXISTS baselineTable;
DROP TEMPORARY TABLE IF EXISTS resultsTable;

SET metricsCopy = userMetricsList;
SET @filterCount = 0;
SET @filterQueryStr = "";
SET inoutFoundResults = 1;

call updateViews();

SET curGroupId = (select group_id FROM viewOfAllCollectionGroupIdPairs WHERE collection_id = userCollectionId);
IF curGroupId IS NULL THEN
	SELECT "Requested collection not found in the collection table.";
	LEAVE keyMetricsProc;
END IF;
SET groupIdString = CONCAT("data_for_group_table_", curGroupId);

SET baseGroupId = (select group_id FROM viewOfAllCollectionGroupIdPairs WHERE collection_id = userBaselineCollectionId);
IF baseGroupId  IS NULL THEN
		SELECT "Requested baseline collection not found in the collection table.";
		LEAVE keyMetricsProc;
END IF;
SET baselineGroupString = CONCAT("data_for_group_table_",baseGroupId);


IF (LENGTH(userVerticalFilter)>0 AND userVerticalFilter IS NOT NULL AND userVerticalFilter != "NULL") OR
   (LENGTH(userHorizontalFilter)>0 AND userHorizontalFilter IS NOT NULL AND userHorizontalFilter != "NULL") THEN 	
	SET hasFilters = TRUE;
END IF;


SET filterStr = "CREATE TEMPORARY TABLE IF NOT EXISTS filterTable AS SELECT *";
SET filterStr = CONCAT(filterStr, " FROM ");

IF hasFilters = TRUE THEN 	
	CALL getFilterQuery(@filterQueryStr,userHorizontalFilter,userVerticalFilter,@filterCount);
END IF;

IF hasFilters = TRUE THEN 	
	SET filterStr = CONCAT(filterStr, groupIdString, " INNER JOIN ");
	SET filterStr = CONCAT(filterStr, @filterQueryStr, " collection_id='",userCollectionId);
	SET filterStr = CONCAT(filterStr,"' AND ",groupIdString, ".dimension_id=infoTable.dimension_id; ");
ELSE 
	SET filterStr = CONCAT(filterStr, groupIdString, "  ");
	SET filterStr = CONCAT(filterStr," WHERE irow_type = 'horizontal' ");
	SET filterStr = CONCAT(filterStr," AND collection_id='",userCollectionId, "';");
END IF;

CALL tpf_execute_immediate(filterStr);

SET baseFilterStr = "CREATE TEMPORARY TABLE IF NOT EXISTS baselineTable AS SELECT *";
IF hasFilters = TRUE THEN 	
	SET baseFilterStr = CONCAT(baseFilterStr , " FROM ",baselineGroupString, " INNER JOIN");
	SET baseFilterStr = CONCAT(baseFilterStr, @filterQueryStr, " collection_id ='",userBaselineCollectionId);
	SET baseFilterStr = CONCAT(baseFilterStr,"' AND ",baselineGroupString, ".dimension_id = infoTable.dimension_id; ");
ELSE 
	SET baseFilterStr = CONCAT(baseFilterStr , " FROM ",baselineGroupString);
	SET baseFilterStr = CONCAT(baseFilterStr," WHERE irow_type = 'horizontal' ");
	SET baseFilterStr = CONCAT(baseFilterStr," AND collection_id='",userBaselineCollectionId, "';");
END IF;

CALL tpf_execute_immediate(baseFilterStr);

OPEN cur1;
	FETCH cur1 INTO colTotal;		
CLOSE cur1;
OPEN cur2;
	FETCH cur2 INTO baseTotal;		
CLOSE cur2;

IF colTotal IS NULL THEN
	SET inoutFoundResults = 1;
	DROP TEMPORARY TABLE IF EXISTS filterTable;
	DROP TEMPORARY TABLE IF EXISTS baselineTable;
	LEAVE keyMetricsProc;
END IF;

IF baseTotal IS NULL THEN
	SET inoutFoundResults = 1;
	DROP TEMPORARY TABLE IF EXISTS filterTable;
	DROP TEMPORARY TABLE IF EXISTS baselineTable;
	LEAVE keyMetricsProc;
END IF;

/*
This loop will go through each requested metric and 
generate the SQL that will read from the two created tables and calcuate the delta.
We will UNION the SQL for each metric.
This will be written to the results table.
*/


SET filterTotalsDataStr = "SELECT collection_id, ";
SET filterGroupDataStr = " SELECT collection_id,message_count,";
metric_loop: LOOP	
	SET separatorLoc = LOCATE(',',userMetricsList);
	IF separatorLoc = 0 THEN
		SET @tempTotal = colTotal + baseTotal;
		SET curMetric = userMetricsList;
		SET filterGroupDataStr = CONCAT(filterGroupDataStr, " (", curMetric, " * message_count");		
		SET filterGroupDataStr = CONCAT(filterGroupDataStr, ")  AS " , curMetric, " ");
		SET filterTotalsDataStr = CONCAT(filterTotalsDataStr, " (SUM( ", curMetric, ") / SUM(message_count)) AS ", curMetric);
		LEAVE metric_loop;	
	ELSE 
		SET curMetric = SUBSTRING_INDEX(userMetricsList, ',',1);
		SET userMetricsList = SUBSTRING(userMetricsList, (separatorLoc + 1));		
	END IF;
	SET filterGroupDataStr = CONCAT(filterGroupDataStr, " (", curMetric, "");
	SET filterGroupDataStr = CONCAT(filterGroupDataStr, " * message_count ",  ") AS " , curMetric, ", ");	
	SET filterTotalsDataStr = CONCAT(filterTotalsDataStr, " (SUM( ", curMetric, ") / SUM(message_count)) AS ",curMetric,",");
END LOOP metric_loop;

SET filterBaselineDataStr = filterGroupDataStr;
SET filterBaselineDataStr = CONCAT(filterGroupDataStr, " FROM baselineTable ");
SET filterGroupDataStr = CONCAT(filterGroupDataStr, " FROM filterTable");
SET filterGroupDataStr = CONCAT(filterGroupDataStr, " UNION ", filterBaselineDataStr);
SET filterTotalsDataStr = CONCAT(filterTotalsDataStr," FROM (",filterGroupDataStr,") AS totalsTableDer GROUP BY collection_id;");
SET filterGroupDataStr = CONCAT("CREATE TEMPORARY TABLE IF NOT EXISTS resultsTable AS ",filterTotalsDataStr, ";");

CALL tpf_execute_immediate(filterGroupDataStr);


SET deltaQueryString = "";
delta_loop: LOOP
	SET separatorLoc = LOCATE(',',metricsCopy);
	IF separatorLoc = 0 THEN
		SET curMetric = metricsCopy;	
	ELSE 
		SET curMetric = SUBSTRING_INDEX(metricsCopy, ',',1);
		SET metricsCopy = SUBSTRING(metricsCopy, (separatorLoc + 1));				
	END IF;	
	
	SET deltaQueryString = CONCAT(deltaQueryString, " SELECT '", curMetric, "' AS METRIC, ");		

	SET compareColIDString = "(SELECT ";
	SET compareColIDString = CONCAT(compareColIDString," ", curMetric, " FROM resultsTable ");
	SET compareColIDString = CONCAT(compareColIDString,  " where collection_id = '", userCollectionId,"') ");
	
	SET baseColIDString = " (SELECT ";
	SET baseColIDString = CONCAT(baseColIDString," ",  curMetric, " FROM resultsTable ");
	SET baseColIDString = CONCAT(baseColIDString,  " where collection_id = '", userBaselineCollectionId, "')");

	SET deltaQueryString = CONCAT(deltaQueryString, "CAST(", compareColIDString, "  AS DECIMAL(17,2)) AS CURRENT_VALUE,");
	SET deltaQueryString = CONCAT(deltaQueryString, "CAST(", baseColIDString, " AS DECIMAL(17,2)) AS BASELINE_VALUE,");
	SET deltaQueryString = CONCAT(deltaQueryString, "CAST((", compareColIDString , " - ");
	SET deltaQueryString = CONCAT(deltaQueryString,  baseColIDString, " ) AS DECIMAL(17,2)) AS DELTA ");
		
	SET deltaQueryString = CONCAT(deltaQueryString, " FROM resultsTable GROUP BY METRIC ");		
	IF separatorLoc = 0 THEN
		LEAVE delta_loop;	
	END IF;
	SET deltaQueryString = CONCAT(deltaQueryString, " UNION ");
END LOOP delta_loop;

CALL tpf_execute_immediate(deltaQueryString);


DROP TEMPORARY TABLE IF EXISTS filterTable;
DROP TEMPORARY TABLE IF EXISTS baselineTable;
DROP TEMPORARY TABLE IF EXISTS resultsTable;

END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.getMetricsAverages
DROP PROCEDURE IF EXISTS `getMetricsAverages`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `getMetricsAverages`(
	IN `userHorizontalInput` BLOB,
	IN `userMetricsList` VARCHAR(100),
	IN `userGroupId` VARCHAR(50),
	IN `userVerticalInput` BLOB,
	INOUT `foundResults` INT




)
metricsProc:BEGIN
/************************************************************************/
/* IBM TEMPLATE: getMetricsAverages                                     */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used to find the averages for specified metrics    */
/* across a group.                                                      */
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* userGroupId varchar(50) - (Required) The group id of the group being */
/*    compared.                                                         */
/* userMetricsList varchar(100) - (Required) A list of comma seperated  */
/*    metrics who's information is desired by the user.                 */
/* userHorizontalInput varchar(100) - (Optional) This is the list of    */
/*    horizontal NVPs the user wants to breakdown percentage by.        */
/* userVerticalInput varchar(100) - (Optional) This is the list of      */
/*    vertical NVPs the user wants to breakdown percentage by.          */
/* foundResults int - a return code.                                    */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (veritable columns):                                           */
/* collection_date - the date and time the metrics were collected.      */
/* the name of the metric searched for - this name could be any valid   */
/*    metric specified in the user's query. Examples are FIND, SWBK,    */
/*    etc...                                                            */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* CALL getMetricsAverages(                                             */
/*    "Msgtype=Shopping,MsgOriginType=Mobile",                          */
/*    "FIND,SWBK",                                                      */
/*    "310987fea03987198c371931237caf",                                 */
/*    "IOwnerMid=Phase1",                                               */
/*    "");                                                              */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/ 
/* foundResults is used to return specific values depending on whether  */
/* or not results were found. This is meant to be used at the layer     */
/* above to assist metabase in displaying meaningful data.              */
/*                                                                      */
/*                                                                      */
/************************************************************************/
DECLARE filterGroupDataStr BLOB(800);
DECLARE getGroupDataStr BLOB(500);
DECLARE separatorLoc INT;
DECLARE curMetric CHAR(200);
DECLARE done INT DEFAULT FALSE;
DECLARE colGroupTotal INT;
DECLARE resultCount INT;
DECLARE tmpGroupID INT;
DECLARE filterQueryStr BLOB(800);
DECLARE filterTotalsDataStr BLOB(800);

DECLARE cur1 CURSOR FOR SELECT SUM(message_count) FROM filterTable;
DECLARE cur2 CURSOR FOR SELECT COUNT(*) FROM filterTable;
DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

SET foundResults = 0;
/* We are going to return an empty table when we do not have the correct input */
/* This is because it looks a lot nicer in the GUI */
DROP TEMPORARY TABLE IF EXISTS filterTable;

SET tmpGroupID  = (SELECT COUNT(group_id) from group_table where group_id = userGroupId);
IF tmpGroupID = 0 THEN
	SET foundResults = 1;
	LEAVE metricsProc;
END IF;

SET getGroupDataStr = "CREATE TEMPORARY TABLE IF NOT EXISTS filterTable AS SELECT *";
SET getGroupDataStr = CONCAT(getGroupDataStr, " FROM data_for_group_table_");
SET getGroupDataStr = CONCAT(getGroupDataStr, userGroupId);

/* Check for filter input to improve performance if no inputs are given */
IF (LENGTH(userVerticalInput)>0 AND userVerticalInput IS NOT NULL AND userVerticalInput != "NULL") OR
   (LENGTH(userHorizontalInput)>0 AND userHorizontalInput IS NOT NULL AND userHorizontalInput != "NULL") THEN 	
	SET getGroupDataStr = CONCAT(getGroupDataStr, " INNER JOIN ");
	SET @filterQueryStr = "";
	CALL getFilterQuery(@filterQueryStr,userHorizontalInput,userVerticalInput,@filterCount);
	SET getGroupDataStr = CONCAT(getGroupDataStr, @filterQueryStr, " data_for_group_table_",userGroupId);
	SET getGroupDataStr = CONCAT(getGroupDataStr,".dimension_id = infoTable.dimension_id ");
	SET getGroupDataStr = CONCAT(getGroupDataStr," ;");
ELSE
	SET getGroupDataStr = CONCAT(getGroupDataStr, " WHERE irow_type = 'horizontal' ");
END IF;

CALL tpf_execute_immediate(getGroupDataStr);

OPEN cur1;
		FETCH cur1 INTO colGroupTotal;		
CLOSE cur1;

OPEN cur2;
		FETCH cur2 INTO resultCount;		
CLOSE cur2;

IF resultCount = 0 THEN	
	DROP TEMPORARY TABLE filterTable;
	SET foundResults = 1;
	LEAVE metricsProc;
END IF;

IF colGroupTotal = 0 THEN	
	DROP TEMPORARY TABLE filterTable;
	SET foundResults = 1;
	LEAVE metricsProc;
END IF;

SET filterTotalsDataStr = "SELECT UNIX_TIMESTAMP(STR_TO_DATE(concat(collection_date, ' ',collection_time), '%Y-%m-%d %H:%i:%s')) as time, ";
SET filterGroupDataStr = "SELECT collection_id,message_count, ";

metric_loop: LOOP
	SET separatorLoc = LOCATE(',',userMetricsList);
	IF separatorLoc = 0 THEN
		SET curMetric = userMetricsList;
		SET filterGroupDataStr = CONCAT(filterGroupDataStr, "(",curMetric, " * message_count) AS ", curMetric);
		SET filterTotalsDataStr = CONCAT(filterTotalsDataStr, " (SUM( ", curMetric, ") / SUM(message_count)) AS ", curMetric );
		LEAVE metric_loop;	
	ELSE 
		SET curMetric = SUBSTRING_INDEX(userMetricsList, ',',1);
		SET userMetricsList = SUBSTRING(userMetricsList, (separatorLoc + 1));		
	END IF;
	SET filterGroupDataStr = CONCAT(filterGroupDataStr, "(",curMetric, " * message_count) AS ", curMetric, ",");
	SET filterTotalsDataStr = CONCAT(filterTotalsDataStr, " (SUM( ", curMetric, ") / SUM(message_count)) AS ", curMetric, "," );
END LOOP metric_loop;
SET filterGroupDataStr = CONCAT(filterGroupDataStr, " FROM filterTable ");
SET filterTotalsDataStr = CONCAT(filterTotalsDataStr, " FROM " );
SET filterTotalsDataStr = CONCAT(filterTotalsDataStr," collection_table INNER JOIN (",filterGroupDataStr);
SET filterTotalsDataStr = CONCAT(filterTotalsDataStr,") AS valTable  USING (collection_id) GROUP BY time;");

CALL tpf_execute_immediate(filterTotalsDataStr);

DROP TEMPORARY TABLE IF EXISTS filterTable;

END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.getMetricsList
DROP PROCEDURE IF EXISTS `getMetricsList`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `getMetricsList`(
	IN `userGroupId` VARCHAR(50)

)
BEGIN
/************************************************************************/
/* IBM TEMPLATE: mbGetMetricsList                                       */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used to get a list of all metrics in a group.      */
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* userGroupId varchar(50) - (Required) The group id of the group being */
/*    compared.                                                         */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (1 column):                                                    */
/* column_name - list of metrics were collected in that group.          */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* CALL mbGetMetricsList(                                               */
/*    "310987fea03987198c371931237caf"                                  */
/*    );                                                                */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/ 
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
DECLARE metricsStr VARCHAR(300);
SET metricsStr = "SELECT DISTINCT COLUMN_NAME ";
SET metricsStr = CONCAT(metricsStr," FROM INFORMATION_SCHEMA.COLUMNS WHERE table_schema = (SELECT DATABASE())");
SET metricsStr = CONCAT(metricsStr," AND table_name = 'data_for_group_table_", userGroupId,"' "); 
SET metricsStr = CONCAT(metricsStr," AND COLUMN_NAME NOT IN ('collection_id', 'tape_number', 'irow_type', 'message_count','dimension_id');");

CALL tpf_execute_immediate(metricsStr);

END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.getMetricsVerticals
DROP PROCEDURE IF EXISTS `getMetricsVerticals`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `getMetricsVerticals`(
	IN `verticalInput` BLOB,
	INOUT `verticalCountStr` BLOB,
	INOUT `verticalTotal` INT




)
BEGIN
/************************************************************************/
/* IBM TEMPLATE: getMetricsVerticals                                    */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure will generate SQL that will get the relationship ID   */
/* for the requested verticals. We use HAVING COUNT to ensure that the  */
/* number of rows for this relationship ID exactly matches the number   */
/* of verticals requested by the user. Once you find the relationship   */
/* ID that matches the requested verticals, get the count of the number */
/* of rows for this relationship ID. If the number of rows matches the  */
/* number of requested verticals, then this is the relationship ID that */
/* matches the filter.                                                  */
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* verticalInput blob - (Required) List of verticals to be used.        */
/* verticalCountStr blob - (Required) input is blank but the returned   */
/*    value is the query desired by the caller.                         */
/* verticalTotal int - (Required) input is blank but the returned is    */
/*    the number of verticals used.                                     */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* None.                                                                */
/* verticalCountStr is returned with proper query.                      */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* CALL getMetricsVerticals(                                            */
/*    "IOwnerMid=Phase1"                                                */
/*    "",                                                               */
/*    ""                                                                */
/*    );                                                                */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/ 
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
/*

*/
DECLARE verticalQueryStr BLOB(500);
DECLARE relationshipStr BLOB(500);
DECLARE curFilter BLOB(500);
DECLARE separatorLoc INT;
DECLARE firstItem BOOLEAN DEFAULT TRUE;
DECLARE lastItem BOOLEAN DEFAULT FALSE;
DECLARE fieldName CHAR(200);
DECLARE fieldValue CHAR(200);
DECLARE done INT DEFAULT FALSE;


DECLARE cur1 CURSOR FOR SELECT COUNT(*) FROM verticalList;
DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

SET verticalTotal = 0;

SET verticalQueryStr = "SELECT DISTINCT relationship_id  FROM viewOfRelationshipTableWithNameAndValueStrings ";
SET relationshipStr  = "SELECT DISTINCT relationship_id FROM viewOfRelationshipTableWithNameAndValueStrings ";
vertical_loop: LOOP
	IF firstItem = TRUE THEN
		SET relationshipStr = " WHERE "; 
		SET verticalTotal = 1;
	ELSE 
		SET relationshipStr = " AND "; 
		SET verticalTotal = verticalTotal + 1;
	END IF;
	SET separatorLoc = LOCATE(',',verticalInput);
	IF separatorLoc = 0 THEN
		SET curFilter = verticalInput;
		SET lastItem = TRUE;
	ELSE
		SET curFilter = SUBSTRING_INDEX(verticalInput, ',',1);
		SET verticalInput = SUBSTRING(verticalInput, (separatorLoc + 1));		
	END IF;
	
	SET relationshipStr = CONCAT(relationshipStr," relationship_ID IN (SELECT relationship_ID  FROM viewOfRelationshipTableWithNameAndValueStrings WHERE ");
	SET fieldName  = SUBSTRING_INDEX(curFilter, '=',1);
	SET fieldValue  = SUBSTRING_INDEX(curFilter, '=',-1);
        SET fieldName  = TRIM(fieldName);
	SET fieldValue  = TRIM(fieldValue);
	SET relationshipStr = CONCAT(relationshipStr," name_string='", fieldName,"'");
	SET relationshipStr = CONCAT(relationshipStr," AND value_string='", fieldValue,"') ");		
	SET firstItem = FALSE;
	SET verticalQueryStr = CONCAT(verticalQueryStr,relationshipStr);
	IF lastItem = TRUE THEN
		LEAVE vertical_loop;	
	END IF;
END LOOP vertical_loop;

SET verticalCountStr = CONCAT(verticalQueryStr," GROUP BY relationship_id HAVING COUNT(*) = ",verticalTotal);

END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.getNamesForDimensionTable
DROP PROCEDURE IF EXISTS `getNamesForDimensionTable`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `getNamesForDimensionTable`(
	IN `userGroupID` VARCHAR(50),
	IN `userCollectionID` VARCHAR(50),
	IN `userHorizontalInput` VARCHAR(250),
	IN `userVerticalInput` VARCHAR(250)
,
	IN `dimensionId` VARCHAR(50)






)
BEGIN
DECLARE nameField CHAR(50);
DECLARE valueField CHAR(50);
DECLARE nameListStr BLOB (500);
DECLARE getGroupDataStr BLOB (500);
DECLARE dataForGroupStr CHAR(200);
DECLARE doneNames INT DEFAULT FALSE;
DECLARE cur1 CURSOR FOR SELECT * FROM namesTable;
DECLARE CONTINUE HANDLER FOR NOT FOUND SET doneNames = TRUE;

DROP TEMPORARY TABLE IF EXISTS namesTable;
DROP TEMPORARY TABLE IF EXISTS dimNamesTable;
DROP TEMPORARY TABLE IF EXISTS filterTable;

SET getGroupDataStr = "CREATE TEMPORARY TABLE IF NOT EXISTS filterTable AS SELECT *";
SET getGroupDataStr = CONCAT(getGroupDataStr, " FROM data_for_group_");
SET getGroupDataStr = CONCAT(getGroupDataStr, userGroupId, " INNER JOIN ");
SET @filterQueryStr = "";
CALL getFilterQuery(@filterQueryStr,userHorizontalInput,userVerticalInput,@filterCount);
SET getGroupDataStr = CONCAT(getGroupDataStr, @filterQueryStr, " data_for_group_",userGroupId);
SET getGroupDataStr = CONCAT(getGroupDataStr,".dimension_id = infoTable.dimension_id ");
SET getGroupDataStr = CONCAT(getGroupDataStr," ;");

CALL tpf_execute_immediate(getGroupDataStr);

SELECT getGroupDataStr;


CREATE TEMPORARY TABLE namesTable AS
(SELECT name_string, value_string
FROM dimension_table inner JOIN relationship_table
on dimension_table.horizontal_relationship_id=relationship_table.relationship_id or dimension_table.vertical_relationship_id=relationship_table.relationship_id
INNER JOIN name_table USING (name_id) INNER JOIN value_table USING (value_id)
where dimension_id=dimensionID);

SET nameListStr = "CREATE TEMPORARY TABLE dimNamesTable AS SELECT ";
SET nameListStr = CONCAT(nameListStr," '", dimensionID, "' AS dimension_id");

OPEN cur1;
nvpLoop: LOOP
	FETCH cur1 INTO nameField,valueField;	
	IF doneNames THEN 
		LEAVE nvpLoop;
	END IF;
	SET nameListStr = CONCAT(nameListStr," ,'", valueField, "' AS ", nameField);
END LOOP;
CLOSE cur1;

CALL tpf_execute_immediate(nameListStr);

SELECT nameListStr;

SET dataForGroupStr = "SELECT * FROM dimNamesTable INNER JOIN filterTable";
SET dataForGroupStr = CONCAT(dataForGroupStr," USING (dimension_id) WHERE collection_id='",userCollectionID,"'");

CALL tpf_execute_immediate(dataForGroupStr);

SELECT dataForGroupStr;

DROP TEMPORARY TABLE IF EXISTS namesTable;
DROP TEMPORARY TABLE IF EXISTS dimNamesTable;
DROP TEMPORARY TABLE IF EXISTS filterTable;
END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.getPercentOfFilteredTrafficByCollection
DROP PROCEDURE IF EXISTS `getPercentOfFilteredTrafficByCollection`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `getPercentOfFilteredTrafficByCollection`(
	IN `userCollectionId` VARCHAR(50),
	IN `userHorizontal` BLOB,
	IN `userVertical` BLOB,
	IN `userBreakdown` VARCHAR(100)









)
getPercentOfFilteredTrafficByCollectionProc:BEGIN 
/************************************************************************/
/*  IBM TEMPLATE: getPercentOfFilteredTrafficByCollection               */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used to query results regarding a single           */
/* collection. Specifically this will find the VALUE results (relating  */
/* to the NAME specified in the BREAKDOWN input) and their corisponding */
/* message count and percentage. The percentage will be calculated by   */
/* taking the message count for that VALUE and dividing it by the total */
/* message count of the message traffic that occured in the FILTERed    */
/* traffic specified in the input.                                      */
/*                                                                      */
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* userHorizontal blob - This is the list of horizontal NVPs  the user  */
/*    wants to breakdown percentage by.                                 */
/* userVertical blob - This is the list of vertical NVPs the user wants */
/*    to breakdown percentage by.                                       */
/* userBreakdown varchar(100) - This is the NVP the user wants to       */
/*    breakdown percentages by.                                         */
/* userCollectionId varchar(50) - The collection id of the collection   */
/*    to query against.                                                 */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (5 columns):                                                   */
/* value_string - The Value portion of the Name Value Pair relating to  */
/*    the Name specified in the breakdown. In other words the Name in   */
/*    the breakdown is made up of these values.                         */
/* message_count - The number of messages for that given value_string.  */
/* collection_id - The collection id in which that value_string exists. */
/* total_message_count_by_collection - the total number of messages in  */
/*    that collection_id when the horizontal/vertical filters are taken */
/*    into account.                                                     */
/* percentage - message_count / total_message_count_by_collection * 100 */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* call getPercentOfFilteredTrafficByCollection(                        */
/*   '264b86d1b37d24533618545db6c18eefe1fdd',                           */
/*   'MsgType=Booking',                                                 */
/*   'IOwnerHi=PackageA',                                               */
/*   'IOwnerMid'                                                        */
/* );                                                                   */
/*                                                                      */
/* call getPercentOfFilteredTrafficByCollection(                        */
/*   'the-collection-id-hash',                                          */
/*   'horizontal-NVP-filter1,horizontal-NVP-filter2',                   */
/*   'vertical-NVP-filter1,vertical-NVP-filter2',                       */
/*   'breakdown'                                                        */
/* );                                                                   */
/*                                                                      */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/


/******************************************************************************/
/******************************** DECLARES ************************************/
/******************************************************************************/

DECLARE subStringLength                                           INT;
DECLARE stringLength                                              INT;
DECLARE relationship_string_subquery                              VARCHAR     (200);
DECLARE relationship_horizontal_string_subquery                   BLOB;
DECLARE relationship_vertical_string_subquery                     BLOB;
DECLARE select_horizontal_value_string                            VARCHAR     (20);
DECLARE select_vertical_value_string                              VARCHAR     (20);
DECLARE breakdown_where_vertical_value_string_is                  VARCHAR     (100);
DECLARE breakdown_where_horizontal_value_string_is                VARCHAR     (100);

DECLARE filtered_horizontal_pairs_subquery                        BLOB;
DECLARE filtered_vertical_pairs_subquery                          BLOB;
DECLARE filtered_horizontal_vertical_pairs_subquery               BLOB;
DECLARE filtered_traffic_dimension_ids_subquery                   BLOB;
DECLARE filtered_horizontal_traffic_where                         BLOB;
DECLARE filtered_vertical_traffic_where                           BLOB;

DECLARE breakdown_horizontal_vertical_pairs_subquery              BLOB;
DECLARE breakdown_traffic_dimension_ids_subquery                  BLOB;
DECLARE breakdown_where                                           BLOB;
DECLARE breakdown_horizontal_pairs_subquery                       BLOB;
DECLARE breakdown_vertical_pairs_subquery                         BLOB;
DECLARE breakdown_horizontal_traffic_where                        BLOB;
DECLARE breakdown_vertical_traffic_where                          BLOB;

DECLARE total_horizontal_vertical_pairs_subquery                  BLOB;
DECLARE total_traffic_dimension_ids_subquery                      BLOB;
DECLARE total_traffic_message_count_by_collection_id              BLOB;
DECLARE filtered_traffic_message_count_by_collection_id           BLOB;
DECLARE breakdown_traffic_message_count_by_collection_id          BLOB;

DECLARE data_for_group_X_table_collection                         VARCHAR     (100);

DECLARE filtered_and_breakdown_traffic_message_count_by_collection_id		BLOB;
DECLARE final_collection_query                                    BLOB;

DECLARE number_of_horizontal_filters                              INT         DEFAULT 0;
DECLARE number_of_vertical_filters                                INT         DEFAULT 0;

/* 0 - none        */
/* 1 - horizontal  */
/* 2 - vertical    */
DECLARE is_breakdown_name_string_in_filter                        BOOL        DEFAULT 0;
DECLARE collection_date_time                                      BLOB;

/******************************************************************************/
/******************************* Initialize ***********************************/
/******************************************************************************/

SET filtered_horizontal_traffic_where = "";
SET filtered_vertical_traffic_where = "";

/******************************************************************************/
/***************************** Input checking *********************************/
/******************************************************************************/

/* refresh views */
call updateViews();

IF !(EXISTS (SELECT collection_id FROM viewOfAllCollectionGroupIdPairs WHERE collection_id = userCollectionId)) THEN
	SELECT "Collection Id doesn't exist";
	LEAVE getPercentOfFilteredTrafficByCollectionProc;
END IF;

IF !(userBreakdown IS NULL OR userBreakdown = "" OR userBreakdown = "NULL") THEN
	CALL tpf_execute_immediate(CONCAT('SELECT relationship_type INTO @isBreakdownHorizontalOrVertical from name_table WHERE name_string = "', userBreakdown,'"'));
	SET breakdown_where = CONCAT("WHERE (name_string = '",TRIM(userBreakdown),"')");
ELSE
	LEAVE getPercentOfFilteredTrafficByCollectionProc;
END IF;

IF !(userCollectionId IS NULL OR userCollectionId = "" OR userCollectionId = "NULL") THEN
	SET data_for_group_X_table_collection = CONCAT("data_for_group_table_", (SELECT group_id from viewOfAllCollectionGroupIdPairs where collection_id = userCollectionId));
ELSE
	LEAVE getPercentOfFilteredTrafficByCollectionProc;
END IF;

/******************************************************************************/
/****************** save the date and time of the collection ******************/
/******************************************************************************/

CALL tpf_execute_immediate(CONCAT("SELECT
	STR_TO_DATE(concat(collection_date, ' ',collection_time), '%Y-%m-%d %H:%i:%s') INTO @collection_date_time
	FROM collection_table where collection_id='", userCollectionId, "'"));

/******************************************************************************/
/******************************** Parsing *************************************/
/******************************************************************************/

read_horizontal_input:
LOOP
	IF (userHorizontal IS NULL OR userHorizontal = "" OR userHorizontal = "NULL") THEN
		LEAVE read_horizontal_input;
	END IF;
	/* Start or continue where statement */
	IF (number_of_horizontal_filters <= 0) THEN
		SET filtered_horizontal_traffic_where = " WHERE ( ";
	ELSE
		SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, " OR "); 
	END IF;
	SET number_of_horizontal_filters = number_of_horizontal_filters + 1;
	SET stringLength = LENGTH(userHorizontal);
	/* start to "=" */
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, " ( name_string = '");
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, TRIM(SUBSTRING_INDEX(userHorizontal, '=', 1)));
	IF (userBreakdown = TRIM(SUBSTRING_INDEX(userHorizontal, '=', 1))) THEN
		SET is_breakdown_name_string_in_filter = 1;
	END IF;
	SET subStringLength = LENGTH(SUBSTRING_INDEX(userHorizontal, '=', 1))+2;
	SET userHorizontal = MID(userHorizontal, subStringLength, stringLength);
	/* "=" to "," */
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, "' AND value_string = '");
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, TRIM(SUBSTRING_INDEX(userHorizontal, ',', 1)));
	SET subStringLength = LENGTH(SUBSTRING_INDEX(userHorizontal, ',', 1))+2;
	SET userHorizontal = MID(userHorizontal, subStringLength, stringLength);
	/* close () */
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, "' ) ");
END LOOP read_horizontal_input;
SET breakdown_horizontal_traffic_where = filtered_horizontal_traffic_where;
IF ( LENGTH(filtered_horizontal_traffic_where) > 0 ) THEN
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, " ) AND cardinality >=",number_of_horizontal_filters," ");
END IF;

IF !(userVertical IS NULL OR userVertical = "" OR userVertical = "NULL") THEN
	read_vertical_input:
	LOOP
		IF (userVertical IS NULL OR userVertical = "" OR userVertical = "NULL") THEN
			LEAVE read_vertical_input;
		END IF;
		/* Start or continue where statement */
		IF (number_of_vertical_filters <= 0) THEN
			SET filtered_vertical_traffic_where = " WHERE ( ";
		ELSE
			SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, " OR "); 
		END IF;
		SET number_of_vertical_filters = number_of_vertical_filters + 1;
		SET stringLength = LENGTH(userVertical);
		/* start to "=" */
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, " ( name_string = '");
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, TRIM(SUBSTRING_INDEX(userVertical, '=', 1)));
		IF (userBreakdown = TRIM(SUBSTRING_INDEX(userVertical, '=', 1))) THEN
			SET is_breakdown_name_string_in_filter = 2;
		END IF;
		SET subStringLength = LENGTH(SUBSTRING_INDEX(userVertical, '=', 1))+2;
		SET userVertical = MID(userVertical, subStringLength, stringLength);
		/* "=" to "," */
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, "' AND value_string = '");
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, TRIM(SUBSTRING_INDEX(userVertical, ',', 1)));
		SET subStringLength = LENGTH(SUBSTRING_INDEX(userVertical, ',', 1))+2;
		SET userVertical = MID(userVertical, subStringLength, stringLength);
		/* close () */
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, "' ) ");
	END LOOP read_vertical_input;
END IF;
SET breakdown_vertical_traffic_where = filtered_vertical_traffic_where;
IF ( LENGTH(filtered_vertical_traffic_where) > 0 ) THEN
	SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, " ) AND cardinality =",number_of_vertical_filters," ");
END IF;


/******************************************************************************/
/**************************** Filters - Collection ****************************/
/******************************************************************************/

/* In the case no horizontal filter is specified, we want to select all horizontals. */
SET filtered_horizontal_pairs_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id
FROM 
	viewOfHorizontalRelationshipTableWithNameAndValueStrings
",filtered_horizontal_traffic_where," 
GROUP BY horizontal_relationship_id HAVING COUNT(horizontal_relationship_id)>=",number_of_horizontal_filters," "
);

/* In the case no vertical filter is specified, we want to select vertical only when they are NULL. */
IF (number_of_vertical_filters > 0) THEN
	SET filtered_vertical_pairs_subquery = CONCAT(
	"SELECT 
		DISTINCT vertical_relationship_id
	FROM 
		viewOfVerticalRelationshipTableWithNameAndValueStrings
	",filtered_vertical_traffic_where," 
	GROUP BY vertical_relationship_id HAVING COUNT(vertical_relationship_id)=",number_of_vertical_filters," "
);
ELSE
	SET filtered_vertical_pairs_subquery = "SELECT NULL AS vertical_relationship_id";
END IF;

SET filtered_horizontal_vertical_pairs_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id, vertical_relationship_id 
FROM ( 
		",filtered_horizontal_pairs_subquery," 
	) AS filtered_horizontal_pairs_subquery,
	( 
		",filtered_vertical_pairs_subquery," 
	) AS filtered_vertical_pairs_subquery"
);

SET filtered_traffic_dimension_ids_subquery = CONCAT(
"SELECT 
	dimension_id 
FROM 
	dimension_table, 
	( 
		",filtered_horizontal_vertical_pairs_subquery," 
	) AS filtered_horizontal_vertical_pairs_subquery 
WHERE 
	( filtered_horizontal_vertical_pairs_subquery.horizontal_relationship_id <=> dimension_table.horizontal_relationship_id ) 
	AND ( filtered_horizontal_vertical_pairs_subquery.vertical_relationship_id <=> dimension_table.vertical_relationship_id )"
);

SET filtered_traffic_message_count_by_collection_id = CONCAT(
"SELECT 
	dimension_id, 'IValueUnassigned' as value_string, collection_id, message_count 
FROM ( 
		",filtered_traffic_dimension_ids_subquery," 
	) as filtered_traffic_dimension_ids_subquery 
	INNER JOIN ",data_for_group_X_table_collection," USING (dimension_id)"
);

/******************************************************************************/
/********************** Total Message Count - Collection **********************/
/******************************************************************************/

/* Total message counts by exit row. */
SET total_traffic_message_count_by_collection_id = CONCAT(
	"SELECT collection_id, SUM(message_count) as total_message_count_by_collection 
	FROM ( ",filtered_traffic_dimension_ids_subquery," ) as total_traffic_dimension_ids_subquery 
		INNER JOIN ",data_for_group_X_table_collection," USING (dimension_id) 
	GROUP BY collection_id"
);

/******************************************************************************/
/**************************** Breakdown - Collection **************************/
/******************************************************************************/
/*add the breakdown to the filter WHERE statements and increase the correct COUNT */
IF (@isBreakdownHorizontalOrVertical = 'vertical' ) THEN
	SET breakdown_where_vertical_value_string_is = CONCAT(" (name_string = '",TRIM(userBreakdown),"')AND ");
	SET breakdown_where_horizontal_value_string_is = "";
	SET select_vertical_value_string = ", value_string"; 
	SET select_horizontal_value_string = "";
	
	/* Check if the breakdown name string appeared in the user's horizontal filters, if it did we won't add the breakdown name string to our search query */
	IF(is_breakdown_name_string_in_filter != 2) THEN
		SET number_of_vertical_filters = number_of_vertical_filters + 1;
		IF !(breakdown_vertical_traffic_where IS NULL OR breakdown_vertical_traffic_where = "") THEN
			SET breakdown_vertical_traffic_where = CONCAT(breakdown_vertical_traffic_where, " OR ( name_string = '", TRIM(userBreakdown), "' ) ");
		ELSE
			SET breakdown_vertical_traffic_where = CONCAT(" WHERE ( ( name_string = '", TRIM(userBreakdown), "') ");
		END IF;
	END IF;
ELSE	
	SET breakdown_where_horizontal_value_string_is = CONCAT(" (name_string = '",TRIM(userBreakdown),"')AND ");
	SET breakdown_where_vertical_value_string_is = "";
	SET select_horizontal_value_string = ", value_string";
	SET select_vertical_value_string = ""; 
	
	/* Check if the breakdown name string appeared in the user's horizontal filters, if it did we won't add the breakdown name string to our search query */
	IF(is_breakdown_name_string_in_filter != 1) THEN
		SET number_of_horizontal_filters = number_of_horizontal_filters + 1;
		IF !(breakdown_horizontal_traffic_where IS NULL OR breakdown_horizontal_traffic_where = "") THEN
			SET breakdown_horizontal_traffic_where = CONCAT(breakdown_horizontal_traffic_where, " OR ( name_string = '", TRIM(userBreakdown), "' ) ");
		ELSE
			SET breakdown_horizontal_traffic_where = CONCAT(" WHERE ( ( name_string = '", TRIM(userBreakdown), "') ");
		END IF;
	END IF;
END IF;

IF (LENGTH(breakdown_vertical_traffic_where) > 0) THEN
	SET breakdown_vertical_traffic_where = CONCAT(breakdown_vertical_traffic_where, " ) AND cardinality =",number_of_vertical_filters," ");
END IF;
IF (LENGTH(breakdown_horizontal_traffic_where) > 0) THEN
	SET breakdown_horizontal_traffic_where = CONCAT(breakdown_horizontal_traffic_where, " ) AND cardinality >=",number_of_horizontal_filters," ");
END IF;

/** Relationship-Name-Value Table Query */
/* Find all the relationship_ids that have at least the filter and the breakdown */
/* similar to filtered_horizontal_pairs_subquery but includes the breakdown */
SET relationship_horizontal_string_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id
FROM 
	viewOfHorizontalRelationshipTableWithNameAndValueStrings
",breakdown_horizontal_traffic_where," 
GROUP BY horizontal_relationship_id HAVING COUNT(horizontal_relationship_id)>=",number_of_horizontal_filters," "
);

/* Find all the relationship_ids that have exactly the filter and the breakdown */
/* similar to filtered_vertical_pairs_subquery but includes the breakdown*/
SET relationship_vertical_string_subquery = CONCAT(
"SELECT 
	DISTINCT vertical_relationship_id
FROM 
	viewOfVerticalRelationshipTableWithNameAndValueStrings
",breakdown_vertical_traffic_where," 
GROUP BY vertical_relationship_id HAVING COUNT(vertical_relationship_id)=",number_of_vertical_filters," "
);

/* In the case no horizontal filter is specified, we want to select all horizontals. */
SET breakdown_horizontal_pairs_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id",select_horizontal_value_string,"
FROM 
	viewOfHorizontalRelationshipTableWithNameAndValueStrings
WHERE
	",breakdown_where_horizontal_value_string_is,"
	horizontal_relationship_id IN (
		",relationship_horizontal_string_subquery,"
	) "
);		

/* In the case no vertical filter is specified, we want to select vertical only when they are NULL. */
IF (number_of_vertical_filters > 0) THEN
	SET breakdown_vertical_pairs_subquery = CONCAT(
	"SELECT 
	DISTINCT vertical_relationship_id",select_vertical_value_string,"
	FROM 
		viewOfVerticalRelationshipTableWithNameAndValueStrings
	WHERE
		",breakdown_where_vertical_value_string_is," 
		vertical_relationship_id IN (
			",relationship_vertical_string_subquery,"
		) "
	);
ELSE
	SET breakdown_vertical_pairs_subquery = " SELECT NULL AS vertical_relationship_id ";
END IF;

SET breakdown_horizontal_vertical_pairs_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id, vertical_relationship_id, value_string
FROM ( 
		",breakdown_horizontal_pairs_subquery," 
	) AS filtered_horizontal_pairs_subquery,
	( 
		",breakdown_vertical_pairs_subquery," 
	) AS filtered_vertical_pairs_subquery"
);

SET breakdown_traffic_dimension_ids_subquery = CONCAT(
"SELECT 
	dimension_id, value_string
FROM 
	dimension_table, 
	( 
		",breakdown_horizontal_vertical_pairs_subquery," 
	) AS filtered_horizontal_vertical_pairs_subquery 
WHERE 
	( filtered_horizontal_vertical_pairs_subquery.horizontal_relationship_id <=> dimension_table.horizontal_relationship_id ) 
	AND ( filtered_horizontal_vertical_pairs_subquery.vertical_relationship_id <=> dimension_table.vertical_relationship_id )"
);

SET breakdown_traffic_message_count_by_collection_id = CONCAT(
"SELECT 
	dimension_id, value_string, collection_id, message_count 
FROM ( 
		",breakdown_traffic_dimension_ids_subquery," 
	) as breakdown_traffic_dimension_ids_subquery 
	INNER JOIN ",data_for_group_X_table_collection," USING (dimension_id)"
);

/******************************************************************************/
/************************** Breakdown & Filter - Collection *******************/
/******************************************************************************/
/* In the case the breakdown is vertical we don't need (left join) the   */
/* breakdown-query (breakdown_traffic_message_count_by_collection_id)    */ 
/* to the filter-query (filtered_traffic_message_count_by_collection_id) */ 
/* (like we have to do in the case only horizontals are selected in      */
/* order to find the IValueUnassigned values). We could just use the     */
/* filter value, this would give the same value and have less queries    */ 
/* to run thus improving efficiency at the cost of some complexity       */

IF (@isBreakdownHorizontalOrVertical = 'vertical' ) THEN
	/* Vertical breakdown */
	SET filtered_and_breakdown_traffic_message_count_by_collection_id = CONCAT(
"SELECT 
	breakdown_traffic_message_count_by_collection_id.value_string AS value_string, 
	breakdown_traffic_message_count_by_collection_id.message_count as message_count, 
	breakdown_traffic_message_count_by_collection_id.dimension_id, 
	breakdown_traffic_message_count_by_collection_id.collection_id 
FROM ( 
		",breakdown_traffic_message_count_by_collection_id,"
	) AS breakdown_traffic_message_count_by_collection_id "
	);
ELSE
	/* Horizontal Breakdown - */
	/* We join the filter and the breakdown because we need to calculate the   */   
	/* horizontal superset separately from breakdown subset. The Horizontals   */
	/* in the (filter) superset that do not appear in the (breakdown) subset   */
	/* are IValueUnassigned values.                                            */
	/* We join the Breakdown and Filter queries at the data_for_group_X_table  */
	/* level and not the dimension_table because we need to join based on      */
	/* collection ID and dimension ID. It's possible a collection doesn't have */ 
	/* a dimension ID that others do.                                          */
	
	SET filtered_and_breakdown_traffic_message_count_by_collection_id = CONCAT(
"SELECT 
	COALESCE(breakdown_traffic_message_count_by_collection_id.value_string, filtered_traffic_message_count_by_collection_id.value_string) AS value_string, 
	COALESCE(breakdown_traffic_message_count_by_collection_id.message_count, filtered_traffic_message_count_by_collection_id.message_count) as message_count, 
	filtered_traffic_message_count_by_collection_id.dimension_id, 
	filtered_traffic_message_count_by_collection_id.collection_id 
FROM ( 
		",filtered_traffic_message_count_by_collection_id," 
	) AS filtered_traffic_message_count_by_collection_id 
	LEFT Join ( 
			",breakdown_traffic_message_count_by_collection_id," 
		) AS breakdown_traffic_message_count_by_collection_id 
	ON (filtered_traffic_message_count_by_collection_id.collection_id = breakdown_traffic_message_count_by_collection_id.collection_id) 
	AND (filtered_traffic_message_count_by_collection_id.dimension_id = breakdown_traffic_message_count_by_collection_id.dimension_id)"
	);
END IF;



/* Outer most query - Collection */
SET final_collection_query = CONCAT(
"
SELECT 
	 UNIX_TIMESTAMP(@collection_date_time) as time, value_string, (message_count/total_message_count_by_collection * 100) AS percentage 
FROM ( 
		SELECT 
			value_string, SUM(message_count) AS message_count, collection_id, total_message_count_by_collection 
		FROM ( 
				",filtered_and_breakdown_traffic_message_count_by_collection_id,"
			) AS filtered_and_breakdown_traffic_message_count_by_collection_id 
			JOIN ( 
				",total_traffic_message_count_by_collection_id," ) AS total_traffic_message_count_by_collection_id USING (collection_id) 
		GROUP BY value_string, collection_id 
		ORDER BY collection_id) as final_collection_query 
	WHERE collection_id = '",userCollectionId, "'");

CALL tpf_execute_immediate(final_collection_query);

END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.getPercentOfFilteredTrafficByCollectionAndBaseline
DROP PROCEDURE IF EXISTS `getPercentOfFilteredTrafficByCollectionAndBaseline`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `getPercentOfFilteredTrafficByCollectionAndBaseline`(
	IN `userCollectionId` CHAR(50),
	IN `userBaseCollectionId` CHAR(50),
	IN `userHorizontal` BLOB,
	IN `userVertical` BLOB,
	IN `userBreakdown` VARCHAR(100)
)
getPercentOfFilteredTrafficByCollectionAndBaselineProc:BEGIN 
/************************************************************************/
/*  IBM TEMPLATE: getPercentOfFilteredTrafficByCollectionAndBasline     */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used to query results regarding a two unique       */
/* collections. Specifically this will find the VALUE results (relating */
/* to the NAME specified in the BREAKDOWN input) and their corisponding */
/* message count and percentage, of each collection then compare the    */
/* two with a delta value. The percentage will be calculated by taking  */
/* the message count for that VALUE and dividing it by the total        */
/* message count of the message traffic that occured in the FILTERed    */
/* traffic specified in the input.                                      */
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* userHorizontal blob - This is the list of horizontal NVPs  the user  */
/*    wants to breakdown percentage by.                                 */
/* userVertical blob - This is the list of vertical NVPs the user wants */
/*    to breakdown percentage by.                                       */
/* userBreakdown varchar(100) - This is the NVP the user wants to       */
/*    breakdown percentages by.                                         */
/* userCollectionId varchar(50) - The collection id of the collection   */
/*    to query against.                                                 */
/* userBaseCollectionId varchar(50) - The collection id of the second   */
/*    collection to compared against the first. This is typically a     */
/*    baseline collection.                                              */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (4 columns):                                                   */
/* value_string - The Value portion of the Name Value Pair relating to  */
/*    the Name specified in the breakdown. In other words the Name in   */
/*    the breakdown is made up of these values.                         */
/* collection_percent - The percent of traffic of the breakdown for the */
/*    collection id when horizontal/vertical filters are taken into     */
/*    account.                                                          */
/* baseline_percent - The percent of traffic of the breakdown for the   */
/*    baseline collection id when horizontal/vertical filters are taken */
/*    into account.                                                     */
/* delta - baseline_percent minus collection_percent.                   */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* call getPercentOfFilteredTrafficByCollectionAndBasline(              */
/*   '264b86d1b37d24533618545db6c18eefe1fdd',                           */
/*   '9076fde8928efde6648ebca23175eba4ba44b',                           */
/*   'MsgType=Booking',                                                 */
/*   'IOwnerHi=PackageA',                                               */
/*   'IOwnerMid'                                                        */
/* );                                                                   */
/*                                                                      */
/* call getPercentOfFilteredTrafficByCollectionAndBasline(              */
/*   'the-collection-id-hash',                                          */
/*   'the-baseline-or-second-collection-id-hash',                       */
/*   'horizontal-NVP-filter1,horizontal-NVP-filter2',                   */
/*   'vertical-NVP-filter1,vertical-NVP-filter2',                       */
/*   'breakdown'                                                        */
/* );                                                                   */
/*                                                                      */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/

/******************************************************************************/
/******************************** DECLARES ************************************/
/******************************************************************************/

DECLARE subStringLength                                           INT;
DECLARE stringLength                                              INT;
DECLARE relationship_string_subquery                              VARCHAR     (200);
DECLARE relationship_horizontal_string_subquery                   BLOB;
DECLARE relationship_vertical_string_subquery                     BLOB;
DECLARE select_horizontal_value_string                            VARCHAR     (20);
DECLARE select_vertical_value_string                              VARCHAR     (20);
DECLARE breakdown_where_vertical_value_string_is                  VARCHAR     (100);
DECLARE breakdown_where_horizontal_value_string_is                VARCHAR     (100);

DECLARE filtered_horizontal_pairs_subquery                        BLOB;
DECLARE filtered_vertical_pairs_subquery                          BLOB;
DECLARE filtered_horizontal_vertical_pairs_subquery               BLOB;
DECLARE filtered_traffic_dimension_ids_subquery                   BLOB;
DECLARE filtered_horizontal_traffic_where                         BLOB;
DECLARE filtered_vertical_traffic_where                           BLOB;

DECLARE breakdown_horizontal_vertical_pairs_subquery              BLOB;
DECLARE breakdown_traffic_dimension_ids_subquery                  BLOB;
DECLARE breakdown_where                                           BLOB;
DECLARE breakdown_horizontal_pairs_subquery                       BLOB;
DECLARE breakdown_vertical_pairs_subquery                         BLOB;
DECLARE breakdown_horizontal_traffic_where                        BLOB;
DECLARE breakdown_vertical_traffic_where                          BLOB;

DECLARE total_horizontal_vertical_pairs_subquery                  BLOB;
DECLARE total_traffic_dimension_ids_subquery                      BLOB;
DECLARE total_traffic_message_count_by_collection_id_BASELINE     BLOB;
DECLARE total_traffic_message_count_by_collection_id_COLLECTION   BLOB;
DECLARE filtered_traffic_message_count_by_collection_id_BASELINE  BLOB;
DECLARE filtered_traffic_message_count_by_collection_id_COLLECTION BLOB;
DECLARE breakdown_traffic_message_count_by_collection_id_BASELINE BLOB;
DECLARE breakdown_traffic_message_count_by_collection_id_COLLECTION BLOB;

DECLARE data_for_group_X_table_collection                         VARCHAR     (100);
DECLARE data_for_group_X_table_baseline                           VARCHAR     (100);

DECLARE filtered_and_breakdown_traffic_message_count_by_collection_id_BASELINE BLOB;
DECLARE filtered_and_breakdown_traffic_message_count_by_collection_id_COLLECTION BLOB;
DECLARE final_collection_query_BASELINE                           BLOB;
DECLARE final_collection_query_COLLECTION                         BLOB;
DECLARE final_baseline_query                                      BLOB;
DECLARE final_query                                               BLOB;

DECLARE number_of_horizontal_filters                              INT         DEFAULT 0;
DECLARE number_of_vertical_filters                                INT         DEFAULT 0;

/* 0 - none        */
/* 1 - horizontal  */
/* 2 - vertical    */
DECLARE is_breakdown_name_string_in_filter                        BOOL        DEFAULT 0;


/******************************************************************************/
/******************************* Initialize ***********************************/
/******************************************************************************/

SET filtered_horizontal_traffic_where = "";
SET filtered_vertical_traffic_where = "";

/******************************************************************************/
/***************************** Input checking *********************************/
/******************************************************************************/
/* refresh views */
call updateViews();

IF !(EXISTS (SELECT collection_id FROM viewOfAllCollectionGroupIdPairs WHERE collection_id = userCollectionId)) THEN
	SELECT "Collection Id doesn't exist";
	LEAVE getPercentOfFilteredTrafficByCollectionAndBaselineProc;
END IF;
IF !(EXISTS (SELECT collection_id FROM viewOfAllCollectionGroupIdPairs WHERE collection_id = userBaseCollectionId)) THEN
	SELECT "Baseline Collection Id doesn't exist";
	LEAVE getPercentOfFilteredTrafficByCollectionAndBaselineProc;
END IF;

IF !(userBreakdown IS NULL OR userBreakdown = "" OR userBreakdown = "NULL") THEN
	CALL tpf_execute_immediate(CONCAT('SELECT relationship_type INTO @isBreakdownHorizontalOrVertical from name_table WHERE name_string = "', userBreakdown,'"'));
	SET breakdown_where = CONCAT("WHERE (name_string = '",TRIM(userBreakdown),"')");
ELSE
	LEAVE getPercentOfFilteredTrafficByCollectionAndBaselineProc;
END IF;

IF !(userCollectionId IS NULL OR userCollectionId = "" OR userCollectionId = "NULL") THEN
	SET data_for_group_X_table_collection = CONCAT("data_for_group_table_", (SELECT group_id from viewOfAllCollectionGroupIdPairs where collection_id = userCollectionId));
ELSE
	LEAVE getPercentOfFilteredTrafficByCollectionAndBaselineProc; 
END IF;

IF !(userBaseCollectionId IS NULL OR userBaseCollectionId = "" OR userBaseCollectionId = "NULL") THEN
	SET data_for_group_X_table_baseline = CONCAT("data_for_group_table_", (SELECT group_id from viewOfAllCollectionGroupIdPairs where collection_id = userBaseCollectionId));
ELSE
	LEAVE getPercentOfFilteredTrafficByCollectionAndBaselineProc; 
END IF;

/******************************************************************************/
/******************************** Parsing *************************************/
/******************************************************************************/

read_horizontal_input:
LOOP
	IF (userHorizontal IS NULL OR userHorizontal = "" OR userHorizontal = "NULL") THEN
		LEAVE read_horizontal_input;
	END IF;
	/* Start or continue where statement */
	IF (number_of_horizontal_filters <= 0) THEN
		SET filtered_horizontal_traffic_where = " WHERE ( ";
	ELSE
		SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, " OR "); 
	END IF;
	SET number_of_horizontal_filters = number_of_horizontal_filters + 1;
	SET stringLength = LENGTH(userHorizontal);
	/* start to "=" */
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, " ( name_string = '");
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, TRIM(SUBSTRING_INDEX(userHorizontal, '=', 1)));
	IF (userBreakdown = TRIM(SUBSTRING_INDEX(userHorizontal, '=', 1))) THEN
		SET is_breakdown_name_string_in_filter = 1;
	END IF;
	SET subStringLength = LENGTH(SUBSTRING_INDEX(userHorizontal, '=', 1))+2;
	SET userHorizontal = MID(userHorizontal, subStringLength, stringLength);
	/* "=" to "," */
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, "' AND value_string = '");
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, TRIM(SUBSTRING_INDEX(userHorizontal, ',', 1)));
	SET subStringLength = LENGTH(SUBSTRING_INDEX(userHorizontal, ',', 1))+2;
	SET userHorizontal = MID(userHorizontal, subStringLength, stringLength);
	/* close () */
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, "' ) ");
END LOOP read_horizontal_input;
SET breakdown_horizontal_traffic_where = filtered_horizontal_traffic_where;
IF ( LENGTH(filtered_horizontal_traffic_where) > 0 ) THEN
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, " ) AND cardinality >=",number_of_horizontal_filters," ");
END IF;

IF !(userVertical IS NULL OR userVertical = "" OR userVertical = "NULL") THEN
	read_vertical_input:
	LOOP
		IF (userVertical IS NULL OR userVertical = "" OR userVertical = "NULL") THEN
			LEAVE read_vertical_input;
		END IF;
		/* Start or continue where statement */
		IF (number_of_vertical_filters <= 0) THEN
			SET filtered_vertical_traffic_where = " WHERE ( ";
		ELSE
			SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, " OR "); 
		END IF;
		SET number_of_vertical_filters = number_of_vertical_filters + 1;
		SET stringLength = LENGTH(userVertical);
		/* start to "=" */
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, " ( name_string = '");
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, TRIM(SUBSTRING_INDEX(userVertical, '=', 1)));
		IF (userBreakdown = TRIM(SUBSTRING_INDEX(userVertical, '=', 1))) THEN
			SET is_breakdown_name_string_in_filter = 2;
		END IF;
		SET subStringLength = LENGTH(SUBSTRING_INDEX(userVertical, '=', 1))+2;
		SET userVertical = MID(userVertical, subStringLength, stringLength);
		/* "=" to "," */
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, "' AND value_string = '");
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, TRIM(SUBSTRING_INDEX(userVertical, ',', 1)));
		SET subStringLength = LENGTH(SUBSTRING_INDEX(userVertical, ',', 1))+2;
		SET userVertical = MID(userVertical, subStringLength, stringLength);
		/* close () */
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, "' ) ");
	END LOOP read_vertical_input;
END IF;
SET breakdown_vertical_traffic_where = filtered_vertical_traffic_where;
IF ( LENGTH(filtered_vertical_traffic_where) > 0 ) THEN
	SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, " ) AND cardinality =",number_of_vertical_filters," ");
END IF;


/******************************************************************************/
/**************************** Filters - Collection ****************************/
/******************************************************************************/

/* In the case no horizontal filter is specified, we want to select all horizontals. */
SET filtered_horizontal_pairs_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id
FROM 
	viewOfHorizontalRelationshipTableWithNameAndValueStrings
",filtered_horizontal_traffic_where," 
GROUP BY horizontal_relationship_id HAVING COUNT(horizontal_relationship_id)>=",number_of_horizontal_filters," "
);

/* In the case no vertical filter is specified, we want to select vertical only when they are NULL. */
IF (number_of_vertical_filters > 0) THEN
	SET filtered_vertical_pairs_subquery = CONCAT(
	"SELECT 
		DISTINCT vertical_relationship_id
	FROM 
		viewOfVerticalRelationshipTableWithNameAndValueStrings
	",filtered_vertical_traffic_where," 
	GROUP BY vertical_relationship_id HAVING COUNT(vertical_relationship_id)=",number_of_vertical_filters," "
);
ELSE
	SET filtered_vertical_pairs_subquery = "SELECT NULL AS vertical_relationship_id";
END IF;

SET filtered_horizontal_vertical_pairs_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id, vertical_relationship_id 
FROM ( 
		",filtered_horizontal_pairs_subquery," 
	) AS filtered_horizontal_pairs_subquery,
	( 
		",filtered_vertical_pairs_subquery," 
	) AS filtered_vertical_pairs_subquery"
);

SET filtered_traffic_dimension_ids_subquery = CONCAT(
"SELECT 
	dimension_id 
FROM 
	dimension_table, 
	( 
		",filtered_horizontal_vertical_pairs_subquery," 
	) AS filtered_horizontal_vertical_pairs_subquery 
WHERE 
	( filtered_horizontal_vertical_pairs_subquery.horizontal_relationship_id <=> dimension_table.horizontal_relationship_id ) 
	AND ( filtered_horizontal_vertical_pairs_subquery.vertical_relationship_id <=> dimension_table.vertical_relationship_id )"
);

SET filtered_traffic_message_count_by_collection_id_BASELINE = CONCAT(
"SELECT 
	dimension_id, 'IValueUnassigned' as value_string, collection_id, message_count 
FROM ( 
		",filtered_traffic_dimension_ids_subquery," 
	) as filtered_traffic_dimension_ids_subquery 
	INNER JOIN ",data_for_group_X_table_baseline," USING (dimension_id)"
);
SET filtered_traffic_message_count_by_collection_id_COLLECTION = CONCAT(
"SELECT 
	dimension_id, 'IValueUnassigned' as value_string, collection_id, message_count 
FROM ( 
		",filtered_traffic_dimension_ids_subquery," 
	) as filtered_traffic_dimension_ids_subquery 
	INNER JOIN ",data_for_group_X_table_collection," USING (dimension_id)"
);

/******************************************************************************/
/********************** Total Message Count - Collection **********************/
/******************************************************************************/

/* Total message counts by exit row. */
SET total_traffic_message_count_by_collection_id_BASELINE = CONCAT(
	"SELECT collection_id, SUM(message_count) as total_message_count_by_collection 
	FROM ( ",filtered_traffic_dimension_ids_subquery," ) as total_traffic_dimension_ids_subquery 
		INNER JOIN ",data_for_group_X_table_baseline," USING (dimension_id) 
	GROUP BY collection_id"
);
SET total_traffic_message_count_by_collection_id_COLLECTION = CONCAT(
	"SELECT collection_id, SUM(message_count) as total_message_count_by_collection 
	FROM ( ",filtered_traffic_dimension_ids_subquery," ) as total_traffic_dimension_ids_subquery 
		INNER JOIN ",data_for_group_X_table_collection," USING (dimension_id) 
	GROUP BY collection_id"
);


/******************************************************************************/
/**************************** Breakdown - Collection **************************/
/******************************************************************************/
/*add the breakdown to the filter WHERE statements and increase the correct COUNT */
IF (@isBreakdownHorizontalOrVertical = 'vertical' ) THEN
	SET breakdown_where_vertical_value_string_is = CONCAT(" (name_string = '",TRIM(userBreakdown),"')AND ");
	SET breakdown_where_horizontal_value_string_is = "";
	SET select_vertical_value_string = ", value_string"; 
	SET select_horizontal_value_string = "";
	
	/* Check if the breakdown name string appeared in the user's horizontal filters, if it did we won't add the breakdown name string to our search query */
	IF(is_breakdown_name_string_in_filter != 2) THEN
		SET number_of_vertical_filters = number_of_vertical_filters + 1;
		IF !(breakdown_vertical_traffic_where IS NULL OR breakdown_vertical_traffic_where = "") THEN
			SET breakdown_vertical_traffic_where = CONCAT(breakdown_vertical_traffic_where, " OR ( name_string = '", TRIM(userBreakdown), "' ) ");
		ELSE
			SET breakdown_vertical_traffic_where = CONCAT(" WHERE ( ( name_string = '", TRIM(userBreakdown), "') ");
		END IF;
	END IF;
ELSE	
	SET breakdown_where_horizontal_value_string_is = CONCAT(" (name_string = '",TRIM(userBreakdown),"')AND ");
	SET breakdown_where_vertical_value_string_is = "";
	SET select_horizontal_value_string = ", value_string";
	SET select_vertical_value_string = ""; 
	
	/* Check if the breakdown name string appeared in the user's horizontal filters, if it did we won't add the breakdown name string to our search query */
	IF(is_breakdown_name_string_in_filter != 1) THEN
		SET number_of_horizontal_filters = number_of_horizontal_filters + 1;
		IF !(breakdown_horizontal_traffic_where IS NULL OR breakdown_horizontal_traffic_where = "") THEN
			SET breakdown_horizontal_traffic_where = CONCAT(breakdown_horizontal_traffic_where, " OR ( name_string = '", TRIM(userBreakdown), "' ) ");
		ELSE
			SET breakdown_horizontal_traffic_where = CONCAT(" WHERE ( ( name_string = '", TRIM(userBreakdown), "') ");
		END IF;
	END IF;
END IF;

IF (LENGTH(breakdown_vertical_traffic_where) > 0) THEN
	SET breakdown_vertical_traffic_where = CONCAT(breakdown_vertical_traffic_where, " ) AND cardinality =",number_of_vertical_filters," ");
END IF;
IF (LENGTH(breakdown_horizontal_traffic_where) > 0) THEN
	SET breakdown_horizontal_traffic_where = CONCAT(breakdown_horizontal_traffic_where, " ) AND cardinality >=",number_of_horizontal_filters," ");
END IF;

/** Relationship-Name-Value Table Query */
/* Find all the relationship_ids that have at least the filter and the breakdown */
/* similar to filtered_horizontal_pairs_subquery but includes the breakdown */
SET relationship_horizontal_string_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id
FROM 
	viewOfHorizontalRelationshipTableWithNameAndValueStrings
",breakdown_horizontal_traffic_where," 
GROUP BY horizontal_relationship_id HAVING COUNT(horizontal_relationship_id)>=",number_of_horizontal_filters," "
);

/* Find all the relationship_ids that have exactly the filter and the breakdown */
/* similar to filtered_vertical_pairs_subquery but includes the breakdown*/
SET relationship_vertical_string_subquery = CONCAT(
"SELECT 
	DISTINCT vertical_relationship_id
FROM 
	viewOfVerticalRelationshipTableWithNameAndValueStrings
",breakdown_vertical_traffic_where," 
GROUP BY vertical_relationship_id HAVING COUNT(vertical_relationship_id)=",number_of_vertical_filters," "
);

/* In the case no horizontal filter is specified, we want to select all horizontals. */
SET breakdown_horizontal_pairs_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id",select_horizontal_value_string,"
FROM 
	viewOfHorizontalRelationshipTableWithNameAndValueStrings
WHERE
	",breakdown_where_horizontal_value_string_is,"
	horizontal_relationship_id IN (
		",relationship_horizontal_string_subquery,"
	) "
);		

/* In the case no vertical filter is specified, we want to select vertical only when they are NULL. */
IF (number_of_vertical_filters > 0) THEN
	SET breakdown_vertical_pairs_subquery = CONCAT(
	"SELECT 
	DISTINCT vertical_relationship_id",select_vertical_value_string,"
	FROM 
		viewOfVerticalRelationshipTableWithNameAndValueStrings
	WHERE
		",breakdown_where_vertical_value_string_is," 
		vertical_relationship_id IN (
			",relationship_vertical_string_subquery,"
		) "
	);
ELSE
	SET breakdown_vertical_pairs_subquery = " SELECT NULL AS vertical_relationship_id ";
END IF;

SET breakdown_horizontal_vertical_pairs_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id, vertical_relationship_id, value_string
FROM ( 
		",breakdown_horizontal_pairs_subquery," 
	) AS filtered_horizontal_pairs_subquery,
	( 
		",breakdown_vertical_pairs_subquery," 
	) AS filtered_vertical_pairs_subquery"
);

SET breakdown_traffic_dimension_ids_subquery = CONCAT(
"SELECT 
	dimension_id, value_string
FROM 
	dimension_table, 
	( 
		",breakdown_horizontal_vertical_pairs_subquery," 
	) AS filtered_horizontal_vertical_pairs_subquery 
WHERE 
	( filtered_horizontal_vertical_pairs_subquery.horizontal_relationship_id <=> dimension_table.horizontal_relationship_id ) 
	AND ( filtered_horizontal_vertical_pairs_subquery.vertical_relationship_id <=> dimension_table.vertical_relationship_id )"
);

SET breakdown_traffic_message_count_by_collection_id_BASELINE = CONCAT(
"SELECT 
	dimension_id, value_string, collection_id, message_count 
FROM ( 
		",breakdown_traffic_dimension_ids_subquery," 
	) as breakdown_traffic_dimension_ids_subquery 
	INNER JOIN ",data_for_group_X_table_baseline," USING (dimension_id)"
);
SET breakdown_traffic_message_count_by_collection_id_COLLECTION = CONCAT(
"SELECT 
	dimension_id, value_string, collection_id, message_count 
FROM ( 
		",breakdown_traffic_dimension_ids_subquery," 
	) as breakdown_traffic_dimension_ids_subquery 
	INNER JOIN ",data_for_group_X_table_collection," USING (dimension_id)"
);

/******************************************************************************/
/************************** Breakdown & Filter - Collection *******************/
/******************************************************************************/
/* In the case the breakdown is vertical we don't need (left join) the   */
/* breakdown-query (breakdown_traffic_message_count_by_collection_id)    */ 
/* to the filter-query (filtered_traffic_message_count_by_collection_id) */ 
/* (like we have to do in the case only horizontals are selected in      */
/* order to find the IValueUnassigned values). We could just use the     */
/* filter value, this would give the same value and have less queries    */ 
/* to run thus improving efficiency at the cost of some complexity       */

IF (@isBreakdownHorizontalOrVertical = 'vertical' ) THEN
	/* Vertical breakdown */
	SET filtered_and_breakdown_traffic_message_count_by_collection_id_BASELINE = CONCAT(
	"SELECT 
		breakdown_traffic_message_count_by_collection_id.value_string AS value_string, 
		breakdown_traffic_message_count_by_collection_id.message_count as message_count, 
		breakdown_traffic_message_count_by_collection_id.dimension_id, 
		breakdown_traffic_message_count_by_collection_id.collection_id 
	FROM ( ",breakdown_traffic_message_count_by_collection_id_BASELINE," ) AS breakdown_traffic_message_count_by_collection_id "
	);
	SET filtered_and_breakdown_traffic_message_count_by_collection_id_COLLECTION = CONCAT(
	"SELECT 
		breakdown_traffic_message_count_by_collection_id.value_string AS value_string, 
		breakdown_traffic_message_count_by_collection_id.message_count as message_count, 
		breakdown_traffic_message_count_by_collection_id.dimension_id, 
		breakdown_traffic_message_count_by_collection_id.collection_id 
	FROM ( ",breakdown_traffic_message_count_by_collection_id_COLLECTION," ) AS breakdown_traffic_message_count_by_collection_id "
	);
ELSE
	/* Horizontal Breakdown - */
	/* We join the filter and the breakdown because we need to calculate the   */   
	/* horizontal superset separately from breakdown subset. The Horizontals   */
	/* in the (filter) superset that do not appear in the (breakdown) subset   */
	/* are IValueUnassigned values.                                            */
	/* We join the Breakdown and Filter queries at the data_for_group_X_table  */
	/* level and not the dimension_table because we need to join based on      */
	/* collection ID and dimension ID. It's possible a collection doesn't have */ 
	/* a dimension ID that others do.                                          */
	
	SET filtered_and_breakdown_traffic_message_count_by_collection_id_BASELINE = CONCAT(
	"SELECT 
		COALESCE(breakdown_traffic_message_count_by_collection_id.value_string, filtered_traffic_message_count_by_collection_id.value_string) AS value_string, 
		COALESCE(breakdown_traffic_message_count_by_collection_id.message_count, filtered_traffic_message_count_by_collection_id.message_count) as message_count, 
		filtered_traffic_message_count_by_collection_id.dimension_id, 
		filtered_traffic_message_count_by_collection_id.collection_id 
	FROM ( ",filtered_traffic_message_count_by_collection_id_BASELINE," ) AS filtered_traffic_message_count_by_collection_id 
		LEFT Join ( ",breakdown_traffic_message_count_by_collection_id_BASELINE," ) AS breakdown_traffic_message_count_by_collection_id 
		ON (filtered_traffic_message_count_by_collection_id.collection_id = breakdown_traffic_message_count_by_collection_id.collection_id) 
		AND (filtered_traffic_message_count_by_collection_id.dimension_id = breakdown_traffic_message_count_by_collection_id.dimension_id)"
	);
	SET filtered_and_breakdown_traffic_message_count_by_collection_id_COLLECTION = CONCAT(
	"SELECT 
		COALESCE(breakdown_traffic_message_count_by_collection_id.value_string, filtered_traffic_message_count_by_collection_id.value_string) AS value_string, 
		COALESCE(breakdown_traffic_message_count_by_collection_id.message_count, filtered_traffic_message_count_by_collection_id.message_count) as message_count, 
		filtered_traffic_message_count_by_collection_id.dimension_id, 
		filtered_traffic_message_count_by_collection_id.collection_id 
	FROM ( ",filtered_traffic_message_count_by_collection_id_COLLECTION," ) AS filtered_traffic_message_count_by_collection_id 
		LEFT Join ( ",breakdown_traffic_message_count_by_collection_id_COLLECTION," ) AS breakdown_traffic_message_count_by_collection_id 
		ON (filtered_traffic_message_count_by_collection_id.collection_id = breakdown_traffic_message_count_by_collection_id.collection_id) 
		AND (filtered_traffic_message_count_by_collection_id.dimension_id = breakdown_traffic_message_count_by_collection_id.dimension_id)"
	);
END IF;

/* Outer most query - Collection */
SET final_collection_query_BASELINE = CONCAT(
	"SELECT *, (message_count/total_message_count_by_collection * 100) AS percentage 
	FROM ( 
		SELECT value_string, SUM(message_count) AS message_count, collection_id, total_message_count_by_collection 
		FROM (",filtered_and_breakdown_traffic_message_count_by_collection_id_BASELINE,") AS filtered_and_breakdown_traffic_message_count_by_collection_id 
					JOIN ( ",total_traffic_message_count_by_collection_id_BASELINE," ) AS total_traffic_message_count_by_collection_id USING (collection_id) 
			GROUP BY value_string, collection_id 
			ORDER BY collection_id) as final_collection_query 
		WHERE collection_id = '",userBaseCollectionId, "'");
		
SET final_collection_query_COLLECTION = CONCAT(
	"SELECT *, (message_count/total_message_count_by_collection * 100) AS percentage 
	FROM ( 
		SELECT value_string, SUM(message_count) AS message_count, collection_id, total_message_count_by_collection 
		FROM (",filtered_and_breakdown_traffic_message_count_by_collection_id_COLLECTION,") AS filtered_and_breakdown_traffic_message_count_by_collection_id 
					JOIN ( ",total_traffic_message_count_by_collection_id_COLLECTION," ) AS total_traffic_message_count_by_collection_id USING (collection_id) 
			GROUP BY value_string, collection_id 
			ORDER BY collection_id) as final_collection_query 
		WHERE collection_id = '",userCollectionId, "'");

SET final_query = CONCAT("SELECT 
						fin_collection.value_string, 
						fin_collection.percentage as collection_percent, 
						fin_base.percentage as baseline_percent, 
						(fin_collection.percentage - fin_base.percentage) as delta 
						FROM (",final_collection_query_BASELINE, ") as fin_base JOIN (", final_collection_query_COLLECTION,") as fin_collection USING (value_string)");

CALL tpf_execute_immediate(final_query);

END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.getPercentOfFilteredTrafficByCollectionMessageCount
DROP PROCEDURE IF EXISTS `getPercentOfFilteredTrafficByCollectionMessageCount`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `getPercentOfFilteredTrafficByCollectionMessageCount`(
	IN `userCollectionId` VARCHAR(50),
	IN `userHorizontal` BLOB,
	IN `userVertical` BLOB,
	IN `userBreakdown` VARCHAR(100)









)
getPercentOfFilteredTrafficByCollectionMessageCountProc:BEGIN 
/************************************************************************/
/*  IBM TEMPLATE: getPercentOfFilteredTrafficByCollectionMessageCount               */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used to query results regarding a single           */
/* collection. Specifically this will find the VALUE results (relating  */
/* to the NAME specified in the BREAKDOWN input) and their corisponding */
/* message count and percentage. The percentage will be calculated by   */
/* taking the message count for that VALUE and dividing it by the total */
/* message count of the message traffic that occured in the FILTERed    */
/* traffic specified in the input.                                      */
/*                                                                      */
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* userHorizontal blob - This is the list of horizontal NVPs  the user  */
/*    wants to breakdown percentage by.                                 */
/* userVertical blob - This is the list of vertical NVPs the user wants */
/*    to breakdown percentage by.                                       */
/* userBreakdown varchar(100) - This is the NVP the user wants to       */
/*    breakdown percentages by.                                         */
/* userCollectionId varchar(50) - The collection id of the collection   */
/*    to query against.                                                 */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (5 columns):                                                   */
/* value_string - The Value portion of the Name Value Pair relating to  */
/*    the Name specified in the breakdown. In other words the Name in   */
/*    the breakdown is made up of these values.                         */
/* message_count - The number of messages for that given value_string.  */
/* collection_id - The collection id in which that value_string exists. */
/* total_message_count_by_collection - the total number of messages in  */
/*    that collection_id when the horizontal/vertical filters are taken */
/*    into account.                                                     */
/* percentage - message_count / total_message_count_by_collection * 100 */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* call getPercentOfFilteredTrafficByCollectionMessageCount(                        */
/*   '264b86d1b37d24533618545db6c18eefe1fdd',                           */
/*   'MsgType=Booking',                                                 */
/*   'IOwnerHi=PackageA',                                               */
/*   'IOwnerMid'                                                        */
/* );                                                                   */
/*                                                                      */
/* call getPercentOfFilteredTrafficByCollectionMessageCount(                        */
/*   'the-collection-id-hash',                                          */
/*   'horizontal-NVP-filter1,horizontal-NVP-filter2',                   */
/*   'vertical-NVP-filter1,vertical-NVP-filter2',                       */
/*   'breakdown'                                                        */
/* );                                                                   */
/*                                                                      */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/


/******************************************************************************/
/******************************** DECLARES ************************************/
/******************************************************************************/

DECLARE subStringLength                                           INT;
DECLARE stringLength                                              INT;
DECLARE relationship_string_subquery                              VARCHAR     (200);
DECLARE relationship_horizontal_string_subquery                   BLOB;
DECLARE relationship_vertical_string_subquery                     BLOB;
DECLARE select_horizontal_value_string                            VARCHAR     (20);
DECLARE select_vertical_value_string                              VARCHAR     (20);
DECLARE breakdown_where_vertical_value_string_is                  VARCHAR     (100);
DECLARE breakdown_where_horizontal_value_string_is                VARCHAR     (100);

DECLARE filtered_horizontal_pairs_subquery                        BLOB;
DECLARE filtered_vertical_pairs_subquery                          BLOB;
DECLARE filtered_horizontal_vertical_pairs_subquery               BLOB;
DECLARE filtered_traffic_dimension_ids_subquery                   BLOB;
DECLARE filtered_horizontal_traffic_where                         BLOB;
DECLARE filtered_vertical_traffic_where                           BLOB;

DECLARE breakdown_horizontal_vertical_pairs_subquery              BLOB;
DECLARE breakdown_traffic_dimension_ids_subquery                  BLOB;
DECLARE breakdown_where                                           BLOB;
DECLARE breakdown_horizontal_pairs_subquery                       BLOB;
DECLARE breakdown_vertical_pairs_subquery                         BLOB;
DECLARE breakdown_horizontal_traffic_where                        BLOB;
DECLARE breakdown_vertical_traffic_where                          BLOB;

DECLARE total_horizontal_vertical_pairs_subquery                  BLOB;
DECLARE total_traffic_dimension_ids_subquery                      BLOB;
DECLARE total_traffic_message_count_by_collection_id              BLOB;
DECLARE filtered_traffic_message_count_by_collection_id           BLOB;
DECLARE breakdown_traffic_message_count_by_collection_id          BLOB;

DECLARE data_for_group_X_table_collection                         VARCHAR     (100);

DECLARE filtered_and_breakdown_traffic_message_count_by_collection_id		BLOB;
DECLARE final_collection_query                                    BLOB;

DECLARE number_of_horizontal_filters                              INT         DEFAULT 0;
DECLARE number_of_vertical_filters                                INT         DEFAULT 0;

/* 0 - none        */
/* 1 - horizontal  */
/* 2 - vertical    */
DECLARE is_breakdown_name_string_in_filter                        BOOL        DEFAULT 0;
DECLARE collection_date_time                                      BLOB;

/******************************************************************************/
/******************************* Initialize ***********************************/
/******************************************************************************/

SET filtered_horizontal_traffic_where = "";
SET filtered_vertical_traffic_where = "";

/******************************************************************************/
/***************************** Input checking *********************************/
/******************************************************************************/

/* refresh views */
call updateViews();

IF !(EXISTS (SELECT collection_id FROM viewOfAllCollectionGroupIdPairs WHERE collection_id = userCollectionId)) THEN
	SELECT "Collection Id doesn't exist";
	LEAVE getPercentOfFilteredTrafficByCollectionMessageCountProc;
END IF;

IF !(userBreakdown IS NULL OR userBreakdown = "" OR userBreakdown = "NULL") THEN
	CALL tpf_execute_immediate(CONCAT('SELECT relationship_type INTO @isBreakdownHorizontalOrVertical from name_table WHERE name_string = "', userBreakdown,'"'));
	SET breakdown_where = CONCAT("WHERE (name_string = '",TRIM(userBreakdown),"')");
ELSE
	LEAVE getPercentOfFilteredTrafficByCollectionMessageCountProc;
END IF;

IF !(userCollectionId IS NULL OR userCollectionId = "" OR userCollectionId = "NULL") THEN
	SET data_for_group_X_table_collection = CONCAT("data_for_group_table_", (SELECT group_id from viewOfAllCollectionGroupIdPairs where collection_id = userCollectionId));
ELSE
	LEAVE getPercentOfFilteredTrafficByCollectionMessageCountProc;
END IF;

/******************************************************************************/
/****************** save the date and time of the collection ******************/
/******************************************************************************/

CALL tpf_execute_immediate(CONCAT("SELECT
	STR_TO_DATE(concat(collection_date, ' ',collection_time), '%Y-%m-%d %H:%i:%s') INTO @collection_date_time
	FROM collection_table where collection_id='", userCollectionId, "'"));

/******************************************************************************/
/******************************** Parsing *************************************/
/******************************************************************************/

read_horizontal_input:
LOOP
	IF (userHorizontal IS NULL OR userHorizontal = "" OR userHorizontal = "NULL") THEN
		LEAVE read_horizontal_input;
	END IF;
	/* Start or continue where statement */
	IF (number_of_horizontal_filters <= 0) THEN
		SET filtered_horizontal_traffic_where = " WHERE ( ";
	ELSE
		SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, " OR "); 
	END IF;
	SET number_of_horizontal_filters = number_of_horizontal_filters + 1;
	SET stringLength = LENGTH(userHorizontal);
	/* start to "=" */
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, " ( name_string = '");
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, TRIM(SUBSTRING_INDEX(userHorizontal, '=', 1)));
	IF (userBreakdown = TRIM(SUBSTRING_INDEX(userHorizontal, '=', 1))) THEN
		SET is_breakdown_name_string_in_filter = 1;
	END IF;
	SET subStringLength = LENGTH(SUBSTRING_INDEX(userHorizontal, '=', 1))+2;
	SET userHorizontal = MID(userHorizontal, subStringLength, stringLength);
	/* "=" to "," */
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, "' AND value_string = '");
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, TRIM(SUBSTRING_INDEX(userHorizontal, ',', 1)));
	SET subStringLength = LENGTH(SUBSTRING_INDEX(userHorizontal, ',', 1))+2;
	SET userHorizontal = MID(userHorizontal, subStringLength, stringLength);
	/* close () */
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, "' ) ");
END LOOP read_horizontal_input;
SET breakdown_horizontal_traffic_where = filtered_horizontal_traffic_where;
IF ( LENGTH(filtered_horizontal_traffic_where) > 0 ) THEN
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, " ) AND cardinality >=",number_of_horizontal_filters," ");
END IF;

IF !(userVertical IS NULL OR userVertical = "" OR userVertical = "NULL") THEN
	read_vertical_input:
	LOOP
		IF (userVertical IS NULL OR userVertical = "" OR userVertical = "NULL") THEN
			LEAVE read_vertical_input;
		END IF;
		/* Start or continue where statement */
		IF (number_of_vertical_filters <= 0) THEN
			SET filtered_vertical_traffic_where = " WHERE ( ";
		ELSE
			SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, " OR "); 
		END IF;
		SET number_of_vertical_filters = number_of_vertical_filters + 1;
		SET stringLength = LENGTH(userVertical);
		/* start to "=" */
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, " ( name_string = '");
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, TRIM(SUBSTRING_INDEX(userVertical, '=', 1)));
		IF (userBreakdown = TRIM(SUBSTRING_INDEX(userVertical, '=', 1))) THEN
			SET is_breakdown_name_string_in_filter = 2;
		END IF;
		SET subStringLength = LENGTH(SUBSTRING_INDEX(userVertical, '=', 1))+2;
		SET userVertical = MID(userVertical, subStringLength, stringLength);
		/* "=" to "," */
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, "' AND value_string = '");
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, TRIM(SUBSTRING_INDEX(userVertical, ',', 1)));
		SET subStringLength = LENGTH(SUBSTRING_INDEX(userVertical, ',', 1))+2;
		SET userVertical = MID(userVertical, subStringLength, stringLength);
		/* close () */
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, "' ) ");
	END LOOP read_vertical_input;
END IF;
SET breakdown_vertical_traffic_where = filtered_vertical_traffic_where;
IF ( LENGTH(filtered_vertical_traffic_where) > 0 ) THEN
	SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, " ) AND cardinality =",number_of_vertical_filters," ");
END IF;


/******************************************************************************/
/**************************** Filters - Collection ****************************/
/******************************************************************************/

/* In the case no horizontal filter is specified, we want to select all horizontals. */
SET filtered_horizontal_pairs_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id
FROM 
	viewOfHorizontalRelationshipTableWithNameAndValueStrings
",filtered_horizontal_traffic_where," 
GROUP BY horizontal_relationship_id HAVING COUNT(horizontal_relationship_id)>=",number_of_horizontal_filters," "
);

/* In the case no vertical filter is specified, we want to select vertical only when they are NULL. */
IF (number_of_vertical_filters > 0) THEN
	SET filtered_vertical_pairs_subquery = CONCAT(
	"SELECT 
		DISTINCT vertical_relationship_id
	FROM 
		viewOfVerticalRelationshipTableWithNameAndValueStrings
	",filtered_vertical_traffic_where," 
	GROUP BY vertical_relationship_id HAVING COUNT(vertical_relationship_id)=",number_of_vertical_filters," "
);
ELSE
	SET filtered_vertical_pairs_subquery = "SELECT NULL AS vertical_relationship_id";
END IF;

SET filtered_horizontal_vertical_pairs_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id, vertical_relationship_id 
FROM ( 
		",filtered_horizontal_pairs_subquery," 
	) AS filtered_horizontal_pairs_subquery,
	( 
		",filtered_vertical_pairs_subquery," 
	) AS filtered_vertical_pairs_subquery"
);

SET filtered_traffic_dimension_ids_subquery = CONCAT(
"SELECT 
	dimension_id 
FROM 
	dimension_table, 
	( 
		",filtered_horizontal_vertical_pairs_subquery," 
	) AS filtered_horizontal_vertical_pairs_subquery 
WHERE 
	( filtered_horizontal_vertical_pairs_subquery.horizontal_relationship_id <=> dimension_table.horizontal_relationship_id ) 
	AND ( filtered_horizontal_vertical_pairs_subquery.vertical_relationship_id <=> dimension_table.vertical_relationship_id )"
);

SET filtered_traffic_message_count_by_collection_id = CONCAT(
"SELECT 
	dimension_id, 'IValueUnassigned' as value_string, collection_id, message_count 
FROM ( 
		",filtered_traffic_dimension_ids_subquery," 
	) as filtered_traffic_dimension_ids_subquery 
	INNER JOIN ",data_for_group_X_table_collection," USING (dimension_id)"
);

/******************************************************************************/
/********************** Total Message Count - Collection **********************/
/******************************************************************************/

/* Total message counts by exit row. */
SET total_traffic_message_count_by_collection_id = CONCAT(
	"SELECT collection_id, SUM(message_count) as total_message_count_by_collection 
	FROM ( ",filtered_traffic_dimension_ids_subquery," ) as total_traffic_dimension_ids_subquery 
		INNER JOIN ",data_for_group_X_table_collection," USING (dimension_id) 
	GROUP BY collection_id"
);

/******************************************************************************/
/**************************** Breakdown - Collection **************************/
/******************************************************************************/
/*add the breakdown to the filter WHERE statements and increase the correct COUNT */
IF (@isBreakdownHorizontalOrVertical = 'vertical' ) THEN
	SET breakdown_where_vertical_value_string_is = CONCAT(" (name_string = '",TRIM(userBreakdown),"')AND ");
	SET breakdown_where_horizontal_value_string_is = "";
	SET select_vertical_value_string = ", value_string"; 
	SET select_horizontal_value_string = "";
	
	/* Check if the breakdown name string appeared in the user's horizontal filters, if it did we won't add the breakdown name string to our search query */
	IF(is_breakdown_name_string_in_filter != 2) THEN
		SET number_of_vertical_filters = number_of_vertical_filters + 1;
		IF !(breakdown_vertical_traffic_where IS NULL OR breakdown_vertical_traffic_where = "") THEN
			SET breakdown_vertical_traffic_where = CONCAT(breakdown_vertical_traffic_where, " OR ( name_string = '", TRIM(userBreakdown), "' ) ");
		ELSE
			SET breakdown_vertical_traffic_where = CONCAT(" WHERE ( ( name_string = '", TRIM(userBreakdown), "') ");
		END IF;
	END IF;
ELSE	
	SET breakdown_where_horizontal_value_string_is = CONCAT(" (name_string = '",TRIM(userBreakdown),"')AND ");
	SET breakdown_where_vertical_value_string_is = "";
	SET select_horizontal_value_string = ", value_string";
	SET select_vertical_value_string = ""; 
	
	/* Check if the breakdown name string appeared in the user's horizontal filters, if it did we won't add the breakdown name string to our search query */
	IF(is_breakdown_name_string_in_filter != 1) THEN
		SET number_of_horizontal_filters = number_of_horizontal_filters + 1;
		IF !(breakdown_horizontal_traffic_where IS NULL OR breakdown_horizontal_traffic_where = "") THEN
			SET breakdown_horizontal_traffic_where = CONCAT(breakdown_horizontal_traffic_where, " OR ( name_string = '", TRIM(userBreakdown), "' ) ");
		ELSE
			SET breakdown_horizontal_traffic_where = CONCAT(" WHERE ( ( name_string = '", TRIM(userBreakdown), "') ");
		END IF;
	END IF;
END IF;

IF (LENGTH(breakdown_vertical_traffic_where) > 0) THEN
	SET breakdown_vertical_traffic_where = CONCAT(breakdown_vertical_traffic_where, " ) AND cardinality =",number_of_vertical_filters," ");
END IF;
IF (LENGTH(breakdown_horizontal_traffic_where) > 0) THEN
	SET breakdown_horizontal_traffic_where = CONCAT(breakdown_horizontal_traffic_where, " ) AND cardinality >=",number_of_horizontal_filters," ");
END IF;

/** Relationship-Name-Value Table Query */
/* Find all the relationship_ids that have at least the filter and the breakdown */
/* similar to filtered_horizontal_pairs_subquery but includes the breakdown */
SET relationship_horizontal_string_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id
FROM 
	viewOfHorizontalRelationshipTableWithNameAndValueStrings
",breakdown_horizontal_traffic_where," 
GROUP BY horizontal_relationship_id HAVING COUNT(horizontal_relationship_id)>=",number_of_horizontal_filters," "
);

/* Find all the relationship_ids that have exactly the filter and the breakdown */
/* similar to filtered_vertical_pairs_subquery but includes the breakdown*/
SET relationship_vertical_string_subquery = CONCAT(
"SELECT 
	DISTINCT vertical_relationship_id
FROM 
	viewOfVerticalRelationshipTableWithNameAndValueStrings
",breakdown_vertical_traffic_where," 
GROUP BY vertical_relationship_id HAVING COUNT(vertical_relationship_id)=",number_of_vertical_filters," "
);

/* In the case no horizontal filter is specified, we want to select all horizontals. */
SET breakdown_horizontal_pairs_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id",select_horizontal_value_string,"
FROM 
	viewOfHorizontalRelationshipTableWithNameAndValueStrings
WHERE
	",breakdown_where_horizontal_value_string_is,"
	horizontal_relationship_id IN (
		",relationship_horizontal_string_subquery,"
	) "
);		

/* In the case no vertical filter is specified, we want to select vertical only when they are NULL. */
IF (number_of_vertical_filters > 0) THEN
	SET breakdown_vertical_pairs_subquery = CONCAT(
	"SELECT 
	DISTINCT vertical_relationship_id",select_vertical_value_string,"
	FROM 
		viewOfVerticalRelationshipTableWithNameAndValueStrings
	WHERE
		",breakdown_where_vertical_value_string_is," 
		vertical_relationship_id IN (
			",relationship_vertical_string_subquery,"
		) "
	);
ELSE
	SET breakdown_vertical_pairs_subquery = " SELECT NULL AS vertical_relationship_id ";
END IF;

SET breakdown_horizontal_vertical_pairs_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id, vertical_relationship_id, value_string
FROM ( 
		",breakdown_horizontal_pairs_subquery," 
	) AS filtered_horizontal_pairs_subquery,
	( 
		",breakdown_vertical_pairs_subquery," 
	) AS filtered_vertical_pairs_subquery"
);

SET breakdown_traffic_dimension_ids_subquery = CONCAT(
"SELECT 
	dimension_id, value_string
FROM 
	dimension_table, 
	( 
		",breakdown_horizontal_vertical_pairs_subquery," 
	) AS filtered_horizontal_vertical_pairs_subquery 
WHERE 
	( filtered_horizontal_vertical_pairs_subquery.horizontal_relationship_id <=> dimension_table.horizontal_relationship_id ) 
	AND ( filtered_horizontal_vertical_pairs_subquery.vertical_relationship_id <=> dimension_table.vertical_relationship_id )"
);

SET breakdown_traffic_message_count_by_collection_id = CONCAT(
"SELECT 
	dimension_id, value_string, collection_id, message_count 
FROM ( 
		",breakdown_traffic_dimension_ids_subquery," 
	) as breakdown_traffic_dimension_ids_subquery 
	INNER JOIN ",data_for_group_X_table_collection," USING (dimension_id)"
);

/******************************************************************************/
/************************** Breakdown & Filter - Collection *******************/
/******************************************************************************/
/* In the case the breakdown is vertical we don't need (left join) the   */
/* breakdown-query (breakdown_traffic_message_count_by_collection_id)    */ 
/* to the filter-query (filtered_traffic_message_count_by_collection_id) */ 
/* (like we have to do in the case only horizontals are selected in      */
/* order to find the IValueUnassigned values). We could just use the     */
/* filter value, this would give the same value and have less queries    */ 
/* to run thus improving efficiency at the cost of some complexity       */

IF (@isBreakdownHorizontalOrVertical = 'vertical' ) THEN
	/* Vertical breakdown */
	SET filtered_and_breakdown_traffic_message_count_by_collection_id = CONCAT(
"SELECT 
	breakdown_traffic_message_count_by_collection_id.value_string AS value_string, 
	breakdown_traffic_message_count_by_collection_id.message_count as message_count, 
	breakdown_traffic_message_count_by_collection_id.dimension_id, 
	breakdown_traffic_message_count_by_collection_id.collection_id 
FROM ( 
		",breakdown_traffic_message_count_by_collection_id,"
	) AS breakdown_traffic_message_count_by_collection_id "
	);
ELSE
	/* Horizontal Breakdown - */
	/* We join the filter and the breakdown because we need to calculate the   */   
	/* horizontal superset separately from breakdown subset. The Horizontals   */
	/* in the (filter) superset that do not appear in the (breakdown) subset   */
	/* are IValueUnassigned values.                                            */
	/* We join the Breakdown and Filter queries at the data_for_group_X_table  */
	/* level and not the dimension_table because we need to join based on      */
	/* collection ID and dimension ID. It's possible a collection doesn't have */ 
	/* a dimension ID that others do.                                          */
	
	SET filtered_and_breakdown_traffic_message_count_by_collection_id = CONCAT(
"SELECT 
	COALESCE(breakdown_traffic_message_count_by_collection_id.value_string, filtered_traffic_message_count_by_collection_id.value_string) AS value_string, 
	COALESCE(breakdown_traffic_message_count_by_collection_id.message_count, filtered_traffic_message_count_by_collection_id.message_count) as message_count, 
	filtered_traffic_message_count_by_collection_id.dimension_id, 
	filtered_traffic_message_count_by_collection_id.collection_id 
FROM ( 
		",filtered_traffic_message_count_by_collection_id," 
	) AS filtered_traffic_message_count_by_collection_id 
	LEFT Join ( 
			",breakdown_traffic_message_count_by_collection_id," 
		) AS breakdown_traffic_message_count_by_collection_id 
	ON (filtered_traffic_message_count_by_collection_id.collection_id = breakdown_traffic_message_count_by_collection_id.collection_id) 
	AND (filtered_traffic_message_count_by_collection_id.dimension_id = breakdown_traffic_message_count_by_collection_id.dimension_id)"
	);
END IF;



/* Outer most query - Collection */
SET final_collection_query = CONCAT(
"
SELECT 
	 UNIX_TIMESTAMP(@collection_date_time) as time, value_string, message_count
FROM ( 
		SELECT 
			value_string, SUM(message_count) AS message_count, collection_id, total_message_count_by_collection 
		FROM ( 
				",filtered_and_breakdown_traffic_message_count_by_collection_id,"
			) AS filtered_and_breakdown_traffic_message_count_by_collection_id 
			JOIN ( 
				",total_traffic_message_count_by_collection_id," ) AS total_traffic_message_count_by_collection_id USING (collection_id) 
		GROUP BY value_string, collection_id 
		ORDER BY collection_id) as final_collection_query 
	WHERE collection_id = '",userCollectionId, "'");

CALL tpf_execute_immediate(final_collection_query);

END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.getPercentOfFilteredTrafficByGroup
DROP PROCEDURE IF EXISTS `getPercentOfFilteredTrafficByGroup`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `getPercentOfFilteredTrafficByGroup`(
	IN `userGroupId` VARCHAR(50),
	IN `userHorizontal` BLOB,
	IN `userVertical` BLOB,
	IN `userBreakdown` VARCHAR(100)











)
getPercentOfFilteredTrafficByGroupProc:BEGIN 
/************************************************************************/
/*  IBM TEMPLATE: getPercentOfFilteredTrafficByGroup                    */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used to query results regarding all collections in */
/* a group. Specifically this will find the VALUE results (relating     */
/* to the NAME specified in the BREAKDOWN input) and their corisponding */
/* percentage. The percentage will be calculated by taking              */
/* the message count for that VALUE and dividing it by the total        */
/* message count of the message traffic that occured in the FILTERed    */
/* traffic specified in the input.                                      */
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* userHorizontal blob - This is the list of horizontal NVPs  the user  */
/*    wants to breakdown percentage by.                                 */
/* userVertical blob - This is the list of vertical NVPs the user wants */
/*    to breakdown percentage by.                                       */
/* userBreakdown varchar(100) - This is the NVP the user wants to       */
/*    breakdown percentages by.                                         */
/* userGroupIds varchar(50) - The group id of the group being compared. */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (3 columns):                                                   */
/* value_string - The Value portion of the Name Value Pair relating to  */
/*    the Name specified in the breakdown. In other words the Name in   */
/*    the breakdown is made up of these values.                         */
/* Date - the date and time at which that Value_String-collection_id    */
/*    set was collected.                                                */
/* percentage - message_count / total_message_count_by_collection * 100 */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* call getPercentOfFilteredTrafficByCollectionAndBasline(              */
/*   '0982ba590eb098fb9082adcff234ef3233331',                           */
/*   'MsgType=Booking',                                                 */
/*   'IOwnerHi=PackageA',                                               */
/*   'IOwnerMid'                                                        */
/* );                                                                   */
/*                                                                      */
/* call getPercentOfFilteredTrafficByCollectionAndBasline(              */
/*   'the-group-id-hash',                                               */
/*   'horizontal-NVP-filter1,horizontal-NVP-filter2',                   */
/*   'vertical-NVP-filter1,vertical-NVP-filter2',                       */
/*   'breakdown'                                                        */
/* );                                                                   */
/*                                                                      */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/

/******************************************************************************/
/******************************** DECLARES ************************************/
/******************************************************************************/

DECLARE subStringLength                                           INT;
DECLARE stringLength                                              INT;
DECLARE relationship_string_subquery                              VARCHAR     (200);
DECLARE relationship_horizontal_string_subquery                   BLOB;
DECLARE relationship_vertical_string_subquery                     BLOB;
DECLARE select_horizontal_value_string                            VARCHAR     (20);
DECLARE select_vertical_value_string                              VARCHAR     (20);
DECLARE breakdown_where_vertical_value_string_is                  VARCHAR     (100);
DECLARE breakdown_where_horizontal_value_string_is                VARCHAR     (100);

DECLARE filtered_horizontal_pairs_subquery                        BLOB;
DECLARE filtered_vertical_pairs_subquery                          BLOB;
DECLARE filtered_horizontal_vertical_pairs_subquery               BLOB;
DECLARE filtered_traffic_dimension_ids_subquery                   BLOB;

DECLARE filtered_horizontal_traffic_where                         BLOB;
DECLARE filtered_vertical_traffic_where                           BLOB;

DECLARE breakdown_horizontal_vertical_pairs_subquery              BLOB;
DECLARE breakdown_traffic_dimension_ids_subquery                  BLOB;
DECLARE breakdown_where                                           BLOB;
DECLARE breakdown_horizontal_pairs_subquery                       BLOB;
DECLARE breakdown_vertical_pairs_subquery                         BLOB;
DECLARE breakdown_horizontal_traffic_where                        BLOB;
DECLARE breakdown_vertical_traffic_where                          BLOB;

DECLARE total_horizontal_vertical_pairs_subquery                  BLOB;
DECLARE total_traffic_dimension_ids_subquery                      BLOB;
DECLARE total_traffic_message_count_by_collection_id              BLOB;
DECLARE filtered_traffic_message_count_by_collection_id           BLOB;
DECLARE breakdown_traffic_message_count_by_collection_id          BLOB;

DECLARE data_for_group_X_table                                    VARCHAR 		(100);



DECLARE filtered_and_breakdown_traffic_message_count_by_collection_id		BLOB;
DECLARE final_collection_query                                    BLOB;

DECLARE number_of_horizontal_filters                              INT         DEFAULT 0;
DECLARE number_of_vertical_filters                                INT         DEFAULT 0;

/* 0 - none        */
/* 1 - horizontal  */
/* 2 - vertical    */
DECLARE is_breakdown_name_string_in_filter                        BOOL        DEFAULT 0;


/******************************************************************************/
/******************************* Initialize ***********************************/
/******************************************************************************/

SET filtered_horizontal_traffic_where = "";
SET filtered_vertical_traffic_where = "";

/******************************************************************************/
/***************************** Input checking *********************************/
/******************************************************************************/

/* refresh views */
call updateViews();

IF !(EXISTS (SELECT group_id FROM viewOfAllCollectionGroupIdPairs WHERE group_id = userGroupId)) THEN
	SELECT "Group Id doesn't exist";
	LEAVE getPercentOfFilteredTrafficByGroupProc;
END IF;

IF !(userBreakdown IS NULL OR userBreakdown = "" OR userBreakdown = "NULL") THEN
	CALL tpf_execute_immediate(CONCAT('SELECT relationship_type INTO @isBreakdownHorizontalOrVertical from name_table WHERE name_string = "', userBreakdown,'"'));
	SET breakdown_where = CONCAT("WHERE (name_string = '",TRIM(userBreakdown),"')");
ELSE
	LEAVE getPercentOfFilteredTrafficByGroupProc;
END IF;

IF !(userGroupId IS NULL OR userGroupId = "" OR userGroupId = "NULL") THEN
	SET data_for_group_X_table = CONCAT("data_for_group_table_", trim(userGroupId));
ELSE
	LEAVE getPercentOfFilteredTrafficByGroupProc;
END IF;

/******************************************************************************/
/******************************** Parsing *************************************/
/******************************************************************************/

read_horizontal_input:
LOOP
	IF (userHorizontal IS NULL OR userHorizontal = "" OR userHorizontal = "NULL") THEN
		LEAVE read_horizontal_input;
	END IF;
	/* Start or continue where statement */
	IF (number_of_horizontal_filters <= 0) THEN
		SET filtered_horizontal_traffic_where = " WHERE ( ";
	ELSE
		SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, " OR "); 
	END IF;
	SET number_of_horizontal_filters = number_of_horizontal_filters + 1;
	SET stringLength = LENGTH(userHorizontal);
	/* start to "=" */
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, " ( name_string = '");
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, TRIM(SUBSTRING_INDEX(userHorizontal, '=', 1)));
	IF (userBreakdown = TRIM(SUBSTRING_INDEX(userHorizontal, '=', 1))) THEN
		SET is_breakdown_name_string_in_filter = 1;
	END IF;
	SET subStringLength = LENGTH(SUBSTRING_INDEX(userHorizontal, '=', 1))+2;
	SET userHorizontal = MID(userHorizontal, subStringLength, stringLength);
	/* "=" to "," */
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, "' AND value_string = '");
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, TRIM(SUBSTRING_INDEX(userHorizontal, ',', 1)));
	SET subStringLength = LENGTH(SUBSTRING_INDEX(userHorizontal, ',', 1))+2;
	SET userHorizontal = MID(userHorizontal, subStringLength, stringLength);
	/* close () */
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, "' ) ");
END LOOP read_horizontal_input;
SET breakdown_horizontal_traffic_where = filtered_horizontal_traffic_where;
IF ( LENGTH(filtered_horizontal_traffic_where) > 0 ) THEN
	SET filtered_horizontal_traffic_where = CONCAT(filtered_horizontal_traffic_where, " ) AND cardinality >=",number_of_horizontal_filters," ");
END IF;

IF !(userVertical IS NULL OR userVertical = "" OR userVertical = "NULL") THEN
	read_vertical_input:
	LOOP
		IF (userVertical IS NULL OR userVertical = "" OR userVertical = "NULL") THEN
			LEAVE read_vertical_input;
		END IF;
		/* Start or continue where statement */
		IF (number_of_vertical_filters <= 0) THEN
			SET filtered_vertical_traffic_where = " WHERE ( ";
		ELSE
			SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, " OR "); 
		END IF;
		SET number_of_vertical_filters = number_of_vertical_filters + 1;
		SET stringLength = LENGTH(userVertical);
		/* start to "=" */
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, " ( name_string = '");
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, TRIM(SUBSTRING_INDEX(userVertical, '=', 1)));
		IF (userBreakdown = TRIM(SUBSTRING_INDEX(userVertical, '=', 1))) THEN
			SET is_breakdown_name_string_in_filter = 2;
		END IF;
		SET subStringLength = LENGTH(SUBSTRING_INDEX(userVertical, '=', 1))+2;
		SET userVertical = MID(userVertical, subStringLength, stringLength);
		/* "=" to "," */
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, "' AND value_string = '");
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, TRIM(SUBSTRING_INDEX(userVertical, ',', 1)));
		SET subStringLength = LENGTH(SUBSTRING_INDEX(userVertical, ',', 1))+2;
		SET userVertical = MID(userVertical, subStringLength, stringLength);
		/* close () */
		SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, "' ) ");
	END LOOP read_vertical_input;
END IF;
SET breakdown_vertical_traffic_where = filtered_vertical_traffic_where;
IF ( LENGTH(filtered_vertical_traffic_where) > 0 ) THEN
	SET filtered_vertical_traffic_where = CONCAT(filtered_vertical_traffic_where, " ) AND cardinality =",number_of_vertical_filters," ");
END IF;


/******************************************************************************/
/**************************** Filters - Collection ****************************/
/******************************************************************************/


	
/* In the case no horizontal filter is specified, we want to select all horizontals. */
SET filtered_horizontal_pairs_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id
FROM 
	viewOfHorizontalRelationshipTableWithNameAndValueStrings
",filtered_horizontal_traffic_where," 
GROUP BY horizontal_relationship_id HAVING COUNT(horizontal_relationship_id)>=",number_of_horizontal_filters," "
);

/* In the case no vertical filter is specified, we want to select vertical only when they are NULL. */
IF (number_of_vertical_filters > 0) THEN
	SET filtered_vertical_pairs_subquery = CONCAT(
	"SELECT 
		DISTINCT vertical_relationship_id
	FROM 
		viewOfVerticalRelationshipTableWithNameAndValueStrings
	",filtered_vertical_traffic_where," 
	GROUP BY vertical_relationship_id HAVING COUNT(vertical_relationship_id)=",number_of_vertical_filters," "
);
ELSE
	SET filtered_vertical_pairs_subquery = "SELECT NULL AS vertical_relationship_id";
END IF;

SET filtered_horizontal_vertical_pairs_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id, vertical_relationship_id 
FROM ( 
		",filtered_horizontal_pairs_subquery," 
	) AS filtered_horizontal_pairs_subquery,
	( 
		",filtered_vertical_pairs_subquery," 
	) AS filtered_vertical_pairs_subquery"
);

SET filtered_traffic_dimension_ids_subquery = CONCAT(
"SELECT 
	dimension_id 
FROM 
	dimension_table, 
	( 
		",filtered_horizontal_vertical_pairs_subquery," 
	) AS filtered_horizontal_vertical_pairs_subquery 
WHERE 
	( filtered_horizontal_vertical_pairs_subquery.horizontal_relationship_id <=> dimension_table.horizontal_relationship_id ) 
	AND ( filtered_horizontal_vertical_pairs_subquery.vertical_relationship_id <=> dimension_table.vertical_relationship_id )"
);

SET filtered_traffic_message_count_by_collection_id = CONCAT(
"SELECT 
	dimension_id, 'IValueUnassigned' as value_string, collection_id, message_count 
FROM ( 
		",filtered_traffic_dimension_ids_subquery," 
	) as filtered_traffic_dimension_ids_subquery 
	INNER JOIN ",data_for_group_X_table," USING (dimension_id)"
);

/******************************************************************************/
/********************** Total Message Count - Collection **********************/
/******************************************************************************/

/* Total message counts by exit row. */
SET total_traffic_message_count_by_collection_id = CONCAT(
	"SELECT collection_id, SUM(message_count) as total_message_count_by_collection 
	FROM ( ",filtered_traffic_dimension_ids_subquery," ) as total_traffic_dimension_ids_subquery 
		INNER JOIN ",data_for_group_X_table," USING (dimension_id) 
	GROUP BY collection_id"
);

/******************************************************************************/
/**************************** Breakdown - Collection **************************/
/******************************************************************************/
/*add the breakdown to the filter WHERE statements and increase the correct COUNT */
IF (@isBreakdownHorizontalOrVertical = 'vertical' ) THEN
	SET breakdown_where_vertical_value_string_is = CONCAT(" (name_string = '",TRIM(userBreakdown),"')AND ");
	SET breakdown_where_horizontal_value_string_is = "";
	SET select_vertical_value_string = ", value_string"; 
	SET select_horizontal_value_string = "";
	
	/* Check if the breakdown name string appeared in the user's horizontal filters, if it did we won't add the breakdown name string to our search query */
	IF(is_breakdown_name_string_in_filter != 2) THEN
		SET number_of_vertical_filters = number_of_vertical_filters + 1;
		IF !(breakdown_vertical_traffic_where IS NULL OR breakdown_vertical_traffic_where = "") THEN
			SET breakdown_vertical_traffic_where = CONCAT(breakdown_vertical_traffic_where, " OR ( name_string = '", TRIM(userBreakdown), "' ) ");
		ELSE
			SET breakdown_vertical_traffic_where = CONCAT(" WHERE ( ( name_string = '", TRIM(userBreakdown), "') ");
		END IF;
	END IF;
ELSE	
	SET breakdown_where_horizontal_value_string_is = CONCAT(" (name_string = '",TRIM(userBreakdown),"')AND ");
	SET breakdown_where_vertical_value_string_is = "";
	SET select_horizontal_value_string = ", value_string";
	SET select_vertical_value_string = ""; 
	
	/* Check if the breakdown name string appeared in the user's horizontal filters, if it did we won't add the breakdown name string to our search query */
	IF(is_breakdown_name_string_in_filter != 1) THEN
		SET number_of_horizontal_filters = number_of_horizontal_filters + 1;
		IF !(breakdown_horizontal_traffic_where IS NULL OR breakdown_horizontal_traffic_where = "") THEN
			SET breakdown_horizontal_traffic_where = CONCAT(breakdown_horizontal_traffic_where, " OR ( name_string = '", TRIM(userBreakdown), "' ) ");
		ELSE
			SET breakdown_horizontal_traffic_where = CONCAT(" WHERE ( ( name_string = '", TRIM(userBreakdown), "') ");
		END IF;
	END IF;
END IF;

IF (LENGTH(breakdown_vertical_traffic_where) > 0) THEN
	SET breakdown_vertical_traffic_where = CONCAT(breakdown_vertical_traffic_where, " ) AND cardinality =",number_of_vertical_filters," ");
END IF;
IF (LENGTH(breakdown_horizontal_traffic_where) > 0) THEN
	SET breakdown_horizontal_traffic_where = CONCAT(breakdown_horizontal_traffic_where, " ) AND cardinality >=",number_of_horizontal_filters," ");
END IF;

/** Relationship-Name-Value Table Query */
/* Find all the relationship_ids that have at least the filter and the breakdown */
/* similar to filtered_horizontal_pairs_subquery but includes the breakdown */
SET relationship_horizontal_string_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id
FROM 
	viewOfHorizontalRelationshipTableWithNameAndValueStrings
",breakdown_horizontal_traffic_where," 
GROUP BY horizontal_relationship_id HAVING COUNT(horizontal_relationship_id)>=",number_of_horizontal_filters," "
);

/* Find all the relationship_ids that have exactly the filter and the breakdown */
/* similar to filtered_vertical_pairs_subquery but includes the breakdown*/
SET relationship_vertical_string_subquery = CONCAT(
"SELECT 
	DISTINCT vertical_relationship_id
FROM 
	viewOfVerticalRelationshipTableWithNameAndValueStrings
",breakdown_vertical_traffic_where," 
GROUP BY vertical_relationship_id HAVING COUNT(vertical_relationship_id)=",number_of_vertical_filters," "
);

/* In the case no horizontal filter is specified, we want to select all horizontals. */
SET breakdown_horizontal_pairs_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id",select_horizontal_value_string,"
FROM 
	viewOfHorizontalRelationshipTableWithNameAndValueStrings
WHERE
	",breakdown_where_horizontal_value_string_is,"
	horizontal_relationship_id IN (
		",relationship_horizontal_string_subquery,"
	) "
);		

/* In the case no vertical filter is specified, we want to select vertical only when they are NULL. */
IF (number_of_vertical_filters > 0) THEN
	SET breakdown_vertical_pairs_subquery = CONCAT(
	"SELECT 
	DISTINCT vertical_relationship_id",select_vertical_value_string,"
	FROM 
		viewOfVerticalRelationshipTableWithNameAndValueStrings
	WHERE
		",breakdown_where_vertical_value_string_is," 
		vertical_relationship_id IN (
			",relationship_vertical_string_subquery,"
		) "
	);
ELSE
	SET breakdown_vertical_pairs_subquery = " SELECT NULL AS vertical_relationship_id ";
END IF;

SET breakdown_horizontal_vertical_pairs_subquery = CONCAT(
"SELECT 
	DISTINCT horizontal_relationship_id, vertical_relationship_id, value_string
FROM ( 
		",breakdown_horizontal_pairs_subquery," 
	) AS filtered_horizontal_pairs_subquery,
	( 
		",breakdown_vertical_pairs_subquery," 
	) AS filtered_vertical_pairs_subquery"
);

SET breakdown_traffic_dimension_ids_subquery = CONCAT(
"SELECT 
	dimension_id, value_string
FROM 
	dimension_table, 
	( 
		",breakdown_horizontal_vertical_pairs_subquery," 
	) AS filtered_horizontal_vertical_pairs_subquery 
WHERE 
	( filtered_horizontal_vertical_pairs_subquery.horizontal_relationship_id <=> dimension_table.horizontal_relationship_id ) 
	AND ( filtered_horizontal_vertical_pairs_subquery.vertical_relationship_id <=> dimension_table.vertical_relationship_id )"
);

SET breakdown_traffic_message_count_by_collection_id = CONCAT(
"SELECT 
	dimension_id, value_string, collection_id, message_count 
FROM ( 
		",breakdown_traffic_dimension_ids_subquery," 
	) as breakdown_traffic_dimension_ids_subquery 
	INNER JOIN ",data_for_group_X_table," USING (dimension_id)"
);

/******************************************************************************/
/************************** Breakdown & Filter - Collection *******************/
/******************************************************************************/
/* In the case the breakdown is vertical we don't need (left join) the   */
/* breakdown-query (breakdown_traffic_message_count_by_collection_id)    */ 
/* to the filter-query (filtered_traffic_message_count_by_collection_id) */ 
/* (like we have to do in the case only horizontals are selected in      */
/* order to find the IValueUnassigned values). We could just use the     */
/* filter value, this would give the same value and have less queries    */ 
/* to run thus improving efficiency at the cost of some complexity       */

IF (@isBreakdownHorizontalOrVertical = 'vertical' ) THEN
	/* Vertical breakdown */
	SET filtered_and_breakdown_traffic_message_count_by_collection_id = CONCAT(
	"SELECT 
		breakdown_traffic_message_count_by_collection_id.value_string AS value_string, 
		breakdown_traffic_message_count_by_collection_id.message_count as message_count, 
		breakdown_traffic_message_count_by_collection_id.dimension_id, 
		breakdown_traffic_message_count_by_collection_id.collection_id 
	FROM ( ",breakdown_traffic_message_count_by_collection_id," ) AS breakdown_traffic_message_count_by_collection_id "
	);
ELSE
	/* Horizontal Breakdown - */
	/* We join the filter and the breakdown because we need to calculate the   */   
	/* horizontal superset separately from breakdown subset. The Horizontals   */
	/* in the (filter) superset that do not appear in the (breakdown) subset   */
	/* are IValueUnassigned values.                                            */
	/* We join the Breakdown and Filter queries at the data_for_group_X_table  */
	/* level and not the dimension_table because we need to join based on      */
	/* collection ID and dimension ID. It's possible a collection doesn't have */ 
	/* a dimension ID that others do.                                          */
	
	SET filtered_and_breakdown_traffic_message_count_by_collection_id = CONCAT(
	"SELECT 
		COALESCE(breakdown_traffic_message_count_by_collection_id.value_string, filtered_traffic_message_count_by_collection_id.value_string) AS value_string, 
		COALESCE(breakdown_traffic_message_count_by_collection_id.message_count, filtered_traffic_message_count_by_collection_id.message_count) as message_count, 
		filtered_traffic_message_count_by_collection_id.dimension_id, 
		filtered_traffic_message_count_by_collection_id.collection_id 
	FROM ( ",filtered_traffic_message_count_by_collection_id," ) AS filtered_traffic_message_count_by_collection_id 
		LEFT Join ( ",breakdown_traffic_message_count_by_collection_id," ) AS breakdown_traffic_message_count_by_collection_id 
		ON (filtered_traffic_message_count_by_collection_id.collection_id = breakdown_traffic_message_count_by_collection_id.collection_id) 
		AND (filtered_traffic_message_count_by_collection_id.dimension_id = breakdown_traffic_message_count_by_collection_id.dimension_id)"
	);
END IF;

/* The STR_TO_DATE function is more than what is needed. Only the concat is required. */
/* However, this gives an example of how one could customize the date output if needed. */								
SET final_collection_query = CONCAT(
	"SELECT UNIX_TIMESTAMP(STR_TO_DATE(concat(collection_date, ' ',collection_time), '%Y-%m-%d %H:%i:%s')) AS time, value_string, (message_count/total_message_count_by_collection * 100) AS percentage
	FROM ( 
		SELECT value_string, SUM(message_count) AS message_count, collection_id, total_message_count_by_collection 
		FROM (",filtered_and_breakdown_traffic_message_count_by_collection_id,") AS filtered_and_breakdown_traffic_message_count_by_collection_id 
					JOIN ( ",total_traffic_message_count_by_collection_id," ) AS total_traffic_message_count_by_collection_id USING (collection_id) 
			GROUP BY value_string, collection_id 
			ORDER BY collection_id) as final_collection_query 
		JOIN
			collection_table USING (collection_id)");

CALL tpf_execute_immediate(final_collection_query);

END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.getValues
DROP PROCEDURE IF EXISTS `getValues`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `getValues`(
	IN `userNameString` VARCHAR(50),
	IN `userGroupId` VARCHAR(50)

)
BEGIN
/************************************************************************/
/* IBM TEMPLATE: getValues                                              */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used to get a list of values relating to a Name.   */  
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* groupId varchar(50) - (Required) The group id of the group being     */
/*    compared.                                                         */
/* valuesList varchar(100) - (Required) Enter a Name to find all        */
/*    possible Values for the Name specified.                           */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (1 column):                                                    */
/* value_string - List of the values in the group, specified in         */
/*    userGroupId, relating to the Name, specified in userNameString.   */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* CALL getValues(                                                      */
/*    "msgtype",                                                        */
/*    "09181fbe0a87ca08df0a987e8f6");                                   */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/ 
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
DECLARE valuesForGroupStr BLOB(400);
SET valuesForGroupStr = "SELECT DISTINCT value_string FROM ";
SET valuesForGroupStr = CONCAT(valuesForGroupStr, " (SELECT  relationship_table.relationship_id,name_string,value_table.value_string ");
SET valuesForGroupStr = CONCAT(valuesForGroupStr, " FROM relationship_table INNER JOIN name_table USING (name_id) INNER JOIN value_table USING (value_id) ");
SET valuesForGroupStr = CONCAT(valuesForGroupStr, " WHERE relationship_id IN	");
SET valuesForGroupStr = CONCAT(valuesForGroupStr, "(SELECT horizontal_relationship_id FROM  dimension_table WHERE dimension_id IN ");
SET valuesForGroupStr = CONCAT(valuesForGroupStr, " (SELECT DISTINCT dimension_id FROM ");
SET valuesForGroupStr = CONCAT(valuesForGroupStr, "data_for_group_table_", userGroupId, ") ");

SET valuesForGroupStr = CONCAT(valuesForGroupStr, " ) OR relationship_id IN	");
SET valuesForGroupStr = CONCAT(valuesForGroupStr, "(SELECT vertical_relationship_id FROM  dimension_table WHERE dimension_id IN ");
SET valuesForGroupStr = CONCAT(valuesForGroupStr, " (SELECT DISTINCT dimension_id FROM ");
SET valuesForGroupStr = CONCAT(valuesForGroupStr, "data_for_group_table_", userGroupId, ") ");

SET valuesForGroupStr = CONCAT(valuesForGroupStr, "  ) ORDER BY name_string) as table1 ");
SET valuesForGroupStr = CONCAT(valuesForGroupStr, "WHERE name_string='",userNameString,"'");

CALL tpf_execute_immediate(valuesForGroupStr);

END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.mbGetBaseCollectionList
DROP PROCEDURE IF EXISTS `mbGetBaseCollectionList`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `mbGetBaseCollectionList`(
	IN `groupId` VARCHAR(50),
	IN `collectionId` VARCHAR(50),
	IN `breakdownName` varchar(100),
	IN `valuesList` varchar(100),
	IN `metricsList` VARCHAR(100),
	IN `baseCollectionId` VARCHAR(50),
	IN `horizontalFilter` varchar(100),
	IN `verticalFilter` VARCHAR(100)

)
mbGetBaseCollectionList:BEGIN
/************************************************************************/
/* IBM TEMPLATE: mbGetBaseCollectionList                                */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used to used by metabase to call                   */
/* mbGetBaseCollectionList to get a list of collections in the          */
/* data_for_group_table_baseline table.                                 */ 
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* groupId varchar(50) - (Not Used) The group id of the group being     */
/*    compared.                                                         */
/* collectionId varchar(50) - (Not Used) The collection id of the       */
/*    collection to query against.                                      */
/* breakdownName varchar(100) - (Not Used) This is the NVP the user     */
/*    wants to breakdown percentages by.                                */
/* valuesList varchar(100) - (Not Used) Enter a Name to find all        */
/*    possible Values for the Name specified.                           */
/* metricsList varchar(100) - (Not Used) A list of comma seperated      */
/*    metrics who's information is desired by the user.                 */
/* baseCollectionId varchar(50) - (Not Used) The collection id of the   */
/*    second collection to compared against the first. This is          */
/*    typically a baseline collection.                                  */
/* horizontalFilter varchar(100) - (Not Used) This is the list of       */
/*    horizontal NVPs the user wants to breakdown percentage by.        */
/* verticalFilter varchar(100) - (Not Used) This is the list of         */
/*    vertical NVPs the user wants to breakdown percentage by.          */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (1 column):                                                    */
/* collection_id - List of the collection_ids in the baseline table     */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* CALL mbGetBaseCollectionList(                                        */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "");                                                              */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/ 
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
CALL getBaseCollectionList();
END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.mbGetCollectionHorizontalNames
DROP PROCEDURE IF EXISTS `mbGetCollectionHorizontalNames`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `mbGetCollectionHorizontalNames`(
	IN `groupId` VARCHAR(50),
	IN `collectionId` VARCHAR(50),
	IN `breakdownName` varchar(100),
	IN `valuesList` varchar(100),
	IN `metricsList` VARCHAR(100),
	IN `baseCollectionId` VARCHAR(50),
	IN `horizontalFilter` varchar(100),
	IN `verticalFilter` VARCHAR(100)

)
mbGetCollectionHorizontalNames:BEGIN
/************************************************************************/
/* IBM TEMPLATE: mbGetCollectionHorizontalNames                         */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used to used by metabase to call                   */
/* mbGetcollectionHorizontalNames to get a list of horizontal names     */
/* used in the group specified.                                         */ 
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* groupId varchar(50) - (Required) The group id of the group being     */
/*    compared.                                                         */
/* collectionId varchar(50) - (Not Used) The collection id of the       */
/*    collection to query against.                                      */
/* breakdownName varchar(100) - (Not Used) This is the NVP the user     */
/*    wants to breakdown percentages by.                                */
/* valuesList varchar(100) - (Not Used) Enter a Name to find all        */
/*    possible Values for the Name specified.                           */
/* metricsList varchar(100) - (Not Used) A list of comma seperated      */
/*    metrics who's information is desired by the user.                 */
/* baseCollectionId varchar(50) - (Not Used) The collection id of the   */
/*    second collection to compared against the first. This is          */
/*    typically a baseline collection.                                  */
/* horizontalFilter varchar(100) - (Not Used) This is the list of       */
/*    horizontal NVPs the user wants to breakdown percentage by.        */
/* verticalFilter varchar(100) - (Not Used) This is the list of         */
/*    vertical NVPs the user wants to breakdown percentage by.          */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (1 column):                                                    */
/* name_string - List of the horizontal names in the group requested    */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* CALL mbGetCollectionHorizontalNames(                                 */
/*    "310987fea03987198c371931237caf",                                 */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "");                                                              */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/ 
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
IF (groupId IS NULL OR groupId = 'NULL') THEN
	/* Return an empty table for clean Metabase interface. */
	SELECT * FROM name_table WHERE FALSE;
	LEAVE mbGetCollectionHorizontalNames;
END IF;
	CALL getCollectionHorizontals(groupId);
END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.mbGetCollectionList
DROP PROCEDURE IF EXISTS `mbGetCollectionList`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `mbGetCollectionList`(
	IN `groupId` VARCHAR(50),
	IN `collectionId` VARCHAR(50),
	IN `breakdownName` varchar(100),
	IN `valuesList` varchar(100),
	IN `metricsList` VARCHAR(100),
	IN `baseCollectionId` VARCHAR(50),
	IN `horizontalFilter` varchar(100),
	IN `verticalFilter` VARCHAR(100)

)
mbGetCollectionList:BEGIN
/************************************************************************/
/* IBM TEMPLATE: mbGetCollectionList                                    */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used by metabase to call                           */
/* getCollectionList to get a list of collections in the                */
/* group specified.                                                     */ 
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* groupId varchar(50) - (Required) The group id of the group being     */
/*    compared.                                                         */
/* collectionId varchar(50) - (Not Used) The collection id of the       */
/*    collection to query against.                                      */
/* breakdownName varchar(100) - (Not Used) This is the NVP the user     */
/*    wants to breakdown percentages by.                                */
/* valuesList varchar(100) - (Not Used) Enter a Name to find all        */
/*    possible Values for the Name specified.                           */
/* metricsList varchar(100) - (Not Used) A list of comma seperated      */
/*    metrics who's information is desired by the user.                 */
/* baseCollectionId varchar(50) - (Not Used) The collection id of the   */
/*    second collection to compared against the first. This is          */
/*    typically a baseline collection.                                  */
/* horizontalFilter varchar(100) - (Not Used) This is the list of       */
/*    horizontal NVPs the user wants to breakdown percentage by.        */
/* verticalFilter varchar(100) - (Not Used) This is the list of         */
/*    vertical NVPs the user wants to breakdown percentage by.          */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (4 columns):                                                   */
/* collection_id - the Id hash of the collection.                       */
/* collection_date_time - the date and time, with seconds, of when the  */
/*    collection was collected.                                         */
/* messages_collected - number of messages collected during collection. */
/* messages_skipped - number of messages skipped during collection.     */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* CALL mbGetCollectionList(                                            */
/*    "310987fea03987198c371931237caf",                                 */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "");                                                              */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/ 
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
IF (groupId IS NULL OR groupId = 'NULL') THEN
	/* Return an empty table for clean Metabase interface. */
	SELECT * FROM name_table WHERE FALSE;
	LEAVE mbGetCollectionList;
END IF;
	CALL getCollectionList(groupId);
END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.mbGetCollectionVerticalNames
DROP PROCEDURE IF EXISTS `mbGetCollectionVerticalNames`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `mbGetCollectionVerticalNames`(
	IN `groupId` VARCHAR(50),
	IN `collectionId` VARCHAR(50),
	IN `breakdownName` varchar(100),
	IN `valuesList` varchar(100),
	IN `metricsList` VARCHAR(100),
	IN `baseCollectionId` VARCHAR(50),
	IN `horizontalFilter` varchar(100),
	IN `verticalFilter` VARCHAR(100)

)
mbGetCollectionVerticalNames:BEGIN
/************************************************************************/
/* IBM TEMPLATE: mbGetCollectionVerticalNames                           */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used to used by metabase to call                   */
/* getCollectionVerticals to get a list of vertical names used in the   */
/* group specified.                                                     */ 
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* groupId varchar(50) - (Required) The group id of the group being     */
/*    compared.                                                         */
/* collectionId varchar(50) - (Not Used) The collection id of the       */
/*    collection to query against.                                      */
/* breakdownName varchar(100) - (Not Used) This is the NVP the user     */
/*    wants to breakdown percentages by.                                */
/* valuesList varchar(100) - (Not Used) Enter a Name to find all        */
/*    possible Values for the Name specified.                           */
/* metricsList varchar(100) - (Not Used) A list of comma seperated      */
/*    metrics who's information is desired by the user.                 */
/* baseCollectionId varchar(50) - (Not Used) The collection id of the   */
/*    second collection to compared against the first. This is          */
/*    typically a baseline collection.                                  */
/* horizontalFilter varchar(100) - (Not Used) This is the list of       */
/*    horizontal NVPs the user wants to breakdown percentage by.        */
/* verticalFilter varchar(100) - (Not Used) This is the list of         */
/*    vertical NVPs the user wants to breakdown percentage by.          */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (1 column):                                                    */
/* name_string - List of the vertical names in the group requested      */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* CALL mbGetCollectionVerticalNames(                                   */
/*    "310987fea03987198c371931237caf",                                 */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "");                                                              */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/ 
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
IF (groupId IS NULL OR groupId = 'NULL') THEN
	/* Return an empty table for clean Metabase interface. */
	SELECT * FROM name_table WHERE FALSE;
	LEAVE mbGetCollectionVerticalNames;
END IF;
	CALL getCollectionVerticals(groupId);
END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.mbGetDimensionInfo
DROP PROCEDURE IF EXISTS `mbGetDimensionInfo`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `mbGetDimensionInfo`(
	IN `groupId` VARCHAR(50),
	IN `collectionId` VARCHAR(50),
	IN `breakdownName` VARCHAR(100),
	IN `valuesList` VARCHAR(100),
	IN `metricsList` VARCHAR(100),
	IN `baseCollectionId` VARCHAR(50),
	IN `horizontalFilter` VARCHAR(100),
	IN `verticalFilter` VARCHAR(100)



)
BEGIN
IF (groupId IS NULL OR groupId = 'NULL') 				OR 
	(collectionID IS NULL OR collectionID = 'NULL') 	THEN
	SELECT * FROM name_table WHERE FALSE;
ELSE
	CALL getDimensionInfo(groupID, collectionId);
END IF;
END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.mbGetGroupTable
DROP PROCEDURE IF EXISTS `mbGetGroupTable`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `mbGetGroupTable`(
	IN `groupId` VARCHAR(50),
	IN `collectionId` VARCHAR(50),
	IN `breakdownName` VARCHAR(100),
	IN `valuesList` VARCHAR(100),
	IN `metricsList` VARCHAR(100),
	IN `baseCollectionId` VARCHAR(50),
	IN `horizontalFilter` VARCHAR(100),
	IN `verticalFilter` VARCHAR(200)



)
mbGetGroup:BEGIN
/************************************************************************/
/* IBM TEMPLATE: mbGetGroupTable                                        */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used by metabase to call                           */
/* getGroupTable to get a list of groups.                               */ 
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* groupId varchar(50) - (Not Used) The group id of the group being     */
/*    compared.                                                         */
/* collectionId varchar(50) - (Not Used) The collection id of the       */
/*    collection to query against.                                      */
/* breakdownName varchar(100) - (Not Used) This is the NVP the user     */
/*    wants to breakdown percentages by.                                */
/* valuesList varchar(100) - (Not Used) Enter a Name to find all        */
/*    possible Values for the Name specified.                           */
/* metricsList varchar(100) - (Not Used) A list of comma seperated      */
/*    metrics who's information is desired by the user.                 */
/* baseCollectionId varchar(50) - (Not Used) The collection id of the   */
/*    second collection to compared against the first. This is          */
/*    typically a baseline collection.                                  */
/* horizontalFilter varchar(100) - (Not Used) This is the list of       */
/*    horizontal NVPs the user wants to breakdown percentage by.        */
/* verticalFilter varchar(100) - (Not Used) This is the list of         */
/*    vertical NVPs the user wants to breakdown percentage by.          */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (2 columns):                                                   */
/* group_id - The hash identifier of the group.                         */
/* group_string - The name of the group (dir).                          */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* CALL mbGetGroupTable(                                                */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "",                                                               */
/*    "");                                                              */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/ 
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
CALL getGroupTable();
END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.mbGetKeyMetrics
DROP PROCEDURE IF EXISTS `mbGetKeyMetrics`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `mbGetKeyMetrics`(
	IN `groupId` VARCHAR(50),
	IN `collectionId` VARCHAR(50),
	IN `breakdownName` VARCHAR(100),
	IN `valuesList` VARCHAR(100),
	IN `metricsList` VARCHAR(100),
	IN `baseCollectionId` VARCHAR(50),
	IN `horizontalFilter` VARCHAR(100),
	IN `verticalFilter` VARCHAR(100)



)
BEGIN

IF (horizontalFilter = 'none') THEN
   set horizontalFilter = 'NULL';
END IF;
IF (verticalFilter = 'none') THEN
   set verticalFilter = 'NULL';
END IF;

IF (baseCollectionId IS NULL OR baseCollectionId = 'NULL')  OR 
	(collectionId IS NULL OR collectionId = 'NULL') 			OR 
	(metricsList IS NULL OR metricsList = 'NULL')				THEN 
	/* Return an empty table for clean Metabase interface. */
	SELECT * FROM name_table WHERE FALSE;
ELSEIF collectionId = baseCollectionId THEN
	/* One possible form of error handling with Metabase. */
   SELECT "Unable to process request. Baseline collection ID cannot match the collection ID.";
ELSE
	CALL getKeyMetrics(horizontalFilter,verticalFilter,metricsList,collectionId,baseCollectionId,@foundResults);
	/* Another form of error handling, via a form of "Return Code" handling. */
	IF @foundResults = 1 THEN
		SELECT * FROM name_table WHERE FALSE;
	END IF;
END IF;

END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.mbGetMetricsAverages
DROP PROCEDURE IF EXISTS `mbGetMetricsAverages`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `mbGetMetricsAverages`(
	IN `groupId` VARCHAR(50),
	IN `collectionId` VARCHAR(50),
	IN `breakdownName` VARCHAR(100),
	IN `valuesList` VARCHAR(100),
	IN `metricsList` VARCHAR(100),
	IN `baseCollectionId` VARCHAR(50),
	IN `horizontalFilter` VARCHAR(100),
	IN `verticalFilter` VARCHAR(100)


)
BEGIN

IF (horizontalFilter = 'none') THEN
   set horizontalFilter = 'NULL';
END IF;
IF (verticalFilter = 'none') THEN
   set verticalFilter = 'NULL';
END IF;

IF (groupId IS NULL OR groupId = 'NULL') 				OR 
	(metricsList IS NULL OR metricsList = 'NULL') 	THEN
	/* Return an empty table for clean Metabase interface. */
	SELECT * FROM name_table WHERE FALSE;
ELSE
	CALL getMetricsAverages(horizontalFilter,metricsList,groupId,verticalFilter,@foundResults);
	/* A form of error handling, via a form of "Return Code" handling. */
	IF (@foundResults = 1) THEN
		SELECT * FROM name_table WHERE FALSE;
	END IF;
END IF;
END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.mbGetMetricsList
DROP PROCEDURE IF EXISTS `mbGetMetricsList`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `mbGetMetricsList`(
	IN `groupId` VARCHAR(50),
	IN `collectionId` VARCHAR(50),
	IN `breakdownName` VARCHAR(100),
	IN `valuesList` VARCHAR(100),
	IN `metricsList` VARCHAR(100),
	IN `baseCollectionId` VARCHAR(50),
	IN `horizontalFilter` VARCHAR(100),
	IN `verticalFilter` VARCHAR(100)

)
IF (groupId IS NULL OR groupId = 'NULL') THEN
	/* Return an empty table for clean Metabase interface. */
	SELECT * FROM name_table WHERE FALSE;
ELSE
	CALL getMetricsList(groupId);
END IF//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.mbGetNamesForDimensionTable
DROP PROCEDURE IF EXISTS `mbGetNamesForDimensionTable`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `mbGetNamesForDimensionTable`(
	IN `groupId` VARCHAR(50),
	IN `collectionId` VARCHAR(50),
	IN `breakdownName` VARCHAR(100),
	IN `valuesList` VARCHAR(100),
	IN `metricsList` VARCHAR(100),
	IN `baseCollectionId` VARCHAR(50),
	IN `horizontalFilter` VARCHAR(100),
	IN `verticalFilter` VARCHAR(100),
	IN `dimensionID` VARCHAR(50)

)
BEGIN
SELECT name_string, value_string
FROM dimension_table inner JOIN relationship_table
on dimension_table.horizontal_relationship_id=relationship_table.relationship_id or dimension_table.vertical_relationship_id=relationship_table.relationship_id
INNER JOIN name_table USING (name_id) INNER JOIN value_table USING (value_id)
where dimension_id=dimensionID;

END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.mbGetPercentOfFilteredTrafficByCollection
DROP PROCEDURE IF EXISTS `mbGetPercentOfFilteredTrafficByCollection`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `mbGetPercentOfFilteredTrafficByCollection`(
	IN `groupId` VARCHAR(50),
	IN `collectionId` VARCHAR(50),
	IN `breakdownName` VARCHAR(100),
	IN `valuesList` VARCHAR(100),
	IN `metricsList` VARCHAR(100),
	IN `baseCollectionId` VARCHAR(50),
	IN `horizontalFilter` VARCHAR(100),
	IN `verticalFilter` VARCHAR(100)

)
BEGIN
/************************************************************************/
/*  IBM TEMPLATE: mbGetPercentOfFilteredTrafficByCollection             */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used by metabase to get results regarding a single */
/* collection. Specifically this will find the VALUE results (relating  */
/* to the NAME specified in the BREAKDOWN input) and their corisponding */
/* message count and percentage. The percentage will be calculated by   */
/* taking the message count for that VALUE and dividing it by the total */
/* message count of the message traffic that occured in the FILTERed    */
/* traffic specified in the input.                                      */
/*                                                                      */
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* groupId varchar(50) - (Required) The group id of the group being     */
/*    compared.                                                         */
/* collectionId varchar(50) - (Required) The collection id of the       */
/*    collection to query against.                                      */
/* breakdownName varchar(100) - (Required) This is the NVP the user     */
/*    wants to breakdown percentages by.                                */
/* valuesList varchar(100) - (Not Used) Enter a Name to find all        */
/*    possible Values for the Name specified.                           */
/* metricsList varchar(100) - (Not Used) A list of comma seperated      */
/*    metrics who's information is desired by the user.                 */
/* baseCollectionId varchar(50) - (Not Used) The collection id of the   */
/*    second collection to compared against the first. This is          */
/*    typically a baseline collection.                                  */
/* horizontalFilter varchar(100) - (Optional) This is the list of       */
/*    horizontal NVPs the user wants to breakdown percentage by.        */
/* verticalFilter varchar(100) - (Optional) This is the list of         */
/*    vertical NVPs the user wants to breakdown percentage by.          */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (5 columns):                                                   */
/* value_string - The Value portion of the Name Value Pair relating to  */
/*    the Name specified in the breakdown. In other words the Name in   */
/*    the breakdown is made up of these values.                         */
/* message_count - The number of messages for that given value_string.  */
/* collection_id - The collection id in which that value_string exists. */
/* total_message_count_by_collection - the total number of messages in  */
/*    that collection_id when the horizontal/vertical filters are taken */
/*    into account.                                                     */
/* percentage - message_count / total_message_count_by_collection * 100 */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* call mbGetPercentOfFilteredTrafficByCollection(                      */
/*   '264b86d1b37d24533618545db6c18eefe1fdd',                           */
/*   '98be908f87a7e276faf76ef221187fb02ba87',                           */
/*   'IOwnerMid'                                                        */
/*   '',                                                                */
/*   '',                                                                */
/*   '',                                                                */
/*   '',                                                                */
/*   'IOwnerHi=PackageA',                                               */
/* );                                                                   */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
IF (horizontalFilter = 'none') THEN
   set horizontalFilter = 'NULL';
END IF;
IF (verticalFilter = 'none') THEN
   set verticalFilter = 'NULL';
END IF;

IF (collectionId IS NULL OR collectionId = 'NULL') 	OR 
	(breakdownName IS NULL OR breakdownName = 'NULL') 	THEN
	/* Return an empty table for clean Metabase interface. */	
	SELECT "No Results" as value_string, 0 as percentage, 0 as message_count;
ELSE
	CALL getPercentOfFilteredTrafficByCollection(collectionId,horizontalFilter,verticalFilter,breakdownName);
END IF;
END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.mbGetPercentOfFilteredTrafficByCollectionAndBaseline
DROP PROCEDURE IF EXISTS `mbGetPercentOfFilteredTrafficByCollectionAndBaseline`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `mbGetPercentOfFilteredTrafficByCollectionAndBaseline`(
	IN `groupId` VARCHAR(50),
	IN `collectionId` VARCHAR(50),
	IN `breakdownName` VARCHAR(100),
	IN `valuesList` VARCHAR(100),
	IN `metricsList` VARCHAR(100),
	IN `baseCollectionId` VARCHAR(50),
	IN `horizontalFilter` VARCHAR(100),
	IN `verticalFilter` VARCHAR(100)

)
BEGIN
/************************************************************************/
/*  IBM TEMPLATE: mbGetPercentOfFilteredTrafficByCollectionAndBasline   */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used by metabase to get results of a two unique    */
/* collections. Specifically this will find the VALUE results (relating */
/* to the NAME specified in the BREAKDOWN input) and their corisponding */
/* message count and percentage, of each collection then compare the    */
/* two with a delta value. The percentage will be calculated by taking  */
/* the message count for that VALUE and dividing it by the total        */
/* message count of the message traffic that occured in the FILTERed    */
/* traffic specified in the input.                                      */
/*                                                                      */
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* groupId varchar(50) - (Not Used) The group id of the group being     */
/*    compared.                                                         */
/* collectionId varchar(50) - (Required) The collection id of the       */
/*    collection to query against.                                      */
/* breakdownName varchar(100) - (Required) This is the NVP the user     */
/*    wants to breakdown percentages by.                                */
/* valuesList varchar(100) - (Not Used) Enter a Name to find all        */
/*    possible Values for the Name specified.                           */
/* metricsList varchar(100) - (Not Used) A list of comma seperated      */
/*    metrics who's information is desired by the user.                 */
/* baseCollectionId varchar(50) - (Required) The collection id of the   */
/*    second collection to compared against the first. This is          */
/*    typically a baseline collection.                                  */
/* horizontalFilter varchar(100) - (Optional) This is the list of       */
/*    horizontal NVPs the user wants to breakdown percentage by.        */
/* verticalFilter varchar(100) - (Optional) This is the list of         */
/*    vertical NVPs the user wants to breakdown percentage by.          */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (4 columns):                                                   */
/* value_string - The Value portion of the Name Value Pair relating to  */
/*    the Name specified in the breakdown. In other words the Name in   */
/*    the breakdown is made up of these values.                         */
/* collection_percent - The percent of traffic of the breakdown for the */
/*    collection id when horizontal/vertical filters are taken into     */
/*    account.                                                          */
/* baseline_percent - The percent of traffic of the breakdown for the   */
/*    baseline collection id when horizontal/vertical filters are taken */
/*    into account.                                                     */
/* delta - baseline_percent minus collection_percent.                   */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* call mbGetPercentOfFilteredTrafficByCollectionAndBasline(            */
/*   '264b86d1b37d24533618545db6c18eefe1fdd',                           */
/*   '98be908f87a7e276faf76ef221187fb02ba87',                           */
/*   'IOwnerMid'                                                        */
/*   '',                                                                */
/*   '',                                                                */
/*   '8eba87beb7ebb327aefdb6732897100d362a',                            */
/*   '',                                                                */
/*   'IOwnerHi=PackageA',                                               */
/* );                                                                   */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
IF (horizontalFilter = 'none') THEN
   set horizontalFilter = 'NULL';
END IF;
IF (verticalFilter = 'none') THEN
   set verticalFilter = 'NULL';
END IF;

IF (collectionId IS NULL OR collectionId = 'NULL') 			OR 
	(baseCollectionId IS NULL OR baseCollectionId = 'NULL') 	OR 
	(breakdownName IS NULL OR breakdownName = 'NULL') 			THEN
	/* Return an empty table for clean Metabase interface. */
	SELECT * FROM name_table WHERE FALSE;
ELSE
	CALL getPercentOfFilteredTrafficByCollectionAndBaseline(collectionId,baseCollectionId,horizontalFilter,verticalFilter,breakdownName);
END IF;
END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.mbgetPercentOfFilteredTrafficByCollectionMessageCount
DROP PROCEDURE IF EXISTS `mbgetPercentOfFilteredTrafficByCollectionMessageCount`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `mbgetPercentOfFilteredTrafficByCollectionMessageCount`(
	IN `groupId` VARCHAR(50),
	IN `collectionId` VARCHAR(50),
	IN `breakdownName` VARCHAR(100),
	IN `valuesList` VARCHAR(100),
	IN `metricsList` VARCHAR(100),
	IN `baseCollectionId` VARCHAR(50),
	IN `horizontalFilter` VARCHAR(100),
	IN `verticalFilter` VARCHAR(100)

)
BEGIN
/************************************************************************/
/*  IBM TEMPLATE: mbgetPercentOfFilteredTrafficByCollectionMessageCount             */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used by metabase to get results regarding a single */
/* collection. Specifically this will find the VALUE results (relating  */
/* to the NAME specified in the BREAKDOWN input) and their corisponding */
/* message count and percentage. The percentage will be calculated by   */
/* taking the message count for that VALUE and dividing it by the total */
/* message count of the message traffic that occured in the FILTERed    */
/* traffic specified in the input.                                      */
/*                                                                      */
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* groupId varchar(50) - (Required) The group id of the group being     */
/*    compared.                                                         */
/* collectionId varchar(50) - (Required) The collection id of the       */
/*    collection to query against.                                      */
/* breakdownName varchar(100) - (Required) This is the NVP the user     */
/*    wants to breakdown percentages by.                                */
/* valuesList varchar(100) - (Not Used) Enter a Name to find all        */
/*    possible Values for the Name specified.                           */
/* metricsList varchar(100) - (Not Used) A list of comma seperated      */
/*    metrics who's information is desired by the user.                 */
/* baseCollectionId varchar(50) - (Not Used) The collection id of the   */
/*    second collection to compared against the first. This is          */
/*    typically a baseline collection.                                  */
/* horizontalFilter varchar(100) - (Optional) This is the list of       */
/*    horizontal NVPs the user wants to breakdown percentage by.        */
/* verticalFilter varchar(100) - (Optional) This is the list of         */
/*    vertical NVPs the user wants to breakdown percentage by.          */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (5 columns):                                                   */
/* value_string - The Value portion of the Name Value Pair relating to  */
/*    the Name specified in the breakdown. In other words the Name in   */
/*    the breakdown is made up of these values.                         */
/* message_count - The number of messages for that given value_string.  */
/* collection_id - The collection id in which that value_string exists. */
/* total_message_count_by_collection - the total number of messages in  */
/*    that collection_id when the horizontal/vertical filters are taken */
/*    into account.                                                     */
/* percentage - message_count / total_message_count_by_collection * 100 */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* call mbgetPercentOfFilteredTrafficByCollectionMessageCount(                      */
/*   '264b86d1b37d24533618545db6c18eefe1fdd',                           */
/*   '98be908f87a7e276faf76ef221187fb02ba87',                           */
/*   'IOwnerMid'                                                        */
/*   '',                                                                */
/*   '',                                                                */
/*   '',                                                                */
/*   '',                                                                */
/*   'IOwnerHi=PackageA',                                               */
/* );                                                                   */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/
IF (horizontalFilter = 'none') THEN
   set horizontalFilter = 'NULL';
END IF;
IF (verticalFilter = 'none') THEN
   set verticalFilter = 'NULL';
END IF;

IF (collectionId IS NULL OR collectionId = 'NULL') 	OR 
	(breakdownName IS NULL OR breakdownName = 'NULL') 	THEN
	/* Return an empty table for clean Metabase interface. */	
	SELECT "No Results" as value_string, 0 as percentage, 0 as message_count;
ELSE
	CALL getPercentOfFilteredTrafficByCollectionMessageCount(collectionId,horizontalFilter,verticalFilter,breakdownName);
END IF;
END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.mbGetPercentOfFilteredTrafficByGroup
DROP PROCEDURE IF EXISTS `mbGetPercentOfFilteredTrafficByGroup`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `mbGetPercentOfFilteredTrafficByGroup`(
	IN `groupId` VARCHAR(50),
	IN `collectionId` VARCHAR(50),
	IN `breakdownName` VARCHAR(100),
	IN `valuesList` VARCHAR(100),
	IN `metricsList` VARCHAR(100),
	IN `baseCollectionId` VARCHAR(50),
	IN `horizontalFilter` VARCHAR(100),
	IN `verticalFilter` VARCHAR(100)


)
BEGIN
/************************************************************************/
/*  IBM TEMPLATE: mbGetPercentOfFilteredTrafficByGroup                  */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure is used by metabase to call                           */
/* getPercentOfFilteredTrafficByGroup. This to queries the results      */
/* regarding all collections in a group.                                */
/* Specifically this will find the VALUE results (relating              */
/* to the NAME specified in the BREAKDOWN input) and their corisponding */
/* percentage. The percentage will be calculated by taking              */
/* the message count for that VALUE and dividing it by the total        */
/* message count of the message traffic that occured in the FILTERed    */
/* traffic specified in the input.                                      */
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* groupId varchar(50) - (Required) The group id of the group being     */
/*    compared.                                                         */
/* collectionId varchar(50) - (Not Used) The collection id of the       */
/*    collection to query against.                                      */
/* breakdownName varchar(100) - (Required) This is the NVP the user     */
/*    wants to breakdown percentages by.                                */
/* valuesList varchar(100) - (Not Used) Enter a Name to find all        */
/*    possible Values for the Name specified.                           */
/* metricsList varchar(100) - (Not Used) A list of comma seperated      */
/*    metrics who's information is desired by the user.                 */
/* baseCollectionId varchar(50) - (Not Used) The collection id of the   */
/*    second collection to compared against the first. This is          */
/*    typically a baseline collection.                                  */
/* horizontalFilter varchar(100) - (Optional) This is the list of       */
/*    horizontal NVPs the user wants to breakdown percentage by.        */
/* verticalFilter varchar(100) - (Optional) This is the list of         */
/*    vertical NVPs the user wants to breakdown percentage by.          */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* TABLE (3 columns):                                                   */
/* value_string - The Value portion of the Name Value Pair relating to  */
/*    the Name specified in the breakdown. In other words the Name in   */
/*    the breakdown is made up of these values.                         */
/* Date - the date and time at which that Value_String-collection_id    */
/*    set was collected.                                                */
/* percentage - message_count / total_message_count_by_collection * 100 */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* call mbGetPercentOfFilteredTrafficByGroup(                           */
/*   '264b86d1b37d24533618545db6c18eefe1fdd',                           */
/*   '',                                                                */
/*   'IOwnerMid'                                                        */
/*   '',                                                                */
/*   '',                                                                */
/*   '',                                                                */
/*   '',                                                                */
/*   'IOwnerHi=PackageA',                                               */
/* );                                                                   */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/

IF (horizontalFilter = 'none') THEN
   set horizontalFilter = 'NULL';
END IF;
IF (verticalFilter = 'none') THEN
   set verticalFilter = 'NULL';
END IF;

IF (groupId IS NULL OR groupId = 'NULL') 					OR 
	(breakdownName  IS NULL OR breakdownName = 'NULL') THEN
	/* Return an empty table for clean Metabase interface. */
	SELECT * FROM name_table WHERE FALSE; 
ELSE
	CALL getPercentOfFilteredTrafficByGroup(groupId,horizontalFilter,verticalFilter,breakdownName);
END IF;

END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.mbGetValues
DROP PROCEDURE IF EXISTS `mbGetValues`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `mbGetValues`(
	IN `groupId` VARCHAR(50),
	IN `collectionId` VARCHAR(50),
	IN `breakdownName` VARCHAR(100),
	IN `valuesList` VARCHAR(100),
	IN `metricsList` VARCHAR(100),
	IN `baseCollectionId` VARCHAR(50),
	IN `horizontalFilter` VARCHAR(100),
	IN `verticalFilter` VARCHAR(100)

)
IF (groupId IS NULL OR groupId = 'NULL') OR
	(valuesList IS NULL OR valuesList = 'NULL') THEN
	/* Return an empty table for clean Metabase interface. */
	SELECT * FROM name_table WHERE FALSE;
ELSE
	CALL getValues(valuesList,groupId);
END IF//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.removeCollection
DROP PROCEDURE IF EXISTS `removeCollection`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `removeCollection`(
	IN `userCollectionId` VARCHAR(50)





)
removeProc:BEGIN
DECLARE groupIdString CHAR(150);
DECLARE queryString CHAR(150);
DECLARE deleteTableString CHAR(150);
DECLARE tmpTableString CHAR(150);
DECLARE tableName CHAR(100);
DECLARE dbName CHAR(100);
DECLARE tableNameQuery CHAR(200);
DECLARE done INT DEFAULT FALSE;
DECLARE curGroupId CHAR(40);

DECLARE cur1 CURSOR FOR SELECT TABLE_NAME FROM tmpTableList;
DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

/* remove all rows from the data for group table */
SET curGroupId = (select group_id FROM collection_table WHERE collection_id = userCollectionId);
IF curGroupId IS NULL THEN
	SET curGroupId = (select DISTINCT (collection_id) FROM data_for_group_table_baseline WHERE collection_id = userCollectionId);
	IF curGroupId IS NULL THEN
		SELECT "Requested collection not found in the collection table.";
		LEAVE removeProc;
	END IF;
	SET curGroupId = "baseline";
END IF;


SET groupIdString = CONCAT("data_for_group_table_", curGroupId);
SET queryString = concat("DELETE FROM ",groupIdString, " WHERE collection_id='", userCollectionId, "'");

CALL tpf_execute_immediate(queryString);

/* remove the entry in the collection table */
DELETE FROM collection_table WHERE collection_id=userCollectionID;


/* Delete all of the temporary tables for this collection */
SELECT DATABASE() INTO dbName;
DROP TEMPORARY TABLE IF EXISTS tmpTableList;
SET tableNameQuery = "CREATE TEMPORARY TABLE IF NOT EXISTS tmpTableList AS SELECT TABLE_NAME FROM information_schema.TABLES ";
SET tableNameQuery = CONCAT(tableNameQuery, " WHERE TABLE_NAME LIKE '%tmp%", userCollectionId, "%'");
SET tableNameQuery = CONCAT(tableNameQuery, " AND TABLE_SCHEMA='", dbName, "'");

CALL tpf_execute_immediate(tableNameQuery);

OPEN cur1;
read_loop: LOOP
	FETCH cur1 INTO tableName;
	IF done THEN
		LEAVE read_loop;
	END IF;		

	SET deleteTableString = "DROP TABLE IF EXISTS ";
	SET deleteTableString = CONCAT(deleteTableString," ",tableName);

	CALL tpf_execute_immediate(deleteTableString);

END LOOP;
CLOSE cur1;



END//
DELIMITER ;

-- Dumping structure for procedure GrafanaDB.updateViews
DROP PROCEDURE IF EXISTS `updateViews`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `updateViews`()
updateViewProc:BEGIN  


/************************************************************************/
/*  IBM TEMPLATE: updateViews                                           */
/*                                                                      */
/*____DESCRIPTION_______________________________________________________*/
/* This procedure makes sure that the views are up to date before being */
/* used. This avoids any SQL Errors that could arrise from a user       */
/* manipulating the database in unpredicted ways.                       */
/*                                                                      */
/*____INPUTS____________________________________________________________*/
/* userCollectionIds blob - The collection id of the collection         */
/*    to query against.                                                 */
/* userBaselineName varchar(40) - The name the user wishes to use to    */
/*    identify the baseline being collection being created.             */
/* userGroupId varchar(40) - The group id of the group being averaged.  */
/*                                                                      */
/*____RETURNS___________________________________________________________*/
/* None                                                                 */
/*                                                                      */
/*____USE CASE; EXAMPLES________________________________________________*/
/*                                                                      */
/* call updateViews();                                                  */
/*                                                                      */
/*____ADDITIONAL NOTES__________________________________________________*/
/*                                                                      */
/*                                                                      */
/*                                                                      */
/************************************************************************/

/* If our views don't exist, we'll create them now */
IF NOT EXISTS(SELECT * FROM information_schema.VIEWS WHERE TABLE_NAME = 'viewOfHorizontalRelationshipTableWithNameAndValueStrings')
THEN

        CREATE OR REPLACE VIEW viewOfHorizontalRelationshipTableWithNameAndValueStrings AS select `relationship_table`.`relationship_id` AS `horizontal_relationship_id`,`name_table`.`name_string` AS `name_string`,`value_table`.`value_string` AS `value_string`, `relationship_table`.`cardinality` AS `cardinality`, `relationship_table`.`relationship_type` AS `relationship_type` from ((`relationship_table` join `name_table` on(`relationship_table`.`name_id` = `name_table`.`name_id`)) join `value_table` on(`relationship_table`.`value_id` = `value_table`.`value_id`)) where `relationship_table`.`relationship_type` = 'horizontal';  

        CREATE OR REPLACE VIEW viewOfRelationshipTableWithNameAndValueStrings AS select `relationship_table`.`relationship_id` AS `relationship_id`,`name_table`.`name_string` AS `name_string`,`value_table`.`value_string` AS `value_string`, `relationship_table`.`cardinality` AS `cardinality`, `relationship_table`.`relationship_type` AS `relationship_type` from ((`relationship_table` join `name_table` on(`relationship_table`.`name_id` = `name_table`.`name_id`)) join `value_table` on(`relationship_table`.`value_id` = `value_table`.`value_id`)); 

        CREATE OR REPLACE VIEW viewOfVerticalRelationshipTableWithNameAndValueStrings AS select `relationship_table`.`relationship_id` AS `vertical_relationship_id`,`name_table`.`name_string` AS `name_string`,`value_table`.`value_string` AS `value_string`,`relationship_table`.`cardinality` AS `cardinality`,`relationship_table`.`relationship_type` AS `relationship_type` from ((`relationship_table` join `name_table` on(`relationship_table`.`name_id` = `name_table`.`name_id`)) join `value_table` on(`relationship_table`.`value_id` = `value_table`.`value_id`)) where `relationship_table`.`relationship_type` = 'vertical';

END IF;

/* Need to update the view to include/remove the baseline collections. */
/* If the data_for_group_table_baseline table is dropped by the user the view will get reset here. */
IF (EXISTS (select * from information_schema.tables where TABLE_NAME = "data_for_group_table_baseline" and TABLE_SCHEMA = DATABASE())) THEN
	CREATE OR REPLACE VIEW viewOfAllCollectionGroupIdPairs AS SELECT DISTINCT collection_id, group_id FROM collection_table 
	UNION SELECT DISTINCT collection_id, 'baseline' AS group_id FROM data_for_group_table_baseline;
ELSE
	CREATE OR REPLACE VIEW viewOfAllCollectionGroupIdPairs AS SELECT DISTINCT collection_id, group_id FROM collection_table;
END IF;

/* viewOfHorizontalRelationshipTableWithNameAndValueStrings does not need to be adjusted for baselines. */

/* viewOfRelationshipTableWithNameAndValueStrings does not need to be adjusted for baselines. */

/* viewOfVerticalRelationshipTableWithNameAndValueStrings does not need to be adjusted for baselines. */


END//


DELIMITER ;
