#Since this script runs in the database container and we might have
#different shells available, no shebang will be specified.
#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2019, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
#---START OF HISTORY--------------------------------------------------#
#   DATE        DESCRIPTION                                           #
#   12182019    Initial implementation of z/TPF Realtime Insights     #
#               Dashboard Starter Kit.                                #
#   12152021    Initial Message Analysis Tool support                 #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   20230719    PJ46982 - z/TPF real-time insights dashboard starter  #
#                         kit installation on Linux on IBM Z          #
#   20230719    PJ46904 - User-Defined Metrics                        #
#   20231004    PJ47156 - prepare config script does not work in all  #
#                         customer environments                       #
#   20240515    PJ48025 - Modify DROP for TEMPORARY tables            #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
#   20240730    PJ48072 - MariaDB large temp file issue               #
#---END OF HISTORY----------------------------------------------------#
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
DB_CMD="mysql"
if ! command -v $DB_CMD &> /dev/null
then
   DB_CMD="mariadb"
fi
echo "Using DB_CMD=$DB_CMD"

echo "Setting up TPFDataSciDB"
eval $DB_CMD --host=127.0.0.1 --port=3306 --user=root --password=admin_passw0rd < /home/tpf_files/SQL/tpf_rtmc_create_db.sql
eval $DB_CMD --host=127.0.0.1 --port=3306 --user=root --password=admin_passw0rd TPFDataSciDB < /home/tpf_files/SQL/tpf_common_procedures.sql
echo "Upgrading database schema"
eval $DB_CMD --host=127.0.0.1 --port=3306 --user=root --password=admin_passw0rd -N TPFDataSciDB < /home/tpf_files/SQL/versioning/tpf_zrtmc_db_schema_upgrade.sql
echo "Loading stored procedures"
eval $DB_CMD --host=127.0.0.1 --port=3306 --user=root --password=admin_passw0rd TPFDataSciDB < /home/tpf_files/SQL/tpf_stored_procedures.sql
eval $DB_CMD --host=127.0.0.1 --port=3306 --user=root --password=admin_passw0rd TPFDataSciDB < /home/tpf_files/SQL/tpf_complex_metrics_json.sql
eval $DB_CMD --host=127.0.0.1 --port=3306 --user=root --password=admin_passw0rd TPFDataSciDB < /home/tpf_files/SQL/tpf_complex_stored_procedures.sql
eval $DB_CMD --host=127.0.0.1 --port=3306 --user=root --password=admin_passw0rd TPFDataSciDB < /home/tpf_files/SQL/tpf_common_dashboard_framework.sql
eval $DB_CMD --host=127.0.0.1 --port=3306 --user=root --password=admin_passw0rd TPFDataSciDB < /home/tpf_files/SQL/tpf_jvm_stored_procedures.sql
echo "Installing common dashboard framework user exits in TPFDataSciDB"
sh /home/tpf_files/SQL/tpf_cdf_user_exits/tpf_cdf_user_exits_install_commands.sh
echo "Loading events"
eval $DB_CMD --host=127.0.0.1 --port=3306 --user=root --password=admin_passw0rd TPFDataSciDB < /home/tpf_files/SQL/tpf_periodic_cleanup_event.sql
echo "Finished setting up TPFDataSciDB"

echo "Setting up TPFNVPCDB"
eval $DB_CMD --host=127.0.0.1 --port=3306 --user=root --password=admin_passw0rd < /home/tpf_files/SQL/tpf_nvpc_create_db.sql
echo "Loading stored procedures"
eval $DB_CMD --host=127.0.0.1 --port=3306 --user=root --password=admin_passw0rd TPFNVPCDB < /home/tpf_files/SQL/tpf_common_procedures.sql
eval $DB_CMD --host=127.0.0.1 --port=3306 --user=root --password=admin_passw0rd TPFNVPCDB < /home/tpf_files/SQL/tpf_nvpc_stored_procedures.sql
echo "Finished setting up TPFNVPCDB"

echo "Setting up TPFMATDB"
eval $DB_CMD --host=127.0.0.1 --port=3306 --user=root --password=admin_passw0rd < /home/tpf_files/SQL/tpf_mat_create_db.sql
echo "Loading stored procedures"
eval $DB_CMD --host=127.0.0.1 --port=3306 --user=root --password=admin_passw0rd TPFMATDB < /home/tpf_files/SQL/tpf_common_procedures.sql
eval $DB_CMD --host=127.0.0.1 --port=3306 --user=root --password=admin_passw0rd TPFMATDB < /home/tpf_files/SQL/tpf_mat_stored_procedures.sql
echo "Finished setting up TPFMATDB"

# SAMPLE PRUNING INSTALL COMMANDS
#sh /home/tpf_files/SQL/tpf_sample_pruning/tpf_sample_pruning_SQL_install_commands.sh
# SAMPLE USER DEFINED METRICS INSTALL COMMANDS
#sh /home/tpf_files/SQL/tpf_sample_udm/tpf_sample_udm_SQL_install_commands.sh
