/*********************************************************************/
/*      Licensed Materials - Property of IBM                         */
/*      "Restricted Materials of IBM"                                */
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2019, 2024                               */
/*      All Rights Reserved                                          */
/*      US Government Users Restricted Rights - Use, duplication     */
/*      or disclosure restricted by GSA ADP Schedule Contract        */
/*      with IBM Corp.                                               */
/*********************************************************************/
/*--START OF HISTORY-------------------------------------------------*/
/*  DATE        DESCRIPTION                                          */
/*  12182019    Initial implementation of z/TPF Realtime Insights    */
/*              Dashboard Starter Kit.                               */
/*  12112020    Updated for PJ45657 and PJ46295.                     */
/*  03242021    Updated for PJ46275 - JVM Monitor Support.           */
/*  08062021    Improve z/TPF real-time insights dashboard starter   */
/*              kit resource usage                                   */
/*  12152021    Initial Message Analysis Tool support                */
/*  20220131    PJ46608 - multi-JVM and system selector support      */
/*  06162022    PJ46737 - complex-wide dashboard and mysql support   */
/*  10052022    PJ46853 - Change skip factor to fractional scale     */
/*                        factor                                     */
/*  20230719    PJ46982 - z/TPF real-time insights dashboard starter */
/*                         kit installation on Linux on IBM Z        */
/*  20230719    PJ46904 - User-Defined Metrics                       */
/*  20240426    PJ47242 - Improve correlation and stats dashboard    */
/*                        performance                                */
/*  20240515    PJ48025 - Modify DROP for TEMPORARY tables           */
/*  20240613    PJ47253 - starter kit improvements                   */
/*  20240716    PJ47254 - RTMC CDC Dashboards                        */
/*--END OF HISTORY---------------------------------------------------*/
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     sql for various stored procedures for the z/TPF     */
/*               Realtime Insights Dashboard Starter Kit.            */
/*                                                                   */
/*********************************************************************/
-- --------------------------------------------------------
-- Server version:               10.2.24-MariaDB - MariaDB Server
-- Server OS:                    Linux
-- HeidiSQL Version:             9.4.0.5125
-- --------------------------------------------------------

/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;
/*!40101 SET NAMES utf8 */;
/*!50503 SET NAMES utf8mb4 */;
/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;
/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO' */;

-- These procedures depend on ONLY_FULL_GROUP_BY. MySQL and MariaDB always use the sql_mode
-- That was active when the procedures were created, so this permanently sets it for these procs.
SET sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));

-- Set the default character set to latin1 for consistency across MariaDB and MySQL
SET NAMES 'latin1';

DROP PROCEDURE IF EXISTS `tpf_cpcs_complexes_available`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_cpcs_complexes_available` (
    IN `in_panel_key` TEXT,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)

tpf_cpcs_complexes_available:BEGIN

DECLARE table_name TEXT;
DECLARE select_stmt TEXT DEFAULT "";
DECLARE has_altered_column INT DEFAULT 0;

-- A lot of queries except plan SELECTS (any write operation, SELECT INTO, CREATE TABLE AS SELECT)
-- require transactions therefore they can lock out queries from other transactions.
-- Changing the ISOLATION LEVEL prevents this (or at least significantly reduces the possibility).
-- We need to do this for procedures called from Grafana. They should only be reading committed data anyway.
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

-- __text, __value create key-value pairs in Grafana dropdown. This way, Grafana passes the system id list back
-- to the stored procedures so we don't have to do another lookup
-- __value is a comma separated list of system_ids in the time window for that cpc_name. We can concat these
-- and pass them back from Grafana so we only need to do the time-based query once.

IF in_panel_key IN ('Alerts', 'System_State')
THEN
   SET in_panel_key = 'system';
ELSEIF in_panel_key IN ('Correlation_Analysis','NVP_Statistics', 'NVP_Metrics')
THEN
   SET in_panel_key = 'nvp';
END IF;
SET table_name = tpf_get_table_name(in_panel_key);
SET has_altered_column = tpf_get_altered_column(in_panel_key);

SET select_stmt = CONCAT(
    " WITH cpcs_complexes_used AS (",
        " SELECT DISTINCT system_id",
        " FROM ",table_name,
        CASE WHEN has_altered_column = 0 THEN CONCAT(" WHERE TIME BETWEEN '",in_lt_time,"' AND '",in_rt_time,"'")
             ELSE CONCAT(" WHERE (TIME BETWEEN '",in_lt_time,"' AND '",in_rt_time,"') OR",
                         " (altered BETWEEN '",in_lt_time,"' AND '",in_rt_time,"') OR",
                         " (altered <= '",in_lt_time,"' AND '",in_rt_time,"' <= time)")
        END,
    " ),",
    " cpcs_complexes_available AS (",
        " SELECT cpc_name, complex_name",
        " FROM tpf_systems t",
        " INNER JOIN cpcs_complexes_used c ON c.system_id=t.id",
    " )",
    " (SELECT CONCAT('CPC: ', cpc_name) AS __text, cpc_name AS __value",
    "  FROM cpcs_complexes_available",
    "  GROUP BY cpc_name)",
    " UNION ALL",
    " (SELECT CONCAT('Complex: ', complex_name) AS __text, complex_name AS __value",
    "  FROM cpcs_complexes_available",
    "  GROUP BY complex_name)",
    " ORDER BY __text;"
);
CALL tpf_execute_immediate(select_stmt);


END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_systems_available`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_systems_available` (
    IN `in_panel_key` TEXT,
    IN `in_cpcs_complexes` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)

tpf_systems_available:BEGIN

DECLARE table_name TEXT;
DECLARE select_stmt TEXT DEFAULT "";
DECLARE  has_altered_column INT DEFAULT 0;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

IF JSON_LENGTH(in_cpcs_complexes) = 0
THEN
    SET in_cpcs_complexes = JSON_ARRAY_APPEND(in_cpcs_complexes, '$', '*');
END IF;

-- __text, __value create key-value pairs in Grafana dropdown. This way, Grafana passes the system id back
-- to the stored procedures so we don't have to do another lookup

IF in_panel_key IN ('Alerts', 'System_State')
THEN
   SET in_panel_key = 'system';
ELSEIF in_panel_key IN ('Correlation_Analysis','NVP_Statistics', 'NVP_Metrics')
THEN
   SET in_panel_key = 'nvp';
END IF;
SET table_name = tpf_get_table_name(in_panel_key);
SET has_altered_column = tpf_get_altered_column(in_panel_key);

SET select_stmt = CONCAT(
    " SELECT CONCAT_WS('~', system_name, complex_name, processor_id, ip_address, environment, cpc_name) AS __text, ",
    "        id AS __value ",
    " FROM tpf_systems ts ",
    " INNER JOIN JSON_TABLE(",  QUOTE(in_cpcs_complexes), ", \"$[*]\" COLUMNS(cpc_complex VARCHAR(16) path \"$\")) icc ON (ts.complex_name=icc.cpc_complex OR ts.cpc_name=icc.cpc_complex OR icc.cpc_complex='*') ",
    " WHERE EXISTS ( ",
        " SELECT 1 ",
        " FROM ",table_name,
        " WHERE id=system_id AND",
        CASE WHEN has_altered_column = 0 THEN CONCAT(" TIME BETWEEN '",in_lt_time,"' AND '",in_rt_time,"'")
             ELSE CONCAT(" (TIME BETWEEN '",in_lt_time,"' AND '",in_rt_time,"') OR",
                         " (altered BETWEEN '",in_lt_time,"' AND '",in_rt_time,"') OR",
                         " (altered <= '",in_lt_time,"' AND '",in_rt_time,"' <= time)")
        END,
        " LIMIT 1 ",
    ") ",
    " GROUP BY ts.id ",
    " ORDER BY __text; "
);

    CALL tpf_execute_immediate(select_stmt);

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_subsystems_available`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_subsystems_available` (
    IN `in_panel_key` TEXT,
    IN `in_tpf_systems` JSON,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)

tpf_subsystems_available:BEGIN

DECLARE table_metrics JSON;
DECLARE table_name TEXT;
DECLARE subsystem_applies_var TEXT;
DECLARE select_stmt LONGTEXT DEFAULT "";
DECLARE has_altered_column INT;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

IF JSON_contains(in_tpf_systems,'[""]') OR in_tpf_systems='""' OR in_tpf_systems="" OR in_tpf_systems="[]"
THEN
   SELECT "N/A" AS __text, 0 AS __value;
   LEAVE tpf_subsystems_available;
END IF;

IF JSON_TYPE(in_tpf_systems) = "STRING"
THEN
    SET in_tpf_systems = JSON_UNQUOTE(in_tpf_systems);
    SET in_tpf_systems = JSON_ARRAY(in_tpf_systems);
END IF;

--  Do everything in UTC time.
SET time_zone = '+00:00';

SET table_metrics = tpf_get_metrics_json();

SELECT DISTINCT subsystem_applies INTO subsystem_applies_var FROM json_table(table_metrics, '$[*]'
  COLUMNS(
    panel_key         VARCHAR(64) PATH '$.panel_key',
    dashboard_key     VARCHAR(64) PATH '$.dashboard_key',
    subsystem_applies VARCHAR(64) PATH '$.subsystem_applies',
    table_name        VARCHAR(64) PATH '$.table_name',
    NESTED PATH '$.attribute_columns[*]' COLUMNS (
      attribute_name VARCHAR(64) PATH '$.attribute_name',
      attribute_table VARCHAR(64) PATH '$.attribute_table',
      table_column VARCHAR(64) PATH '$.table_column'
    ),
    metric_type       VARCHAR(64) PATH '$.metric_type',
    NESTED PATH '$.metrics[*]' COLUMNS (
      metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.virtual_metrics[*]' COLUMNS (
      virtual_metrics VARCHAR(64) PATH '$'
    ),
    NESTED PATH '$.extra_cols[*]' COLUMNS (
      extra_cols VARCHAR(64) PATH '$'
    ),
    altered_column   VARCHAR(64) PATH '$.altered_column'
  )
) AS jt
WHERE jt.panel_key=in_panel_key
LIMIT 1;

IF subsystem_applies_var = "no"
THEN
   SELECT "N/A" AS __text, 0 AS __value;
   LEAVE tpf_subsystems_available;
END IF;

SET has_altered_column = tpf_get_altered_column(in_panel_key);
SET table_name = tpf_get_table_name(in_panel_key);

SET select_stmt = CONCAT(
" SELECT DISTINCT tss.subsystem AS __text, tss.id AS __value",
" FROM ",table_name," t ",
" INNER JOIN JSON_TABLE('", in_tpf_systems,"', '$[*]' COLUMNS(id INT path '$')) its ON t.system_id=its.id",
" INNER JOIN tpf_subsystems tss ON t.subsystem_id=tss.id",
CASE WHEN has_altered_column = 0 THEN CONCAT(" WHERE TIME BETWEEN '",in_lt_time,"' AND '",in_rt_time,"'")
     ELSE CONCAT(" WHERE (TIME BETWEEN '",in_lt_time,"' AND '",in_rt_time,"') OR",
                 " (altered BETWEEN '",in_lt_time,"' AND '",in_rt_time,"') OR",
                 " (altered <= '",in_lt_time,"' AND '",in_rt_time,"' <= time)")
END,  " ORDER BY __text");
CALL tpf_execute_immediate(select_stmt);

END//
DELIMITER ;



DROP PROCEDURE IF EXISTS `tpf_app_dimensions_available`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_app_dimensions_available` (
    IN `in_panel_key` ENUM('Correlation_Analysis','NVP_Statistics'),
    IN `in_system_id` INT,
    IN `in_lt_time` DATETIME,
    IN `in_rt_time` DATETIME
)

tpf_app_dimensions_available:BEGIN

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

--  Do everything in UTC time.
SET time_zone = '+00:00';

    SELECT DISTINCT dimension
    FROM app_dimensions d INNER JOIN app_dimension_values v ON d.id=v.dimension_id
    WHERE d.dimension != 'TCPIP' AND EXISTS (
        SELECT 1
        FROM nvp_message_metrics_table n
        WHERE n.system_id=in_system_id AND n.value_id=v.id AND n.TIME BETWEEN in_lt_time AND in_rt_time
        LIMIT 1
    )
    ORDER BY dimension;

END//
DELIMITER ;




DROP FUNCTION IF EXISTS `tpf_get_dimension_id`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_dimension_id` (
    `app_dimension` VARCHAR(64)
)
RETURNS INT
READS SQL DATA
BEGIN

DECLARE app_dimension_id INT;

SELECT `id`
INTO app_dimension_id
FROM app_dimensions
WHERE dimension=app_dimension;

RETURN app_dimension_id;

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_correlation_panel_titles`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_correlation_panel_titles` (
    IN `in_system_metric` ENUM('Actual_CPU', "InUse_ECBs", "InUse_IOBs", 'LP_CPU', 'Workload_CPU', 'Sys_Pseudo_CPU', 'Fenced_Pseudo_CPU',
                               'Est_Msg_CPU', 'Est_Msg_Exist', 'Est_Msg_Find_File', 'Est_Msg_Rate'),
    IN `in_app_dimension` VARCHAR(64),
    IN `in_panel_orientation` ENUM('Left', 'Right')
)

tpf_correlation_panel_titles:BEGIN

DECLARE panel_title_mappings JSON DEFAULT '{
    "Actual_CPU": {
        "NVP": {
            "Left": "NVP Rate Correlated to Actual System CPU",
            "Right": "Avg CPU by NVP per Message Correlated to Actual System CPU"
        },
        "TCPIP": {
            "Left": "Messages Sent and Received by App Name Correlated to Actual System CPU",
            "Right": "Bytes Sent and Received by App Name Correlated to Actual System CPU"
        }
    },
    "InUse_ECBs": {
        "NVP": {
            "Left": "NVP Rate Correlated to Inuse System ECBs",
            "Right": "Avg Existence Time by NVP per Message Correlated to Inuse System ECBs"
        },
        "TCPIP": {
            "Left": "Messages Sent and Received by App Name Correlated to Inuse System ECBs",
            "Right": "Bytes Sent and Received by App Name Correlated to Inuse System ECBs"
        }
    },
    "InUse_IOBs": {
        "NVP": {
            "Left": "NVP Rate Correlated to Inuse System IOBs",
            "Right": "Avg FIND/FILE by NVP per Message Correlated to Inuse System IOBs"
        },
        "TCPIP": {
            "Left": "Messages Sent and Received by App Name Correlated to Inuse System IOBs",
            "Right": "Bytes Sent and Received by App Name Correlated to Inuse System IOBs"
        }
    },
    "LP_CPU": {
        "NVP": {
            "Left": "NVP Rate Correlated to System Low Priority CPU",
            "Right": "Avg CPU by NVP per Message Correlated to System Low Priority CPU"
        },
        "TCPIP": {
            "Left": "Messages Sent and Received by App Name Correlated to System Low Priority CPU",
            "Right": "Bytes Sent and Received by App Name Correlated to System Low Priority CPU"
        }
    },
    "Workload_CPU": {
        "NVP": {
            "Left": "NVP Rate Correlated to System Workload CPU",
            "Right": "Avg CPU by NVP per Message Correlated to System Workload CPU"
        },
        "TCPIP": {
            "Left": "Messages Sent and Received by App Name Correlated to System Workload CPU",
            "Right": "Bytes Sent and Received by App Name Correlated to System Workload CPU"
        }
    },
    "Sys_Pseudo_CPU": {
        "NVP": {
            "Left": "NVP Rate Correlated to System Pseudo CPU",
            "Right": "Avg CPU by NVP per Message Correlated to System Pseudo CPU"
        },
        "TCPIP": {
            "Left": "Messages Sent and Received by App Name Correlated to System Pseudo CPU",
            "Right": "Bytes Sent and Received by App Name Correlated to System Pseudo CPU"
        }
    },
    "Fenced_Pseudo_CPU": {
        "NVP": {
            "Left": "NVP Rate Correlated to Fenced Pseudo CPU",
            "Right": "Avg CPU by NVP per Message Correlated to Fenced Pseudo CPU"
        },
        "TCPIP": {
            "Left": "Messages Sent and Received by App Name Correlated to Fenced Pseudo CPU",
            "Right": "Bytes Sent and Received by App Name Correlated to Fenced Pseudo CPU"
        }
    },
    "Est_Msg_CPU": {
        "NVP": {
            "Left": "NVP Rate Correlated to System Estimated Message CPU",
            "Right": "Avg CPU by NVP per Message Correlated to System Estimated Message CPU"
        },
        "TCPIP": {
            "Left": "Messages Sent and Received by App Name Correlated to System Estimated Message CPU",
            "Right": "Bytes Sent and Received by App Name Correlated to System Estimated Message CPU"
        }
    },
    "Est_Msg_Exist": {
        "NVP": {
            "Left": "NVP Rate Correlated to System Estimated Message Exist",
            "Right": "Avg Existence Time by NVP per Message Correlated to System Estimated Message Exist"
        },
        "TCPIP": {
            "Left": "Messages Sent and Received by App Name Correlated to System Estimated Message Exist",
            "Right": "Bytes Sent and Received by App Name Correlated to System Estimated Message Exist"
        }
    },
    "Est_Msg_Find_File": {
        "NVP": {
            "Left": "NVP Rate Correlated to System Estimated Message Find/File",
            "Right": "Avg FIND/FILE by NVP per Message Correlated to System Estimated Message Find/File"
        },
        "TCPIP": {
            "Left": "Messages Sent and Received by App Name Correlated to System Estimated Message Find/File",
            "Right": "Bytes Sent and Received by App Name Correlated to System Estimated Message Find/File"
        }
    },
    "Est_Msg_Rate": {
        "NVP": {
            "Left": "NVP Rate Correlated to System Estimated Message Rate",
            "Right": "Avg CPU by NVP per Message Correlated to System Estimated Message Rate"
        },
        "TCPIP": {
            "Left": "Messages Sent and Received by App Name Correlated to System Estimated Message Rate",
            "Right": "Bytes Sent and Received by App Name Correlated to System Estimated Message Rate"
        }
    }
}';

SELECT REPLACE(
    tpf_json_value(panel_title_mappings, CONCAT("$.", in_system_metric,".\"",  IF(in_app_dimension = 'TCPIP', 'TCPIP', 'NVP'), "\".", in_panel_orientation)),
    "NVP",
    in_app_dimension
);


END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_alerts_graph_titles`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_alerts_graph_titles` (
    IN `in_system_metrics` JSON
)

tpf_alerts_graph_titles:BEGIN

SELECT CONCAT(
    "System Level Metrics",
    CASE WHEN JSON_SEARCH(in_system_metrics, 'one', 'actual_cpu') IS NOT NULL THEN " (Including CPU Model Based Upon Point in Time Message Mix)"
         ELSE ""
    END
) AS title;


END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_nvp_statistics_panel_titles`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_nvp_statistics_panel_titles` (
    IN `in_app_metric` ENUM('MsgCnt', 'AvgCPU', 'TotCPU', 'AvgExstTime', 'TotExstTime', 'AvgIO', 'TotIO'),
    IN `in_app_dimension` VARCHAR(64)
)
tpf_nvp_statistics_panel_titles:BEGIN

DECLARE app_metric_title_mappings JSON DEFAULT '{
    "MsgCnt": "Number of Collected Samples",
    "AvgCPU": "Avg CPU per Message",
    "TotCPU": "Total CPU",
    "AvgExstTime": "Avg Existence Time per Message",
    "TotExstTime": "Total Existence Time",
    "AvgIO": "Avg FIND/FILE per Message",
    "TotIO": "Total FIND/FILE"
}';

SELECT CONCAT(
    tpf_json_value(app_metric_title_mappings, CONCAT("$.", in_app_metric)),
    " by ",
    in_app_dimension
);


END//
DELIMITER ;


-- Dumping structure for procedure TPF_RTMC_DEV.tpf_alerts_results_builder
DROP PROCEDURE IF EXISTS `tpf_alerts_results_builder`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_alerts_results_builder`()
tpf_alerts_results_builder:BEGIN

/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     Return table of alerts.                             */
/*                                                                   */
/* Description:  This functionality is still under development.  As  */
/*               such, this is simply a stand in to return a useful  */
/*               informational message for now.                      */
/*                                                                   */
/*********************************************************************/

SELECT "Example feature: This panel is used to alert you of conditions on the system and provides insights as to the next dashboard to use to investigate." as Message;

    
END//
DELIMITER ;

-- Dumping structure for procedure TPF_RTMC_DEV.tpf_changepoint_result_builder
DROP PROCEDURE IF EXISTS `tpf_changepoint_result_builder`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_changepoint_result_builder`()
tpf_changepoint_result_builder:BEGIN

/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     Return table of changepoint and trend rates.        */
/*                                                                   */
/* Description:  This functionality is still under development.  As  */
/*               such, this is simply a stand in to return a useful  */
/*               informational message for now.                      */
/*                                                                   */
/*********************************************************************/

SELECT "Example feature: This panel is used to identify resource use changes over time that have significant impacts and provides insights as to the next dashboard to use to investigate." as Message;

END//
DELIMITER ;

-- Dumping structure for procedure TPF_RTMC_DEV.tpf_correlation_result_builder
DROP PROCEDURE IF EXISTS `tpf_correlation_result_builder`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_correlation_result_builder`(
    IN `in_analysis_type` VARCHAR(64),
    IN `in_app_dimension` VARCHAR(64),
    IN `in_lt_time` VARCHAR(50),
    IN `in_rt_time` VARCHAR(50),
    IN `in_num_intervals` INT,
    IN `in_seconds_per_interval` INT,
    IN `in_num_rows_to_return` INT,
    IN `in_filter_string` VARCHAR(100),
    IN `in_system_id` INT
)
tpf_correlation_result_builder:BEGIN
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     Returns correlations and associated metrics.        */
/*                                                                   */
/* Description:  Returns correlations and other metrics for the two  */
/*               time windows specified by the inputs.  The results  */
/*               are returned in the form of a table specific to the */
/*               tables and analysis requested.                      */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* INTERFACE:                                                        */
/*                                                                   */
/* Dependencies: Assumes caller is Grafana.                          */
/*               Assumes Grafana will handle the number of decimals  */
/*               to be displayed.                                    */
/*                                                                   */
/* Input Parameters:                                                 */
/*      in_analysis_type       - enum of analysis to perform         */
/*      in_app_dimension       - dimension in app_dimensions table   */
/*      in_lt_time             - leftmost time to analyze            */
/*      in_rt_time             - rightmost time to analyze           */
/*      in_num_intervals       - # intervals for our analysis        */
/*      in_seconds_per_interval- # seconds in an interval            */
/*      in_num_rows_to_return  - limits our results to return        */
/*      in_filter_string       - filters our NVPs, etc results       */
/*                                                                   */
/* Tables accessed: Depending upon the request.                      */
/*      nvp_message_metrics_table                                    */
/*      tpf_tcpip_metrics_table                                           */
/*                                                                   */
/* Output Parameters:                                                */
/*      None.                                                        */
/*                                                                   */
/* Returned table of data:                                           */
/*      search_id    - String like booking for msgtype            */
/*      lt_corr_coef  - correlation coefficient for left time window */
/*      rt_corr_coef  - correlation coefficient for right time window*/
/*      MatDelta        - delta between the two correlation          */
/*                        coefficients                               */
/*      lt_insuf        - either insufficient system or NVPC data in */
/*                        the desired intervals on left.             */
/*      rt_insuf        - either insufficient system or NVPC data in */
/*                        the desired intervals on right.            */
/*      lt_metric       - metric for left time window like cpu_used  */
/*      rt_metric       - metric for right time window like cpu_used */
/*      metric_delta    - delta between the two metrics              */
/*      ordered by corr_coef_delta in descending fashion             */
/*                                                                   */  
/* SAMPLE CALLS:                                                     */ 
/*      Grafana:                                                     */
/*         CALL tpf_correlation_result_builder('MsgCnt_to_SysECB',   */
/*              'Message Type',$__timeFrom(),$__timeTo(),60,1,10,"");*/
/*         CALL tpf_correlation_result_builder('MsgCnt_to_SysECB',   */
/*             'Message Type',$__timeFrom(),$__timeTo(),48,15,10,"");*/
/*      Direct Query:                                                */     
/*         CALL tpf_correlation_result_builder('MsgCnt_to_SysECB',   */
/*              'Message Type','2018-11-05T17:43:00Z',               */
/*              '2018-11-05T17:49:00Z',60,1,10,"");                  */
/*         CALL tpf_correlation_result_builder('MsgCnt_to_SysECB',   */
/*              'Message Type','2018-11-05T17:43:00Z',               */
/*              '2018-11-05T17:49:00Z',48,15,10,"");                 */
/*                                                                   */ 
/*********************************************************************/
/*                                                                   */
/* NOTES:                                                            */
/*     The handling of time below assumes time is captured in 1      */
/*     second intervals and inputs are specified in 1 second         */
/*     intervals. If otherwise the time handling needs to be changed.*/
/*                                                                   */
/*********************************************************************/   

-- ###########################################################################################
--  This routine calculates table of correlations against various metrics for a given time.
--
--  IMPORTANT NOTE!
--
--    We generally recommend 1 second intervals and correlating 60 second windows.  Fewer
--    samples is highly reactive to noise in the data and for noisy data, it's unable to
--    approximate the actual relationship.
-- ###########################################################################################



-- ## variable for our selects or other statements we'll build.
DECLARE select_stmt BLOB;

DECLARE corr_min_interval INT DEFAULT 25;

-- ## time variables
DECLARE lt_end_time DATETIME;
DECLARE rt_end_time DATETIME;
DECLARE lt_start_time DATETIME;
DECLARE rt_start_time DATETIME;

-- ## These variables are used when calling the aggregator_lib function
-- ## to get the desired content.
DECLARE app_metric_table_name ENUM('nvp_message_metrics_table','tpf_tcpip_metrics_table');
DECLARE app_metric_column_name ENUM('msg_cnt','cpu_used','find_file','exist_time','msg_cnt*cpu_used','msg_cnt*find_file','msg_cnt*exist_time','tot_bytes');
DECLARE app_column_compare_name ENUM('value_id', 'app_name_id');
DECLARE sys_metric_name ENUM('actual_cpu','inuse_ecb','inuse_iob', 'lp_cpu', 'workload_cpu', 'sys_pseudo_cpu', 'fenced_pseudo_cpu', 'est_msg_cpu', 'est_msg_exist', 'est_msg_find_file', 'est_msg_rate');
DECLARE app_metric_total_or_per_msg_avg ENUM('total','per_msg_avg');
DECLARE calc_norm_metrics BOOLEAN DEFAULT FALSE;
DECLARE missing_metrics_form ENUM('AsMissingDataPoints','AsNULLs','AsZEROs','AsCorrelationDataPoints');

-- ## These variables are used for display purposes in our results.
-- ## These can't be enums since we're printing them.
DECLARE lt_metric_name VARCHAR(50);
DECLARE rt_metric_name VARCHAR(50);
DECLARE metric_delta_name VARCHAR(50);
DECLARE lt_additional_metric_name VARCHAR(50);
DECLARE rt_additional_metric_name VARCHAR(50);

-- ## These variables for controls inside this SQL.
DECLARE show_metric_as_percent_of_total BOOLEAN;
DECLARE show_additional_metric BOOLEAN;
DECLARE additional_metric_string TEXT;
DECLARE rt_aggregate_table_row_count BIGINT;
DECLARE lt_aggregate_table_row_count BIGINT;
DECLARE rt_system_metrics_cnt_intervals_without_data INT;
DECLARE lt_system_metrics_cnt_intervals_without_data INT;
DECLARE app_dimension_id INT;

DECLARE num_search_ids INT DEFAULT 0;

-- MySQL does not allow temp tables to be used more than once.
-- We could use CTE, but that's less efficient since the search table will be used a bunch.
-- We could duplicate the search table over and over and over.
-- Allocating the search table with a UUID in the name is probably the best approach.
-- But we don't want to create a table leak so we'll catch the error here.
DECLARE search_table_name TEXT DEFAULT NULL;
DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
    -- we don't want to hide the error as that will make debugging more difficult.
    RESIGNAL;
END;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

-- ## Grafana always sends down the time in UTC.  MariaDB attempts to
-- ## automatically handle timezones and has some interesting rules
-- ## about timestamp vs datetime behavior.  All of this yields some
-- ## interesting behavior between the two.
-- ## For simplicity:
-- ##    TPF will record data/time as UTC.
-- ##    Our procedures and connections will use UTC.
-- ##    Date and time will be stored as datetime variables.
SET time_zone = '+00:00';

-- ## Unfortunately, I've forgotten what problem adding this solved.
SET sql_mode = 'NO_UNSIGNED_SUBTRACTION';

-- ## If we have less that 25 intervals for our correlation calculations,
-- ## we'll bail out because the results are generally considered
-- ## to be unreliable.
SET corr_min_interval = 25;

-- ## Initialize this boolean since most types won't use this except TCPIP
SET show_additional_metric = FALSE;

-- ######################################################################
-- ##
-- ## First thing we'll do is validate all of our inputs and set up the
-- ## various variables we're going to need.
-- ##
-- ######################################################################


-- ## in_analysis_type - does not need to be validated since we are
-- ## using an enum.  So we'll just set up our table specific
-- ## variables.

-- ####################################################################
-- ## Set up our metric/analysis specific variables
-- ####################################################################
IF in_analysis_type = 'MsgCnt_to_SysCPU'
THEN
    -- ## Print Names
    SET lt_metric_name = '%Trfc-Lt';
    SET rt_metric_name = '%Trfc-Rt';
    SET show_metric_as_percent_of_total = TRUE;
  
    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'msg_cnt';
    SET sys_metric_name = 'actual_cpu';
  
    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'total';
ELSEIF in_analysis_type = 'MsgCnt_to_SysECB'
THEN
    -- ## Print Names
    SET lt_metric_name = '%Trfc-Lt';
    SET rt_metric_name = '%Trfc-Rt';
    SET show_metric_as_percent_of_total = TRUE;
  
    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'msg_cnt';
    SET sys_metric_name = 'inuse_ecb';
  
    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'total';
ELSEIF in_analysis_type = 'MsgCnt_to_SysIOB'
THEN
    -- ## Print Names
    SET lt_metric_name = '%Trfc-Lt';
    SET rt_metric_name = '%Trfc-Rt';
    SET show_metric_as_percent_of_total = TRUE;
  
    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'msg_cnt';
    SET sys_metric_name = 'inuse_iob';
  
    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'total';
ELSEIF in_analysis_type = 'MsgCnt_to_SysLpCPU'
THEN
    -- ## Print Names
    SET lt_metric_name = '%Trfc-Lt';
    SET rt_metric_name = '%Trfc-Rt';
    SET show_metric_as_percent_of_total = TRUE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'msg_cnt';
    SET sys_metric_name = 'lp_cpu';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'total';
ELSEIF in_analysis_type = 'MsgCnt_to_SysWrkldCPU'
THEN
    -- ## Print Names
    SET lt_metric_name = '%Trfc-Lt';
    SET rt_metric_name = '%Trfc-Rt';
    SET show_metric_as_percent_of_total = TRUE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'msg_cnt';
    SET sys_metric_name = 'workload_cpu';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'total';
ELSEIF in_analysis_type = 'MsgCnt_to_SysPseudoCPU'
THEN
    -- ## Print Names
    SET lt_metric_name = '%Trfc-Lt';
    SET rt_metric_name = '%Trfc-Rt';
    SET show_metric_as_percent_of_total = TRUE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'msg_cnt';
    SET sys_metric_name = 'sys_pseudo_cpu';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'total';
ELSEIF in_analysis_type = 'MsgCnt_to_FencedPseudoCPU'
THEN
    -- ## Print Names
    SET lt_metric_name = '%Trfc-Lt';
    SET rt_metric_name = '%Trfc-Rt';
    SET show_metric_as_percent_of_total = TRUE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'msg_cnt';
    SET sys_metric_name = 'fenced_pseudo_cpu';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'total';
ELSEIF in_analysis_type = 'MsgCnt_to_EstMsgCpu'
THEN
    -- ## Print Names
    SET lt_metric_name = '%Trfc-Lt';
    SET rt_metric_name = '%Trfc-Rt';
    SET show_metric_as_percent_of_total = TRUE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'msg_cnt';
    SET sys_metric_name = 'est_msg_cpu';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'total';
ELSEIF in_analysis_type = 'MsgCnt_to_EstMsgExist'
THEN
    -- ## Print Names
    SET lt_metric_name = '%Trfc-Lt';
    SET rt_metric_name = '%Trfc-Rt';
    SET show_metric_as_percent_of_total = TRUE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'msg_cnt';
    SET sys_metric_name = 'est_msg_exist';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'total';
ELSEIF in_analysis_type = 'MsgCnt_to_EstMsgFindFile'
THEN
    -- ## Print Names
    SET lt_metric_name = '%Trfc-Lt';
    SET rt_metric_name = '%Trfc-Rt';
    SET show_metric_as_percent_of_total = TRUE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'msg_cnt';
    SET sys_metric_name = 'est_msg_find_file';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'total';
ELSEIF in_analysis_type = 'MsgCnt_to_EstMsgRate'
THEN
    -- ## Print Names
    SET lt_metric_name = '%Trfc-Lt';
    SET rt_metric_name = '%Trfc-Rt';
    SET show_metric_as_percent_of_total = TRUE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'msg_cnt';
    SET sys_metric_name = 'est_msg_rate';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'total';
ELSEIF in_analysis_type = 'AppAvgCPU_to_SysCPU'
THEN
    -- ## Print Names
    SET lt_metric_name = 'CPU-Lt';
    SET rt_metric_name = 'CPU-Rt';
    SET show_metric_as_percent_of_total = FALSE;
  
    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'cpu_used';
    SET sys_metric_name = 'actual_cpu';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AppAvgCPU_to_SysLpCPU'
THEN
    -- ## Print Names
    SET lt_metric_name = 'CPU-Lt';
    SET rt_metric_name = 'CPU-Rt';
    SET show_metric_as_percent_of_total = FALSE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'cpu_used';
    SET sys_metric_name = 'lp_cpu';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AppAvgCPU_to_SysWrkldCPU'
THEN
    -- ## Print Names
    SET lt_metric_name = 'CPU-Lt';
    SET rt_metric_name = 'CPU-Rt';
    SET show_metric_as_percent_of_total = FALSE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'cpu_used';
    SET sys_metric_name = 'workload_cpu';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AppAvgCPU_to_SysPseudoCPU'
THEN
    -- ## Print Names
    SET lt_metric_name = 'CPU-Lt';
    SET rt_metric_name = 'CPU-Rt';
    SET show_metric_as_percent_of_total = FALSE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'cpu_used';
    SET sys_metric_name = 'sys_pseudo_cpu';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AppAvgCPU_to_FencedPseudoCPU'
THEN
    -- ## Print Names
    SET lt_metric_name = 'CPU-Lt';
    SET rt_metric_name = 'CPU-Rt';
    SET show_metric_as_percent_of_total = FALSE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'cpu_used';
    SET sys_metric_name = 'fenced_pseudo_cpu';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AppAvgCPU_to_EstMsgCpu'
THEN
    -- ## Print Names
    SET lt_metric_name = 'CPU-Lt';
    SET rt_metric_name = 'CPU-Rt';
    SET show_metric_as_percent_of_total = FALSE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'cpu_used';
    SET sys_metric_name = 'est_msg_cpu';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AppAvgCPU_to_EstMsgRate'
THEN
    -- ## Print Names
    SET lt_metric_name = 'CPU-Lt';
    SET rt_metric_name = 'CPU-Rt';
    SET show_metric_as_percent_of_total = FALSE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'cpu_used';
    SET sys_metric_name = 'est_msg_rate';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AvgBytes_to_SysCPU'
THEN
    -- ## Print Names
    SET lt_metric_name = 'AvgBytes-Lt';
    SET rt_metric_name = 'AvgBytes-Rt';
    SET show_metric_as_percent_of_total = FALSE;
  
    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'tot_bytes';
    SET sys_metric_name = 'actual_cpu';  
  
    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AvgBytes_to_SysECB'
THEN
    -- ## Print Names
    SET lt_metric_name = 'AvgBytes-Lt';
    SET rt_metric_name = 'AvgBytes-Rt';
    SET show_metric_as_percent_of_total = FALSE;
  
    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'tot_bytes';
    SET sys_metric_name = 'inuse_ecb';  

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AvgBytes_to_SysIOB'
THEN
    -- ## Print Names
    SET lt_metric_name = 'AvgBytes-Lt';
    SET rt_metric_name = 'AvgBytes-Rt';
    SET show_metric_as_percent_of_total = FALSE;
  
    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'tot_bytes';
    SET sys_metric_name = 'inuse_iob';  

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AvgBytes_to_SysLpCPU'
THEN
    -- ## Print Names
    SET lt_metric_name = 'AvgBytes-Lt';
    SET rt_metric_name = 'AvgBytes-Rt';
    SET show_metric_as_percent_of_total = FALSE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'tot_bytes';
    SET sys_metric_name = 'lp_cpu';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AvgBytes_to_SysWrkldCPU'
THEN
    -- ## Print Names
    SET lt_metric_name = 'AvgBytes-Lt';
    SET rt_metric_name = 'AvgBytes-Rt';
    SET show_metric_as_percent_of_total = FALSE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'tot_bytes';
    SET sys_metric_name = 'workload_cpu';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AvgBytes_to_SysPseudoCPU'
THEN
    -- ## Print Names
    SET lt_metric_name = 'AvgBytes-Lt';
    SET rt_metric_name = 'AvgBytes-Rt';
    SET show_metric_as_percent_of_total = FALSE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'tot_bytes';
    SET sys_metric_name = 'sys_pseudo_cpu';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AvgBytes_to_FencedPseudoCPU'
THEN
    -- ## Print Names
    SET lt_metric_name = 'AvgBytes-Lt';
    SET rt_metric_name = 'AvgBytes-Rt';
    SET show_metric_as_percent_of_total = FALSE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'tot_bytes';
    SET sys_metric_name = 'fenced_pseudo_cpu';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AvgBytes_to_EstMsgCpu'
THEN
    -- ## Print Names
    SET lt_metric_name = 'AvgBytes-Lt';
    SET rt_metric_name = 'AvgBytes-Rt';
    SET show_metric_as_percent_of_total = FALSE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'tot_bytes';
    SET sys_metric_name = 'est_msg_cpu';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AvgBytes_to_EstMsgExist'
THEN
    -- ## Print Names
    SET lt_metric_name = 'AvgBytes-Lt';
    SET rt_metric_name = 'AvgBytes-Rt';
    SET show_metric_as_percent_of_total = FALSE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'tot_bytes';
    SET sys_metric_name = 'est_msg_exist';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AvgBytes_to_EstMsgFindFile'
THEN
    -- ## Print Names
    SET lt_metric_name = 'AvgBytes-Lt';
    SET rt_metric_name = 'AvgBytes-Rt';
    SET show_metric_as_percent_of_total = FALSE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'tot_bytes';
    SET sys_metric_name = 'est_msg_find_file';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AvgBytes_to_EstMsgRate'
THEN
    -- ## Print Names
    SET lt_metric_name = 'AvgBytes-Lt';
    SET rt_metric_name = 'AvgBytes-Rt';
    SET show_metric_as_percent_of_total = FALSE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'tot_bytes';
    SET sys_metric_name = 'est_msg_rate';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AppExstTime_to_SysECB'
THEN
    -- ## Print Names
    SET lt_metric_name = 'Time-Lt';
    SET rt_metric_name = 'Time-Rt';
    SET show_metric_as_percent_of_total = FALSE;
  
    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'exist_time';
    SET sys_metric_name = 'inuse_ecb';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AppExstTime_to_EstMsgExist'
THEN
    -- ## Print Names
    SET lt_metric_name = 'CPU-Lt';
    SET rt_metric_name = 'CPU-Rt';
    SET show_metric_as_percent_of_total = FALSE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'exist_time';
    SET sys_metric_name = 'est_msg_exist';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';

ELSEIF in_analysis_type = 'AppIO_to_SysIOB'
THEN
    -- ## Print Names
    SET lt_metric_name = 'IO-Lt';
    SET rt_metric_name = 'IO-Rt';
    SET show_metric_as_percent_of_total = FALSE;
  
    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'find_file';
    SET sys_metric_name = 'inuse_iob';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSEIF in_analysis_type = 'AppIO_to_EstMsgFindFile'
THEN
    -- ## Print Names
    SET lt_metric_name = 'IO-Lt';
    SET rt_metric_name = 'IO-Rt';
    SET show_metric_as_percent_of_total = FALSE;

    -- ## Table details - what metrics are we comparing
    SET app_metric_column_name = 'find_file';
    SET sys_metric_name = 'est_msg_find_file';

    -- ## Should the metrics be summed to total or taken as averages
    SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
ELSE
    -- return error and exit
    SELECT "The analysis type specified on the in_analysis_type parameter of the call to tpf_correlation_result_builder does not exist." AS 'Error Message';
    LEAVE tpf_correlation_result_builder;
END IF;

-- # This is the same for everyone
SET metric_delta_name = 'Delta';


-- Validate in_app_dimension
SET app_dimension_id = tpf_get_dimension_id(in_app_dimension);

-- If in_app_dimension is "", there is no data. Let the below error handling display the correct message
IF app_dimension_id IS NULL AND in_app_dimension != ""
THEN
    SELECT CONCAT("Dimension ", QUOTE(in_app_dimension), " not found.") AS 'Error Message';
    LEAVE tpf_correlation_result_builder;
ELSEIF app_dimension_id IS NULL
THEN
    LEAVE tpf_correlation_result_builder;
END IF;

-- Set table name and dimension value column name
SET app_metric_table_name = CASE WHEN in_app_dimension = "TCPIP" THEN "tpf_tcpip_metrics_table" ELSE "nvp_message_metrics_table" END;
SET app_column_compare_name = CASE WHEN in_app_dimension = "TCPIP" THEN "app_name_id" ELSE "value_id" END;

-- We'll used this for TCPIP message count base correlations to show the message counts.
IF in_app_dimension = 'TCPIP'
THEN
    SET lt_additional_metric_name = 'Cnt-Lt';
    SET rt_additional_metric_name = 'Cnt-Rt';
    SET show_additional_metric = TRUE;
END IF;

-- For code packages, % of traffic is difficult to understand and kinda weird
-- It's easier to just show msg_cnt
IF LOCATE("code package",in_app_dimension) > 0 AND app_metric_column_name = 'msg_cnt'
THEN
    -- ## Print Names
    SET lt_metric_name = 'msg_cnt-Lt';
    SET rt_metric_name = 'msg_cnt-Rt';
    SET show_metric_as_percent_of_total = FALSE;
END IF;

-- ####################################################################
-- ## Validate our time inputs
-- ####################################################################
SET lt_end_time = STR_TO_DATE(in_lt_time, '%Y-%m-%d %H:%i:%s');
SET rt_end_time = STR_TO_DATE(in_rt_time, '%Y-%m-%d %H:%i:%s');
IF lt_end_time IS NULL OR rt_end_time IS NULL
THEN
    SELECT "Time input values are not formatted correctly." AS 'Error Message';
    LEAVE tpf_correlation_result_builder;
END IF;

IF lt_end_time > rt_end_time
THEN
    SELECT "'lt' time input is larger than 'rt' time input." AS 'Error Message';
    LEAVE tpf_correlation_result_builder;
END IF;


-- ####################################################################
-- ## Validate our seconds to aggregate input
-- ####################################################################
IF in_seconds_per_interval IS NULL OR in_seconds_per_interval < 1
THEN
    -- return error and exit
    SELECT "The number of seconds to aggregate is less than 1.  Please fill in a value greater than 0." AS 'Error Message';
    LEAVE tpf_correlation_result_builder;
END IF;



-- ####################################################################
-- ## Validate our number of intervals
-- ####################################################################
IF in_num_intervals IS NULL OR in_num_intervals < corr_min_interval
THEN
    -- return error and exit
    SELECT CONCAT("The number of intervals is less than ",corr_min_interval," which tends to result in unreliable correlation results. Please specify a larger number of intervals.") AS 'Error Message';
    LEAVE tpf_correlation_result_builder;
END IF;



-- ####################################################################
-- ## Validate our number of rows to return
-- ####################################################################
IF in_num_rows_to_return IS NULL OR in_num_rows_to_return < 1
THEN
    -- return error and exit
    SELECT "The number of rows to return is less than 1.  Please fill in a value greater than 0." AS 'Error Message';
    LEAVE tpf_correlation_result_builder;
END IF;

-- ####################################################################
-- ## adjust our input times to be on the in_seconds_to_agg mod boundary
-- ####################################################################
SET rt_end_time = FROM_UNIXTIME(UNIX_TIMESTAMP(rt_end_time) DIV in_seconds_per_interval * in_seconds_per_interval + in_seconds_per_interval-1);
SET lt_end_time = FROM_UNIXTIME(UNIX_TIMESTAMP(lt_end_time) DIV in_seconds_per_interval * in_seconds_per_interval-1);

-- ####################################################################
-- ## set up our time related variables
-- ####################################################################
SET lt_start_time = SUBDATE(lt_end_time, INTERVAL in_seconds_per_interval*in_num_intervals-1 SECOND);
SET rt_start_time = SUBDATE(rt_end_time, INTERVAL in_seconds_per_interval*in_num_intervals-1 SECOND);



-- ######################################################################
-- ##
-- ## Next we'll set up our temporary working tables.
-- ##
-- ######################################################################
CALL tpf_allocate_aggregator_search_table(in_app_dimension,app_dimension_id,in_filter_string,in_system_id,lt_start_time,lt_end_time,rt_start_time,rt_end_time,in_seconds_per_interval,search_table_name);
-- ## How many entries do we have?  Handle errors
SET select_stmt=CONCAT(" SELECT COUNT(*) INTO @num_search_ids FROM `",search_table_name,"`");
CALL tpf_execute_immediate(select_stmt);
IF @num_search_ids IS NULL OR @num_search_ids = 0
THEN
    SELECT CONCAT(
        "No strings found for ",
        QUOTE(in_app_dimension),
        " in the requested time ranges" ,
        CASE WHEN in_filter_string != "" THEN CONCAT(" with filter ", QUOTE(in_filter_string)) ELSE "" END,
        ". There is probably no data in the time ranges."
    ) AS 'Error Message';
    LEAVE tpf_correlation_result_builder;
END IF;

-- ## Only do the work of calculating the normalization metrics if we want them
SET missing_metrics_form = 'AsCorrelationDataPoints';
IF app_metric_total_or_per_msg_avg = 'total' AND in_app_dimension != 'TCPIP'
THEN
    SET calc_norm_metrics = TRUE;
END IF;
    

-- ######################################################################
-- ##
-- ## Next we'll gather our aggregated stats for both of our time windows
-- ##
-- ######################################################################
CALL tpf_allocate_aggregate_table("rt_aggregate_table");
CALL tpf_allocate_aggregate_table("lt_aggregate_table");

CALL tpf_metric_aggregator_lib(search_table_name,app_metric_table_name,app_metric_column_name,app_column_compare_name,sys_metric_name,rt_end_time,in_seconds_per_interval,in_num_intervals,app_metric_total_or_per_msg_avg,missing_metrics_form,"rt_aggregate_table","rt_cnt_intervals_with_data",rt_system_metrics_cnt_intervals_without_data, in_system_id, app_dimension_id);
CALL tpf_metric_aggregator_lib(search_table_name,app_metric_table_name,app_metric_column_name,app_column_compare_name,sys_metric_name,lt_end_time,in_seconds_per_interval,in_num_intervals,app_metric_total_or_per_msg_avg,missing_metrics_form,"lt_aggregate_table","lt_cnt_intervals_with_data",lt_system_metrics_cnt_intervals_without_data, in_system_id, app_dimension_id);

IF rt_system_metrics_cnt_intervals_without_data IS NULL THEN SET rt_system_metrics_cnt_intervals_without_data = 0; END IF;
IF lt_system_metrics_cnt_intervals_without_data IS NULL THEN SET lt_system_metrics_cnt_intervals_without_data = 0; END IF;

-- ######################################################################
-- ##
-- ## Make sure we have some data to work with in at least one of our tables
-- ##
-- ######################################################################
SELECT COUNT(*) INTO @rt_aggregate_table_row_count FROM rt_aggregate_table;
IF @rt_aggregate_table_row_count IS NULL THEN SET @rt_aggregate_table_row_count = 0; END IF;

SELECT COUNT(*) INTO @lt_aggregate_table_row_count FROM lt_aggregate_table;
IF @lt_aggregate_table_row_count IS NULL THEN SET @lt_aggregate_table_row_count = 0; END IF;

IF @rt_aggregate_table_row_count = 0 AND @lt_aggregate_table_row_count = 0
THEN
    SELECT "No data in the requested time ranges." AS 'Error Message';
    LEAVE tpf_correlation_result_builder;
END IF;

-- ######################################################################
-- ##
-- ## Perform correlation calculations
-- ##
-- ######################################################################
CALL tpf_allocate_correlation_table("rt_corr_results_table");
CALL tpf_allocate_correlation_table("lt_corr_results_table");

CALL tpf_calculation_correlation(search_table_name,"rt_aggregate_table","rt_corr_results_table",calc_norm_metrics,show_metric_as_percent_of_total,corr_min_interval,"rt_cnt_intervals_with_data",rt_system_metrics_cnt_intervals_without_data,in_num_intervals);
CALL tpf_calculation_correlation(search_table_name,"lt_aggregate_table","lt_corr_results_table",calc_norm_metrics,show_metric_as_percent_of_total,corr_min_interval,"lt_cnt_intervals_with_data",lt_system_metrics_cnt_intervals_without_data,in_num_intervals);

-- ######################################################################
-- ##
-- ## Create our final table of results
-- ##
-- ######################################################################
SET additional_metric_string = "";
IF show_additional_metric=TRUE AND show_metric_as_percent_of_total=TRUE
THEN
   SET additional_metric_string = CONCAT(", lt.additional_metric AS '", lt_additional_metric_name, "', rt.additional_metric AS '", rt_additional_metric_name, "'");
END IF;

SET select_stmt = CONCAT(
    " SELECT rt.search_string AS ", QUOTE(in_app_dimension), ",",
    " IF(lt.inadequate_data_across_intervals!=0,NULL,lt.corr_coef) AS 'Coef-Lt',",
    " IF(rt.inadequate_data_across_intervals!=0,NULL,rt.corr_coef) AS 'Coef-Rt',",
    " CASE ",
    "      WHEN rt.inadequate_data_across_intervals !=0 OR lt.inadequate_data_across_intervals != 0 THEN NULL"
    "      WHEN rt.corr_coef > lt.corr_coef THEN ABS((POWER(rt.corr_coef,3)-POWER(lt.corr_coef,3))/3)"
    "      ELSE ABS((POWER(lt.corr_coef,3)-POWER(rt.corr_coef,3))/3)"
    " END AS 'MatDelta',",
    " lt.inadequate_data_across_intervals AS 'Insuf-Lt', rt.inadequate_data_across_intervals AS 'Insuf-Rt',",
    " IF(lt.metric=0,NULL,lt.metric) AS '", lt_metric_name, "',",
    " IF(rt.metric=0,NULL,rt.metric) AS '", rt_metric_name, "',",
    " IF(lt.metric=0 OR rt.metric=0,NULL,rt.metric-lt.metric) AS '",metric_delta_name,"'",
    additional_metric_string,
    " FROM rt_corr_results_table rt",
    " INNER JOIN lt_corr_results_table lt ON rt.search_string=lt.search_string",
    " ORDER BY MatDelta DESC,rt.corr_coef DESC",
    " LIMIT ", in_num_rows_to_return
);
-- SELECT select_stmt;
CALL tpf_execute_immediate(select_stmt);
END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_grafana_call_correlation_result_builder`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_grafana_call_correlation_result_builder` (
    IN `in_system_metric` ENUM('Actual_CPU', "InUse_ECBs", "InUse_IOBs", 'LP_CPU', 'Workload_CPU', 'Sys_Pseudo_CPU', 'Fenced_Pseudo_CPU',
                               'Est_Msg_CPU', 'Est_Msg_Exist', 'Est_Msg_Find_File', 'Est_Msg_Rate'),
    IN `in_app_dimension` VARCHAR(64),
    IN `in_panel_orientation` ENUM('Left', 'Right'),
    IN `in_lt_time` VARCHAR(50),
    IN `in_rt_time` VARCHAR(50),
    IN `in_num_intervals` INT,
    IN `in_seconds_per_interval` INT,
    IN `in_num_rows_to_return` INT,
    IN `in_filter_string` VARCHAR(100),
    IN `in_system_id` INT
)

tpf_grafana_call_correlation_result_builder:BEGIN

DECLARE analysis_type VARCHAR(64);

DECLARE analysis_type_mappings JSON DEFAULT '{
    "NVP": {
        "Actual_CPU": {
            "Left": "MsgCnt_to_SysCPU",
            "Right": "AppAvgCPU_to_SysCPU"
        },
        "InUse_ECBs": {
            "Left": "MsgCnt_to_SysECB",
            "Right": "AppExstTime_to_SysECB"
        },
        "InUse_IOBs": {
            "Left": "MsgCnt_to_SysIOB",
            "Right": "AppIO_to_SysIOB"
        },
        "LP_CPU": {
            "Left": "MsgCnt_to_SysLpCPU",
            "Right": "AppAvgCPU_to_SysLpCPU"
        },
        "Workload_CPU": {
            "Left": "MsgCnt_to_SysWrkldCPU",
            "Right": "AppAvgCPU_to_SysWrkldCPU"
        },
        "Sys_Pseudo_CPU": {
            "Left": "MsgCnt_to_SysPseudoCPU",
            "Right": "AppAvgCPU_to_SysPseudoCPU"
        },
        "Fenced_Pseudo_CPU": {
            "Left": "MsgCnt_to_FencedPseudoCPU",
            "Right": "AppAvgCPU_to_FencedPseudoCPU"
        },
        "Est_Msg_CPU": {
            "Left": "MsgCnt_to_EstMsgCpu",
            "Right": "AppAvgCPU_to_EstMsgCpu"
        },
        "Est_Msg_Exist": {
            "Left": "MsgCnt_to_EstMsgExist",
            "Right": "AppExstTime_to_EstMsgExist"
        },
        "Est_Msg_Find_File": {
            "Left": "MsgCnt_to_EstMsgFindFile",
            "Right": "AppIO_to_EstMsgFindFile"
        },
        "Est_Msg_Rate": {
            "Left": "MsgCnt_to_EstMsgRate",
            "Right": "AppAvgCPU_to_EstMsgRate"
        }
    },
    "TCPIP": {
         "Actual_CPU": {
             "Left": "MsgCnt_to_SysCPU",
             "Right": "AvgBytes_to_SysCPU"
         },
         "InUse_ECBs": {
             "Left": "MsgCnt_to_SysECB",
             "Right": "AvgBytes_to_SysECB"
         },
         "InUse_IOBs": {
             "Left": "MsgCnt_to_SysIOB",
             "Right": "AvgBytes_to_SysIOB"
         },
         "LP_CPU": {
             "Left": "MsgCnt_to_SysCPU",
             "Right": "AvgBytes_to_SysLpCPU"
         },
         "Workload_CPU": {
             "Left": "MsgCnt_to_SysECB",
             "Right": "AvgBytes_to_SysWrkldCPU"
         },
         "Sys_Pseudo_CPU": {
             "Left": "MsgCnt_to_SysIOB",
             "Right": "AvgBytes_to_SysPseudoCPU"
         },
         "Fenced_Pseudo_CPU": {
             "Left": "MsgCnt_to_SysCPU",
             "Right": "AvgBytes_to_FencedPseudoCPU"
         },
         "Est_Msg_CPU": {
             "Left": "MsgCnt_to_SysECB",
             "Right": "AvgBytes_to_EstMsgCpu"
         },
         "Est_Msg_Exist": {
             "Left": "MsgCnt_to_SysIOB",
             "Right": "AvgBytes_to_EstMsgExist"
         },
         "Est_Msg_Find_File": {
             "Left": "MsgCnt_to_SysECB",
             "Right": "AvgBytes_to_EstMsgFindFile"
         },
         "Est_Msg_Rate": {
             "Left": "MsgCnt_to_SysIOB",
             "Right": "AvgBytes_to_EstMsgRate"
         }
    }
}';

SET analysis_type = tpf_json_value(analysis_type_mappings, CONCAT("$.\"", IF(in_app_dimension = 'TCPIP', 'TCPIP', 'NVP'), "\".", in_system_metric, ".", in_panel_orientation));

CALL tpf_correlation_result_builder(analysis_type, in_app_dimension, in_lt_time, in_rt_time, in_num_intervals, in_seconds_per_interval, in_num_rows_to_return, in_filter_string, in_system_id);


END//
DELIMITER ;

-- Dumping structure for procedure TPF_RTMC_DEV.tpf_metric_aggregator_lib
DROP PROCEDURE IF EXISTS `tpf_metric_aggregator_lib`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_metric_aggregator_lib`(
    IN `in_search_table_name` VARCHAR(100),
	IN `in_app_metric_table_name` ENUM('nvp_message_metrics_table', 'tpf_tcpip_metrics_table'),
	IN `in_app_metric_column_name` ENUM('msg_cnt','cpu_used','find_file','exist_time','msg_cnt*cpu_used','msg_cnt*find_file','msg_cnt*exist_time','tot_bytes'),
	IN `in_app_column_compare_name` ENUM('value_id', 'app_name_id'),
	IN `in_sys_metric_name` ENUM('actual_cpu','inuse_ecb','inuse_iob', 'lp_cpu', 'workload_cpu', 'sys_pseudo_cpu', 'fenced_pseudo_cpu', 'est_msg_cpu', 'est_msg_exist', 'est_msg_find_file', 'est_msg_rate'),
	IN `in_rt_time` DATETIME,
	IN `in_seconds_per_interval` INT,
	IN `in_num_intervals` INT,
	IN `in_app_metric_total_or_per_msg_avg` ENUM('total','per_msg_avg'),
	IN `in_missing_metrics_form` ENUM('AsMissingDataPoints','AsNULLs','AsZEROs','AsCorrelationDataPoints'),
	IN `out_table_name` VARCHAR(50),
	IN `out_search_table_cnt_intervals_without_data_column_name` VARCHAR(50),
	OUT `out_system_metrics_cnt_intervals_without_data_column` INT,
	IN `in_system_id` INT,
	IN `in_app_dimension_id` INT
)
tpf_metric_aggregator_lib:BEGIN
    

/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     Returns data aggregated by time windows.            */
/*                                                                   */
/* Description:  Returns metric data aggregated by intervals of      */
/*               seconds in the specified time window.  These results*/
/*               are used by the statistics and correlation functions*/
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* INTERFACE:                                                        */
/*                                                                   */
/* Dependencies: Assumes caller will handle the number of decimals   */
/*               to be displayed.                                    */
/*                                                                   */
/* Input Parameters:                                                 */
/*      in_search_table_name       - valid search ids to operate on  */
/*      in_app_metric_table_name   - metric table to operate on      */
/*                                   such as msg_types_table         */
/*      in_app_metric_column_name  - metric column in table to use   */
/*                                   such as cpu_used                */
/*      in_app_column_compare_name - column identifying app          */
/*                                   dimension values                */
/*      in_sys_metric_name         - metric from system_metrics_table*/
/*                                   such as actual_cpu              */
/*      in_rt_time                 - right most time                 */
/*      in_seconds_per_interval    - # secs per interval             */
/*      in_num_intervals           - # intervals going back in time  */
/*      in_app_metric_total_or_per_msg_avg - how to normalize data   */
/*      out_table_name             - where to store the results      */
/*      out_search_table_cnt_intervals_without_data_column_name -    */
/*                                 - specifies the column name in the*/
/*                                   search_table where we'll        */
/*                                   record then number of intervals */
/*                                   that don't have any NVP data.   */
/*      in_system_id               - the system_id                   */
/*      in_app_dimension_id        - the dimension_id                */
/*                                                                   */
/* Tables accessed: Depending upon the request.                      */
/*      msg_types_table                                              */
/*      horizontal_nvps_table                                        */
/*      code_packages_table                                          */
/*      msg_types_and_code_packages_table                            */
/*                                                                   */
/* Output Parameters:                                                */
/*      out_system_metrics_cnt_intervals_without_data_column -       */
/*                                 - returns the number of intervals */
/*                                   that have no system metric data */
/*                                                                   */
/* Returned table of data:                                           */
/*     `time` DATETIME(3)     - time for this interval               */
/*     `secs_per_interval` INT(10)     - # seconds in interval       */
/*     `search_id` VARCHAR(500) - search str instance such as     */
/*                                   booking for msgtype             */
/*     `system_metric` INT    - system metric total for this interval*/
/*     `avg_nvp_scale_factor` FLOAT - avg scale factor from samples  */
/*                                 in the interval                   */
/*     `app_metric` INT       - total or avg metric for search str   */
/*                              for samples in this interval         */
/*     `app_metric_total` INT - total of metrics for all search strs */
/*                              for samples in this interval         */
/*     `windowed_scale_factor` DOUBLE - smoothed scale factor        */
/*                                     calculated using a window of  */
/*                                     nearby intervals              */
/*     `windowed_app_metric` DOUBLE - app metric calculated using    */
/*                                    a window of nearby intervals   */
/*                                    and the windowed scale factor  */
/*                                                                   */  
/* SAMPLE CALLS:                                                     */ 
/*      Grafana:                                                     */
/*         Not called by Grafana                                     */
/*      Direct Query:                                                */     
/*         Must be called by a stored procedure.                     */
/*                                                                   */ 
/*********************************************************************/
/*                                                                   */
/* NOTES:                                                            */
/*     The handling of time below assumes time is captured in 1      */
/*     second intervals and inputs are specified in 1 second         */
/*     intervals. If otherwise the time handling needs to be changed.*/
/*                                                                   */
/*********************************************************************/   

DECLARE select_stmt BLOB;

DECLARE num_search_ids INT DEFAULT 0;

DECLARE interval_rt_time DATETIME;
DECLARE interval_lt_time DATETIME;

DECLARE app_metric_calculation_string TEXT;
DECLARE app_total_calculation_string TEXT;
DECLARE search_id_list TEXT;
DECLARE full_id_list TEXT;

DECLARE window_metrics_table TEXT;
DECLARE window_metrics_join TEXT;
DECLARE window_metrics_app TEXT;
DECLARE window_metrics_nvp_scale TEXT;
DECLARE window_total_metrics_table TEXT;
DECLARE window_total_metrics_join TEXT;
DECLARE window_total_metrics_app TEXT;
DECLARE window_metrics_cret TEXT;

-- select in_app_metric_table_name,in_app_metric_column_name,in_app_column_compare_name,in_sys_metric_name,in_rt_time,in_seconds_per_interval,in_num_intervals,
-- in_app_metric_total_or_per_msg_avg,in_missing_metrics_form,in_system_id,in_app_dimension_id;

-- ######################################################################
-- ##
-- ## We're going to walk backwards from the right most time for the number
-- ## of intervals specified on the input.  Then for each interval we'll
-- ## loop through the strings in the search table.  So for each string
-- ## in each interval we'll grab all of the samples, create an average or
-- ## total of the metrics.  We'll include other various metrics as well.
-- ##
-- ######################################################################

-- ## get the number of strings from the search table: such as booking, shopping, etc for msgtype
SET select_stmt = CONCAT("SELECT COUNT(*) FROM `",in_search_table_name,"` INTO @num_search_ids;");
CALL tpf_execute_immediate(select_stmt);

-- ## if we don't have any strings, then we're done
IF @num_search_ids IS NULL OR @num_search_ids = 0
THEN
    LEAVE tpf_metric_aggregator_lib;
END IF;

-- ## check for silly input errors
IF in_seconds_per_interval = 0 OR in_num_intervals = 0
THEN 
    LEAVE tpf_metric_aggregator_lib;
END IF;

-- ## save our in_rt_time to a working variable
SET interval_rt_time = in_rt_time;
SET out_system_metrics_cnt_intervals_without_data_column=0;

SET interval_lt_time = SUBDATE(interval_rt_time, INTERVAL in_seconds_per_interval*in_num_intervals-1 SECOND);
-- select in_seconds_per_interval,in_app_metric_table_name,interval_lt_time, interval_rt_time,in_system_id,out_table_name;

IF in_app_metric_total_or_per_msg_avg = 'total'
THEN
    SET app_metric_calculation_string=CONCAT(" SUM(",in_app_metric_column_name,")");
    SET app_total_calculation_string=CONCAT(" SUM(",in_app_metric_column_name,")");
ELSE
    IF in_app_metric_table_name = 'tpf_tcpip_metrics_table' AND in_app_metric_column_name = 'tot_bytes'
    THEN
       SET app_metric_calculation_string=CONCAT(" IF(SUM(msg_cnt)=0,0,SUM(",in_app_metric_column_name,")/SUM(msg_cnt))");
       -- we are summing across different value_ids so use distinct time instead of average
       SET app_total_calculation_string=CONCAT(" SUM(",in_app_metric_column_name,")/COUNT(DISTINCT time)");
    ELSE
       SET app_metric_calculation_string=CONCAT(" AVG(",in_app_metric_column_name,")");
       -- we are summing across different value_ids so use distinct time instead of average
       SET app_total_calculation_string=CONCAT(" SUM(",in_app_metric_column_name,")/COUNT(DISTINCT time)");
    END IF;
END IF;

SET select_stmt = CONCAT("SELECT GROUP_CONCAT(DISTINCT search_id) INTO @search_id_list FROM `",in_search_table_name,"`;");
CALL tpf_execute_immediate(select_stmt);

SET select_stmt = CONCAT("SELECT GROUP_CONCAT(DISTINCT id) INTO @full_id_list FROM app_dimension_values WHERE dimension_id=",in_app_dimension_id,";");
CALL tpf_execute_immediate(select_stmt);

IF in_seconds_per_interval <= 4
THEN
   SET window_metrics_table = CONCAT(
       " window_metrics AS (",
       " SELECT time_range.TIME,dim_range.id AS search_id,",
       " AVG(app.app_metric) OVER (PARTITION BY dim_range.id ORDER BY UNIX_TIMESTAMP(time_range.time) RANGE 4 PRECEDING) AS app_metric,",
       IF(in_app_metric_table_name='nvp_message_metrics_table'," AVG(app.avg_nvp_scale_factor) OVER (PARTITION BY dim_range.id ORDER BY UNIX_TIMESTAMP(time_range.time) RANGE 4 PRECEDING) AS avg_nvp_scale_factor"," 1 AS avg_nvp_scale_factor"),",",
       IF(in_app_metric_table_name='nvp_message_metrics_table'," AVG(app.cret) OVER (PARTITION BY dim_range.id ORDER BY UNIX_TIMESTAMP(time_range.time) RANGE 4 PRECEDING) AS cret"," 0 AS cret"),
       " FROM time_range ",
       " INNER JOIN dim_range ON dim_range.dummy_column=time_range.dummy_column",
       " LEFT JOIN app ON time_range.time=app.time AND dim_range.id=app.search_id",
       " GROUP BY time_range.TIME,dim_range.id",
       " ),"
   );
   SET window_total_metrics_table = CONCAT(
       " window_total_metrics AS (",
       " SELECT TIME,SUM(app_metric) AS app_metric_total",
       " FROM window_metrics",
       " GROUP BY TIME"
       " ),"
   );
   SET window_metrics_join = CONCAT(
       " LEFT JOIN window_metrics ON time_range.time=window_metrics.time AND dim_range.id=window_metrics.search_id"
   );
   SET window_total_metrics_join = CONCAT(
       " LEFT JOIN window_total_metrics ON time_range.time=window_total_metrics.time"
   );
   SET window_metrics_app = CONCAT(
       " ",IF(in_missing_metrics_form = 'AsZEROs' OR in_missing_metrics_form = 'AsCorrelationDataPoints',"IFNULL(window_metrics.app_metric,0)","window_metrics.app_metric")
   );
   SET window_metrics_nvp_scale = CONCAT(
        " ",IF(in_missing_metrics_form = 'AsZEROs' OR in_missing_metrics_form = 'AsCorrelationDataPoints',"IFNULL(window_metrics.avg_nvp_scale_factor,0)","window_metrics.avg_nvp_scale_factor")
   );
   SET window_metrics_cret = CONCAT(
        " ",IF(in_missing_metrics_form = 'AsZEROs' OR in_missing_metrics_form = 'AsCorrelationDataPoints',"IFNULL(window_metrics.cret,0)","window_metrics.cret")
   );
   SET window_total_metrics_app = CONCAT(
       " ",IF(in_missing_metrics_form = 'AsZEROs' OR in_missing_metrics_form = 'AsCorrelationDataPoints',"IFNULL(window_total_metrics.app_metric_total,0)","window_total_metrics.app_metric_total")
   );
ELSE -- if we're already averaging 5 or more seconds, we'll just reuse the values we calculated as the normalized values
   SET window_metrics_app = CONCAT(
       " ",IF(in_missing_metrics_form = 'AsZEROs' OR in_missing_metrics_form = 'AsCorrelationDataPoints',"IFNULL(app.app_metric,0)","app.app_metric")
   );
   SET window_metrics_nvp_scale = CONCAT(
        " ",IF(in_missing_metrics_form = 'AsZEROs' OR in_missing_metrics_form = 'AsCorrelationDataPoints',"IFNULL(app.avg_nvp_scale_factor,0)","app.avg_nvp_scale_factor")
   );
   SET window_metrics_cret = CONCAT(
        " ",IF(in_missing_metrics_form = 'AsZEROs' OR in_missing_metrics_form = 'AsCorrelationDataPoints',"IFNULL(app.cret,0)","app.cret")
   );
   SET window_total_metrics_app = CONCAT(
       " ",IF(in_missing_metrics_form = 'AsZEROs' OR in_missing_metrics_form = 'AsCorrelationDataPoints',"IFNULL(app_total.app_metric_total,0)","app_total.app_metric_total")
   );
END IF;

SET select_stmt = CONCAT(

 " INSERT INTO ", out_table_name, " (time,secs_per_interval,search_id,search_string,system_metric,app_metric,app_metric_total,avg_nvp_scale_factor,windowed_app_metric,windowed_scale_factor,windowed_total_app_metric,windowed_cret_metric)","\n\n",

    " WITH RECURSIVE time_range (TIME,dummy_column) AS (",
    " SELECT CAST('",interval_lt_time,"' AS DATETIME),1",
    " UNION ALL",
    "     SELECT DATE_ADD(time,INTERVAL ",in_seconds_per_interval," SECOND),1",
    "     FROM time_range ",
    "     WHERE DATE_ADD(time,INTERVAL ",in_seconds_per_interval," SECOND)<DATE_ADD('",interval_rt_time,"', INTERVAL 1 SECOND)",
    " ),","\n\n",

    " dim_range (id,str,dummy_column) AS (",
    "     SELECT search_id,search_string,1 FROM `",in_search_table_name,"`",
    " ),","\n\n",

    " app AS (",
    -- time calculation repeated here in case we're missing data points in an interval
    " SELECT from_unixtime(UNIX_TIMESTAMP(TIME) DIV (",in_seconds_per_interval,")*",in_seconds_per_interval,") AS TIME,",
    in_app_column_compare_name, " AS search_id,",
    app_metric_calculation_string, " AS app_metric,",
    IF(in_app_metric_table_name='nvp_message_metrics_table'," AVG(avg_nvp_scale_factor) AS avg_nvp_scale_factor"," 1 AS avg_nvp_scale_factor"),",",
    IF(in_app_metric_table_name='nvp_message_metrics_table'," AVG(cret) AS cret"," 0 AS cret"),
    " FROM ",in_app_metric_table_name," ",
    " WHERE TIME BETWEEN '",interval_lt_time,"' AND '",interval_rt_time,"' AND system_id=", in_system_id, " AND ",in_app_column_compare_name," IN (",@search_id_list,")",
    " GROUP BY from_unixtime(UNIX_TIMESTAMP(TIME) DIV (",in_seconds_per_interval,")*",in_seconds_per_interval,"),",in_app_column_compare_name,
    " ),","\n\n",

    -- if the window of time is small we'll smooth across 5 seconds
    IF(in_seconds_per_interval <= 4,window_metrics_table,""),"\n\n",
    IF(in_seconds_per_interval <= 4,window_total_metrics_table,""),"\n\n",

    " app_total AS (",
    " SELECT from_unixtime(UNIX_TIMESTAMP(TIME) DIV (",in_seconds_per_interval,")*",in_seconds_per_interval,") AS TIME,",
    app_total_calculation_string, " AS app_metric_total",
    " FROM ",in_app_metric_table_name," ",
    " WHERE TIME BETWEEN '",interval_lt_time,"' AND '",interval_rt_time,"' AND system_id=", in_system_id," AND ",in_app_column_compare_name," IN (",@full_id_list,")",
    " GROUP BY from_unixtime(UNIX_TIMESTAMP(TIME) DIV (",in_seconds_per_interval,")*",in_seconds_per_interval,")",
    " ),","\n\n",

    " sys AS (",
    " SELECT from_unixtime(UNIX_TIMESTAMP(TIME) DIV (",in_seconds_per_interval,")*",in_seconds_per_interval,") AS TIME,",
    " AVG(",in_sys_metric_name,") AS system_metric",
    " FROM system_metrics_table ",
    " WHERE TIME BETWEEN '",interval_lt_time,"' AND '",interval_rt_time,"' AND system_id=", in_system_id,
    " GROUP BY from_unixtime(UNIX_TIMESTAMP(TIME) DIV (",in_seconds_per_interval,")*",in_seconds_per_interval,")",
    " )","\n\n",

 " SELECT time_range.time,",in_seconds_per_interval,",dim_range.id,dim_range.str,",
        " ",IF(in_missing_metrics_form = 'AsZEROs' OR in_missing_metrics_form = 'AsCorrelationDataPoints',"IFNULL(sys.system_metric,0)","sys.system_metric"),",",
        " ",IF(in_missing_metrics_form = 'AsZEROs' OR in_missing_metrics_form = 'AsCorrelationDataPoints',"IFNULL(app.app_metric,0)","app.app_metric"),",",
        " ",IF(in_missing_metrics_form = 'AsZEROs' OR in_missing_metrics_form = 'AsCorrelationDataPoints',"IFNULL(app_total.app_metric_total,0)","app_total.app_metric_total"),",",
        " ",IF(in_missing_metrics_form = 'AsZEROs' OR in_missing_metrics_form = 'AsCorrelationDataPoints',"IFNULL(app.avg_nvp_scale_factor,0)","app.avg_nvp_scale_factor"),",",
        window_metrics_app,",",
        window_metrics_nvp_scale,",",
        window_total_metrics_app,",",
        window_metrics_cret,"\n\n",
 " FROM time_range","\n\n",
 " INNER JOIN dim_range ON dim_range.dummy_column=time_range.dummy_column","\n",
 " ",IF(in_missing_metrics_form = 'AsMissingDataPoints',"INNER","LEFT"), " JOIN app ON time_range.time=app.time AND dim_range.id=app.search_id","\n",
 IF(in_seconds_per_interval <= 4,window_metrics_join,""),"\n",
 IF(in_seconds_per_interval <= 4,window_total_metrics_join,""),"\n",
 " ",IF(in_missing_metrics_form = 'AsMissingDataPoints',"INNER","LEFT"), " JOIN app_total ON time_range.time=app_total.time","\n",
 " ",IF(in_missing_metrics_form = 'AsMissingDataPoints',"INNER","LEFT"), " JOIN sys ON time_range.time=sys.time","\n",
 " ORDER BY time_range.TIME DESC;"
 );

-- select select_stmt;
-- leave tpf_metric_aggregator_lib;
CALL tpf_execute_immediate(select_stmt);
-- CALL tpf_execute_immediate(CONCAT(" SELECT * FROM ", out_table_name,";"));

-- figure out how many intervals are missing our system metric
SET select_stmt = CONCAT(
    " SELECT COUNT(DISTINCT from_unixtime(UNIX_TIMESTAMP(TIME) DIV (",in_seconds_per_interval,")*",in_seconds_per_interval,"))",
    " INTO @out_system_metrics_cnt_intervals_without_data_column",
    " FROM system_metrics_table ",
    " WHERE TIME BETWEEN '",interval_lt_time,"' AND '",interval_rt_time,"' AND system_id=", in_system_id
);
CALL tpf_execute_immediate(select_stmt);
IF @out_system_metrics_cnt_intervals_without_data_column IS NULL
THEN
   SET out_system_metrics_cnt_intervals_without_data_column=in_num_intervals;
ELSE
   SET out_system_metrics_cnt_intervals_without_data_column=in_num_intervals - @out_system_metrics_cnt_intervals_without_data_column;
END IF;

END//
DELIMITER ;

-- Dumping structure for procedure TPF_RTMC_DEV.tpf_statistics_result_builder
DROP PROCEDURE IF EXISTS `tpf_statistics_result_builder`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_statistics_result_builder`(
    IN `in_analysis_type` ENUM('MsgCnt_Stats','AvgCPU_Stats','AvgIO_Stats','AvgExstTime_Stats','TotCPU_Stats','TotIO_Stats','TotExstTime_Stats','MsgCnt_Graph','AvgCPU_Graph','AvgIO_Graph','AvgExstTime_Graph','TotCPU_Graph','TotIO_Graph','TotExstTime_Graph'),
    IN `in_app_dimension` VARCHAR(64),
	IN `in_lt_time` VARCHAR(50),
	IN `in_rt_time` VARCHAR(50),
	IN `in_seconds_to_agg` INT,
	IN `in_num_rows_to_return` INT,
	IN `in_filter_string` VARCHAR(100),
	IN `in_raw_or_projected` ENUM('TimeAveragedScaleFactorProjected','RawData','RawDataScaleFactorProjected'),
    IN `in_system_id` INT
)
tpf_statistics_result_builder:BEGIN

/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     Returns statistics about the metrics.               */
/*                                                                   */
/* Description:  Returns statistics by aggregated windows of seconds */
/*               in the specified time window.  This can be in the   */
/*               form of a graph or table format.                    */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* INTERFACE:                                                        */
/*                                                                   */
/* Dependencies: Assumes caller is Grafana and so the time column    */
/*               must be returned as "time" or "time_sec" for graphs.*/
/*               Assumes Grafana will handle the number of decimals  */
/*               to be displayed.                                    */
/*                                                                   */
/* Input Parameters:                                                 */
/*      in_analysis_type       - enum of analysis to perform         */
/*      in_app_dimension       - dimension in app_dimensions table   */
/*      in_lt_time             - start of time frame to return       */
/*      in_rt_time             - end of time frame to return         */
/*      in_seconds_to_agg      - # seconds in an interval            */
/*      in_num_rows_to_return  - limits our results to return        */
/*      in_filter_string       - filters our NVPs, etc results       */
/*      in_raw_or_projected    - enum of display raw or scale factor */
/*                               projected data for msg_counts       */
/*                                                                   */
/* Tables accessed: Depending upon the request.                      */
/*      nvp_message_metrics_table                                    */
/*                                                                   */
/* Output Parameters:                                                */
/*      None.                                                        */
/*                                                                   */
/* Returned table of data:                                           */
/*      time,*Various table and graph formats*                       */
/*      ordered by time in ascending fashion                         */
/*                                                                   */  
/* SAMPLE CALLS:                                                     */ 
/*      Grafana:                                                     */
/*         CALL tpf_statistics_result_builder('MsgCnt_Stats',        */
/*              'Message Type',$__timeFrom(),$__timeTo(),20,10,'')   */
/*      Direct Query:                                                */     
/*         CALL tpf_statistics_result_builder('MsgCnt_Stats',        */
/*              'Message Type','2018-11-05T17:43:00Z',               */
/*              '2018-11-05T17:49:00Z',20,10,'')                     */
/*                                                                   */ 
/*********************************************************************/
/*                                                                   */
/* NOTES:                                                            */
/*     None.                                                         */
/*                                                                   */
/*********************************************************************/   

-- ## rudimentary working variables
DECLARE select_stmt BLOB;
DECLARE error_string_column_name VARCHAR(50);

-- ## time handling related variables
DECLARE lt_time DATETIME;
DECLARE rt_time DATETIME;
DECLARE num_seconds DOUBLE;
DECLARE num_intervals INT;
DECLARE time_window_total_seconds INT;
DECLARE rt_time_adder INT;

-- ## variables for working with various data
DECLARE calc_norm_metrics BOOLEAN DEFAULT FALSE;
DECLARE missing_metrics_form ENUM('AsMissingDataPoints','AsNULLs','AsZEROs');

-- ## These variables all relate to our generic handling of the tables.
DECLARE agg_metric_name VARCHAR(50);
DECLARE app_metric_column_name VARCHAR(50);
DECLARE sys_metric_name VARCHAR(50);
DECLARE app_metric_total_or_per_msg_avg ENUM('total','per_msg_avg');
DECLARE output_table_type ENUM('statistics','graph');
DECLARE num_aggregate_results BIGINT DEFAULT 0;
DECLARE system_metrics_cnt_intervals_without_data INT;
DECLARE app_dimension_id INT;
DECLARE app_metric_string TEXT;
DECLARE min_app_metric_string TEXT;
DECLARE max_app_metric_string TEXT;
DECLARE avg_app_metric_string TEXT;
DECLARE stddev_app_metric_string TEXT;
DECLARE pct_delta_string TEXT;
DECLARE windowed_app_metric_string TEXT;

-- ## Variables for iterating through lists
DECLARE num_search_ids BIGINT DEFAULT 0;

-- MySQL does not allow temp tables to be used more than once.
-- We could use CTE, but that's less efficient since the search table will be used a bunch.
-- We could duplicate the search table over and over and over.
-- Allocating the search table with a UUID in the name is probably the best approach.
-- But we don't want to create a table leak so we'll catch the error here.
DECLARE search_table_name TEXT DEFAULT NULL;
DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
    -- we don't want to hide the error as that will make debugging more difficult.
    RESIGNAL;
END;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

-- ## Grafana always sends down the time in UTC.  MariaDB attempts to
-- ## automatically handle timezones and has some interesting rules
-- ## about timestamp vs datetime behavior.  All of this yields some
-- ## interesting behavior between the two.
-- ## For simplicity:
-- ##    TPF will record data/time as UTC.
-- ##    Our procedures and connections will use UTC.
-- ##    Date and time will be stored as datetime variables.
SET time_zone = '+00:00';


-- ######################################################################
-- ##
-- ## First thing we'll do is validate all of our inputs and set up the
-- ## various variables we're going to need.
-- ##
-- ######################################################################

-- Validate in_app_dimension
SET app_dimension_id = tpf_get_dimension_id(in_app_dimension);

-- If in_app_dimension is "", there is no data. Let the below error handling display the correct message
IF app_dimension_id IS NULL AND in_app_dimension != ""
THEN
    SELECT CONCAT("Dimension ", QUOTE(in_app_dimension), " not found.") AS 'Error Message';
    LEAVE tpf_statistics_result_builder;
ELSEIF app_dimension_id IS NULL
THEN
    LEAVE tpf_statistics_result_builder;
END IF;


-- ## in_analysis_type - does not need to be validated since we are
-- ## using an enum.  So we'll just set up our table specific
-- ## variables.

-- ####################################################################
-- ## Simple stats
-- ####################################################################
IF in_analysis_type = 'MsgCnt_Stats'
THEN
  -- ## Table details - what metrics are we comparing
  SET app_metric_column_name = 'msg_cnt';
  SET sys_metric_name = 'actual_cpu'; # Not Used
  
  -- ## Should the metrics be summed to total or taken as averages
  SET app_metric_total_or_per_msg_avg = 'total';
  
  SET output_table_type = 'statistics';
ELSEIF in_analysis_type = 'AvgCPU_Stats'
THEN
  -- ## Table details - what metrics are we comparing
  SET app_metric_column_name = 'cpu_used';
  SET sys_metric_name = 'actual_cpu'; # Not Used

  -- ## Should the metrics be summed to total or taken as averages
  SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
  SET output_table_type = 'statistics';
ELSEIF in_analysis_type = 'AvgIO_Stats'
THEN
  -- ## Table details - what metrics are we comparing
  SET app_metric_column_name = 'find_file';
  SET sys_metric_name = 'inuse_iob'; # Not Used
  
  -- ## Should the metrics be summed to total or taken as averages
  SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
  SET output_table_type = 'statistics';
ELSEIF in_analysis_type = 'AvgExstTime_Stats'
THEN
  -- ## Table details - what metrics are we comparing
  SET app_metric_column_name = 'exist_time';
  SET sys_metric_name = 'inuse_ecb'; # Not Used

  -- ## Should the metrics be summed to total or taken as averages
  SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
  SET output_table_type = 'statistics';

-- ####################################################################
-- ## msg count * stats
-- ####################################################################

ELSEIF in_analysis_type = 'TotCPU_Stats'
THEN
  -- ## Table details - what metrics are we comparing
  SET app_metric_column_name = 'msg_cnt*cpu_used';
  SET sys_metric_name = 'actual_cpu'; # Not Used

  -- ## Should the metrics be summed to total or taken as averages
  SET app_metric_total_or_per_msg_avg = 'total';
  
  SET output_table_type = 'statistics';
ELSEIF in_analysis_type = 'TotIO_Stats'
THEN
  -- ## Table details - what metrics are we comparing
  SET app_metric_column_name = 'msg_cnt*find_file';
  SET sys_metric_name = 'inuse_iob'; # Not Used
  
  -- ## Should the metrics be summed to total or taken as averages
  SET app_metric_total_or_per_msg_avg = 'total';
  
  SET output_table_type = 'statistics';
ELSEIF in_analysis_type = 'TotExstTime_Stats'
THEN
  -- ## Table details - what metrics are we comparing
  SET app_metric_column_name = 'msg_cnt*exist_time';
  SET sys_metric_name = 'inuse_ecb'; # Not Used

  -- ## Should the metrics be summed to total or taken as averages
  SET app_metric_total_or_per_msg_avg = 'total';
  
  SET output_table_type = 'statistics';

-- ####################################################################
-- ## graph data points of the stats themselves
-- ####################################################################

ELSEIF in_analysis_type = 'MsgCnt_Graph'
THEN
  -- ## Print Names
  SET agg_metric_name = 'AvgSmpCnt';
  
  -- ## Table details - what metrics are we comparing
  SET app_metric_column_name = 'msg_cnt';
  SET sys_metric_name = 'actual_cpu'; # Not Used
  
  -- ## Should the metrics be summed to total or taken as averages
  SET app_metric_total_or_per_msg_avg = 'total';
  
  SET output_table_type = 'graph';
ELSEIF in_analysis_type = 'AvgCPU_Graph'
THEN
  -- ## Print Names
  SET agg_metric_name = 'AvgCPU';
  
  -- ## Table details - what metrics are we comparing
  SET app_metric_column_name = 'cpu_used';
  SET sys_metric_name = 'actual_cpu'; # Not Used

  -- ## Should the metrics be summed to total or taken as averages
  SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
  SET output_table_type = 'graph';
ELSEIF in_analysis_type = 'AvgIO_Graph'
THEN
  -- ## Print Names
  SET agg_metric_name = 'AvgIO';
  
  -- ## Table details - what metrics are we comparing
  SET app_metric_column_name = 'find_file';
  SET sys_metric_name = 'inuse_iob'; # Not Used
  
  -- ## Should the metrics be summed to total or taken as averages
  SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
  SET output_table_type = 'graph';
ELSEIF in_analysis_type = 'AvgExstTime_Graph'
THEN
  -- ## Print Names
  SET agg_metric_name = 'AvgExstTime';
  
  -- ## Table details - what metrics are we comparing
  SET app_metric_column_name = 'exist_time';
  SET sys_metric_name = 'inuse_ecb'; # Not Used

  -- ## Should the metrics be summed to total or taken as averages
  SET app_metric_total_or_per_msg_avg = 'per_msg_avg';
  SET output_table_type = 'graph';

-- ####################################################################
-- ## graph data points of the MsgCnt * stats
-- ####################################################################

ELSEIF in_analysis_type = 'TotCPU_Graph'
THEN
  -- ## Print Names
  SET agg_metric_name = 'AvgSmpCntCPU';
  
  -- ## Table details - what metrics are we comparing
  SET app_metric_column_name = 'msg_cnt*cpu_used';
  SET sys_metric_name = 'actual_cpu'; # Not Used

  -- ## Should the metrics be summed to total or taken as averages
  SET app_metric_total_or_per_msg_avg = 'total';
  
  SET output_table_type = 'graph';
ELSEIF in_analysis_type = 'TotIO_Graph'
THEN
  -- ## Print Names
  SET agg_metric_name = 'TotIO';
  
  -- ## Table details - what metrics are we comparing
  SET app_metric_column_name = 'msg_cnt*find_file';
  SET sys_metric_name = 'inuse_iob'; # Not Used
  
  -- ## Should the metrics be summed to total or taken as averages
  SET app_metric_total_or_per_msg_avg = 'total';
  
  SET output_table_type = 'graph';
ELSEIF in_analysis_type = 'TotExstTime_Graph'
THEN
  -- ## Print Names
  SET agg_metric_name = 'TotExstTime';
  
  -- ## Table details - what metrics are we comparing
  SET app_metric_column_name = 'msg_cnt*exist_time';
  SET sys_metric_name = 'inuse_ecb'; # Not Used

  -- ## Should the metrics be summed to total or taken as averages
  SET app_metric_total_or_per_msg_avg = 'total';
  
  SET output_table_type = 'graph';

-- ####################################################################
-- ## Otherwise, we have an error of some kind, but with enums this can't happen
-- ####################################################################

ELSE
  -- return error and exit
  -- enum inputs prevent us from getting here!!!!!!!!!!!
  SELECT "The analysis type specified on the in_analysis_type parameter of the call to tpf_corr_table_builder does not exist." AS 'Error Message';
  LEAVE tpf_statistics_result_builder;
END IF;

-- ####################################################################
-- ## set up our error column name.
-- ## If the content is displaying in a graph, we'll use the name time so the end user can see the error in grafana
-- ## Otherwise, it's a table and we'll use "error message" which will appear as the column header in a nice obvious way.
-- ####################################################################
IF output_table_type = 'graph'
THEN
   SET error_string_column_name = 'time';
ELSE
   SET error_string_column_name = 'Error Message';
END IF;

-- ####################################################################
-- ## Validate our seconds to aggregate input
-- ####################################################################
IF in_seconds_to_agg IS NULL OR in_seconds_to_agg < 1
THEN
    -- return error and exit
    CALL tpf_execute_immediate(CONCAT(
        " SELECT 'The number of seconds to aggregate is less than 1.  Please fill in a value greater than 0.' AS '", error_string_column_name, "'"
    ));

    LEAVE tpf_statistics_result_builder;
END IF;

-- ####################################################################
-- ## Validate our seconds to aggregate input
-- ####################################################################
IF in_num_rows_to_return IS NULL OR in_num_rows_to_return < 1
THEN
    -- return error and exit
    CALL tpf_execute_immediate(CONCAT(
        " SELECT 'The number of rows to return is less than 1.  Please fill in a value greater than 0.' AS '", error_string_column_name, "'"
    ));
    LEAVE tpf_statistics_result_builder;
END IF;

-- ####################################################################
-- ## Validate our time inputs
-- ####################################################################
SET lt_time = STR_TO_DATE(in_lt_time, '%Y-%m-%d %H:%i:%s');
SET rt_time = STR_TO_DATE(in_rt_time, '%Y-%m-%d %H:%i:%s');
IF lt_time IS NULL OR rt_time IS NULL
THEN
    CALL tpf_execute_immediate(CONCAT(
        " SELECT 'Time input values are not formatted correctly.' AS '", error_string_column_name, "'"
    ));
    LEAVE tpf_statistics_result_builder;
END IF;

IF lt_time > rt_time
THEN
    CALL tpf_execute_immediate(CONCAT(
        " SELECT \"'lt' time input is larger than 'rt' time input.\" AS '", error_string_column_name, "'"
    ));
    LEAVE tpf_statistics_result_builder;
END IF;

-- ####################################################################
-- ## adjust our input times to be on the in_seconds_to_agg mod boundary
-- ## We'll shift our right boundary by 2 additional window so we
-- ## don't end up with a gap on the right side of our graph.
-- ## Otherwise, for the stats we only bump 1 so it aligns with the graph displayed
-- ####################################################################
IF output_table_type = 'graph'
THEN
   SET rt_time_adder=in_seconds_to_agg + in_seconds_to_agg-1;
ELSE
   SET rt_time_adder=in_seconds_to_agg-1;
END IF;
SET rt_time = FROM_UNIXTIME(UNIX_TIMESTAMP(rt_time) DIV in_seconds_to_agg * in_seconds_to_agg + rt_time_adder);
SET lt_time = FROM_UNIXTIME(UNIX_TIMESTAMP(lt_time) DIV in_seconds_to_agg * in_seconds_to_agg);


-- ########################################################################
-- ## Validate our seconds to aggregate is large enough for our time window, otherwise Grafana is too slow
-- ########################################################################
SET time_window_total_seconds = TIMESTAMPDIFF(SECOND, lt_time, rt_time);

IF time_window_total_seconds / in_seconds_to_agg > 600
THEN
    CALL tpf_execute_immediate(CONCAT(
        " SELECT 'Must have fewer than 600 intervals.' AS '", error_string_column_name, "' UNION"
        " SELECT 'Use the time picker to select a smaller range,' UNION",
        " SELECT 'or set SecondsToAggregate to at least ", CEILING(time_window_total_seconds/600), ".'"
    ));
    LEAVE tpf_statistics_result_builder;
END IF;


-- ####################################################################
-- ## set up our time related variables
-- ####################################################################
SET num_seconds=TIMESTAMPDIFF(SECOND,lt_time,rt_time);

-- ####################################################################
-- ## Need the +1 here so that the graph extends through the left time
-- ## bound in Grafana
-- ####################################################################
SET num_intervals=FLOOR(num_seconds/in_seconds_to_agg) + 1;


-- ######################################################################
-- ##
-- ## Next we'll set up our temporary working tables.
-- ##
-- ######################################################################

CALL tpf_allocate_aggregator_search_table(in_app_dimension,app_dimension_id,in_filter_string,in_system_id,lt_time,rt_time,NULL,NULL,num_intervals,search_table_name);
-- ## How many entries do we have?  Handle errors
SET select_stmt=CONCAT(" SELECT COUNT(*) INTO @num_search_ids FROM `",search_table_name,"`");
CALL tpf_execute_immediate(select_stmt);
IF @num_search_ids IS NULL OR @num_search_ids = 0
THEN
    SELECT CONCAT(
        "No strings found for ",
        QUOTE(in_app_dimension),
        " in the requested time ranges" ,
        CASE WHEN in_filter_string != "" THEN CONCAT(" with filter ", QUOTE(in_filter_string)) ELSE "" END,
        ". There is probably no data in the time ranges."
    ) AS 'Error Message';

    LEAVE tpf_statistics_result_builder;
END IF;

-- ## In this case, we want aggregator_lib to calculate the normalization metrics
SET missing_metrics_form = 'AsNULLs';
IF in_raw_or_projected = 'RawData' 
THEN
    SET calc_norm_metrics = FALSE;
    IF output_table_type = 'graph'
    THEN 
       SET missing_metrics_form = 'AsNULLs';
    ELSE
       SET missing_metrics_form = 'AsMissingDataPoints'; 
    END IF;
ELSEIF in_raw_or_projected = 'RawDataScaleFactorProjected'
THEN
    IF app_metric_total_or_per_msg_avg = 'total'
    THEN
       SET calc_norm_metrics = TRUE;
    END IF;
    IF output_table_type = 'graph'
    THEN 
       SET missing_metrics_form = 'AsNULLs';
    ELSE
       SET missing_metrics_form = 'AsMissingDataPoints'; 
    END IF;
ELSEIF in_raw_or_projected = 'TimeAveragedScaleFactorProjected'
THEN
    IF app_metric_total_or_per_msg_avg = 'total'
    THEN
       SET calc_norm_metrics = TRUE;
    END IF;
    SET missing_metrics_form = 'AsZEROs';
END IF;

-- ######################################################################
-- ##
-- ## We'll call the aggregator lib to condense our working set into the
-- ## seconds to aggregate for us.  The results of this call are populated
-- ## in the aggregate_table.
-- ##
-- ######################################################################
CALL tpf_allocate_aggregate_table("aggregate_table");
CALL tpf_metric_aggregator_lib(search_table_name,"nvp_message_metrics_table",app_metric_column_name,"value_id",sys_metric_name,rt_time,in_seconds_to_agg,num_intervals,app_metric_total_or_per_msg_avg,missing_metrics_form,"aggregate_table","lt_cnt_intervals_with_data",system_metrics_cnt_intervals_without_data,in_system_id, app_dimension_id);


-- ######################################################################
-- ##
-- ## We'll check to see if we have any data
-- ##
-- ######################################################################
SELECT COUNT(*) INTO num_aggregate_results FROM aggregate_table;



-- ######################################################################
-- ##
-- ## If we have no data, just exit now.
-- ##
-- ######################################################################
IF num_aggregate_results IS NULL OR num_aggregate_results = 0
THEN
    CALL tpf_execute_immediate(CONCAT(
        " SELECT 'No data in the requested time range.' AS '", error_string_column_name, "'"
    ));
    LEAVE tpf_statistics_result_builder;
END IF;


-- ######################################################################
-- ##
-- ## If we're returning results for a graph, we'll play a little game so
-- ## our graphs look nice in grafana.  But otherwise, we'll just return
-- ## the results we have.
-- ##
-- ######################################################################
IF output_table_type = 'graph'
THEN
	IF calc_norm_metrics = TRUE
    THEN
       SET app_metric_string="windowed_scale_factor * windowed_app_metric";
    ELSE
       SET app_metric_string="app_metric";
    END IF;

    CALL tpf_execute_immediate(CONCAT(
        " SELECT time,search_string AS ",QUOTE(in_app_dimension),", ",app_metric_string," AS ",agg_metric_name,
        " FROM aggregate_table",
        " ORDER BY time ASC"
    ));

    LEAVE tpf_statistics_result_builder;
END IF;


-- ######################################################################
-- ##
-- ## Our result is in table format, so we have some math to do to create our results.
-- ## For our table of results, we need to calculate the min, max, avg, sddev, etc
-- ## we'll also capture the results at the left and right and compture a delta
-- ## We'll store all of these results into the results table and output this at the end.
-- ##
-- ######################################################################

-- ## Calc min, max, avg, stdev from ALL entries in our table for "this" entry (search_id) in the search table.
-- ## The msg_count related metrics are adjusted by the average nvp_scale factor in windows surrounding each interval in our data set
-- ## If our metric involves message counts ie totals (total messages, or total messages * avg CPU), we adjust by the average
-- ## scale factor in a window around each interval. The scale factor can change over time and using an average from the window around
-- ## each interval helps smooth out message rates for infrequent messages.
-- ## Otherwise, we are looking to show the average metric like CPU and we just show it as is.
IF calc_norm_metrics = TRUE
THEN
    SET windowed_app_metric_string="a.windowed_scale_factor * a.windowed_app_metric";
    SET min_app_metric_string="MIN(a.windowed_scale_factor * a.windowed_app_metric)";
    SET max_app_metric_string="MAX(a.windowed_scale_factor * a.windowed_app_metric)";
    SET avg_app_metric_string="AVG(a.windowed_scale_factor * a.windowed_app_metric)";
    SET stddev_app_metric_string="STDDEV(a.windowed_scale_factor * a.windowed_app_metric)";
ELSE
    SET windowed_app_metric_string="a.windowed_app_metric";
    SET min_app_metric_string="MIN(a.app_metric)";
    SET max_app_metric_string="MAX(a.app_metric)";
    SET avg_app_metric_string="AVG(a.app_metric)";
    SET stddev_app_metric_string="STDDEV(a.app_metric)";
END IF;

SET pct_delta_string=CONCAT(
    " CASE WHEN lt.value IS NULL AND rt.value IS NULL THEN 0",
    "      WHEN lt.value IS NULL AND rt.value IS NOT NULL AND rt.value<=0 THEN 0",
    "      WHEN lt.value IS NULL AND rt.value IS NOT NULL AND rt.value>0 THEN 100",
    "      WHEN lt.value IS NOT NULL AND lt.value<=0 AND rt.value IS NULL THEN 0",
    "      WHEN lt.value IS NOT NULL AND lt.value>0 AND rt.value IS NULL THEN -100",
    "      WHEN lt.value<=0 AND rt.value<=0 THEN 0",
    "      WHEN lt.value<=0 AND rt.value>0 THEN 100",
    "      WHEN lt.value>0 AND rt.value<=0 THEN -100",
    "      ELSE (rt.value-lt.value)/lt.value*100",
    " END "
);

DROP TEMPORARY TABLE IF EXISTS tpf_rt_most_values_table;
CREATE TEMPORARY TABLE tpf_rt_most_values_table (
    `search_id` INT NULL DEFAULT NULL,
    `value` BIGINT NULL DEFAULT NULL,
    `cret` BIGINT NULL DEFAULT NULL,
    INDEX `search_id_index` (`search_id`)
)
ENGINE=MEMORY;
DROP TEMPORARY TABLE IF EXISTS tpf_lt_most_values_table;
CREATE TEMPORARY TABLE tpf_lt_most_values_table LIKE tpf_rt_most_values_table;

SELECT MAX(time),MIN(time) INTO rt_time,lt_time FROM aggregate_table;

SET select_stmt=CONCAT(
" INSERT INTO tpf_rt_most_values_table (search_id,value,cret)",
" SELECT a.search_id,",windowed_app_metric_string,",a.windowed_cret_metric",
" FROM aggregate_table a",
" WHERE TIME='",rt_time,"'"
);
CALL tpf_execute_immediate(select_stmt);

SET select_stmt=CONCAT(
" INSERT INTO tpf_lt_most_values_table (search_id,value,cret)",
" SELECT a.search_id,",windowed_app_metric_string,",a.windowed_cret_metric",
" FROM aggregate_table a",
" WHERE TIME='",lt_time,"'"
);
-- SELECT select_stmt;
CALL tpf_execute_immediate(select_stmt);

SET select_stmt=CONCAT(
    " SELECT a.search_string AS ", QUOTE(in_app_dimension),",",

    " lt.value AS 'Lt',",
    " rt.value AS 'Rt',",
    pct_delta_string," AS '%-Delta',",
    " ABS(",pct_delta_string,") AS hltr,",

    " ",min_app_metric_string," AS 'Min',",
    " ",max_app_metric_string," AS 'Max',",
    " ",avg_app_metric_string," AS 'Avg',",
    " ",stddev_app_metric_string," AS 'StdDev',",

    " lt.cret+1 AS LtEcbCnt,",
    " rt.cret+1 AS RtEcbCnt",
    
    " FROM aggregate_table a",
    " LEFT JOIN tpf_rt_most_values_table rt ON rt.search_id=a.search_id"
    " LEFT JOIN tpf_lt_most_values_table lt ON lt.search_id=a.search_id"
    " GROUP BY a.search_id",
    " ORDER BY ABS(",pct_delta_string,") DESC",
    " LIMIT ", in_num_rows_to_return
);
-- SELECT select_stmt;
CALL tpf_execute_immediate(select_stmt);

END//
DELIMITER ;

-- Dumping structure for procedure TPF_RTMC_DEV.tpf_system_stats_result_builder
DROP PROCEDURE IF EXISTS `tpf_system_stats_result_builder`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_system_stats_result_builder`(
    IN `in_system_metrics` JSON,
	IN `in_lt_time` DATETIME,
	IN `in_rt_time` DATETIME,
	IN `in_system_id` INT,
	IN `in_sec_per_interval` INT
)
tpf_system_stats_result_builder:BEGIN

/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     Return time based table of system wide (CDC) data.  */
/*                                                                   */
/* Description:  Returns CPU, IOB, ECB, or ALL CDC data. The data is */
/*               assumed to be displayed in a time based graph.      */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* INTERFACE:                                                        */
/*                                                                   */
/* Dependencies: Assumes caller is Grafana and so the time column    */
/*               must be returned as "time" or "time_sec".           */
/*               Assumes Grafana will handle the number of decimals  */
/*               to be displayed.                                    */
/*                                                                   */
/* Input Parameters:                                                 */
/*      in_system_metrics - json list of system metrics to graph     */
/*      in_lt_time     - start of time frame to return               */
/*      in_rt_time     - end of time frame to return                 */
/*      in_system_id   - the system id to show metrics for           */
/*      in_sec_per_interval - number of seconds per interval to      */
/*                            average metrics (to reduce # points)   */
/*                                                                   */
/* Tables accessed:                                                  */
/*      system_metrics_table                                         */
/*                                                                   */
/* Output Parameters:                                                */
/*      None.                                                        */
/*                                                                   */
/* Returned table of data:                                           */
/*      time,*Desired Metrics*                                       */
/*      ordered by time in ascending fashion                         */
/*                                                                   */  
/* SAMPLE CALLS:                                                     */ 
/*      Grafana:                                                     */
/*         CALL tpf_system_stats_result_builder("ALL",$__timeFrom(), */
/*                                              $__timeTo())         */ 
/*      Direct Query:                                                */     
/*         CALL tpf_system_stats_result_builder("ALL",               */
/*              '2018-11-05T17:43:00Z','2018-11-05T17:49:00Z')       */     
/*                                                                   */ 
/*********************************************************************/
/*                                                                   */
/* NOTES:                                                            */
/*     None.                                                         */
/*                                                                   */
/*********************************************************************/

DECLARE system_metric_cols TEXT;

DECLARE select_stmt BLOB;
DECLARE row_count BIGINT;

DECLARE group_by_time_shift INT;
DECLARE lt_time_shift INT;

-- Set this for procedures called from Grafana
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;

-- ## Grafana always sends down the time in UTC.  MariaDB attempts to
-- ## automatically handle timezones and has some interesting rules
-- ## about timestamp vs datetime behavior.  All of this yields some
-- ## interesting behavior between the two.
-- ## For simplicity:
-- ##    TPF will record data/time as UTC.
-- ##    Our procedures and connections will use UTC.
-- ##    Date and time will be stored as datetime variables.
SET time_zone = '+00:00';
IF  in_lt_time > in_rt_time
THEN 
    SELECT "'lt' time input is larger than 'rt' time input." AS time;
    LEAVE tpf_system_stats_result_builder;
END IF;

SELECT GROUP_CONCAT(
    CASE WHEN metric = 'actual_cpu' THEN "if(AVG(modeled_cpu) > 1000,1000/10,AVG(modeled_cpu)/10) AS modeled_cpu,AVG(actual_cpu)/10 AS actual_cpu,if(AVG(calculated_cpu) > 1000,1000/10,NULL) as modeled_cpu_exceeds_capacity"
         WHEN metric = 'InUse_IOBs' THEN "AVG(inuse_iob) AS inuse_iob"
         WHEN metric = 'InUse_ECBs' THEN "AVG(inuse_ecb) AS inuse_ecb"
         WHEN metric != 'est_msg_cpu' AND metric LIKE '%_cpu%' THEN REPLACE("AVG(metric)/10 AS metric", "metric", LOWER(metric))
         ELSE REPLACE("AVG(metric) AS metric", "metric", LOWER(metric))
    END
)
INTO system_metric_cols
FROM JSON_TABLE(in_system_metrics, "$[*]" COLUMNS(metric VARCHAR(64) path "$")) ism;

IF system_metric_cols IS NULL
THEN
    LEAVE tpf_system_stats_result_builder;
END IF;

SET group_by_time_shift = tpf_calc_group_by_time_shift(in_rt_time, in_sec_per_interval);
SET lt_time_shift = tpf_calc_lt_time_shift(in_lt_time, in_sec_per_interval, group_by_time_shift);

CALL tpf_execute_immediate(CONCAT(
    " SELECT MIN(time) AS time,", system_metric_cols,
    " FROM system_metrics_table",
    " WHERE system_id=", in_system_id, " AND time BETWEEN ", QUOTE(DATE_SUB(in_lt_time, INTERVAL lt_time_shift SECOND)), " AND ", QUOTE(in_rt_time),
    -- Average values over n second intervals to keep from returning too many data points. Grafana will supply n based on the width of the graph and the width of the time window.
    " GROUP BY (UNIX_TIMESTAMP(time) + ", group_by_time_shift, ") DIV ", in_sec_per_interval,
    " ORDER BY time ASC"
));
END//
DELIMITER ;

-- Dumping structure for procedure TPFDataSciDB.tpf_modeled_cpu_lib
DROP PROCEDURE IF EXISTS `tpf_modeled_cpu_lib`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_modeled_cpu_lib`(
    IN `in_app_dimension` VARCHAR(64),
	IN `in_lt_time` DATETIME,
	IN `in_rt_time` DATETIME
)
tpf_modeled_cpu_lib:BEGIN
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     Calculates and inserts the modeled_cpu.             */
/*                                                                   */
/* Description:  tpf_zrtmc_analyzer calculates the CPU each second   */
/*               based upon msg_cnt of each message type * the       */
/*               cpu_used for each  message type.  This value is      */
/*               written to the system_metrics_table as              */
/*               calculated_cpu.  In reality, this is recording when */
/*               the ECBs exited, not really when they were          */
/*               executing. We show the modeled_cpu which is our     */
/*               predicted expectation of what the actual_cpu will   */
/*               be, on graphs with the actual_cpu as reported by    */
/*               CDC.  The actual_cpu is exponentially smoothed so   */
/*               our modeled_cpu shows comparitively high variance.  */
/*               It just doesn't look right.  So this routine will   */
/*               calculate the modeled_cpu as a smoothed version of  */
/*               the calculated_cpu.                                 */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* INTERFACE:                                                        */
/*                                                                   */
/* Dependencies: Assumes caller is tpf_system_stats_result_builder.  */
/*                                                                   */
/* Input Parameters:                                                 */
/*      in_app_dimension       - the app dimension to use            */
/*                               for calculating modeled_cpu         */
/*      in_lt_time             - leftmost time to analyze            */
/*      in_rt_time             - rightmost time to analyze           */
/*                                                                   */
/* Tables accessed:                                                  */
/*      tpf_tcpip_metrics_table                                      */
/*                                                                   */
/* Output Parameters:                                                */
/*      None.                                                        */
/*                                                                   */
/* Result:                                                           */
/*      modeled_cpu   - Modified with smoothed calculated_cpu value  */
/*      modeled_cpu_data_points - indicates how many data points were*/
/*                                were used when calculating the     */
/*                                modeled_cpu.  Also used to know    */
/*                                when modeled_cpu has been          */
/*                                written to a row.                  */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/* NOTES:                                                            */
/*     The handling of time below assumes time is captured in 1      */
/*     second intervals and inputs are specified in 1 second         */
/*     intervals. If otherwise the time handling needs to be changed.*/
/*                                                                   */
/*********************************************************************/

DECLARE app_dimension_id INT;

-- Do everything in UTC time
SET time_zone = '+00:00';


-- Validate in_app_dimension
SET app_dimension_id = tpf_get_dimension_id(in_app_dimension);

-- If in_app_dimension is "", there is no data. Let the below error handling display the correct message
IF app_dimension_id IS NULL AND in_app_dimension != ""
THEN
    SELECT CONCAT("Dimension ", QUOTE(in_app_dimension), " not found.") AS 'Error Message';
    LEAVE tpf_modeled_cpu_lib;
ELSEIF app_dimension_id IS NULL
THEN
    LEAVE tpf_modeled_cpu_lib;
END IF;

DROP TEMPORARY TABLE IF EXISTS temp_istreams_table;
CREATE TEMPORARY TABLE temp_istreams_table (
    `TIME` DATETIME(3) NOT NULL,
    `system_id` SMALLINT UNSIGNED NOT NULL,
    `istreams` INT UNSIGNED NOT NULL
) ENGINE=MEMORY;

INSERT INTO temp_istreams_table (`TIME`, `system_id`, `istreams`)
    SELECT s.TIME, s.system_id, GREATEST(s.inuse_istreams, s.istream_cap)
    FROM system_metrics_table s
    -- Include an extra 3 seconds on either side for the smoothing calculation
    WHERE s.TIME BETWEEN DATE_SUB(in_lt_time, INTERVAL 3 SECOND) AND DATE_ADD(in_rt_time, INTERVAL 3 SECOND);

DROP TEMPORARY TABLE IF EXISTS temp_modeled_cpu_calc_cpu_table;
CREATE TEMPORARY TABLE temp_modeled_cpu_calc_cpu_table (
    `TIME` DATETIME(3) NOT NULL,
    `system_id` SMALLINT UNSIGNED NOT NULL,
    `calc_cpu` INT UNSIGNED NOT NULL
) ENGINE=MEMORY;

INSERT INTO temp_modeled_cpu_calc_cpu_table (`TIME`, `system_id`, `calc_cpu`)
    SELECT m.TIME, m.system_id,
           SUM(m.msg_cnt*m.cpu_used)*AVG(m.avg_nvp_scale_factor)/AVG(m.duration*1000*s.istreams) AS calc_cpu
    FROM nvp_message_metrics_table m
         INNER JOIN app_dimension_values v ON m.value_id=v.id
         INNER JOIN temp_istreams_table s ON (s.system_id=m.system_id AND s.TIME=m.TIME)
         -- Include an extra 3 seconds on either side for the smoothing calculation
    WHERE v.dimension_id=app_dimension_id AND m.TIME BETWEEN DATE_SUB(in_lt_time, INTERVAL 3 SECOND) AND DATE_ADD(in_rt_time, INTERVAL 3 SECOND)
    GROUP BY m.system_id, m.TIME;

-- ## Window function as opposed to a loop is faster and saves stress on the database with multiple, single row inserts

-- ## We intended to do exponential smoothing of the modeled CPU such
-- ## that the modedled CPU was smoothed in the same manner as
-- ## how CDC reports actual CPU.  ie n-1 * .8 + n * .2
-- ## However, at this time, this can not be accomplished in tpf_zrtmc_analyzer.
-- ## Doing so in this procedure would result in values changing dramatically if
-- ## we didn't process from the start of the data.  And how would we
-- ## handle missing entries?  They significantly skew the results
-- ## shown to the user.  Instead we use a rolling average of at least 5 entries,
-- ## two in the past and two the future.  This results in a value that
-- ## does not change as the windows of available data change, a value
-- ## that is more reactive as it only requires 5 seconds of data, etc.
-- ## If we don't have 5 data points, we won't write out a value for
-- ## the cpu value.  But ultimately, we'll try to get 7 data points for our
-- ## average to make it more smooth, 3 in the past, 3 in the future.


DROP TEMPORARY TABLE IF EXISTS temp_modeled_cpu_data;
CREATE TEMPORARY TABLE temp_modeled_cpu_data (
    `TIME` DATETIME(3) NOT NULL,
    `system_id` SMALLINT UNSIGNED NOT NULL,
    `calc_cpu` INT UNSIGNED NOT NULL,
    `cpu_used` INT UNSIGNED NOT NULL,
    `calc_cpu_entry_cnt` INT UNSIGNED NOT NULL
) ENGINE=MEMORY;

INSERT INTO temp_modeled_cpu_data (`system_id`, `time`, `calc_cpu`, `cpu_used`, `calc_cpu_entry_cnt`)
    SELECT system_id, time, calc_cpu,
           -- Get the average of the data points in our time window
           -- as well as the number of rows we have in this time window
           -- Time windows includes 3 seconds behind and ahead of each row
           AVG(calc_cpu) OVER (PARTITION BY system_id ORDER BY UNIX_TIMESTAMP(time) RANGE BETWEEN 3 PRECEDING AND 3 FOLLOWING) AS cpu_used,
           COUNT(calc_cpu) OVER (PARTITION BY system_id ORDER BY UNIX_TIMESTAMP(time) RANGE BETWEEN 3 PRECEDING AND 3 FOLLOWING) AS calc_cpu_entry_cnt
    FROM temp_modeled_cpu_calc_cpu_table;

-- With an update, we don't do any unnecessary writes. The DB won't write out any values that are the same.
UPDATE system_metrics_table s, temp_modeled_cpu_data t
SET s.calculated_cpu=t.calc_cpu, s.modeled_cpu=t.cpu_used, s.modeled_cpu_data_points=t.calc_cpu_entry_cnt
-- If we have at least 5 data points in our 7 second window of time,
-- then we'll write our average into the row with the number of data points
-- If we have less than 7 data points, we'll try again next time and hope to
-- smooth things a bit better.
WHERE s.system_id=t.system_id AND s.TIME=t.TIME AND t.calc_cpu_entry_cnt >= 5 AND s.TIME BETWEEN in_lt_time AND in_rt_time;

END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_estimate_system_metrics`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_estimate_system_metrics`(
    IN `in_app_dimension` VARCHAR(64),
	IN `in_lt_time` DATETIME,
	IN `in_rt_time` DATETIME
)
tpf_estimate_system_metrics:BEGIN

DECLARE app_dimension_id INT;

-- Do everything in UTC time
SET time_zone = '+00:00';

-- Validate in_app_dimension
SET app_dimension_id = tpf_get_dimension_id(in_app_dimension);

-- If in_app_dimension is "", there is no data. Let the below error handling display the correct message
IF app_dimension_id IS NULL AND in_app_dimension != ""
THEN
    SELECT CONCAT("Dimension ", QUOTE(in_app_dimension), " not found.") AS 'Error Message';
    LEAVE tpf_estimate_system_metrics;
ELSEIF app_dimension_id IS NULL
THEN
    LEAVE tpf_estimate_system_metrics;
END IF;

-- Calculated estimated metrics. They are weighted averages by msg_cnt for each metrics, except rate, which is a SUM
-- of the rates.
-- Use DIV instead of / because DIV is safe for BIGINT
-- With an update, we don't do any unnecessary writes. The DB won't write out any values that are the same.
-- Table w calculates a smoothed version of the est_metrics. It averages over an interval of 7 seconds: 3 before and 3 after
UPDATE system_metrics_table s, (
    SELECT t.TIME, t.system_id,
           AVG(t.est_cpu) OVER (PARTITION BY t.system_id ORDER BY UNIX_TIMESTAMP(t.time) RANGE BETWEEN 3 PRECEDING AND 3 FOLLOWING) AS est_cpu,
           AVG(t.est_exist) OVER (PARTITION BY t.system_id ORDER BY UNIX_TIMESTAMP(t.time) RANGE BETWEEN 3 PRECEDING AND 3 FOLLOWING) AS est_exist,
           AVG(t.est_find_file) OVER (PARTITION BY t.system_id ORDER BY UNIX_TIMESTAMP(t.time) RANGE BETWEEN 3 PRECEDING AND 3 FOLLOWING) AS est_find_file,
           AVG(t.est_rate) OVER (PARTITION BY t.system_id ORDER BY UNIX_TIMESTAMP(t.time) RANGE BETWEEN 3 PRECEDING AND 3 FOLLOWING) AS est_rate
    FROM (
        SELECT m.TIME, m.system_id,
               SUM(m.cpu_used*m.msg_cnt) DIV SUM(msg_cnt) AS est_cpu,
               SUM(m.exist_time*m.msg_cnt) DIV SUM(msg_cnt) AS est_exist,
               SUM(m.find_file*m.msg_cnt) DIV SUM(msg_cnt) AS est_find_file,
               SUM(m.est_msg_rate) AS est_rate
        FROM nvp_message_metrics_table m INNER JOIN app_dimension_values v ON m.value_id=v.id
        -- Include an extra 3 seconds on either side for the smoothing calculation
        WHERE v.dimension_id=app_dimension_id AND m.TIME BETWEEN DATE_SUB(in_lt_time, INTERVAL 3 SECOND) AND DATE_ADD(in_rt_time, INTERVAL 3 SECOND)
        GROUP BY m.system_id, m.TIME
    ) t
) w
SET s.est_msg_cpu=w.est_cpu, s.est_msg_exist=w.est_exist, s.est_msg_find_file=w.est_find_file, s.est_msg_rate=w.est_rate
WHERE s.system_id=w.system_id AND s.TIME=w.TIME AND s.TIME BETWEEN in_lt_time AND in_rt_time;


END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_allocate_aggregator_search_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_allocate_aggregator_search_table`(
    IN `in_app_dimension` VARCHAR(64),
    IN `in_app_dimension_id` INT,
    IN `in_filter_string` VARCHAR(100),
    IN `in_system_id` INT,
	IN `in_lt_start_time` DATETIME, -- if NULL, ignored
	IN `in_lt_end_time` DATETIME, -- if NULL, ignored
	IN `in_rt_start_time` DATETIME, -- if NULL, ignored
	IN `in_rt_end_time` DATETIME, -- if NULL, ignored
	IN `in_seconds_per_interval` INT,
	OUT `out_table_name` TEXT
)
tpf_allocate_aggregator_search_table:BEGIN

DECLARE select_stmt BLOB;
DECLARE id_column_name_string TEXT;
DECLARE table_name_string TEXT;
DECLARE time_comparison_string TEXT;
DECLARE lt_not_null_cnt_stmt TEXT;
DECLARE lt_null_cnt_stmt TEXT;
DECLARE rt_not_null_cnt_stmt TEXT;
DECLARE rt_null_cnt_stmt TEXT;

-- ## The search table is used to determine all possible values in our
-- ## data set of interest.  For example, msgtypes available in the
-- ## time window are Booking, Shopping, etc.  We use this with our
-- ## in_filter_string to very crudely filter our results.
-- ## the tpf_periodic_clean_up event will delete the tables that are
-- ## created here.  That way, user's don't need DROP permissions.
SET out_table_name=CONCAT("tpf_search_table_",UUID());
SET select_stmt=CONCAT(
   " CREATE TABLE `",out_table_name,"` (",
   "     `search_id` INT NOT NULL,",
   "     `search_string` VARCHAR(500) NOT NULL,",
   "     `rt_cnt_intervals_with_data` INT NULL DEFAULT NULL,",
   "     `lt_cnt_intervals_with_data` INT NULL DEFAULT NULL,",
   "     PRIMARY KEY (`search_id`)",
   " )",
   " ENGINE=MEMORY",
   " ;"
);
CALL tpf_execute_immediate(select_stmt);

-- ######################################################################
-- ##
-- ## determine the possible groups of
-- ## strings we are are dealing with in our two time ranges.  Like msgtype =
-- ## Booking or Shopping or etc.
-- ##
-- ######################################################################
IF in_app_dimension = 'TCPIP'
THEN
    SET id_column_name_string="app_name_id";
    SET table_name_string="tpf_tcpip_metrics_table";
ELSE
    SET id_column_name_string="value_id";
    SET table_name_string="nvp_message_metrics_table";
END IF;

SET time_comparison_string= CASE WHEN in_lt_start_time IS NOT NULL AND in_rt_start_time IS NOT NULL THEN CONCAT(" AND ((t.time BETWEEN '",in_lt_start_time,"' AND '",in_lt_end_time,"') OR (t.time BETWEEN '",in_rt_start_time,"' AND '",in_rt_end_time,"'))")
                                 WHEN in_lt_start_time IS NOT NULL THEN CONCAT(" AND (t.time BETWEEN '",in_lt_start_time,"' AND '",in_lt_end_time,"')")
                                 WHEN in_rt_start_time IS NOT NULL THEN CONCAT(" AND (t.time BETWEEN '",in_rt_start_time,"' AND '",in_rt_end_time,"')")
                                 ELSE ""
                            END;
SET lt_not_null_cnt_stmt=CONCAT(
   " SELECT t.ID,COUNT(t.time) AS CNT",
   " FROM (SELECT DISTINCT from_unixtime(UNIX_TIMESTAMP(time) DIV (",in_seconds_per_interval,")*",in_seconds_per_interval,") AS TIME,",id_column_name_string," AS ID",
         " FROM ",table_name_string," WHERE system_id=",in_system_id," AND time BETWEEN '",in_lt_start_time,"' AND '",in_lt_end_time,"' ) AS t",
   " GROUP BY t.ID"
);
SET lt_null_cnt_stmt=CONCAT(
   " SELECT v.id AS ID, 0 AS CNT",
   " FROM app_dimension_values v",
   " WHERE v.dimension_id=",in_app_dimension_id," AND v.value LIKE CONCAT('%', '",in_filter_string,"', '%')"
);
SET rt_not_null_cnt_stmt=CONCAT(
   " SELECT t.ID,COUNT(t.time) AS CNT",
   " FROM (SELECT DISTINCT from_unixtime(UNIX_TIMESTAMP(time) DIV (",in_seconds_per_interval,")*",in_seconds_per_interval,") AS TIME,",id_column_name_string," AS ID",
         " FROM ",table_name_string," WHERE system_id=",in_system_id," AND time BETWEEN '",in_rt_start_time,"' AND '",in_rt_end_time,"' ) AS t",
   " GROUP BY t.ID"
);
SET rt_null_cnt_stmt=CONCAT(
   " SELECT v.id as ID, 0 AS CNT",
   " FROM app_dimension_values v",
   " WHERE v.dimension_id=",in_app_dimension_id," AND v.value LIKE CONCAT('%', '",in_filter_string,"', '%')"
);

SET select_stmt = CONCAT(
    " INSERT INTO `",out_table_name,"` (search_id,search_string,rt_cnt_intervals_with_data,lt_cnt_intervals_with_data)",
    " WITH lt AS (",
    IF(in_lt_start_time IS NOT NULL,lt_not_null_cnt_stmt,lt_null_cnt_stmt),
    " ),",
    " rt AS (",
    IF(in_rt_start_time IS NOT NULL,rt_not_null_cnt_stmt,rt_null_cnt_stmt),
    " )",
    " SELECT v.id, v.value, lt.cnt, rt.cnt",
    " FROM app_dimension_values v",
    " INNER JOIN lt ON v.id=lt.id",
    " INNER JOIN rt ON v.id=rt.id",
    " WHERE v.dimension_id=",in_app_dimension_id," AND v.value LIKE CONCAT('%', '",in_filter_string,"', '%') AND",
          " EXISTS (",
              " SELECT 1",
              " FROM ",table_name_string," t",
              " WHERE t.system_id=",in_system_id," AND t.",id_column_name_string,"=v.id",
              time_comparison_string,
              " LIMIT 1",
          " );"
);
-- SELECT select_stmt;
CALL tpf_execute_immediate(select_stmt);
-- SET select_stmt = CONCAT(" SELECT * FROM `",out_table_name,"`;");
-- CALL tpf_execute_immediate(select_stmt);


END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_allocate_aggregate_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_allocate_aggregate_table` (
    IN `in_table_name` VARCHAR(100)
)
tpf_allocate_aggregate_table:BEGIN

DECLARE select_stmt BLOB;

-- ## This is a table of results returned by the procedure
-- ## tpf_metric_aggregator_lib which we'll use to do some work for us.
SET select_stmt = CONCAT(
   " DROP TEMPORARY TABLE IF EXISTS ",in_table_name,";"
);
CALL tpf_execute_immediate(select_stmt);

SET select_stmt = CONCAT(
   " CREATE TEMPORARY TABLE `",in_table_name,"` ("
   "     `time` DATETIME(3) ,"
   "     `secs_per_interval` INT(10) UNSIGNED ZEROFILL NULL DEFAULT NULL,"
   "     `search_id` INT ,"
   "     `search_string` VARCHAR(500) NULL DEFAULT NULL,"
   "     `system_metric` DOUBLE NULL DEFAULT NULL,"
   "     `avg_nvp_scale_factor` FLOAT UNSIGNED NULL DEFAULT NULL,"
   "     `app_metric` DOUBLE NULL DEFAULT NULL,"
   "     `app_metric_total` DOUBLE NULL DEFAULT NULL,"
   "     `windowed_scale_factor` DOUBLE UNSIGNED NULL DEFAULT NULL,"
   "     `windowed_app_metric` DOUBLE NULL DEFAULT NULL,"
   "     `windowed_total_app_metric` DOUBLE NULL DEFAULT NULL,"
   "     `windowed_cret_metric` DOUBLE NULL DEFAULT NULL,"
   "     PRIMARY KEY (`time`,`search_id`),"
   "     INDEX `search_id_index` (`search_id`)"
   " )"
   " ENGINE=MEMORY;"
);
CALL tpf_execute_immediate(select_stmt);


END//
DELIMITER ;

DROP PROCEDURE IF EXISTS `tpf_allocate_correlation_table`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_allocate_correlation_table` (
    IN `in_table_name` VARCHAR(100)
)
tpf_allocate_correlation_table:BEGIN

DECLARE select_stmt BLOB;

-- ## This is a table of results returned by the procedure
-- ## tpf_metric_aggregator_lib which we'll use to do some work for us.
SET select_stmt = CONCAT(
   " DROP TEMPORARY TABLE IF EXISTS ",in_table_name,";"
);
CALL tpf_execute_immediate(select_stmt);

SET select_stmt = CONCAT(
   " CREATE TEMPORARY TABLE `",in_table_name,"` ("
   "     `search_string` VARCHAR(500) NOT NULL,"
   "     `corr_coef` DOUBLE NULL,"
   "     `inadequate_data_across_intervals` INT NULL,"
   "     `metric` DOUBLE NULL,"
   "     `additional_metric` DOUBLE NULL,"
   "     PRIMARY KEY (`search_string`)"
   " )"
   " ENGINE=MEMORY;"
);
CALL tpf_execute_immediate(select_stmt);


END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_calculation_correlation`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_calculation_correlation` (
    IN `in_search_table_name` VARCHAR(100),
    IN `in_aggregate_table_name` VARCHAR(100),
    IN `in_correlation_table_name` VARCHAR(100),
	IN `in_calc_norm_metrics` BOOLEAN,
	IN `in_show_metric_as_percent_of_total` BOOLEAN,
	IN `in_corr_min_interval` INT,
	IN `in_cnt_app_intervals_with_data_column_name` TEXT, -- column name in search_table
	IN `in_system_metrics_cnt_intervals_without_data` INT,
	IN `in_num_intervals` INT
)
tpf_calculation_correlation:BEGIN

DECLARE select_stmt BLOB;

DECLARE app_metric_mean TEXT;
DECLARE app_metric_stddev TEXT;
DECLARE system_app_product_avg TEXT;
DECLARE system_metric_mean TEXT;
DECLARE system_metric_stddev TEXT;
DECLARE minimum_variation_required TEXT;

SET minimum_variation_required="0.05";

IF in_calc_norm_metrics = TRUE
THEN
   SET app_metric_mean="AVG(a.windowed_scale_factor*a.windowed_app_metric)";
   SET app_metric_stddev="STDDEV(a.windowed_scale_factor*a.windowed_app_metric)";
   SET system_app_product_avg="AVG(a.system_metric*a.windowed_scale_factor*a.windowed_app_metric)";
ELSE
   SET app_metric_mean="AVG(a.app_metric)";
   SET app_metric_stddev="STDDEV(a.app_metric)";
   SET system_app_product_avg="AVG(a.system_metric*a.app_metric)";
END IF;
SET system_metric_mean="AVG(a.system_metric)";
SET system_metric_stddev="STDDEV(a.system_metric)";

DROP TEMPORARY TABLE IF EXISTS tpf_corr_most_recent_metric_table;
CREATE TEMPORARY TABLE tpf_corr_most_recent_metric_table (
    `search_id` INT NULL DEFAULT NULL,
    `metric` DOUBLE NULL DEFAULT NULL,
    `additional_metric` DOUBLE NULL DEFAULT NULL,
    INDEX `search_id_index` (`search_id`)
)
ENGINE=InnoDB;
SET select_stmt = CONCAT(
" INSERT INTO tpf_corr_most_recent_metric_table (search_id,metric,additional_metric)",
" SELECT DISTINCT a.search_id,",
   " FIRST_VALUE( CASE "
   "      WHEN ",in_calc_norm_metrics,"=TRUE AND ",in_show_metric_as_percent_of_total,"=TRUE THEN IF(a.windowed_total_app_metric=0,0,(a.windowed_app_metric)/(a.windowed_total_app_metric)*100)"
   "      WHEN ",in_calc_norm_metrics,"=TRUE AND ",in_show_metric_as_percent_of_total,"=FALSE THEN (a.windowed_app_metric*a.windowed_scale_factor)"
   "      WHEN ",in_calc_norm_metrics,"=FALSE AND ",in_show_metric_as_percent_of_total,"=TRUE THEN IF(a.app_metric_total=0,0,(a.app_metric)/(a.app_metric_total)*100)"
   "      WHEN ",in_calc_norm_metrics,"=FALSE AND ",in_show_metric_as_percent_of_total,"=FALSE THEN (a.app_metric)"
   " END) OVER (PARTITION BY a.search_id ORDER BY a.TIME DESC),"
   "FIRST_VALUE(a.windowed_app_metric) OVER (PARTITION BY a.search_id ORDER BY a.TIME DESC)"
" FROM ",in_aggregate_table_name," a;"
);
-- SELECT select_stmt;
CALL tpf_execute_immediate(select_stmt);

SET select_stmt = CONCAT(
   " INSERT INTO ",in_correlation_table_name," (search_string,corr_coef,inadequate_data_across_intervals,metric,additional_metric)",
   " SELECT a.search_string,",
-- Calculate the correlation coefficient
   " CASE "
   "      WHEN (",in_num_intervals,"-",in_system_metrics_cnt_intervals_without_data,")<",in_corr_min_interval," THEN 0"
   -- We use MAX on in_cnt_app_intervals_with_data_column_name due to the group by below, but it doesn't really matter since they are the same value for a given search_id
   "      WHEN (MAX(s.",in_cnt_app_intervals_with_data_column_name,"))<",in_corr_min_interval," THEN 0"
   "      WHEN ",app_metric_mean," IS NULL OR ",system_metric_mean," IS NULL THEN 0"
   "      WHEN ",app_metric_mean,"=0 OR ",system_metric_mean,"=0 THEN 0"
   "      WHEN ",app_metric_stddev,"/",app_metric_mean,"<",minimum_variation_required," OR ",system_metric_stddev,"/",system_metric_mean,"<",minimum_variation_required," THEN 0"
   "      ELSE (",system_app_product_avg,"-(",app_metric_mean,"*",system_metric_mean,"))/(",app_metric_stddev,"*",system_metric_stddev,")"
   " END,"
-- ## Set up our indicators for if too many intervals don't have enough data points
-- ## for a reliable correlation calculation
-- ## 0 will be translated as - with no highlighting
-- ## 1 will be translated as SYSTEM with red highlighting
-- ## 2 will be translated as NVPC with red highlighting
-- ## 3 will be translated as VARIATION with red highlighting
   " CASE "
   "      WHEN (",in_num_intervals,"-",in_system_metrics_cnt_intervals_without_data,")<",in_corr_min_interval," THEN 1"
   -- We use MAX on in_cnt_app_intervals_with_data_column_name due to the group by below, but it doesn't really matter since they are the same value for a given search_id
   "      WHEN (MAX(s.",in_cnt_app_intervals_with_data_column_name,"))<",in_corr_min_interval," THEN 2"
   "      WHEN ",system_metric_mean," IS NULL OR ",system_metric_mean,"=0 THEN 1"
   "      WHEN ",app_metric_mean," IS NULL OR ",app_metric_mean,"=0 THEN 2"
   "      WHEN ",app_metric_stddev,"/",app_metric_mean,"<",minimum_variation_required," OR ",system_metric_stddev,"/",system_metric_mean,"<",minimum_variation_required," THEN 3"
   "      ELSE 0"
   " END,"
   " r.metric,r.additional_metric"
   " FROM ",in_aggregate_table_name," a"
   " INNER JOIN `",in_search_table_name,"` s ON a.search_id=s.search_id"
   " INNER JOIN tpf_corr_most_recent_metric_table r ON a.search_id=r.search_id"
   " GROUP BY a.search_id;"
);

-- SELECT select_stmt;
CALL tpf_execute_immediate(select_stmt);
-- CALL tpf_execute_immediate(CONCAT("SELECT * FROM ",in_correlation_table_name,";"));


END//
DELIMITER ;


DROP PROCEDURE IF EXISTS `tpf_periodic_cleanup`;

DELIMITER //
CREATE DEFINER=`root`@`%` PROCEDURE `tpf_periodic_cleanup`()
tpf_periodic_cleanup:BEGIN

DECLARE select_stmt BLOB;
DECLARE table_string_array TEXT DEFAULT "";
DECLARE temp_systems_table_string_array TEXT DEFAULT "";
DECLARE array_of_table_name_index INT;
DECLARE search_table_name TEXT;

-- --------------------------------------------------------------------------------------
-- DROP the temporary search tables used by correlation and stats dashboards
-- --------------------------------------------------------------------------------------

SELECT JSON_ARRAYAGG(t.TABLE_NAME)
INTO table_string_array
FROM information_schema.TABLES t
WHERE t.TABLE_NAME LIKE "tpf_search_table_%" AND t.CREATE_TIME < DATE_SUB(NOW(),INTERVAL 1 MINUTE);

SELECT JSON_ARRAYAGG(t.TABLE_NAME)
INTO temp_systems_table_string_array
FROM information_schema.TABLES t
WHERE t.TABLE_NAME LIKE "tpf_temp_selected_systems_%" AND t.CREATE_TIME < DATE_SUB(NOW(),INTERVAL 1 MINUTE);

-- Merge the tpf_temp_selected_systems_ to the list of tpf_search_table_ names, if there are any.
-- If there are no items in the list of tpf_search_table_ names, set the
-- table string array to the list in temp_systems_table_string_array.
IF table_string_array IS NULL OR  JSON_LENGTH(table_string_array) = 0
THEN
    SET table_string_array = temp_systems_table_string_array;
ELSEIF temp_systems_table_string_array IS NOT NULL AND
      JSON_LENGTH(temp_systems_table_string_array) > 0
THEN
    SET table_string_array = JSON_MERGE(table_string_array,temp_systems_table_string_array);
END IF;

IF table_string_array IS NOT NULL AND JSON_LENGTH(table_string_array) > 0
THEN
   -- loop through the search table names to drop
   SET array_of_table_name_index = 0;
   WHILE array_of_table_name_index < JSON_LENGTH(table_string_array) DO

      SET search_table_name = JSON_UNQUOTE(JSON_EXTRACT(table_string_array,CONCAT('$[',array_of_table_name_index,']')));
      SET array_of_table_name_index = array_of_table_name_index + 1;
      IF search_table_name IS NOT NULL AND search_table_name!=""
      THEN
         SET select_stmt = CONCAT(" DROP TABLE IF EXISTS `",search_table_name,"`;");
         CALL tpf_execute_immediate(select_stmt);
      END IF;
   END WHILE;
END IF;


END//
DELIMITER ;

/*!40101 SET SQL_MODE=IFNULL(@OLD_SQL_MODE, '') */;
/*!40014 SET FOREIGN_KEY_CHECKS=IF(@OLD_FOREIGN_KEY_CHECKS IS NULL, 1, @OLD_FOREIGN_KEY_CHECKS) */;
/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;
