#!/bin/bash
#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2019, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
#---START OF HISTORY--------------------------------------------------#
#   DATE        DESCRIPTION                                           #
#   12182019    Initial implementation of z/TPF Realtime Insights     #
#               Dashboard Starter Kit.                                #
#   03242021    Updated for PJ46275 - JVM Monitor Support.            #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20230719    PJ46982 - z/TPF real-time insights dashboard starter  #
#                         kit installation on Linux on IBM Z          #
#   20230719    PJ46904 - User-Defined Metrics                        #
#   20231004    PJ47156 - prepare config script does not work in all  #
#                         customer environments                       #
#   20240613    PJ47253 - starter kit improvements                    #
#---END OF HISTORY----------------------------------------------------#
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

#delete our topics
declare -a topics=("TPF_CDC_DATA" "TPF_NVPC_DATA" "TPF_NVPC_VERT_DATA" "TPF_JVM_DATA" "TPF_UDM_DATA")

SCRIPT_OR_EXE=$(docker exec tpf-kafka-broker ls /home/kafka/bin/kafka-configs.sh 2>/dev/null)

# Loop through all topics
for j in "${topics[@]}"
do
      # If Apache Kafka - Linux on IBM Z
      if [ "$SCRIPT_OR_EXE" != "" ]; then
         docker exec -i tpf-kafka-broker /home/kafka/bin/kafka-topics.sh --delete --bootstrap-server localhost:9092 --topic $j
      else # Confluent Kafka - x86
         docker exec -i tpf-kafka-broker /usr/bin/kafka-topics --delete --bootstrap-server localhost:9092 --topic $j
      fi
done

