#!/bin/bash
#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2019, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
#---START OF HISTORY--------------------------------------------------#
#   DATE        DESCRIPTION                                           #
#   12182019    Initial implementation of z/TPF Realtime Insights     #
#               Dashboard Starter Kit.                                #
#   12112020    Updated for PJ45657 and PJ46295.                      #
#   03242021    Updated for PJ46275 - JVM Monitor Support.            #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20230307    PJ47008 - Runtime Metrics Collection Kafka Encryption #
#                         Support                                     #
#   20230719    PJ46982 - z/TPF real-time insights dashboard starter  #
#                         kit installation on Linux on IBM Z          #
#   20230719    PJ46904 - User-Defined Metrics                        #
#   20231004    PJ47156 - prepare config script does not work in all  #
#                         customer environments                       #
#   20240613    PJ47253 - starter kit improvements                    #
#---END OF HISTORY----------------------------------------------------#
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

if [ $# -lt 1 ]; then
   echo ""
   echo "ERROR: Missing input parameter(s)!"
   echo "   tpf_modify_kafka_topics.sh your.rtmc.server.com:9093"
   echo "   where your.rtmc.server.com:9093 will be found in tpf_data_sci/user_files/kafka_hosts.yml"
   exit 1
fi

SCRIPT_OR_EXE=$(docker exec tpf-kafka-broker ls /home/kafka/bin/kafka-configs.sh 2>/dev/null)

KAFKA_HOST_FILE="../user_files/kafka_hosts.yml"
KAFKA_HOST_GET_UTIL="python3 ./tpf_utility_scripts/tpf_kafka_hosts_get_utility.py $KAFKA_HOST_FILE"
MODIFY_SETTINGS=`$KAFKA_HOST_GET_UTIL modify_script_variables $1`

if [ "$MODIFY_SETTINGS" == "NOT_FOUND" ]; then
      echo "ERROR: Line number ${LINENO}"
      echo "   The specified server was not found in tpf_data_sci/user_files/kafka_hosts.yml: $1"
      exit 1
fi

MODIFY_SETTINGS=`sed "s/[{}{\' ]//g" <<< $MODIFY_SETTINGS`
MODIFY_SETTINGS=`sed 's/[,]/ /g' <<< $MODIFY_SETTINGS`
MODIFY_SETTINGS=`sed 's/[:]/=/g' <<< $MODIFY_SETTINGS`
declare -a topics=("TPF_CDC_DATA" "TPF_NVPC_DATA" "TPF_NVPC_VERT_DATA" "TPF_JVM_DATA" "TPF_UDM_DATA")
for i in "${topics[@]}"
do
   for j in $MODIFY_SETTINGS
   do
      
      echo "MODIFY topic: $i config: $j"
      
      # If Apache Kafka - Linux on IBM Z
      if [ "$SCRIPT_OR_EXE" != "" ]; then
         docker exec -i tpf-kafka-broker /home/kafka/bin/kafka-configs.sh --bootstrap-server localhost:9092 --alter --entity-type topics --entity-name $i --add-config $j
      else # Confluent Kafka - x86
         docker exec -i tpf-kafka-broker /usr/bin/kafka-configs --bootstrap-server localhost:9092 --alter --entity-type topics --entity-name $i --add-config $j
      fi
   done
done

#display topics are create and have been configured correctly
for i in "${topics[@]}"
do
      # If Apache Kafka - Linux on IBM Z
      if [ "$SCRIPT_OR_EXE" != "" ]; then
         docker exec -i tpf-kafka-broker /home/kafka/bin/kafka-configs.sh --bootstrap-server localhost:9092 --describe --topic $i
      else # Confluent Kafka - x86
         docker exec -i tpf-kafka-broker /usr/bin/kafka-topics --bootstrap-server localhost:9092 --describe --topic $i
      fi
done
