#!/bin/bash
#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2023                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
#---START OF HISTORY--------------------------------------------------#
#   DATE        DESCRIPTION                                           #
#   03242021    Updated for PJ46275 - JVM Monitor Support.            #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20230307    PJ47008 - Runtime Metrics Collection Kafka Encryption #
#                         Support                                     #
#   20230719    PJ46982 - z/TPF real-time insights dashboard starter  #
#                         kit installation on Linux on IBM Z          #
#   20230719    PJ46904 - User-Defined Metrics                        #
#   20231004    PJ47156 - prepare config script does not work in all  #
#                         customer environments                       #
#---END OF HISTORY----------------------------------------------------#
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
disp_help() {
   echo "Syntax:"
   echo "   tpf_monitor_kafka_topic.sh JVM/CDC/NVP/VERT/UDM"
}

if [ ! $# -eq 1 ]; then
   echo "Missing input parameter!"
   disp_help
   exit 1
fi

TOPIC=""
case $1 in
   JVM|jvm)
      TOPIC="TPF_JVM_DATA"
      ;;
   CDC|cdc)
      TOPIC="TPF_CDC_DATA"
      ;;
   NVP|nvp|NVPC|nvpc)
      TOPIC="TPF_NVPC_DATA"
      ;;
   VERT|vert)
      TOPIC="TPF_NVPC_VERT_DATA"
      ;;
   UDM|udm)
      TOPIC="TPF_UDM_DATA"
      ;;
   *)
      echo "INCORRECT input parameter!"
      disp_help
      exit 1
      ;;
esac

SCRIPT_OR_EXE=$(docker exec tpf-kafka-broker ls /home/kafka/bin/kafka-configs.sh 2>/dev/null)
if [ "$SCRIPT_OR_EXE" != "" ]; then
   docker exec -it tpf-kafka-broker /home/kafka/bin/kafka-console-consumer.sh --bootstrap-server localhost:9092 --topic $TOPIC --from-beginning
else
   docker exec -it tpf-kafka-zookeeper /usr/bin/kafka-console-consumer --bootstrap-server tpf-kafka-broker:9092 --topic $TOPIC --from-beginning
fi
