#!/bin/bash
#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2019, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
#---START OF HISTORY--------------------------------------------------#
#   DATE        DESCRIPTION                                           #
#   12182019    Initial implementation of z/TPF Realtime Insights     #
#               Dashboard Starter Kit.                                #
#   12112020    Updated for PJ45657 and PJ46295.                      #
#   03242021    Updated for PJ46275 - JVM Monitor Support.            #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   12152021    Initial Message Analysis Tool support                 #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   20220930    PJ46876 - improve mysql memory usage                  #
#   20230307    PJ47008 - Runtime Metrics Collection Kafka Encryption #
#                         Support                                     #
#   20230719    PJ46982 - z/TPF real-time insights dashboard starter  #
#                         kit installation on Linux on IBM Z          #
#   20230719    PJ46904 - User-Defined Metrics                        #
#   20231004    PJ47156 - prepare config script does not work in all  #
#                         customer environments                       #
#   20231025    PJ47175 - Specified time zone support                 #
#   20240613    PJ47253 - starter kit improvements                    #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
#   20240730    PJ48072 - MariaDB large temp file issue               #
#---END OF HISTORY----------------------------------------------------#
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

disp_help() {
   echo "Syntax:"
   echo "  tpf_prepare_configurations.sh"
   echo ""
   echo "Settings are pulled from tpf_data_sci/user_files/tpf_prepare_configurations.yml"
   echo ""
}

modify_docker_from_statement() {
   DOCKER_REGISTRY=""
   # if the user specified docker registry, then add a slash at the end for our from statement.
   if [ "$1" != "$DEFAULT_DOCKER_REGISTRY" ]; then
      DOCKER_REGISTRY="$1\/"
   fi

   FROM_DESTINATION="$2"

   # do the substitution
   sed -i "s,FROM.*,FROM $DOCKER_REGISTRY$FROM_DESTINATION," $3
}

modify_docker_user_statement() {
OUT_FILE="$1"
if [ "$IN_USER_ID" != "root" ]; then
   sed -i '/^#RUN groupadd.*/s/#RUN/RUN/' $OUT_FILE
   sed -i '/^#RUN useradd.*/s/#RUN/RUN/' $OUT_FILE
   sed -i '/^#RUN adduser.*/s/#RUN/RUN/' $OUT_FILE
   sed -i '/^#USER.*/s/#USER/USER/' $OUT_FILE
   if [ "$IN_LINUX_PLATFORM" == "x86" ];then
      sed -i "/^RUN groupadd.*/s/RUN groupadd.*/RUN groupadd $IN_USER_ID_GNAME -g $IN_USER_ID_GID; exit 0/" $OUT_FILE
   else
      # on Linux on IBM Z, the group users already existed and so we're ignoring the error
      sed -i "/^RUN groupadd.*/s/RUN groupadd.*/RUN groupadd $IN_USER_ID_GNAME -g $IN_USER_ID_GID; exit 0/" $OUT_FILE
   fi
   sed -i "/^RUN useradd.*/s/RUN useradd.*/RUN useradd $IN_USER_ID -u $IN_USER_ID_UID -g $IN_USER_ID_GID -s \/bin\/bash/" $OUT_FILE
   sed -i "/^RUN adduser.*/s/RUN adduser.*/RUN adduser $IN_USER_ID -u $IN_USER_ID_UID -g $IN_USER_ID_GID -s \/bin\/bash -D/" $OUT_FILE
   sed -i "/^USER.*/s/USER.*/USER $IN_USER_ID/" $OUT_FILE
else
   sed -i '/^RUN groupadd.*/s/RUN/#RUN/' $OUT_FILE
   sed -i '/^RUN useradd.*/s/RUN/#RUN/' $OUT_FILE
   sed -i '/^USER.*/s/USER/#USER/' $OUT_FILE
fi

}

if [ $# -gt 0 ]; then
   echo ""
   echo "ERROR: Specifying any parameters is not valid"
   echo ""
   disp_help
   exit 1
fi

IN_USER_ID="root"
IN_USER_ID_UID=""
IN_USER_ID_GID=""
IN_USER_ID_GNAME=""
IN_RTMC_SERVER=""
IN_ANALYTICS_SERVER=""
IN_LOGGING="no"
IN_DB_TYPE="mariadb"
IN_LINUX_PLATFORM="x86"
IN_USE_TRUSTED_REPOSITORY="no"
DEFAULT_DOCKER_REGISTRY="your.docker.registry.server.com:5000"
IN_DOCKER_REGISTRY="$DEFAULT_DOCKER_REGISTRY"
IN_IMPLEMENT_SAMPLE_PRUNING="no"
IN_CONFIGURE_KAFKA_CONTAINERS="yes"
IN_KAFKA_HOST_SETTING="use"
IN_IMPLEMENT_SAMPLE_UDM="no"
IN_CONFIGURE_KAFKA_CONTAINER_PORT="9093"
IN_HSC_ENCRYPTION_FILES_FOR_RTMC="none"
IN_TIME_ZONE_TEXT="UTC"
DOCKERFILE_replay_script_FROM=""
DOCKERFILE_rtmc_realtime_FROM=""
DOCKERFILE_rtmc_tape_FROM=""
DOCKERFILE_zmatc_analyzer_FROM=""
DOCKERFILE_grafana_FROM=""
DOCKERFILE_kafka_FROM=""
DOCKERFILE_zookeeper_FROM=""
DOCKERFILE_rest_proxy_FROM=""
DOCKERFILE_kafka_ui_FROM=""
DOCKERFILE_mariadb_FROM=""
MARIADB_LD_PRELOAD=""
DOCKERFILE_mysql_FROM=""
DOCKERFILE_zrtmc_analyzer_FROM=""

IN_GRAFANA_ORG_ID=1

readarray rows < ../user_files/tpf_prepare_configurations.yml
for row in "${rows[@]}";do
    row_without_assignment=${row//:/ }
    row_array=(${row_without_assignment})
    case "${row_array[0]}" in
       USER_ID)
          IN_USER_ID="${row_array[1]}";
          IN_USER_ID_UID="$(id -u $IN_USER_ID)"
          IN_USER_ID_GID="$(id -g $IN_USER_ID)"
          IN_USER_ID_GNAME="$(getent group $IN_USER_ID_GID | cut -d: -f1)"
          ;;
       RTMC_SERVER)
          IN_RTMC_SERVER="${row_array[1]}";
          ;;
       ANALYTICS_SERVER)
          IN_ANALYTICS_SERVER="${row_array[1]}";
          ;;
       LOGGING)
          IN_LOGGING="${row_array[1]}";
          if [[ "$IN_LOGGING" != "yes" && "$IN_LOGGING" != "no" ]]; then
             echo ""
             echo "ERROR: Line number ${LINENO}"
             echo "LOGGING parameter specified a value ($IN_LOGGING) that is not valid."
             echo "See help information in tpf_data_sci/user_files/tpf_prepare_configurations.yml"
             echo ""
             exit 1
          fi
          ;;
       DB_TYPE)
          IN_DB_TYPE="${row_array[1]}";
          if [[ "$IN_DB_TYPE" != "mariadb" && "$IN_DB_TYPE" != "mysql" ]]; then
             echo ""
             echo "ERROR: Line number ${LINENO}"
             echo "DB_TYPE parameter specified a value ($IN_DB_TYPE) that is not valid."
             echo "See help information in tpf_data_sci/user_files/tpf_prepare_configurations.yml"
             echo ""
             exit 1
          fi
          ;;
       LINUX_PLATFORM)
          IN_LINUX_PLATFORM="${row_array[1]}";
          if [[ "$IN_LINUX_PLATFORM" != "x86" && "$IN_LINUX_PLATFORM" != "linux_on_ibm_Z" ]]; then
             echo ""
             echo "ERROR: Line number ${LINENO}"
             echo "LINUX_PLATFORM parameter specified a value ($IN_LINUX_PLATFORM) that is not valid."
             echo "See help information in tpf_data_sci/user_files/tpf_prepare_configurations.yml"
             echo ""
             exit 1
          fi
          ;;
       USE_TRUSTED_REPOSITORY)
          IN_USE_TRUSTED_REPOSITORY="${row_array[1]}";
          if [[ "$IN_USE_TRUSTED_REPOSITORY" != "yes" && "$IN_USE_TRUSTED_REPOSITORY" != "no" ]]; then
             echo ""
             echo "ERROR: Line number ${LINENO}"
             echo "USE_TRUSTED_REPOSITORY parameter specified a value ($IN_USE_TRUSTED_REPOSITORY) that is not valid."
             echo "See help information in tpf_data_sci/user_files/tpf_prepare_configurations.yml"
             echo ""
             exit 1
          fi
          ;;
       DOCKER_REGISTRY)
          if [ "${row_array[2]}" != "" ]; then
             IN_DOCKER_REGISTRY="${row_array[1]}:${row_array[2]}";
          else
             IN_DOCKER_REGISTRY="${row_array[1]}";
          fi
          ;;
       IMPLEMENT_SAMPLE_PRUNING)
          IN_IMPLEMENT_SAMPLE_PRUNING="${row_array[1]}";
          if [[ "$IN_IMPLEMENT_SAMPLE_PRUNING" != "yes" && "$IN_IMPLEMENT_SAMPLE_PRUNING" != "no" ]]; then
             echo ""
             echo "ERROR: Line number ${LINENO}"
             echo "IMPLEMENT_SAMPLE_PRUNING parameter specified a value ($IN_IMPLEMENT_SAMPLE_PRUNING) that is not valid."
             echo "See help information in tpf_data_sci/user_files/tpf_prepare_configurations.yml"
             echo ""
             exit 1
          fi
          ;;
       KAFKA_HOST_SETTING)
          IN_KAFKA_HOST_SETTING="${row_array[1]}";
          case "$IN_KAFKA_HOST_SETTING" in
             use|USE)
                IN_KAFKA_HOST_SETTING=use
                ;;
             update|UPDATE)
                IN_KAFKA_HOST_SETTING=update
                ;;
             *)
                echo "ERROR: Line number ${LINENO}"
                echo "KAFKA_HOST_SETTING parameter specified a value ($IN_KAFKA_HOST_SETTING) that is not valid."
                echo "See help information in tpf_data_sci/user_files/tpf_prepare_configurations.yml"
                exit 1
                ;;
          esac
          ;;
       IMPLEMENT_SAMPLE_UDM)
          IN_IMPLEMENT_SAMPLE_UDM="${row_array[1]}";
          if [[ "$IN_IMPLEMENT_SAMPLE_UDM" != "yes" && "$IN_IMPLEMENT_SAMPLE_UDM" != "no" ]]; then
             echo ""
             echo "ERROR: Line number ${LINENO}"
             echo "IMPLEMENT_SAMPLE_UDM parameter specified a value ($IN_IMPLEMENT_SAMPLE_UDM) that is not valid."
             echo "See help information in tpf_data_sci/user_files/tpf_prepare_configurations.yml"
             echo ""
             exit 1
          fi
          ;;
       CONFIGURE_KAFKA_CONTAINERS)
          IN_CONFIGURE_KAFKA_CONTAINERS="${row_array[1]}";
          if [[ "$IN_CONFIGURE_KAFKA_CONTAINERS" != "yes" && "$IN_CONFIGURE_KAFKA_CONTAINERS" != "no" ]]; then
             echo ""
             echo "ERROR: Line number ${LINENO}"
             echo "CONFIGURE_KAFKA_CONTAINERS parameter specified a value ($IN_CONFIGURE_KAFKA_CONTAINERS) that is not valid."
             echo "See help information in tpf_data_sci/user_files/tpf_prepare_configurations.yml"
             echo ""
             exit 1
          fi
          if [[ "$IN_RTMC_SERVER" != `hostname --fqdn` ]]; then
             IN_CONFIGURE_KAFKA_CONTAINERS="no"
             echo "Hostname --fqdn does not match $IN_RTMC_SERVER, CONFIGURE_KAFKA_CONTAINERS is set to no"
          fi
          ;;
       CONFIGURE_KAFKA_CONTAINER_PORT)
          IN_CONFIGURE_KAFKA_CONTAINER_PORT="${row_array[1]}";
          ;;
       HSC_ENCRYPTION_FILES_FOR_RTMC)
          if [[ "${row_array[1]}" != "/home/your_id/encryption_files/" ]]; then
             IN_HSC_ENCRYPTION_FILES_FOR_RTMC="${row_array[1]}"
          fi
          ;;
       TIME_ZONE_TEXT)
          IN_TIME_ZONE_TEXT="${row_array[1]}"
          ;;
       JDK_x86)
          IN_JDK_x86="${row_array[1]}:${row_array[2]}";
          ;;
       JDK_LoIZ)
          IN_JDK_LoIZ="${row_array[1]}:${row_array[2]}";
          ;;
       Grafana_x86)
          IN_Grafana_x86="${row_array[1]}:${row_array[2]}";
          ;;
       Grafana_LoIZ)
          IN_Grafana_LoIZ="${row_array[1]}:${row_array[2]}";
          ;;
       Kafka_x86)
          IN_Kafka_x86="${row_array[1]}:${row_array[2]}";
          ;;
       Kafka_LoIZ)
          IN_Kafka_LoIZ="${row_array[1]}:${row_array[2]}";
          ;;
       Kafka_zookeeper_x86)
          IN_Kafka_zookeeper_x86="${row_array[1]}:${row_array[2]}";
          ;;
       Kafka_rest_proxy_x86)
          IN_Kafka_rest_proxy_x86="${row_array[1]}:${row_array[2]}";
          ;;
       Kafka_ui_x86)
          IN_Kafka_ui_x86="${row_array[1]}:${row_array[2]}";
          ;;
       MariaDB_x86)
          IN_MariaDB_x86="${row_array[1]}:${row_array[2]}";
          ;;
       MaraiDB_LoIZ)
          IN_MaraiDB_LoIZ="${row_array[1]}:${row_array[2]}";
          ;;
       MariaDB_ld_preload_path_x86)
          IN_MariaDB_ld_preload_path_x86="${row_array[1]}";
          ;;
       MariaDB_ld_preload_path_LoIZ)
          IN_MariaDB_ld_preload_path_LoIZ="${row_array[1]}";
          ;;
       MySQL_x86)
          IN_MySQL_x86="${row_array[1]}:${row_array[2]}";
          ;;
       MySQL_LoIZ)
          IN_MySQL_LoIZ="${row_array[1]}:${row_array[2]}";
          ;;
       Python_x86)
          IN_Python_x86="${row_array[1]}:${row_array[2]}";
          ;;
       Python_LoIZ)
          IN_Python_LoIZ="${row_array[1]}:${row_array[2]}";
          ;;
       GRAFANA_ORG_ID)
          IN_GRAFANA_ORG_ID="${row_array[1]}";
          ;;
    esac
done


if [ "$IN_LINUX_PLATFORM" = "x86" ];then
   DOCKERFILE_replay_script_FROM="$IN_JDK_x86"
   DOCKERFILE_rtmc_realtime_FROM="$IN_JDK_x86"
   DOCKERFILE_rtmc_tape_FROM="$IN_JDK_x86"
   DOCKERFILE_zmatc_analyzer_FROM="$IN_JDK_x86"

   DOCKERFILE_grafana_FROM="$IN_Grafana_x86"

   DOCKERFILE_kafka_FROM="$IN_Kafka_x86"
   DOCKERFILE_zookeeper_FROM="$IN_Kafka_zookeeper_x86"
   DOCKERFILE_rest_proxy_FROM="$IN_Kafka_rest_proxy_x86"
   DOCKERFILE_kafka_ui_FROM="$IN_Kafka_ui_x86"

   DOCKERFILE_mariadb_FROM="$IN_MariaDB_x86"
   MARIADB_LD_PRELOAD="$IN_MariaDB_ld_preload_path_x86"

   DOCKERFILE_mysql_FROM="$IN_MySQL_x86"

   DOCKERFILE_zrtmc_analyzer_FROM="$IN_Python_x86"
else
   DOCKERFILE_replay_script_FROM="$IN_JDK_LoIZ"
   DOCKERFILE_rtmc_realtime_FROM="$IN_JDK_LoIZ"
   DOCKERFILE_rtmc_tape_FROM="$IN_JDK_LoIZ"
   DOCKERFILE_zmatc_analyzer_FROM="$IN_JDK_LoIZ"

   DOCKERFILE_grafana_FROM="$IN_Grafana_LoIZ"

   DOCKERFILE_kafka_FROM="$IN_Kafka_LoIZ"

   DOCKERFILE_mariadb_FROM="$IN_MaraiDB_LoIZ"
   MARIADB_LD_PRELOAD="$IN_MariaDB_ld_preload_path_LoIZ"

   DOCKERFILE_mysql_FROM="$IN_MySQL_LoIZ"

   DOCKERFILE_zrtmc_analyzer_FROM="$IN_Python_LoIZ"
fi

if [[ "$IN_RTMC_SERVER" = "" || "$IN_RTMC_SERVER" = "your.rtmc.server.com" ]]; then
   echo ""
   echo "ERROR: Line number ${LINENO}"
   echo "RTMC_SERVER value was not specified."
   echo "See help information in tpf_data_sci/user_files/tpf_prepare_configurations.yml"
   echo ""
   exit 1
fi

if [[ "$IN_ANALYTICS_SERVER" = "" || "$IN_ANALYTICS_SERVER" = "your.analytics.server.com" ]]; then
   echo ""
   echo "ERROR: Line number ${LINENO}"
   echo "ANALYTICS_SERVER value was not specified."
   echo "See help information in tpf_data_sci/user_files/tpf_prepare_configurations.yml"
   echo ""
   exit 1
fi

if [[ "$IN_DB_TYPE" = "mysql" && "$IN_LINUX_PLATFORM" = "linux_on_ibm_Z" ]]; then
   echo ""
   echo "ERROR: Line number ${LINENO}"
   echo "No viable Linux on Z docker container image exists."
   echo "You must build this container yourself or install on bare metal."
   echo "See MySQL installation documentation."
   echo ""
   exit 1
fi

if [[ "$IN_TIME_ZONE_TEXT" = "" ]]; then
   echo ""
   echo "ERROR: Line number ${LINENO}"
   echo " TIME_ZONE must be set."
   echo ""
   exit 1
fi

#######################################################################
#                                                                     #
# Interrogate or modify the kafka_hosts.yml file                      #
#                                                                     #
#######################################################################
SET_UP_KAFKA_ENCRYPTION="no"
PREPARE_SCRIPT_ENCRYPTION_FILES_DIR_LIST=""

KAFKA_HOST_FILE="../user_files/kafka_hosts.yml"
KAFKA_HOST_GET_UTIL="python3 ./tpf_utility_scripts/tpf_kafka_hosts_get_utility.py $KAFKA_HOST_FILE"

KAFKA_HOST_VERSION=`$KAFKA_HOST_GET_UTIL kafka_hosts_version`
if [ "$KAFKA_HOST_VERSION" != 2 ]; then
   echo "ERROR: Line number ${LINENO}"
   echo "   $KAFKA_HOST_FILE is not version 2: $KAFKA_HOST_VERSION"
   exit 1
fi

if [ "$IN_KAFKA_HOST_SETTING" = "update" ]; then
   # create a back up of the original for posterity sake
   if [ ! -f $KAFKA_HOST_FILE.orig ]; then
      cp $KAFKA_HOST_FILE $KAFKA_HOST_FILE.orig
   fi

   # verify we're looking at the expected kafka_hosts.yml file
   #    1 entry for host your.rtmc.server.com with no encryption
   HOST_COUNT=`$KAFKA_HOST_GET_UTIL kafka_hosts_list_count`
   if [ "$HOST_COUNT" != "1" ]; then
      echo "ERROR: Line number ${LINENO}"
      echo "   $KAFKA_HOST_FILE does not have default 1 host: $HOST_COUNT"
      exit 1
   fi
   HOST_LIST=`$KAFKA_HOST_GET_UTIL kafka_hosts_list 0`
   if [[ "$HOST_LIST" != "['your.rtmc.server.com:9093']" ]] &&
      [[ "$HOST_LIST" != "[your.rtmc.server.com:9093]" ]] &&
      [[ "$HOST_LIST" != "['$IN_RTMC_SERVER:$IN_CONFIGURE_KAFKA_CONTAINER_PORT']" ]] &&
      [[ "$HOST_LIST" != "[$IN_RTMC_SERVER:$IN_CONFIGURE_KAFKA_CONTAINER_PORT]" ]]; then
      echo "ERROR: Line number ${LINENO}"
      echo "   $KAFKA_HOST_FILE does not have default host your.rtmc.server.com:9093 or your rtmc server $IN_RTMC_SERVER: $HOST_LIST"
      exit 1
   fi

   # passed our checks, go ahead and update the host to the rtmc server specified
   ./tpf_utility_scripts/tpf_set_yaml_value.sh kafka_hosts.hosts "[$IN_RTMC_SERVER:$IN_CONFIGURE_KAFKA_CONTAINER_PORT]" $KAFKA_HOST_FILE
fi

# build the list of hosts for our different purposes
KAFKA_LISTENERS="PLAINTEXT://0.0.0.0:9092, EXTERNAL://0.0.0.0:$IN_CONFIGURE_KAFKA_CONTAINER_PORT"
KAFKA_SECURITY_PROTOCOL_MAP="PLAINTEXT:PLAINTEXT, EXTERNAL:PLAINTEXT"
KAFKA_HOST_PREFIX=""
KAFKA_SECURITY_PROTOCOL=""
KAFKA_SSL_KEYSTORE_FILENAME=""
KAFKA_SSL_KEYSTORE_CREDENTIALS=""
KAFKA_SSL_KEYSTORE_CREDENTIALS=""
KAFKA_SSL_KEY_CREDENTIALS=""
KAFKA_SSL_KEY_CREDENTIALS=""
KAFKA_SSL_TRUSTSTORE_FILENAME=""
KAFKA_SSL_TRUSTSTORE_CREDENTIALS=""
KAFKA_SSL_TRUSTSTORE_CREDENTIALS=""

HOST_COUNT=`$KAFKA_HOST_GET_UTIL kafka_hosts_list_count`
KAFKA_LISTENER_LIST=""
KAFKA_LISTENER_LIST_NON_SSL=""
KAFKA_HOST_LIST=""
KAFKA_HOST_LIST_NON_SSL=""
KAFKA_HOST_COUNT=0
KAFKA_HOST_COUNT_NON_SSL=0
DUP_HOST_LIST=""
HOST_LIST_ADD_COMMA="false"
HOST_LIST_ADD_COMMA_NON_SSL="false"
LISTENER_LIST_ADD_COMMA="false"
LISTENER_LIST_NON_SSL_ADD_COMMA="false"
LISTENER_LIST_COUNT=0
for (( i=0; i<$HOST_COUNT; i++ ))
do
   TEMP_HOST_LIST=`$KAFKA_HOST_GET_UTIL kafka_hosts_list $i`
   # remove bracket [] and comma characters
   TEMP_HOST_LIST=`sed 's/[][,]/ /g' <<< $TEMP_HOST_LIST`

   TEMP_KAFKA_ENCRYPTION=`$KAFKA_HOST_GET_UTIL encryption $i`
   KAFKA_HOST_PREFIX="EXTERNAL://"

   if [[ "$TEMP_KAFKA_ENCRYPTION" != "" ]]; then
      SET_UP_KAFKA_ENCRYPTION="yes"
      KAFKA_HOST_PREFIX="EXTERNAL_SSL://"
      TEMP_PREPARE_SCRIPT_ENCRYPTION_FILES=`$KAFKA_HOST_GET_UTIL prepare_script_encryption_files $i`
      PREPARE_SCRIPT_ENCRYPTION_FILES_DIR_LIST="$PREPARE_SCRIPT_ENCRYPTION_FILES_DIR_LIST $TEMP_PREPARE_SCRIPT_ENCRYPTION_FILES"


      if [[ "$IN_CONFIGURE_KAFKA_CONTAINERS" == "yes" ]] &&
         [[ "$TEMP_HOST_LIST" == *"$IN_RTMC_SERVER:$IN_CONFIGURE_KAFKA_CONTAINER_PORT"* ]]; then
         SECURITY_SETTINGS_COUNT=`$KAFKA_HOST_GET_UTIL security_settings_count $i`
         if [ "$SECURITY_SETTINGS_COUNT" -gt 1 ]; then
            echo "ERROR: Line number ${LINENO}"
            echo "   CONFIGURE_KAFKA_CONTAINERS is set to yes but there are $SECURITY_SETTINGS_COUNT security_settings."
            echo "   Processing ends in error, please correct the kafka_hosts.yml to have only 1 security setting element"
            echo "   for $IN_RTMC_SERVER:$IN_CONFIGURE_KAFKA_CONTAINER_PORT when using CONFIGURE_KAFKA_CONTAINERS: yes."
            exit 1
         fi

         KAFKA_SECURITY_PROTOCOL=`$KAFKA_HOST_GET_UTIL security_protocol $i`
         KAFKA_SSL_KEYSTORE_FILENAME=`$KAFKA_HOST_GET_UTIL kafka_keystore_file $i`
         KAFKA_SSL_KEYSTORE_CREDENTIALS=`$KAFKA_HOST_GET_UTIL keystore_password_file $i`
         KAFKA_SSL_KEYSTORE_CREDENTIALS=`sed 's/\/.*\///g' <<< $KAFKA_SSL_KEYSTORE_CREDENTIALS`
         KAFKA_SSL_KEY_CREDENTIALS=`$KAFKA_HOST_GET_UTIL key_password_file $i`
         KAFKA_SSL_KEY_CREDENTIALS=`sed 's/\/.*\///g' <<< $KAFKA_SSL_KEY_CREDENTIALS`
         KAFKA_SSL_TRUSTSTORE_FILENAME=`$KAFKA_HOST_GET_UTIL kafka_truststore_file $i`
         KAFKA_SSL_TRUSTSTORE_CREDENTIALS=`$KAFKA_HOST_GET_UTIL truststore_password_file $i`
         KAFKA_SSL_TRUSTSTORE_CREDENTIALS=`sed 's/\/.*\///g' <<< $KAFKA_SSL_TRUSTSTORE_CREDENTIALS`
         KAFKA_LISTENERS="PLAINTEXT://0.0.0.0:9092, EXTERNAL_SSL://0.0.0.0:$IN_CONFIGURE_KAFKA_CONTAINER_PORT"
         KAFKA_SECURITY_PROTOCOL_MAP="PLAINTEXT:PLAINTEXT, EXTERNAL_SSL:SSL"
      fi
   fi

   for HOST in $TEMP_HOST_LIST
   do
      HOST=`sed "s/[ ']//g" <<< $HOST`
      # Is this host already in our list?  Case insenstive compare
      for HOST_IN_DUP_LIST in $DUP_HOST_LIST
      do
         if [ "${HOST,,}" == "${HOST_IN_DUP_LIST,,}" ]; then
            echo "ERROR: Line number ${LINENO}"
            echo "   duplicate host name found: $HOST"
            exit 1
         fi
      done

      if [ "$HOST" == "your.rtmc.server.com:9093" ]; then
            echo "ERROR: Line number ${LINENO}"
            echo "   Host is the default: $HOST"
            echo "   You probably didn't set up $KAFKA_HOST_FILE"
            exit 1
      fi
      DUP_HOST_LIST="$DUP_HOST_LIST $HOST"

      if [ "$TEMP_KAFKA_ENCRYPTION" != "" ]; then
         KAFKA_HOST_COUNT=$((++KAFKA_HOST_COUNT))
         if [ "$HOST_LIST_ADD_COMMA" == "true" ]; then
            KAFKA_HOST_LIST="$KAFKA_HOST_LIST,"
         fi
         KAFKA_HOST_LIST="$KAFKA_HOST_LIST$HOST"
         HOST_LIST_ADD_COMMA="true"

         if [ "$HOST" == "$IN_RTMC_SERVER:$IN_CONFIGURE_KAFKA_CONTAINER_PORT" ]; then
            if [ "$LISTENER_LIST_ADD_COMMA" == "true" ]; then
               KAFKA_LISTENER_LIST="$KAFKA_LISTENER_LIST,"
            fi
            KAFKA_LISTENER_LIST="$KAFKA_LISTENER_LIST$KAFKA_HOST_PREFIX$HOST"
            LISTENER_LIST_ADD_COMMA="true"
            LISTENER_LIST_COUNT=$((++LISTENER_LIST_COUNT))
         fi

      else
         KAFKA_HOST_COUNT_NON_SSL=$((++KAFKA_HOST_COUNT_NON_SSL))
         if [ "$HOST_LIST_ADD_COMMA_NON_SSL" == "true" ]; then
            KAFKA_HOST_LIST_NON_SSL="$KAFKA_HOST_LIST_NON_SSL,"
         fi
         KAFKA_HOST_LIST_NON_SSL="$KAFKA_HOST_LIST_NON_SSL$HOST"
         HOST_LIST_ADD_COMMA_NON_SSL="true"

         if [ "$HOST" == "$IN_RTMC_SERVER:$IN_CONFIGURE_KAFKA_CONTAINER_PORT" ]; then
            if [ "$LISTENER_LIST_NON_SSL_ADD_COMMA" == "true" ]; then
               KAFKA_LISTENER_LIST_NON_SSL="$KAFKA_LISTENER_LIST_NON_SSL,"
            fi
            KAFKA_LISTENER_LIST_NON_SSL="$KAFKA_LISTENER_LIST_NON_SSL$KAFKA_HOST_PREFIX$HOST"
            LISTENER_LIST_NON_SSL_ADD_COMMA="true"
            LISTENER_LIST_COUNT=$((++LISTENER_LIST_COUNT))
         fi

      fi
   done
done
#combine our list of hosts SSL,non-SSL
if [ $KAFKA_HOST_COUNT -eq 0 ]; then
   KAFKA_HOST_COUNT=$KAFKA_HOST_COUNT_NON_SSL
   KAFKA_LISTENER_LIST=$KAFKA_LISTENER_LIST_NON_SSL
   KAFKA_HOST_LIST=$KAFKA_HOST_LIST_NON_SSL
elif [ $KAFKA_HOST_COUNT_NON_SSL -ne 0 ]; then
   KAFKA_HOST_COUNT=$((KAFKA_HOST_COUNT+KAFKA_HOST_COUNT_NON_SSL))
   KAFKA_LISTENER_LIST="$KAFKA_LISTENER_LIST,$KAFKA_LISTENER_LIST_NON_SSL"
   KAFKA_HOST_LIST="$KAFKA_HOST_LIST,$KAFKA_HOST_LIST_NON_SSL"
#else We only had SSL hosts and the lists are already set up
fi
KAFKA_LISTENER_LIST="PLAINTEXT://tpf-kafka-broker:9092,$KAFKA_LISTENER_LIST"

if [ $KAFKA_HOST_COUNT -gt 10 ]; then
   echo "ERROR: Line number ${LINENO}"
   echo "   No more than 10 Kafka hosts can be specified:$KAFKA_HOST_COUNT:$KAFKA_HOST_LIST"
   exit 1
fi

if [[ $LISTENER_LIST_COUNT -ne 1 ]] && [[ "$IN_CONFIGURE_KAFKA_CONTAINERS" == "yes" ]]; then
   echo "ERROR: Line number ${LINENO}"
   echo "   Exactly 1 kafka host can be specified when building kafka:$LISTENER_LIST_COUNT"
   echo "   CONFIGURE_KAFKA_CONTAINERS: $IN_CONFIGURE_KAFKA_CONTAINERS"
   echo "   KAFKA_LISTENER_LIST: $KAFKA_LISTENER_LIST"
   exit 1
fi

#echo "$KAFKA_HOST_COUNT"
#echo "$KAFKA_LISTENER_LIST"
#echo "$KAFKA_HOST_LIST"


#######################################################################
#                                                                     #
# Next, we'll modify the docker compose files for the host name.      #
#                                                                     #
#######################################################################
if [[ "$IN_CONFIGURE_KAFKA_CONTAINERS" == "yes" ]]; then
   rm -f tpf_kafka.yml
   if [ "$IN_LINUX_PLATFORM" = "x86" ]; then
      ln -s tpf_kafka_x86.yml tpf_kafka.yml

      #set the kafka external server name
      sed -i "s/9093:9093/$IN_CONFIGURE_KAFKA_CONTAINER_PORT:$IN_CONFIGURE_KAFKA_CONTAINER_PORT/" tpf_kafka.yml
      ./tpf_utility_scripts/tpf_set_yaml_value.sh services.tpf-kafka-broker.environment.KAFKA_LISTENERS "$KAFKA_LISTENERS" tpf_kafka.yml
      ./tpf_utility_scripts/tpf_set_yaml_value.sh services.tpf-kafka-broker.environment.KAFKA_ADVERTISED_LISTENERS "$KAFKA_LISTENER_LIST" tpf_kafka.yml
      ./tpf_utility_scripts/tpf_set_yaml_value.sh services.tpf-kafka-broker.environment.KAFKA_LISTENER_SECURITY_PROTOCOL_MAP "$KAFKA_SECURITY_PROTOCOL_MAP" tpf_kafka.yml

      #handle encryption
      if [ "$SET_UP_KAFKA_ENCRYPTION" == "yes" ]; then
        mkdir -p ./tpf_kafka_docker_files/encryption_files_zookeeper
        mkdir -p ./tpf_kafka_docker_files/encryption_files_broker
        for DIRECTORY in $PREPARE_SCRIPT_ENCRYPTION_FILES_DIR_LIST
        do
           cp $DIRECTORY/* ./tpf_kafka_docker_files/encryption_files_zookeeper/.
           cp $DIRECTORY/* ./tpf_kafka_docker_files/encryption_files_broker/.
        done

        sed -i '/^#            KAFKA_SSL.*/s/#            KAFKA_SSL/            KAFKA_SSL/' tpf_kafka.yml
        sed -i '/^#            KAFKA_SECURITY.*/s/#            KAFKA_SECURITY/            KAFKA_SECURITY/' tpf_kafka.yml
        ./tpf_utility_scripts/tpf_set_yaml_value.sh services.tpf-kafka-broker.environment.KAFKA_SSL_KEYSTORE_FILENAME "$KAFKA_SSL_KEYSTORE_FILENAME" tpf_kafka.yml
        ./tpf_utility_scripts/tpf_set_yaml_value.sh services.tpf-kafka-broker.environment.KAFKA_SSL_KEYSTORE_CREDENTIALS "$KAFKA_SSL_KEYSTORE_CREDENTIALS" tpf_kafka.yml
        ./tpf_utility_scripts/tpf_set_yaml_value.sh services.tpf-kafka-broker.environment.KAFKA_SSL_KEY_CREDENTIALS "$KAFKA_SSL_KEY_CREDENTIALS" tpf_kafka.yml
        ./tpf_utility_scripts/tpf_set_yaml_value.sh services.tpf-kafka-broker.environment.KAFKA_SSL_TRUSTSTORE_FILENAME "$KAFKA_SSL_TRUSTSTORE_FILENAME" tpf_kafka.yml
        ./tpf_utility_scripts/tpf_set_yaml_value.sh services.tpf-kafka-broker.environment.KAFKA_SSL_TRUSTSTORE_CREDENTIALS "$KAFKA_SSL_TRUSTSTORE_CREDENTIALS" tpf_kafka.yml
        ./tpf_utility_scripts/tpf_set_yaml_value.sh services.tpf-kafka-broker.environment.KAFKA_SECURITY_PROTOCOL "$KAFKA_SECURITY_PROTOCOL" tpf_kafka.yml

      else
        sed -i '/^            KAFKA_SSL.*/s/            KAFKA_SSL/#            KAFKA_SSL/' tpf_kafka.yml
        sed -i '/^            KAFKA_SECURITY.*/s/            KAFKA_SECURITY/#            KAFKA_SECURITY/' tpf_kafka.yml
      fi
   else
      ln -s tpf_kafka_LoIZ.yml tpf_kafka.yml

      mkdir -p ./tpf_kafka_docker_files/volumes/kafka-logs

      #set the kafka external server name
      # '/' characters need to be escaped for sed command
      LOIZ_KAFKA_LISTENERS=${KAFKA_LISTENERS//\//\\\/}
      LOIZ_KAFKA_LISTENER_LIST=${KAFKA_LISTENER_LIST//\//\\\/}
      LOIZ_KAFKA_SECURITY_PROTOCOL_MAP=${KAFKA_SECURITY_PROTOCOL_MAP//\//\\\/}
      sed -i "/^listeners=.*/s/listeners=.*/listeners=$LOIZ_KAFKA_LISTENERS/" ./tpf_kafka_docker_files/server.properties.LoIZ
      sed -i "/^advertised.listeners=.*/s/advertised.listeners=.*/advertised.listeners=$LOIZ_KAFKA_LISTENER_LIST/" ./tpf_kafka_docker_files/server.properties.LoIZ
      sed -i "/^listener.security.protocol.map=.*/s/listener.security.protocol.map=.*/listener.security.protocol.map=$LOIZ_KAFKA_SECURITY_PROTOCOL_MAP/" ./tpf_kafka_docker_files/server.properties.LoIZ

      #handle encryption
      if [ "$SET_UP_KAFKA_ENCRYPTION" == "yes" ]; then
        mkdir -p ./tpf_kafka_docker_files/encryption_files_broker
        for DIRECTORY in $PREPARE_SCRIPT_ENCRYPTION_FILES_DIR_LIST
        do
           cp $DIRECTORY/* ./tpf_kafka_docker_files/encryption_files_broker/.
        done

        sed -i "/^#security.protocol.*/s/#security.protocol/security.protocol/" ./tpf_kafka_docker_files/server.properties.LoIZ
        sed -i "/^security.protocol.*/s/security.protocol.*/security.protocol: SSL /" ./tpf_kafka_docker_files/server.properties.LoIZ

        # extract the file extension and upper case it
        KAFKA_SSL_KEYSTORE_TYPE=${KAFKA_SSL_KEYSTORE_FILENAME##*.}
        KAFKA_SSL_KEYSTORE_TYPE=${KAFKA_SSL_KEYSTORE_TYPE^^}
        sed -i "/^#ssl.keystore.type.*/s/#ssl.keystore.type/ssl.keystore.type/" ./tpf_kafka_docker_files/server.properties.LoIZ
        sed -i "/^ssl.keystore.type.*/s/ssl.keystore.type.*/ssl.keystore.type: $KAFKA_SSL_KEYSTORE_TYPE /" ./tpf_kafka_docker_files/server.properties.LoIZ
        sed -i "/^#ssl.keystore.location.*/s/#ssl.keystore.location/ssl.keystore.location/" ./tpf_kafka_docker_files/server.properties.LoIZ
        sed -i "/^ssl.keystore.location.*/s/ssl.keystore.location.*/ssl.keystore.location: $KAFKA_SSL_KEYSTORE_FILENAME /" ./tpf_kafka_docker_files/server.properties.LoIZ

        # extract the file extension and upper case it
        KAFKA_SSL_TRUSTSTORE_TYPE=${KAFKA_SSL_TRUSTSTORE_FILENAME##*.}
        KAFKA_SSL_TRUSTSTORE_TYPE=${KAFKA_SSL_TRUSTSTORE_TYPE^^}
        sed -i "/^#ssl.truststore.type.*/s/#ssl.truststore.type/ssl.truststore.type/" ./tpf_kafka_docker_files/server.properties.LoIZ
        sed -i "/^ssl.truststore.type.*/s/ssl.truststore.type.*/ssl.truststore.type: $KAFKA_SSL_TRUSTSTORE_TYPE /" ./tpf_kafka_docker_files/server.properties.LoIZ
        sed -i "/^#ssl.truststore.location.*/s/#ssl.truststore.location/ssl.truststore.location/" ./tpf_kafka_docker_files/server.properties.LoIZ
        sed -i "/^ssl.truststore.location.*/s/ssl.truststore.location.*/ssl.truststore.location: $KAFKA_SSL_TRUSTSTORE_FILENAME /" ./tpf_kafka_docker_files/server.properties.LoIZ

        # extract the password from the file (get the file name after the last /)
        readarray -t KAFKA_SSL_KEYSTORE_PASSWORD < $PREPARE_SCRIPT_ENCRYPTION_FILES/${KAFKA_SSL_KEYSTORE_CREDENTIALS##*/}
        sed -i "/^#ssl.keystore.password.*/s/#ssl.keystore.password/ssl.keystore.password/" ./tpf_kafka_docker_files/server.properties.LoIZ
        sed -i "/^ssl.keystore.password.*/s/ssl.keystore.password.*/ssl.keystore.password: ${KAFKA_SSL_KEYSTORE_PASSWORD[0]} /" ./tpf_kafka_docker_files/server.properties.LoIZ

        # extract the password from the file (get the file name after the last /)
        readarray -t KAFKA_SSL_KEY_PASSWORD < $PREPARE_SCRIPT_ENCRYPTION_FILES/${KAFKA_SSL_KEY_CREDENTIALS##*/}
        sed -i "/^#ssl.key.password.*/s/#ssl.key.password/ssl.key.password/" ./tpf_kafka_docker_files/server.properties.LoIZ
        sed -i "/^ssl.key.password.*/s/ssl.key.password.*/ssl.key.password: $KAFKA_SSL_KEY_PASSWORD /" ./tpf_kafka_docker_files/server.properties.LoIZ

        # extract the password from the file (get the file name after the last /)
        readarray -t KAFKA_SSL_TRUSTSTORE_PASSWORD < $PREPARE_SCRIPT_ENCRYPTION_FILES/${KAFKA_SSL_TRUSTSTORE_CREDENTIALS##*/}
        sed -i "/^#ssl.truststore.password.*/s/#ssl.truststore.password/ssl.truststore.password/" ./tpf_kafka_docker_files/server.properties.LoIZ
        sed -i "/^ssl.truststore.password.*/s/ssl.truststore.password.*/ssl.truststore.password: $KAFKA_SSL_TRUSTSTORE_PASSWORD /" ./tpf_kafka_docker_files/server.properties.LoIZ

        # uncomment the copy encryption files line
        sed -i '/^#COPY encryption_files_broker.*/s/#COPY/COPY/' ./tpf_kafka_docker_files/Dockerfile.LoIZ.kafka
      else
        sed -i "/^ssl.keystore.type.*/s/ssl.keystore.type/#ssl.keystore.type/" ./tpf_kafka_docker_files/server.properties.LoIZ
        sed -i "/^ssl.keystore.location.*/s/ssl.keystore.location/#ssl.keystore.location/" ./tpf_kafka_docker_files/server.properties.LoIZ
        sed -i "/^ssl.truststore.type.*/s/ssl.truststore.type/#ssl.truststore.type/" ./tpf_kafka_docker_files/server.properties.LoIZ
        sed -i "/^ssl.truststore.location.*/s/ssl.truststore.location/#ssl.truststore.location/" ./tpf_kafka_docker_files/server.properties.LoIZ
        sed -i "/^ssl.keystore.password.*/s/ssl.keystore.password/#ssl.keystore.password/" ./tpf_kafka_docker_files/server.properties.LoIZ
        sed -i "/^ssl.key.password.*/s/ssl.key.password/#ssl.key.password/" ./tpf_kafka_docker_files/server.properties.LoIZ
        sed -i "/^ssl.truststore.password.*/s/ssl.truststore.password/#ssl.truststore.password/" ./tpf_kafka_docker_files/server.properties.LoIZ

        # comment the copy encryption files line
        sed -i '/^COPY encryption_files_broker.*/s/COPY/#COPY/' ./tpf_kafka_docker_files/Dockerfile.LoIZ.kafka
      fi
   fi
fi
# set the appropriate FROM statement in the Dockerfiles
if [ "$IN_LINUX_PLATFORM" = "x86" ];then
   modify_docker_from_statement $IN_DOCKER_REGISTRY $DOCKERFILE_kafka_FROM ./tpf_kafka_docker_files/Dockerfile.x86.kafka
   modify_docker_from_statement $IN_DOCKER_REGISTRY $DOCKERFILE_zookeeper_FROM ./tpf_kafka_docker_files/Dockerfile.x86.zookeeper
   modify_docker_from_statement $IN_DOCKER_REGISTRY $DOCKERFILE_rest_proxy_FROM ./tpf_kafka_docker_files/Dockerfile.x86.rest_proxy
   modify_docker_from_statement $IN_DOCKER_REGISTRY $DOCKERFILE_kafka_ui_FROM ./tpf_kafka_docker_files/Dockerfile.x86.ui
else
   modify_docker_from_statement $IN_DOCKER_REGISTRY $DOCKERFILE_kafka_FROM ./tpf_kafka_docker_files/Dockerfile.LoIZ.kafka
   # set the user in the docker file so log and data files are not written as root
   modify_docker_user_statement ./tpf_kafka_docker_files/Dockerfile.LoIZ.kafka

   # If local repos are provided, modify the Dockerfile to copy those files in.
   if [ "$IN_USE_TRUSTED_REPOSITORY" == "yes" ]; then
     rm -f ./tpf_kafka_docker_files/local_apt_repository.list
     ln ../user_files/local_apt_repository.list ./tpf_kafka_docker_files/local_apt_repository.list
     sed -i '/#COPY.*local_apt_repository.list*/s/#COPY/COPY/' ./tpf_kafka_docker_files/Dockerfile.LoIZ.kafka
   else
     sed -i '/^COPY.*local_apt_repository.list*/s/^COPY/#COPY/' ./tpf_kafka_docker_files/Dockerfile.LoIZ.kafka
   fi
fi
#######################################################################
#                                                                     #
# Modify the zrtmc analyzer files                                     #
#                                                                     #
#######################################################################

OUT_FILE=./tpf_zrtmc_analyzer_docker_files/profile/tpf_zrtmc_analyzer_profile.yml

if [ -f $OUT_FILE ]; then
   # fix the db server name
   ./tpf_utility_scripts/tpf_set_yaml_value.sh database.host $IN_ANALYTICS_SERVER $OUT_FILE
fi

mkdir -p ./tpf_zrtmc_analyzer_docker_files/kafka_hosts
cp $KAFKA_HOST_FILE ./tpf_zrtmc_analyzer_docker_files/kafka_hosts/.

if [ "$SET_UP_KAFKA_ENCRYPTION" == "yes" ]; then
  mkdir -p ./tpf_zrtmc_analyzer_docker_files/encryption_files
  for DIRECTORY in $PREPARE_SCRIPT_ENCRYPTION_FILES_DIR_LIST
  do
     cp $DIRECTORY/* ./tpf_zrtmc_analyzer_docker_files/encryption_files/.
  done
  sed -i '/^#RUN.*encryption_files/s/#RUN/RUN/' ./tpf_zrtmc_analyzer_docker_files/Dockerfile
  sed -i '/^#COPY.*encryption_files/s/#COPY/COPY/' ./tpf_zrtmc_analyzer_docker_files/Dockerfile
else
   sed -i '/^RUN.*encryption_files/s/RUN/#RUN/' ./tpf_zrtmc_analyzer_docker_files/Dockerfile
   sed -i '/^COPY.*encryption_files/s/COPY/#COPY/' ./tpf_zrtmc_analyzer_docker_files/Dockerfile
fi

# set the appropriate FROM statement in the Dockerfile
OUT_FILE=./tpf_zrtmc_analyzer_docker_files/Dockerfile
modify_docker_from_statement $IN_DOCKER_REGISTRY $DOCKERFILE_zrtmc_analyzer_FROM $OUT_FILE
if [ "$IN_LINUX_PLATFORM" = "x86" ];then
   sed -i '/^#cryptography.*/s/#cryptography/cryptography/' ./tpf_zrtmc_analyzer_docker_files/requirements.txt
   #if the apk line doesn't include cargo, add it back in
   sed -i '/^RUN apk.*cargo/! s/RUN apk.*/& cargo/' ./tpf_zrtmc_analyzer_docker_files/Dockerfile
else
   sed -i '/^cryptography.*/s/cryptography/#cryptography/' ./tpf_zrtmc_analyzer_docker_files/requirements.txt
   sed -i '/^RUN apk.*cargo/s/cargo//' ./tpf_zrtmc_analyzer_docker_files/Dockerfile
fi

# set the user in the docker file so log and data files are not written as root
modify_docker_user_statement $OUT_FILE

# If local repos are provided, modify the Dockerfile to copy those files in.
if [ "$IN_USE_TRUSTED_REPOSITORY" == "yes" ]; then
  rm -f ./tpf_zrtmc_analyzer_docker_files/local_apk_repository
  ln ../user_files/local_apk_repository ./tpf_zrtmc_analyzer_docker_files/local_apk_repository

  rm -f ./tpf_zrtmc_analyzer_docker_files/pip.conf
  ln ../user_files/pip.conf ./tpf_zrtmc_analyzer_docker_files/pip.conf
  sed -i '/#COPY.*\(local_apk_repository\|pip.conf\)/s/#COPY/COPY/' $OUT_FILE
  sed -i 's/RUN apk -U add gcc/RUN apk -U add --repositories-file=\/etc\/apk\/local_apk_repository gcc/' $OUT_FILE
else
  sed -i '/^COPY.*\(local_apk_repository\|pip.conf\)/s/^COPY/#COPY/' $OUT_FILE
  sed -i 's/RUN apk -U add --repositories-file=\/etc\/apk\/local_apk_repository gcc/RUN apk -U add gcc/' $OUT_FILE
fi

#######################################################################
#                                                                     #
# Create the directories and update files for tpfrtmc realtime        #
#                                                                     #
#######################################################################

mkdir -p ./tpf_rtmc_docker_files/volumes/realtime/rtmc-logs

# Set the hosts in the RTMC Properties file
OUT_FILE=./tpf_rtmc_docker_files/RTMCProperties.realtime.yaml
./tpf_utility_scripts/tpf_set_yaml_value.sh database.host $IN_RTMC_SERVER $OUT_FILE
./tpf_utility_scripts/tpf_set_yaml_value.sh daemon.realtime.messageanalysistool.database.host $IN_ANALYTICS_SERVER $OUT_FILE
if [ "$IN_LOGGING" == "yes" ]; then
  sed -i '/#.*\(debug\|level\|filters\)/s/#\+//' $OUT_FILE
else
  sed -i '/^ \+\(debug\|level\|filters\)/s/^ /# /' $OUT_FILE
fi

mkdir -p ./tpf_rtmc_docker_files/kafka_hosts
cp $KAFKA_HOST_FILE ./tpf_rtmc_docker_files/kafka_hosts/.

if [ "$SET_UP_KAFKA_ENCRYPTION" == "yes" ]; then
  mkdir -p ./tpf_rtmc_docker_files/encryption_files
  for DIRECTORY in $PREPARE_SCRIPT_ENCRYPTION_FILES_DIR_LIST
  do
     cp $DIRECTORY/* ./tpf_rtmc_docker_files/encryption_files/.
  done
fi
if [ "$IN_HSC_ENCRYPTION_FILES_FOR_RTMC" != "none" ]; then
  mkdir -p ./tpf_rtmc_docker_files/encryption_files
  cp $IN_HSC_ENCRYPTION_FILES_FOR_RTMC/* ./tpf_rtmc_docker_files/encryption_files/.
fi
if [[ "$SET_UP_KAFKA_ENCRYPTION" == "yes" ]] || [[ "$IN_HSC_ENCRYPTION_FILES_FOR_RTMC" != "none" ]]; then
  sed -i '/^#RUN.*encryption_files/s/#RUN/RUN/' ./tpf_rtmc_docker_files/Dockerfile.realtime
  sed -i '/^#COPY.*encryption_files/s/#COPY/COPY/' ./tpf_rtmc_docker_files/Dockerfile.realtime
else
   sed -i '/^RUN.*encryption_files/s/RUN/#RUN/' ./tpf_rtmc_docker_files/Dockerfile.realtime
   sed -i '/^COPY.*encryption_files/s/COPY/#COPY/' ./tpf_rtmc_docker_files/Dockerfile.realtime
fi


# set the appropriate FROM statement in the Dockerfile
OUT_FILE=./tpf_rtmc_docker_files/Dockerfile.realtime
modify_docker_from_statement $IN_DOCKER_REGISTRY $DOCKERFILE_rtmc_realtime_FROM $OUT_FILE

# set the user in the docker file so log and data files are not written as root
modify_docker_user_statement $OUT_FILE

#######################################################################
#                                                                     #
# Create the directories and update files for replay script           #
#                                                                     #
#######################################################################


mkdir -p ./tpf_replay_script_docker_files/kafka_hosts
cp $KAFKA_HOST_FILE ./tpf_replay_script_docker_files/kafka_hosts/.

if [ "$SET_UP_KAFKA_ENCRYPTION" == "yes" ]; then
  mkdir -p ./tpf_replay_script_docker_files/encryption_files
  for DIRECTORY in $PREPARE_SCRIPT_ENCRYPTION_FILES_DIR_LIST
  do
     cp $DIRECTORY/* ./tpf_replay_script_docker_files/encryption_files/.
  done
fi

if [[ "$SET_UP_KAFKA_ENCRYPTION" == "yes" ]]; then
  sed -i '/^#COPY.*encryption_files/s/#COPY/COPY/' ./tpf_replay_script_docker_files/Dockerfile
else
   sed -i '/^COPY.*encryption_files/s/COPY/#COPY/' ./tpf_replay_script_docker_files/Dockerfile
fi


#######################################################################
#                                                                     #
# Create the directories and update files for tpfrtmc tape            #
#                                                                     #
#######################################################################
mkdir -p ./tpf_rtmc_docker_files/volumes/tape/rtmc-logs
mkdir -p ./tpf_rtmc_docker_files/volumes/tape/binary-tapes
mkdir -p ./tpf_rtmc_docker_files/volumes/tape/transfer-backup-tapes

# Set the host in the RTMC Properties file
OUT_FILE=./tpf_rtmc_docker_files/RTMCProperties.tape.yaml
./tpf_utility_scripts/tpf_set_yaml_value.sh database.host $IN_ANALYTICS_SERVER $OUT_FILE

# set the appropriate FROM statement in the Dockerfile
OUT_FILE=./tpf_rtmc_docker_files/Dockerfile.tape
modify_docker_from_statement $IN_DOCKER_REGISTRY $DOCKERFILE_rtmc_tape_FROM $OUT_FILE

# set the user in the docker file so log and data files are not written as root
modify_docker_user_statement $OUT_FILE

#######################################################################
#                                                                     #
# Create the directories and update files for zmatc analyzer          #
#                                                                     #
#######################################################################
mkdir -p ./tpf_zmatc_analyzer_docker_files/volumes/logs

OUT_FILE=./tpf_zmatc_analyzer_docker_files/tpf_zmatc_properties.yaml

if [ -f $OUT_FILE ]; then
   # fix the db server name
   ./tpf_utility_scripts/tpf_set_yaml_value.sh database.host $IN_ANALYTICS_SERVER $OUT_FILE
   if [ "$IN_LOGGING" == "yes" ]; then
     sed -i '/#.*\(debug\|level\|filters\)/s/#\+//' $OUT_FILE
   else
     sed -i '/^ \+\(debug\|level\|filters\)/s/^ /# /' $OUT_FILE
   fi
fi

# set the appropriate FROM statement in the Dockerfile
OUT_FILE=./tpf_zmatc_analyzer_docker_files/Dockerfile
modify_docker_from_statement $IN_DOCKER_REGISTRY $DOCKERFILE_zmatc_analyzer_FROM $OUT_FILE

# set the user in the docker file so log and data files are not written as root
modify_docker_user_statement $OUT_FILE

#######################################################################
#                                                                     #
# Create the directories and update files for replay scripts          #
#                                                                     #
#######################################################################

# set the appropriate FROM statement in the Dockerfile
OUT_FILE=./tpf_replay_script_docker_files/Dockerfile
modify_docker_from_statement $IN_DOCKER_REGISTRY $DOCKERFILE_replay_script_FROM $OUT_FILE

# set the user in the docker file so log and data files are not written as root
modify_docker_user_statement $OUT_FILE

#######################################################################
#                                                                     #
# Create the directories and update files for db type selected        #
#                                                                     #
#######################################################################
mkdir -p ./tpf_db_docker_files/volume_mysql
mkdir -p ./tpf_db_docker_files/volume_mariadb
if [ "$IN_DB_TYPE" == "mariadb" ]; then

   #set various profiles and properties files to mariadb
   sed -i '/mysql/s/mysql/mariadb/'  ./tpf_rtmc_docker_files/RTMCProperties.realtime.yaml
   sed -i '/mysql/s/mysql/mariadb/'  ./tpf_rtmc_docker_files/RTMCProperties.tape.yaml
   sed -i '/mysql/s/mysql/mariadb/'  ./tpf_zmatc_analyzer_docker_files/tpf_zmatc_properties.yaml
   sed -i '/system/s/mysql/mariadb/' ./tpf_zrtmc_analyzer_docker_files/profile/tpf_zrtmc_analyzer_profile.yml

   # set the appropriate FROM statement in the Dockerfile
   OUT_FILE=./tpf_db_docker_files/Dockerfile.mariadb
   modify_docker_from_statement $IN_DOCKER_REGISTRY $DOCKERFILE_mariadb_FROM $OUT_FILE
   ./tpf_utility_scripts/tpf_set_yaml_value.sh services.tpf-mariadb.environment.LD_PRELOAD $MARIADB_LD_PRELOAD ./tpf_mariadb.yml

   # set the user in the docker file so log and data files are not written as root
   modify_docker_user_statement $OUT_FILE

else

   #set various profiles and properties files to mysql
   sed -i '/mariadb/s/mariadb/mysql/' ./tpf_rtmc_docker_files/RTMCProperties.realtime.yaml
   sed -i '/mariadb/s/mariadb/mysql/' ./tpf_rtmc_docker_files/RTMCProperties.tape.yaml
   sed -i '/mariadb/s/mariadb/mysql/' ./tpf_zmatc_analyzer_docker_files/tpf_zmatc_properties.yaml
   sed -i '/system/s/mariadb/mysql/'  ./tpf_zrtmc_analyzer_docker_files/profile/tpf_zrtmc_analyzer_profile.yml

   # set the appropriate FROM statement in the Dockerfile
   OUT_FILE=./tpf_db_docker_files/Dockerfile.mysql
   modify_docker_from_statement $IN_DOCKER_REGISTRY $DOCKERFILE_mysql_FROM $OUT_FILE

   # set the user in the docker file so log and data files are not written as root
   modify_docker_user_statement $OUT_FILE

   IN_MYSQL_JEMALLOC_REPO=""
   IN_MYSQL_JEMALLOC_RPM=""
   IN_JEMALLOC_LIBRARY_LOCATION=""
   IN_IGNORE_JEMALLOC_CERTIFICATE=""
   readarray rows < ../user_files/mysql_jemalloc_library.yml
   for row in "${rows[@]}";do
       row_without_assignment=${row//:/ }
       row_array=(${row_without_assignment})
       case "${row_array[0]}" in
          MYSQL_JEMALLOC_REPO)
             IN_MYSQL_JEMALLOC_REPO="${row_array[1]}:${row_array[2]}";
             ;;
          MYSQL_JEMALLOC_RPM)
             IN_MYSQL_JEMALLOC_RPM="${row_array[1]}";
             ;;
          JEMALLOC_LIBRARY_LOCATION)
             IN_JEMALLOC_LIBRARY_LOCATION="${row_array[1]}";
             ;;
          IGNORE_JEMALLOC_CERTIFICATE)
             IN_IGNORE_JEMALLOC_CERTIFICATE="${row_array[1]}";
             if [ "$IN_IGNORE_JEMALLOC_CERTIFICATE" == "yes" ];
             then
                IN_IGNORE_JEMALLOC_CERTIFICATE="--no-check-certificate";
             else
                IN_IGNORE_JEMALLOC_CERTIFICATE=""
             fi
             ;;
       esac
   done

   cd ./tpf_db_docker_files/
   wget $IN_IGNORE_JEMALLOC_CERTIFICATE $IN_MYSQL_JEMALLOC_REPO/$IN_MYSQL_JEMALLOC_RPM
   cd ..

   ./tpf_utility_scripts/tpf_set_yaml_value.sh services.tpf-mysql.environment.LD_PRELOAD $IN_JEMALLOC_LIBRARY_LOCATION tpf_mysql.yml
   ./tpf_utility_scripts/tpf_set_yaml_value.sh services.tpf-mysql.build.args.MYSQL_JEMALLOC_RPM $IN_MYSQL_JEMALLOC_RPM tpf_mysql.yml
fi

# manage the sample pruning install actions described in tpf_data_sci/user_files/tpf_prepare_configurations.yml
if [ "$IN_IMPLEMENT_SAMPLE_PRUNING" == "yes" ]; then
   rm -rf ./tpf_db_docker_files/tpf_sample_pruning
   cp -R ../user_files/tpf_sample_pruning ./tpf_db_docker_files/tpf_sample_pruning
   sed -i '/#sh.*tpf_sample_pruning_SQL_install_commands.sh/s/#sh/sh/' ./tpf_db_docker_files/tpf_setup_db.sh
   sed -i '/#COPY tpf_sample_pruning.*/s/#COPY/COPY/' ./tpf_db_docker_files/Dockerfile.mariadb
   sed -i '/#COPY tpf_sample_pruning.*/s/#COPY/COPY/' ./tpf_db_docker_files/Dockerfile.mysql
else
   rm -rf ./tpf_db_docker_files/tpf_sample_pruning
   sed -i '/^sh.*tpf_sample_pruning_SQL_install_commands.sh/s/^sh/#sh/' ./tpf_db_docker_files/tpf_setup_db.sh
   sed -i '/^COPY tpf_sample_pruning.*/s/^COPY/#COPY/' ./tpf_db_docker_files/Dockerfile.mariadb
   sed -i '/^COPY tpf_sample_pruning.*/s/^COPY/#COPY/' ./tpf_db_docker_files/Dockerfile.mysql
fi


#######################################################################
#                                                                     #
# Update files for Grafana                                            #
#                                                                     #
#######################################################################

# modify the database host in the provisioned data source file
OUT_FILE=./tpf_grafana_docker_files/provisioning/datasources/datasource.yaml
if [ -f $OUT_FILE ]; then
   # fix the db server name
   sed -i "/url: .*/s/url: .*/url: $IN_ANALYTICS_SERVER:3306/" $OUT_FILE
fi

# set the appropriate FROM statement in the Dockerfile
OUT_FILE=./tpf_grafana_docker_files/Dockerfile
modify_docker_from_statement $IN_DOCKER_REGISTRY $DOCKERFILE_grafana_FROM $OUT_FILE

# set the grafana org id as specified by the user
sed -i "s/orgId: .*/orgId: $IN_GRAFANA_ORG_ID/" ./tpf_grafana_docker_files/provisioning/dashboards/*
sed -i "s/orgId: .*/orgId: $IN_GRAFANA_ORG_ID/" ./tpf_grafana_docker_files/provisioning/datasources/*
sed -i "s/orgId: .*/orgId: $IN_GRAFANA_ORG_ID/" ../user_files/tpf_sample_user_defined_metrics/grafana/udm.yaml

#######################################################################
#                                                                     #
# Implement changes for User-Defined Metrics Support (UDM)            #
#                                                                     #
#######################################################################
if [ "$IN_IMPLEMENT_SAMPLE_UDM" = "yes" ]; then
   ./tpf_utility_scripts/tpf_set_yaml_value.sh data_sources.udm.active "true" ./tpf_zrtmc_analyzer_docker_files/profile/tpf_zrtmc_analyzer_profile.yml

   if [ ! -f ./tpf_zrtmc_analyzer_docker_files/tpf_user_exit/tpf_zrtmc_analyzer_user_exit.py.orig ]; then
      mv ./tpf_zrtmc_analyzer_docker_files/tpf_user_exit/tpf_zrtmc_analyzer_user_exit.py ./tpf_zrtmc_analyzer_docker_files/tpf_user_exit/tpf_zrtmc_analyzer_user_exit.py.orig
   fi

   rm -rf ./tpf_zrtmc_analyzer_docker_files/tpf_user_exit/tpf_user_*
   rm -rf ./tpf_zrtmc_analyzer_docker_files/tpf_user_exit/udm*

   cp -R ../user_files/tpf_sample_user_defined_metrics/tpf_zrtmc_analyzer/tpf_user_exit/tpf_user_* ./tpf_zrtmc_analyzer_docker_files/tpf_user_exit/.
   cp -R ../user_files/tpf_sample_user_defined_metrics/tpf_zrtmc_analyzer/tpf_user_exit/udm* ./tpf_zrtmc_analyzer_docker_files/tpf_user_exit/.
   cp -R ../user_files/tpf_sample_user_defined_metrics/tpf_zrtmc_analyzer/tpf_user_exit/tpf_zrtmc_analyzer_user_exit.py ./tpf_zrtmc_analyzer_docker_files/tpf_user_exit/.

   rm -rf ./tpf_db_docker_files/tpf_sample_udm
   mkdir -p ./tpf_db_docker_files/tpf_sample_udm
   cp -R ../user_files/tpf_sample_user_defined_metrics/database/* ./tpf_db_docker_files/tpf_sample_udm/.
   sed -i '/#sh.*tpf_sample_udm_SQL_install_commands.sh/s/#sh/sh/' ./tpf_db_docker_files/tpf_setup_db.sh
   sed -i '/#COPY tpf_sample_udm.*/s/#COPY/COPY/' ./tpf_db_docker_files/Dockerfile.mariadb
   sed -i '/#COPY tpf_sample_udm.*/s/#COPY/COPY/' ./tpf_db_docker_files/Dockerfile.mysql

   rm -rf ./tpf_grafana_docker_files/dashboards/udm
   mkdir ./tpf_grafana_docker_files/dashboards/udm
   cp ../user_files/tpf_sample_user_defined_metrics/grafana/tpf_User_Defined_Metrics_Sample.json ./tpf_grafana_docker_files/dashboards/udm/.
   cp ../user_files/tpf_sample_user_defined_metrics/grafana/udm.yaml ./tpf_grafana_docker_files/provisioning/dashboards/.

   ./tpf_utility_scripts/tpf_set_yaml_value.sh daemon.realtime.udm.isActive "true" ./tpf_rtmc_docker_files/RTMCProperties.realtime.yaml
else
   ./tpf_utility_scripts/tpf_set_yaml_value.sh data_sources.udm.active "false" ./tpf_zrtmc_analyzer_docker_files/profile/tpf_zrtmc_analyzer_profile.yml

   if [ -f ./tpf_zrtmc_analyzer_docker_files/tpf_user_exit/tpf_zrtmc_analyzer_user_exit.py.orig ]; then
      rm -f ./tpf_zrtmc_analyzer_docker_files/tpf_user_exit/tpf_zrtmc_analyzer_user_exit.py
      mv ./tpf_zrtmc_analyzer_docker_files/tpf_user_exit/tpf_zrtmc_analyzer_user_exit.py.orig ./tpf_zrtmc_analyzer_docker_files/tpf_user_exit/tpf_zrtmc_analyzer_user_exit.py
   fi

   rm -rf ./tpf_zrtmc_analyzer_docker_files/tpf_user_exit/tpf_user_*
   rm -rf ./tpf_zrtmc_analyzer_docker_files/tpf_user_exit/udm*

   rm -rf ./tpf_db_docker_files/tpf_sample_udm
   sed -i '/^sh.*tpf_sample_udm_SQL_install_commands.sh/s/^sh/#sh/' ./tpf_db_docker_files/tpf_setup_db.sh
   sed -i '/^COPY tpf_sample_udm.*/s/^COPY/#COPY/' ./tpf_db_docker_files/Dockerfile.mariadb
   sed -i '/^COPY tpf_sample_udm.*/s/^COPY/#COPY/' ./tpf_db_docker_files/Dockerfile.mysql

   rm -rf ./tpf_grafana_docker_files/dashboards/udm
   rm -f  ./tpf_grafana_docker_files/provisioning/dashboards/udm.yaml

   ./tpf_utility_scripts/tpf_set_yaml_value.sh daemon.realtime.udm.isActive "false" ./tpf_rtmc_docker_files/RTMCProperties.realtime.yaml
fi

#######################################################################
#                                                                     #
# Set the requested time zone in the configuration files              #
#                                                                     #
#######################################################################

sed -i "s,^timezone.*,timezone = $IN_TIME_ZONE_TEXT," tpf_db_docker_files/*.cnf
sed -i "s,^ENV TZ=.*,ENV TZ=$IN_TIME_ZONE_TEXT," tpf_kafka_docker_files/Dockerfile.LoIZ.kafka

if [ "$IN_TIME_ZONE_TEXT" != "UTC" ]; then
  for JSON_FILE in `find tpf_grafana_docker_files/dashboards * -name *.json`
  do
    sed -i "s,\"timezone\"\:.*,\"timezone\"\: \"$IN_TIME_ZONE_TEXT\"\,," $JSON_FILE
  done
else
  for JSON_FILE in `find tpf_grafana_docker_files/dashboards * -name *.json`
  do
    sed -i "s,\"timezone\"\:.*,\"timezone\"\: \"\"\,," $JSON_FILE
  done
fi

for YMLFILENAME in `ls *.yml`
do
sed -i "s,\(TZ. *\).*,\1$IN_TIME_ZONE_TEXT," $YMLFILENAME
done


#######################################################################
#                                                                     #
# Copy common dashboard framework (CDF) user exits to db fold         #
#                                                                     #
#######################################################################
   rm -rf ./tpf_db_docker_files/tpf_cdf_user_exits
   mkdir -p ./tpf_db_docker_files/tpf_cdf_user_exits
   cp -R ../user_files/tpf_cdf_user_exits/* ./tpf_db_docker_files/tpf_cdf_user_exits/.



#######################################################################
#                                                                     #
# Indicate that processing is done                                    #
#                                                                     #
#######################################################################

   echo ""
   echo "tpf_prepare_configurations.sh processing is complete."
   echo ""

