#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2023                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20230307    PJ47008 - Runtime Metrics Collection Kafka Encryption #
#                         Support                                     #
#   20230719    PJ46982 - z/TPF real-time insights dashboard starter  #
#                         kit installation on Linux on IBM Z          #
#   20230719    PJ46904 - User-Defined Metrics                        #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################


import sys
import yaml

def main(argv,argc):
    try:
        if argc < 3:
            return None

        file_name = argv[1];
        action = argv[2];

        with open(file_name) as f:
            doc = yaml.safe_load(f)

        if action == "kafka_hosts_list_count":
            print(f'{len(doc["kafka_hosts"])}')
            return 0
        elif action == "kafka_hosts_version":
            print(f'{doc["kafkaHostVersion"]}')
            return 0
        elif action == "kafka_hosts_list":
            kafka_hosts_index = argv[3]
            print(f'{doc["kafka_hosts"][int(kafka_hosts_index)]["hosts"]}')
            return 0
        elif action == "modify_script_variables":
            host_found = False
            host = argv[3]
            host_count = len(doc["kafka_hosts"])
            for i in range(host_count):
               if host in doc["kafka_hosts"][i]["hosts"]:
                  host_found = True
                  break;
            if host_found == False:
               print(f'NOT_FOUND')
            else:
               print(f'{doc["kafka_hosts"][i]["modify_script_variables"]}')
            return 0
        elif action == "encryption":
            kafka_hosts_index = argv[3]
            print(f'{doc["kafka_hosts"][int(kafka_hosts_index)]["encryption"]}')
            return 0
        elif action == "prepare_script_encryption_files":
            kafka_hosts_index = argv[3]
            print(f'{doc["kafka_hosts"][int(kafka_hosts_index)]["encryption"]["prepare_script_encryption_files"]}')
            return 0
        elif action == "security_settings_count":
            kafka_hosts_index = argv[3]
            print(f'{len(doc["kafka_hosts"][int(kafka_hosts_index)]["encryption"]["security_settings"])}')
            return 0
        elif action == "kafka_keystore_file":
            kafka_hosts_index = argv[3]
            print(f'{doc["kafka_hosts"][int(kafka_hosts_index)]["encryption"]["security_settings"][0]["kafka_keystore_file"]}')
            return 0
        elif action == "keystore_password_file":
            kafka_hosts_index = argv[3]
            print(f'{doc["kafka_hosts"][int(kafka_hosts_index)]["encryption"]["security_settings"][0]["keystore_password_file"]}')
            return 0
        elif action == "key_password_file":
            kafka_hosts_index = argv[3]
            print(f'{doc["kafka_hosts"][int(kafka_hosts_index)]["encryption"]["security_settings"][0]["key_password_file"]}')
            return 0
        elif action == "kafka_truststore_file":
            kafka_hosts_index = argv[3]
            print(f'{doc["kafka_hosts"][int(kafka_hosts_index)]["encryption"]["security_settings"][0]["kafka_truststore_file"]}')
            return 0
        elif action == "truststore_password_file":
            kafka_hosts_index = argv[3]
            print(f'{doc["kafka_hosts"][int(kafka_hosts_index)]["encryption"]["security_settings"][0]["truststore_password_file"]}')
            return 0
        elif action == "security_protocol":
            kafka_hosts_index = argv[3]
            print(f'{doc["kafka_hosts"][int(kafka_hosts_index)]["encryption"]["security_protocol"]}')
            return 0

    except (OSError, yaml.YAMLError) as e:
        print(f'{e}')
        return None
    except KeyError as e :
        return None

if __name__ == "__main__":
    main(sys.argv,len(sys.argv))
