#!/bin/bash
# *******************************************************************
#      5748-T15
#      Copyright IBM Corp. 2023
# *******************************************************************
# --START OF HISTORY-------------------------------------------------
#   DATE        DESCRIPTION
#   20230719    PJ46982 - z/TPF real-time insights dashboard starter
#                         kit installation on Linux on IBM Z
#   20231004    PJ47156 - prepare config script does not work in all  #
#                         customer environments                       #
# --END OF HISTORY---------------------------------------------------
# *******************************************************************
#
#  GENERAL INFORMATION:
#
#  Disclaimer:   This package is provided on an "as is" basis. There
#                are no warranties, express or implied, including the
#                implied warranties of merchantability and fitness
#                for a particular purpose. IBM has no obligation to
#                provide service, defect correction, or any
#                maintenance for the package.  IBM has no obligation
#                to supply any updates or enhancements for the
#                package to you even if such are or later become
#                available.
#
#  Abstract:     scipt to modify yaml files
#
# *******************************************************************

# This will only set the value for an existing key

# Validate the input and provide help info
if [ ! $# -eq 3 ]; then
   echo "ERROR: Incorrect number of input parameters"
   echo "Syntax:"
   echo "  tpf_set_yaml_value.sh <field> <value> <file>"
   echo "For example:"
   echo "  tpf_set_yaml_value.sh database.host rtmc.server.ibm.com ./tpf_rtmc_docker_files/RTMCProperties.realtime.yaml"
   exit 1
fi

printf "%s\n" "$(
  awk -v FIELD="$1" -v VALUE="$2" '
    BEGIN {num_fields=split(FIELD, fields, "."); nest=1}
    {
      if (match($0, /^\w/)) {
        sections[1]=$1;
        sub(/:/, "", sections[1]);
        indents[1]=1;
        nest=1;
      } else if (match($0, /^[^:]*:\s*$/)) {
        indent=match($0, /[^ ]/);
        if (indent>indents[nest]) {
          nest++;
          sections[nest]=$1;
          sub(/:/, "", sections[nest]);
          indents[nest]=indent;
        } else {
          for (i=nest; i>1; i--) {
            if (indent>indents[i-1]) {
              nest=i;
              sections[nest]=$1;
              sub(/:/, "", sections[nest]);
              indents[nest]=indent;
              break;
            }
          }
        }
      } else {
        key=$1;
        sub(/:/, "", key);
        is_match="false";
        if (key==fields[num_fields] && num_fields==nest+1) {
          is_match="true";
          for (i=1; i<num_fields; i++) {
            if (fields[i] != sections[i]) {
              is_match="false";
            }
          }
        }
        if (is_match=="true") {
          replace=key": "VALUE;
          sub(/[^ ].*/, replace);
        }
      }
      print $0;
    }
    ' "$3"
  )" > "$3"
