#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2023                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20230719    PJ46904 - User-Defined Metrics                        #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from tpf_data_manager.data_source import DataSource


class AnalyzerBatch(object):
    def __init__(self, cdc=[], nvpc_horizontal=[], nvpc_vertical=[], jvm=[], udm=[]):
        """
        sets self.cdc, self.nvpc_vertical, self.nvpc_horizontal, self.jvm
        :param cdc: a list of CdcData objects
        :param nvpc_horizontal: a list of NvpcData objects
        :param nvpc_vertical: a list of NvpcData objects
        :param jvm: a list of JvmData objects
        :return an instance of an AnalyzerBatch object
        """

        self.cdc = cdc
        self.nvpc_horizontal = nvpc_horizontal
        self.nvpc_vertical = nvpc_vertical
        self.jvm = jvm
        self.udm = udm

    def get_analyzer_data(self, data_source: DataSource):
        """
        Get the data from this AnalyzerBatch instance for a particular DataSource.

        :param data_source: a DataSource enum
        :return: list of objects for the given DataSource
        :rtype: List
        """
        if data_source is DataSource.CDC:
            return self.cdc
        if data_source is DataSource.NVPC_HORIZONTAL:
            return self.nvpc_horizontal
        if data_source is DataSource.NVPC_VERTICAL:
            return self.nvpc_vertical
        if data_source is DataSource.JVM:
            return self.jvm
        if data_source is DataSource.UDM:
            return self.udm

    def is_empty(self):
        """
        Check if there is any data present for any of the data sources in the DataSource class.

        :return: bool showing that there is data or no data
        """
        for data_source in DataSource:
            data = self.get_analyzer_data(data_source)
            if data is not None and len(data) > 0:
                return False
        return True
