#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2023                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20230719    PJ46904 - User-Defined Metrics                        #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from tpf_batch.batch import AnalyzerBatch


class AnalyzerBatchProcessor(object):
    def deserialize_batch(self, batch: AnalyzerBatch):
        """
        Override this function in your subclass which implements your custom
        processing logic. process_batch is invoked upon each return from a
        poll to Kafka for data. The starter kit code uses the same interface.
        See tpf_data_manager/zrtmc_manager.py for an example of how to implement
        this function.

        The caller does not handle any exceptions that may be raised by this
        function.

        :param batch: AnalyzerBatch object
        """
        return

    def process_batch(self, batch: AnalyzerBatch):
        """
        Override this function in your subclass which implements your custom
        processing logic. process_batch is invoked upon each return from a
        poll to Kafka for data. The starter kit code uses the same interface.
        See tpf_data_manager/zrtmc_manager.py for an example of how to implement
        this function.

        The caller does not handle any exceptions that may be raised by this
        function.

        :param batch: AnalyzerBatch object
        """
        return
