#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file inserts DASDServiceTime objects to a relational database.#
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from pymysql.cursors import Cursor
from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.cdc_record_types.dasd_service_time import DASDServiceTimes, DASDServiceTime
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert

logger = logging.getLogger('tpf_zrtmc_analyzer')

class DASDServiceTimeHandler(DbDataHandler, CdcDbHelper):

    table_name = 'tpf_dasd_service_time_metrics_table'
    db_field_names = ['time', 'duration', 'system_id', 'ssid', 'lowest_online_sda_on_lss', 'average_service_time']
    db_key_fields = {'time', 'system_id', 'ssid'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    # This def is called from tpf_cdc_manager.py
    def get_mysql_insert_manys(self, db_cursor: Cursor, data: DASDServiceTimes, parent: CdcData = None):

        #logger.info(f'DASDServiceTimeHandler pre_sql_insert: {self.pre_sql_insert}')

        rows = []

        for dasd_service_time in data.cdc_record_array:

            # Arguments must be in order of db_field_names
            row = [dasd_service_time.cdc_record_header.timestamp,
                   dasd_service_time.cdc_record_header.collection_interval_length,
                   self.get_tpf_system_id(parent.header, db_cursor),
                   dasd_service_time.ssid,
                   dasd_service_time.lowest_online_sda_on_lss,
                   dasd_service_time.average_service_time]

            #logger.info(f'DASDServiceTimeHandler row: {row}')

            rows.append(row)

        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]

