#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file writes LODIC_Util objects to a relational database.      #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from pymysql.cursors import Cursor
from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.cdc_record_types.lodic_utilization import LodicUtilizations, LodicUtilization
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_data_manager import id_manager

logger = logging.getLogger('tpf_zrtmc_analyzer')

class LodicUtilizationHandler(DbDataHandler, CdcDbHelper):
    # Local variables to insert into the tpf_lodic_utilization_metrics_table
    table_name = 'tpf_lodic_utilization_metrics_table'
    db_field_names = ['time', 'duration', 'system_id', 'class_name_id', 'ecbs_using_class']
    db_key_fields = {'time', 'system_id', 'class_name_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: LodicUtilizations, parent: CdcData = None):

        rows = []

        for LodicUtilization in data.cdc_record_array:
           # order matters here! it must match with db_field_names
           row = [data.cdc_record_header.timestamp,
              data.cdc_record_header.collection_interval_length,
              self.get_tpf_system_id(parent.header, db_cursor),
              self.get_lodic_utilization_name_id(db_cursor, LodicUtilization.PRIORITY_UTILITY_className),
              LodicUtilization.PRIORITY_UTILITY_CLASS_AMOUNTEcb_USING_CLASS]

           #logger.info(f'LodicUtilizationHandler row: {row}')

        rows.append(row)

        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]

    def get_lodic_utilization_name_id(self, db_cursor: Cursor, name):
        return id_manager.get_lodic_utilization_name_id(name, db_cursor)
