#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2024                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#  This file generates SQL INSERT statements to insert into the       #
#  tpf_system_lists_metrics_table.                                    #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from tpf_cdc.cdc_db_handlers.cdc_db_helper import CdcDbHelper
from tpf_cdc.tpf_cdc_data import CdcData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_utils.db_utils import create_pre_sql_insert
from tpf_cdc.cdc_record_types.system_lists import SystemLists
from tpf_rtmc_db.mysql_handler import DbDataHandler
from pymysql.cursors import Cursor

logger = logging.getLogger('tpf_zrtmc_analyzer')

class SystemListsHandler(DbDataHandler, CdcDbHelper):

    # Local variables to insert the system lists metrics into the tpf_system_lists_metrics_table
    sl_table_name = 'tpf_system_lists_metrics_table'
    sl_db_field_names = ['time', 'duration', 'system_id',
                      'cross_list', 'ready_list', 'input_list', 'deferred_list',
                      'suspend_list', 'vct_list']
    sl_db_key_fields = {'time', 'system_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    sl_pre_sql_insert = create_pre_sql_insert(sl_table_name, sl_db_field_names, sl_db_key_fields)

    sb_table_name = 'tpf_system_blocks_metrics_table'
    sb_db_field_names = ['time', 'system_id',
                         'active_ecbs', 'low_priority_ecbs']
    sb_db_key_fields = {'time', 'system_id'}
    # Generate an SQL INSERT statement for the table, fields, key fields
    sb_pre_sql_insert = create_pre_sql_insert(sb_table_name, sb_db_field_names, sb_db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: SystemLists, parent: CdcData = None):

        insert_manys: List[InsertMany] = []

        # Base version
        v_insert = self.sl_pre_sql_insert

        # ---------------------------------------------------------------------
        # First create the InsertMany statement for the tpf_system_lists_metrics_table
        # ---------------------------------------------------------------------
        sl_rows = []
        # Arguments must be in order of sl_db_field_names
        sl_row = [data.cdc_record_header.timestamp,
               data.cdc_record_header.collection_interval_length,
               self.get_tpf_system_id(parent.header, db_cursor),
               data.cross_list,
               data.ready_list,
               data.input_list,
               data.deferred_list,
               data.suspend_list,
               data.vct_list]

        #logger.info(f'SystemListsHandler system lists row: {sl_row}')

        sl_rows = [sl_row]
        insert_manys.append(InsertMany(self.sl_pre_sql_insert, rows=sl_rows))


        # ---------------------------------------------------------------------
        # Second create the InsertMany statement for the tpf_system_blocks_metrics_table
        # ---------------------------------------------------------------------
        sb_rows = []
        # Arguments must be in order of sb_db_field_names
        sb_row = [data.cdc_record_header.timestamp,
               self.get_tpf_system_id(parent.header, db_cursor),
               data.active_ecbs,
               data.low_priority_ecbs]

        #logger.info(f'SystemListsHandler system blocks row: {sb_row}')

        sb_rows = [sb_row]
        insert_manys.append(InsertMany(self.sb_pre_sql_insert, rows=sb_rows))


        # Finally return the array of InsertMany statements
        return insert_manys

    @classmethod
    def get_table_names(cls):
        return [cls.sl_table_name, cls.sb_table_name]
