#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2022                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import logging
from exceptions.json_err import MissingFieldError
from tpf_cdc.tpf_cdc_body import CdcBody
from tpf_cdc.tpf_cdc_header import CdcHeader
from tpf_utils.json_utils import get_missing_keys


logger = logging.getLogger('tpf_zrtmc_analyzer')


class CdcData(object):

    required_fields = ["header", "body"]

    def __init__(self, json_cdc):

        """
        Initializes a CdcData object.

        :param json_cdc: full CDC JSON structure.This data should be composed of "header" and "body" python
        dictionaries.
        :raise MissingFieldError: If the data is missing a required field.
        """

        try:
            self.header = CdcHeader(json_cdc["header"])
            self.body = CdcBody(json_cdc["body"])

        except KeyError:
            log_message = f'Unable to deserialize CDC data. Missing fields: ' \
                          f'{get_missing_keys(json_cdc, self.required_fields)}. Data: {json_cdc}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)
