#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2022                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import logging
from tpf_cdc.tpf_cdc_record_header import CdcRecordHeader

logger = logging.getLogger('tpf_zrtmc_analyzer')


class CdcRecord(object):

    def __init__(self, json_cdc_record_array, cdc_record_header: CdcRecordHeader):

        """
        Initializes a CdcRecord object.

        :param json_cdc_record_array: list of dictionaries, the cdcRecordArray of a CDC record in the recordList.
        :param cdc_record_header: CdcRecordHeader object from CDC JSON structure
        """

        self.cdc_record_header = cdc_record_header
        self.cdc_record_array_raw_json = json_cdc_record_array

        # The following is for backward compatibility of SYSTEM_MESSAGES and SYSTEM_BLOCKS. self.cdc_record_raw_json
        # used to contain the first (and only) item in the cdcRecordArray. Now, a CdcRecord object contains the
        # entire cdcRecordArray, rather than one CdcRecord object for each item in the cdcRecordArray. This changes
        # the way this code interacts with TCP_IP. The above attributes should be used, not self.cdc_record_raw_json

        self.cdc_record_raw_json = None

        if isinstance(json_cdc_record_array, list) and len(json_cdc_record_array) > 0:
            self.cdc_record_raw_json = json_cdc_record_array[0]
        elif isinstance(json_cdc_record_array, dict):
            self.cdc_record_raw_json = json_cdc_record_array
