#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import logging
from exceptions.json_err import MissingFieldError
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')


class CdcRecordHeader(object):

    required_fields = ["timestamp", "collectionIntervalLength", "version", "cdcRecordName"]

    def __init__(self, json_cdc_record_header):

        """
        Initializes a CdcRecordHeader object.

        :param json_cdc_record_header: "cdcRecordHeader" from CDC JSON structure python dictionary.This dictionary
        should be composed of "timestamp", "collection_interval_length", "version", and "cdc_record_name".
        :raise MissingFieldError: If the data is missing a required field.
        """

        try:
            # Use timestamp and collection interval length for any calculations
            self.timestamp = json_cdc_record_header["timestamp"]
            self.collection_interval_length = json_cdc_record_header["collectionIntervalLength"]
            self.version = json_cdc_record_header["version"]
            self.cdc_record_name = json_cdc_record_header["cdcRecordName"]

        except KeyError:
            log_message = f'Unable to deserialize CDC Record Header. Missing fields: ' \
                          f'{get_missing_keys(json_cdc_record_header, self.required_fields)}. Data: {json_cdc_record_header}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)

