#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from exceptions.json_err import MissingFieldError
from tpf_jvm.tpf_jvm_data_type import JvmDataType
from tpf_utils.json_utils import get_missing_keys
import logging

logger = logging.getLogger('tpf_zrtmc_analyzer')


class JvmCpuData(JvmDataType):
    required_fields = ['eventTime', 'systemUse', 'processUse']

    def __init__(self, jvm_data: dict):

        # Sets self.relational_data and self.event_data
        super().__init__(jvm_data)

        try:
            self.event_time = self.event_data['eventTime']
            self.system_use = self.event_data['systemUse']
            self.process_use = self.event_data['processUse']
        except KeyError:
            log_message = f'Unable to deserialize JVM CPU data. Missing fields: ' \
                          f'{get_missing_keys(self.event_data, self.required_fields)}. Data: {self.event_data}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)
