#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20240716    PJ47254 - RTMC CDC Dashboards                         #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from exceptions.json_err import MissingFieldError
from tpf_jvm.tpf_jvm_data_type import JvmDataType
from tpf_utils.json_utils import get_missing_keys
import logging

logger = logging.getLogger('tpf_zrtmc_analyzer')


class JvmGcData(JvmDataType):
    required_fields = ['eventTime', 'latestThreads', 'avgInterval', 'avgPause', 'avgIntervalGlobal', 'avgPauseGlobal',
                       'usedHeapAfterGC', 'heapSize']

# PJ47254 - some of these fields were not present in the data so they were removed from required_fields since they are not used:
# 'tenuredSize', 'freeTenuredSize','nurserySize', 'freeNurserySize', 'tenureAge', 'amountFlipped', 'objectsFlipped', 'freeTenuredSizeBeforeGC', 'freeNurserySizeBeforeGC',
# The following field were removed since they were not used so they don't cause issues in the future
# 'freeHeapAfterGC', 'reason', 'type', 'gcmode', 'loaSize', 'loaFreeSize', 'finalReferencesCleared', 'markTime', 'sweepTime', 'compactTime', 'freeHeapBeforeGC','pauseTime'

    def __init__(self, jvm_data: dict):

        super().__init__(jvm_data)

        try:
            self.event_time = self.event_data['eventTime']
            self.num_jvm_threads_latest = self.event_data['latestThreads']
            self.avg_interval = self.event_data['avgInterval']
            self.avg_pause = self.event_data['avgPause']
            self.avg_interval_global = self.event_data['avgIntervalGlobal']
            self.avg_pause_global = self.event_data['avgPauseGlobal']
            self.used_heap = self.event_data['usedHeapAfterGC']
            self.heap_size = self.event_data['heapSize']

#            self.free_heap_after_gc = self.event_data['freeHeapAfterGC']
#            self.reason = self.event_data['reason']
#            self.type = self.event_data['type']
#            self.gc_mode = self.event_data['gcmode']
#            self.loa_size = self.event_data['loaSize']
#            self.loa_free_size = self.event_data['loaFreeSize']
#            self.final_references_cleared = self.event_data['finalReferencesCleared']
#            self.mark_time = self.event_data['markTime']
#            self.sweep_time = self.event_data['sweepTime']
#            self.compact_time = self.event_data['compactTime']
#            self.free_heap_before_gc = self.event_data['freeHeapBeforeGC']
#            self.pause_time = self.event_data['pauseTime']

#            self.tenured_size = self.event_data['tenuredSize']
#            self.free_tenured_size = self.event_data['freeTenuredSize']
#            self.nursery_size = self.event_data['nurserySize']
#            self.free_nursery_size = self.event_data['freeNurserySize']
#            self.tenure_age = self.event_data['tenureAge']
#            self.amount_flipped = self.event_data['amountFlipped']
#            self.objects_flipped = self.event_data['objectsFlipped']
#            self.free_tenured_size_before_gc = self.event_data['freeTenuredSizeBeforeGC']
#            self.free_nursey_size_before_gc = self.event_data['freeNurserySizeBeforeGC']

        except KeyError:
            log_message = f'Unable to deserialize JVM GC data. Missing fields: ' \
                          f'{get_missing_keys(self.event_data, self.required_fields)}. Data: {self.event_data}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)

