#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021                                      #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from exceptions.json_err import MissingFieldError
from tpf_jvm.tpf_jvm_data_type import JvmDataType
from tpf_utils.json_utils import get_missing_keys
import logging


logger = logging.getLogger('tpf_zrtmc_analyzer')


class JvmLockData(JvmDataType):

    required_fields = ['eventTime', 'lockName', 'percentMiss', 'percentUtilisation', 'totalGets', 'slowAcquire',
                       'recursiveAcquires', 'averageHoldTime', 'tier2Spins', 'tier3Spins', 'javaMonitor']

    def __init__(self, jvm_data: dict):

        super().__init__(jvm_data)

        try:
            # Lock name looks like "  [0000000024492138] JVM_RawMonitor"
            # Split apart, using [0000000024492138] as name and "JVM_RawMonitor" as description
            lock_components = self.event_data['lockName'].strip().split(maxsplit=1)

            self.lock_name = self.event_data['lockName']

            self.lock_addr = lock_components[0]
            if len(lock_components) > 1:
                self.lock_desc = lock_components[1]
            else:
                self.lock_desc = None

            self.event_time = self.event_data['eventTime']
            self.miss_pct = self.event_data['percentMiss']
            self.util_pct = self.event_data['percentUtilisation']
            self.gets = self.event_data['totalGets']
            self.gets_slow = self.event_data['slowAcquire']
            self.gets_rec = self.event_data['recursiveAcquires']
            self.hold_time = self.event_data['averageHoldTime']
            self.tier2_spins = self.event_data['tier2Spins']
            self.tier3_spins = self.event_data['tier3Spins']
            self.java_monitor = self.event_data['javaMonitor']

        except KeyError:
            log_message = f'Unable to deserialize JVM Lock data. Missing fields: ' \
                          f'{get_missing_keys(self.event_data, self.required_fields)}. Data: {self.event_data}'
            logger.error(log_message)
            raise MissingFieldError(log_message)
