#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2022                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from pymysql.cursors import Cursor

from tpf_jvm.jvm_data_types.jvm_cpu_data import JvmCpuData
from tpf_jvm.jvm_db_handlers.jvm_db_helper import JvmDbHelper
from tpf_jvm.tpf_jvm_data import JvmData
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert, ms_to_datetime


class JvmCpuHandler(DbDataHandler, JvmDbHelper):
    table_name = 'jvm_jvm_metrics_table'
    db_field_names = ['time', 'jam_id', 'pid', 'cpu']
    db_key_fields = {'time', 'jam_id', 'pid'}
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, jvm_cpu_data: JvmCpuData, parent: JvmData = None):
        # Arguments must be in order of db_field_names
        rows = [[ms_to_datetime(jvm_cpu_data.event_time),
                 self.get_tpf_jam_id(jvm_cpu_data.relational_data, parent, db_cursor),
                 jvm_cpu_data.relational_data.pid,
                 jvm_cpu_data.process_use]]
        return [InsertMany(self.pre_sql_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]
