#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2022, 2023                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   10152023    PJ47106 - Display multiple Kafka producers            #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
from pymysql.cursors import Cursor

from tpf_data_manager import id_manager
from tpf_jvm.tpf_jvm_data import JvmData
from tpf_jvm.tpf_relational_data import RelationalData


class JvmDbHelper(object):
    jam_db_field_len = 32
    op_id_db_field_len = 128
    app_db_field_len = 128
    lock_db_field_len = 18
    lock_description_db_field_len = 256

    def get_tpf_jam_id(self, relational_data: RelationalData, parent: JvmData, db_cursor: Cursor):
        return id_manager.get_tpf_jam_id(
                   id_manager.get_tpf_system_id(
                       parent.header.complex_name,
                       parent.header.cpc_name,
                       parent.header.system_name,
                       parent.header.processor_id,
                       parent.header.ip_address,
                       parent.header.environment,
                       db_cursor
                   ),
                   parent.header.ss_name,
                   relational_data.get_db_group_name(self.jam_db_field_len),
                   relational_data.is_jam_app,
                   db_cursor
               )

    def get_jvm_operation_id(self, op_name, db_cursor: Cursor):
        return id_manager.get_jvm_operation_id(
            op_name[:self.op_id_db_field_len],
            db_cursor
        )

    def get_jmx_producer_id(self, producer_name, db_cursor: Cursor):
        return id_manager.get_jmx_producer_id(
            producer_name,
            db_cursor
        )
