#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2022                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from exceptions.json_err import MissingFieldError
from tpf_jvm.tpf_jvm_body import JvmBody
from tpf_jvm.tpf_jvm_header import JvmHeader
from tpf_utils.json_utils import get_missing_keys
import logging

logger = logging.getLogger('tpf_zrtmc_analyzer')


class JvmData(object):

    required_fields = ['header', 'body']

    def __init__(self, json_jvm):
        """
        Initializes a JvmData object.

        :param json_jvm: full JVM JSON structure. This data should be composed of "header" and "body" python
        dictionaries.
        :raise MissingFieldError: If the data is missing a required field.
        """
        try:
            self.header = JvmHeader(json_jvm['header'])
            self.body = JvmBody(json_jvm['body'], self.header.data_type)
        except KeyError:
            log_message = f'Unable to deserialize JVM data. Missing fields: ' \
                          f'{get_missing_keys(json_jvm, self.required_fields)}. Data: {json_jvm}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)
