#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2023                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   10152023    PJ47106 - Display multiple Kafka producers            #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from exceptions.json_err import MissingFieldError
from tpf_jvm.tpf_relational_data import RelationalData
from tpf_utils.json_utils import get_missing_keys
import logging

logger = logging.getLogger('tpf_zrtmc_analyzer')


class JvmDataType(object):


    required_fields = ["relationalData", "eventData"]
    optional_fields = ["info"]

    def __init__(self, json_jvm_data: dict):
        """
        Initializes a JvmDataType.

        :param json_jvm_data: A dict containing the jvm data.This data should be
        composed of "relationalData" and "eventData".
        :raise MissingFieldError: If the data is missing a required field.
        """

        try:
            self.relational_data = RelationalData(json_jvm_data['relationalData'])
            self.event_data = json_jvm_data['eventData']
        except KeyError:
            log_message = f'Unable to deserialize JVM data type. Missing fields: ' \
                          f'{get_missing_keys(json_jvm_data, self.required_fields)}. Data: {json_jvm_data}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)
        self.info_data = json_jvm_data.get('info')
        if (self.info_data != None):
            self.version_number = self.info_data['version']
        else:
            self.version_number = 1
