#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2023                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20230719    PJ46904 - User-Defined Metrics                        #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
from constants.profile_constants import SECURITY_PROTOCOL, SECURITY_SETTINGS, CA_ROOT_PEM, CA_CERT_PEM, KEY_PASSWORD_PEM, \
    PASSWORD_FOR_KEY_PEM


class KafkaConfiguration(object):
    """
        Handles configuration for kafka settings
    """

    def __init__(self, host, kafka_settings, kafka_encrypt_config):
        # print(host, "\n--", kafka_settings,"\n--", kafka_encrypt_config)
        self.protocol = kafka_settings[SECURITY_PROTOCOL]
        self.encrypted = True
        self.hosts = str(host)
        self.ssl_cafile = kafka_encrypt_config[CA_ROOT_PEM]
        self.ssl_certfile = kafka_encrypt_config[CA_CERT_PEM]
        self.ssl_keyfile = kafka_encrypt_config[KEY_PASSWORD_PEM]
        self.ssl_password_file = kafka_encrypt_config[PASSWORD_FOR_KEY_PEM]

    def __repr__(self):
        return f"Protocol: {self.protocol}\n" \
               f"encrypted: {self.encrypted}\n" \
               f"hosts: {self.hosts}\n" \
               f"ssl_cafile: {self.ssl_cafile}\n" \
               f"ssl_certfile: {self.ssl_certfile}\n" \
               f"ssl_keyfile: {self.ssl_keyfile}\n" \
               f"ssl_password_file: {self.ssl_password_file}\n"
