#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2022                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
from datetime import datetime, timedelta

from pymysql.cursors import Cursor

from tpf_nvpc.nvpc_db_handlers.nvpc_db_helper import NvpcDbHelper
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import time_conversion, default_date_format

system_indicator_procs = ['tpf_modeled_cpu_lib', 'tpf_estimate_system_metrics']


class SystemIndicatorHandler(DbDataHandler, NvpcDbHelper):
    table_name = 'system_metrics_table'

    def __init__(self, proc_name, sys_indicator_dimension):
        self.pre_sql_insert = f'CALL {proc_name}(%s,%s,%s)'
        self.sys_indicator = sys_indicator_dimension
        self.rt_time = datetime.utcnow()
        self.lt_time = self.rt_time - timedelta(minutes=2)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data, parent=None):
        cur_time = time_conversion(data.header.collection_interval_utc_time)

        # Call the proc every ~5 seconds.
        # Overlap time windows (lt_time = rt_time-5sec) in case of a system that is a straggler.
        # Also call the proc for 30 seconds in the past, just for safety. Proc should only update rows if the
        # value has changed
        # Use cur_time+5 to set the new rt_time so we keep up with the data
        if (cur_time - self.rt_time).total_seconds() >= 5:
            proc_params = [
                [
                    self.sys_indicator,
                    self.lt_time.strftime(default_date_format),
                    self.rt_time.strftime(default_date_format)
                ],
                [
                    self.sys_indicator,
                    (self.lt_time - timedelta(seconds=30)).strftime(default_date_format),
                    (self.rt_time - timedelta(seconds=30)).strftime(default_date_format)
                ]
            ]

            self.lt_time = self.rt_time - timedelta(seconds=5)
            self.rt_time = cur_time
            return [InsertMany(self.pre_sql_insert, rows=proc_params)]
        else:
            return []

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]
