#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2022                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
#   10052022    PJ46853 - Change skip factor to fractional scale      #
#                         factor                                      #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import pymysql

from tpf_nvpc.tpf_nvpc_body import NvpcBody
from tpf_nvpc.tpf_nvpc_header import NvpcHeader
import logging
from exceptions.json_err import MissingFieldError
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')


class NvpcData(object):
    required_fields = ['header', 'body']

    def __init__(self, json_nvpc):
        """
        Initialize instance of NvpcData object

        :param json_nvpc: full NVPC JSON structure
        :raise MissingFieldError: if required field is missing from JSON structure
        """

        try:
            self.header = NvpcHeader(json_nvpc['header'])
            self.body = NvpcBody(json_nvpc['body'])
        except KeyError:
            msg = f'Unable to deserialize NVPC data. Missing fields: ' \
                  f'{get_missing_keys(json_nvpc, self.required_fields)}. Data: {json_nvpc}'
            logger.warning(msg)
            raise MissingFieldError(msg)

        self.is_vertical = False

        if len(self.body.record_list) > 0:
            for nvp in self.body.record_list[0].name_value_pairs:
                if nvp.orientation == "vertical":
                    self.is_vertical = True
                    break

    def contains_invalid_negatives(self):
        return (self.header.average_skip_factor is not None and self.header.average_skip_factor < 0) or \
               (self.header.average_scale_factor is not None and self.header.average_scale_factor < 0) or \
               (self.header.inuse_istreams is not None and self.header.inuse_istreams < 0) or \
               (self.header.active_istreams is not None and self.header.active_istreams < 0)
