#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2022                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   20220131    PJ46608 - multi-JVM and system selector support       #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import logging

from exceptions.json_err import MissingFieldError
from tpf_nvpc.tpf_nvpc_namevaluepair import NameValuePair
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')


class NvpcRecord(object):

    required_fields = ["message_count", "metrics", "name_value_pairs"]

    def __init__(self, json_nvpc_record):
        """
        Initialize instance of NvpcRecord object.

        :param json_nvpc_record: one object in the "recordList" array, the raw JSON dictionary
        :raise MissingFieldError: if required field is missing from NVPC JSON structure
        """

        try:
            self.message_count = json_nvpc_record["message_count"]
            self.metrics = json_nvpc_record["metrics"]
            name_value_pair_json = json_nvpc_record["name_value_pairs"]
        except KeyError:
            msg = f'Unable to deserialize NVPC record. Missing fields: ' \
                  f'{get_missing_keys(json_nvpc_record, self.required_fields)}. Data: {json_nvpc_record}'
            logger.warning(msg)
            raise MissingFieldError(msg)

        self.name_value_pairs = [NameValuePair(item) for item in name_value_pair_json]
