#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2023                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20230719    PJ46904 - User-Defined Metrics                        #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import logging
from exceptions.json_err import MissingFieldError
from tpf_utils.json_utils import get_missing_keys, get_or_default_or_error, fix_json_cpc_name

from pymysql.cursors import Cursor
from tpf_data_manager import id_manager

logger = logging.getLogger('tpf_zrtmc_analyzer')


class TPF_UDM_RTMC_Header(object):

    required_fields = ["version", "complex_name", "processor_id","system_name", "cpc_name", "ip_address", "environment", "parameters", "data_type"]


    def __init__(self, rtmc_header):

        """

        :param rtmc_header: "header" python dictionary from JSON structure.
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.version_num = 0

        try:
            self.version = rtmc_header["version"]
            self.version_num = int(float(self.version))
            self.complex_name = rtmc_header["complex_name"]
            self.processor_id = rtmc_header["processor_id"]
            self.system_name = rtmc_header["system_name"]
            self.cpc_name = rtmc_header["cpc_name"]
            self.ip_address = rtmc_header["ip_address"]
            self.environment = rtmc_header["environment"]
            self.data_type = str(rtmc_header["parameters"]["data_type"])
            #NOTE "collection_start_utc_time" is the time that the send API was called on TPF.
            #Do not use collection_start_utc_time.  Instead include a time and collection interval
            #in your UDM data payload in a header.  See tpf_udm_sample_header.py
        except KeyError:
            log_message = f'Unable to deserialize TPF RTMC header. ' \
                          f'Missing fields: {get_missing_keys(rtmc_header, required_fields)}. ' \
                          f'Data: {rtmc_header}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)

    def get_tpf_system_id(self, db_cursor: Cursor):
        return id_manager.get_tpf_system_id(
            self.complex_name,
            self.cpc_name,
            self.system_name,
            self.processor_id,
            self.ip_address,
            self.environment,
            db_cursor
        )
