#######################################################################
#       Licensed Materials - Property of IBM                          #
#       "Restricted Materials of IBM"                                 #
#       5748-T15                                                      #
#       Copyright IBM Corp. 2021, 2024                                #
#       All Rights Reserved                                           #
#       US Government Users Restricted Rights - Use, duplication      #
#       or disclosure restricted by GSA ADP Schedule Contract         #
#       with IBM Corp.                                                #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   08062021    Improve z/TPF real-time insights dashboard starter    #
#               kit resource usage                                    #
#   06162022    PJ46737 - complex-wide dashboard and mysql support    #
#   20230307    PJ47008 - Runtime Metrics Collection Kafka Encryption #
#                         Support                                     #
#   20230719    PJ46904 - User-Defined Metrics                        #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import logging
from typing import List
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')


def parameter_to_section(param, profile_section):
    if profile_section is not None:
        return f'{profile_section}.{param}'
    else:
        return param


def create_missing_parameter_list(missing_parameters, profile_section):
    if profile_section is not None:
        return [parameter_to_section(p, profile_section) for p in missing_parameters]
    else:
        return missing_parameters


def create_missing_profile_parameter_msg(missing_parameters, profile_section):
    missing_parameters = create_missing_parameter_list(missing_parameters, profile_section)
    return f'The following parameters in the RTMC Analytics profile were missing: {missing_parameters}'


def create_invalid_profile_parameter_msg(param, profile_section, invalid_value, valid_constraint):
    return f'Value "{invalid_value}" for "{parameter_to_section(param, profile_section)}" does not meet the ' \
           f'following constraint: {valid_constraint}'


def are_profile_parameters_present(profile_dict, parameters: List, profile_section, log_missing=True, warn_missing=False):
    """
        Process if profile contains the @parameters list of data
    """
    missing_parameters = get_missing_keys(profile_dict, parameters)
    if len(missing_parameters) > 0:
        if log_missing:
            logger.error(create_missing_profile_parameter_msg(missing_parameters, profile_section))
        elif warn_missing:
            logger.warning(create_missing_profile_parameter_msg(missing_parameters, profile_section))
        return False
    else:
        return True
