/*********************************************************************/
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2024                                     */
/*********************************************************************/
/*--START OF HISTORY-------------------------------------------------*/
/*  DATE        DESCRIPTION                                          */
/*  20240716    PJ47254 - RTMC CDC Dashboards                        */
/*--END OF HISTORY---------------------------------------------------*/
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     TPFDF Stored procedures                             */
/*                                                                   */
/*********************************************************************/

-- These procedures depend on ONLY_FULL_GROUP_BY. MySQL and MariaDB always use the sql_mode
-- That was active when the procedures were created, so this permanently sets it for these procs.
SET sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));

-- Set the default character set to latin1 for consistency across MariaDB and MySQL
SET NAMES 'latin1';


DROP FUNCTION IF EXISTS `tpf_get_metric_profiles_available_func`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_metric_profiles_available_func` (
)
RETURNS JSON
DETERMINISTIC
BEGIN

DECLARE result JSON;

SET result = '[
{
    "datatype":"tpfdf",
    "name":"Main",
    "metrics":["dbopn_per_sec","dbred_per_sec", "dbadd_per_sec", "dbdel_per_sec", "dbrep_per_sec", "dbkey_per_sec"],
    "virtual_metrics":[]
},
{
    "datatype":"tpfdf",
    "name":"IO",
    "metrics":["pfind_per_sec","pfile_per_sec", "cfind_per_sec", "cfile_per_sec", "relfc_per_sec", "getfc_per_sec"],
    "virtual_metrics":[]
},
{
    "datatype":"tpfdf",
    "name":"LLR",
    "metrics":["lrfnd_per_sec","lrfil_per_sec"],
    "virtual_metrics":[]
}
]';
RETURN result;

END//
DELIMITER ;


DROP FUNCTION IF EXISTS `tpf_get_available_analysis_func`;

DELIMITER //
CREATE DEFINER=`root`@`%` FUNCTION `tpf_get_available_analysis_func` (
)
RETURNS JSON
DETERMINISTIC
BEGIN

DECLARE result JSON;
SET result = '
[
{
    "datatype":"all",
    "name":"Top [limit] by [analysis metric]",
    "table_procedure":"tpf_common_table",
    "bar_procedure":"tpf_common_bar_chart",
    "line_procedure":"tpf_common_graph",
    "attributes":[]
},
{
    "datatype":"all",
    "name":"[analysis metric] > [user input]",
    "table_procedure":"tpf_user_input_table",
    "bar_procedure":"tpf_user_input_bar_chart",
    "line_procedure":"tpf_user_input_graph",
    "attributes":[]
},
{
    "datatype":"tpfdf",
    "name":"AllFilesOpened",
    "table_procedure":"tpf_df_files_table",
    "bar_procedure":"tpf_df_files_bar_chart",
    "line_procedure":"tpf_df_files_graph",
    "attributes":[]
},
{
    "datatype":"tpfdf",
    "name":"ModifyAPIsCalled",
    "table_procedure":"tpf_df_files_table",
    "bar_procedure":"tpf_df_files_bar_chart",
    "line_procedure":"tpf_df_files_graph",
    "attributes":[]
 },
 {
    "datatype":"tpfdf",
    "name":"NoModifyAPIsCalled",
    "table_procedure":"tpf_df_files_table",
    "bar_procedure":"tpf_df_files_bar_chart",
    "line_procedure":"tpf_df_files_graph",
    "attributes":[]
 }

]';
RETURN result;

END//
DELIMITER ;
