/*********************************************************************/
/*      5748-T15                                                     */
/*      Copyright IBM Corp. 2023                                     */
/*********************************************************************/
/*--START OF HISTORY-------------------------------------------------*/
/*  DATE        DESCRIPTION                                          */
/*  20230719    PJ46982 - z/TPF real-time insights dashboard starter */
/*                         kit installation on Linux on IBM Z        */
/*--END OF HISTORY---------------------------------------------------*/
/*********************************************************************/
/*                                                                   */
/* GENERAL INFORMATION:                                              */
/*                                                                   */
/* Disclaimer:   This package is provided on an "as is" basis. There */
/*               are no warranties, express or implied, including the*/
/*               implied warranties of merchantability and fitness   */
/*               for a particular purpose. IBM has no obligation to  */
/*               provide service, defect correction, or any          */
/*               maintenance for the package.  IBM has no obligation */
/*               to supply any updates or enhancements for the       */
/*               package to you even if such are or later become     */
/*               available.                                          */
/*                                                                   */
/* Abstract:     Stored procedures common to all databases           */
/*                                                                   */
/*********************************************************************/

--  Do everything in UTC time.
SET time_zone = '+00:00';

-- events won't run without event_scheduler ON. This SET command is not persistent across server restarts.
-- Make sure event_scheduler=ON exists in your my.cnf file if you want this on every time you start the database server
SET GLOBAL event_scheduler = ON;

-- remove the event if it exists so we can replace it.
DROP EVENT IF EXISTS tpf_sample_prune_daily;

-- create the event
CREATE EVENT tpf_sample_prune_daily
ON SCHEDULE EVERY 1 DAY
STARTS DATE_ADD(DATE(UTC_TIMESTAMP()), INTERVAL '1 00:01' DAY_MINUTE)  -- Start UTC 12:01 AM of the following day
-- Specify the desired retention periods:
-- Keep 5 days of per second data. Keep days 6 to 10 as per minute data.  Delete all data > 10 days old
DO CALL tpf_sample_prune_data(5, 10);

-- show the event that was created for double check
SHOW EVENTS LIKE 'tpf_sample_prune_daily';

--
