#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2023                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20230719    PJ46904 - User-Defined Metrics                        #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

from tpf_user_exit.tpf_user_data_managers.tpf_user_zrtmc_manager import UserZrtmcManager

def get_analyzer_batch_processor(analyzer_profile):
    """
    Implement this function to return a subclass of AnalyzerBatchProcessor. This function
    gets called in tpf_zrtmc_analyzer.py upon startup of the application. This object
    should contain your custom processing logic. The analyzer_profile parameter is a
    dictionary containing the configuration parameters in tpf_zrtmc_analyzer_profile.yml.
    You can add new sections to this file to pass configuration parameters to your
    AnalyzerBatchProcessor object. Return None from this function if for any reason the
    tpf_zrtmc_analyzer should exit (for example, if there are invalid parameters in the
    configuration profile). The starter kit code uses the same interface. See
    tpf_zrtmc_analyzer_ibm_processing.py for an example of how to implement this function.

    The caller does not handle any exceptions that may be raised by this function.

    The files containing the implementation of your AnalyzerBatchProcessor subclass and any
    other code you add should be placed in tpf_user_exit/ or a subdirectory thereof.

    :param analyzer_profile: Python dictionary version of the tpf_zrtmc_analyzer_profile
    :return: A subclass of AnalyzerBatchProcessor that implements your custom processing
    via the process_batch function, or None if the application should exit.
    :rtype: AnalyzerBatchProcessor
    """
    return UserZrtmcManager.get_instance(analyzer_profile)
