#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2023                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20230719    PJ46904 - User-Defined Metrics                        #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
from pymysql.cursors import Cursor

from tpf_user_exit.udm.tpf_udm_payload import TPF_UDM_Payload
from tpf_user_exit.udm.udm_record_types.qmwr_record_type import QMWREntry,QMWRGroupEntry
from tpf_user_exit.udm.udm_sample_data import UDMSampleData
from tpf_user_exit.udm import udm_id_manager
from tpf_rtmc_db.insert_many import InsertMany
from tpf_rtmc_db.mysql_handler import DbDataHandler
from tpf_utils.db_utils import create_pre_sql_insert


class QMWRHandler(DbDataHandler):
    table_name = 'udm_metrics_table'

    db_field_names = ['time', 'system_id', 'data_type_id', 'group_name_id', 'duration', 'metric1', 'metric2', 'metric3']
    db_key_fields = {'time', 'system_id', 'data_type_id', 'group_name_id'}
    pre_sql_insert = create_pre_sql_insert(table_name, db_field_names, db_key_fields)

    def get_mysql_insert_manys(self, db_cursor: Cursor, data: QMWREntry, parent: TPF_UDM_Payload):
        rows = []

        for entry in data.udm_record_array:
            # Arguments must be in order of db_field_names
            # Base version db fields
            row = [entry.udm_sample_header.timestamp,
                   parent.header.get_tpf_system_id(db_cursor),
                   self.get_data_type_id(db_cursor, entry),
                   self.get_group_name_id(db_cursor, entry),
                   entry.udm_sample_header.intervalLength,
                   entry.metric1,
                   entry.metric2,
                   entry.metric3]

            rows.append(row)

        # insert
        v_insert = self.pre_sql_insert

        return [InsertMany(v_insert, rows=rows)]

    @classmethod
    def get_table_names(cls):
        return [cls.table_name]

    def get_data_type_id(self, db_cursor: Cursor, data: QMWRGroupEntry):
        return udm_id_manager.get_udm_data_type_id(
            data.udm_sample_header.data_type,
            db_cursor
        )

    def get_group_name_id(self, db_cursor: Cursor, data: QMWRGroupEntry):
        return udm_id_manager.get_udm_group_name_id(
            data.group_name,
            db_cursor
        )
