#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2023                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20230719    PJ46904 - User-Defined Metrics                        #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import logging
from tpf_data_manager import id_manager
from tpf_data_manager.id_manager import IdCols
from tpf_data_manager.id_manager import id_table_cols
from tpf_data_manager.id_manager import id_tables
from typing import Dict
from pymysql.cursors import Cursor

from tpf_rtmc_db.db_writer import last_executed_query
from tpf_utils.db_utils import create_id_pre_sql_select, create_id_pre_sql_insert

user_id_table_cols = {
    'udm_data_types': IdCols(
        key_cols=['data_type'],
        extra_cols=[]
    ),
    'udm_group_names': IdCols(
        key_cols=['group_name'],
        extra_cols=[]
    )
}

id_table_cols.update(user_id_table_cols)
id_tables.update(
  {
    table: {
        'insert': create_id_pre_sql_insert(table, id_cols.key_cols + id_cols.extra_cols),
        'select': create_id_pre_sql_select(table, id_cols.key_cols),
        'ids': {}
    }
    for table, id_cols in user_id_table_cols.items()
  }
)

def get_udm_data_type_id(udm_data_type, db_cursor: Cursor):
    return id_manager._get_id('udm_data_types', db_cursor, udm_data_type or unknown_col)


def get_udm_group_name_id(group_name, db_cursor: Cursor):
    return id_manager._get_id('udm_group_names', db_cursor, group_name or unknown_col)

