#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2023                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################
import logging

from exceptions.json_err import MissingFieldError
from tpf_user_exit.udm.udm_sample_data import UDMSampleData
from tpf_user_exit.udm.udm_sample_header import UDMSampleHeader
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')


class QMWRGroupEntry(object):

    required_fields = ['groupName', 'metric1', 'metric2', 'metric3']

    def __init__(self, json_record, udm_sample_header: UDMSampleHeader):
        """
        :param json_record: dictionary of a single object in the UDMRecordArray of a UDM record.
        :param udm_sample_header: UDMSampleHeader object from UDM JSON structure
        :raise MissingFieldError: If the data is missing a required field.
        """

        self.udm_sample_header = udm_sample_header
        self.udm_raw_json = json_record

        try:
            self.group_name = str(json_record["groupName"]).strip()
            self.metric1 = json_record["metric1"]
            self.metric2 = json_record["metric2"]
            self.metric3 = json_record["metric3"]
        except KeyError:
            msg = f'Unable to deserialize UDM record. Missing fields: ' \
                  f'{get_missing_keys(json_record, self.required_fields)}. Data: {json_record}'
            logger.warning(msg)
            raise MissingFieldError(msg)


class QMWREntry(UDMSampleData):

    def __init__(self, json_record_array, udm_sample_header: UDMSampleHeader):
        """
        :param json_record_array: list of dictionaries, the udmRecordArray of a UDM record.
        :param udm_sample_header: UDMSampleHeader object for this object.
        :raise MissingFieldError: If the data is missing a required field.
        """

        # sets self.udm_sample_header and self.udm_raw_json
        super().__init__(json_record_array, udm_sample_header)

        self.udm_record_array = [QMWRGroupEntry(json_record, udm_sample_header) for json_record in json_record_array]
