#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2023                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20230719    PJ46904 - User-Defined Metrics                        #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import logging
from exceptions.json_err import MissingFieldError
from tpf_utils.json_utils import get_missing_keys

logger = logging.getLogger('tpf_zrtmc_analyzer')


class UDMSampleHeader(object):

    required_fields = ["version", "timestamp", "intervalLength", "count"]

    def __init__(self, json_udm_sample_header, data_type):

        """
        Initializes a UDMSampleHeader object.

        :param json_udm_sample_header: "header" from UDM JSON structure python dictionary.
        :raise MissingFieldError: If the data is missing a required field.
        """

        try:
            # Use timestamp and collection interval length for any calculations
            self.version = json_udm_sample_header["version"]
            self.timestamp = json_udm_sample_header["timestamp"]
            self.intervalLength = json_udm_sample_header["intervalLength"]
            self.count = json_udm_sample_header["count"]
            self.data_type = data_type

        except KeyError:
            log_message = f'Unable to deserialize UDM sample Header. Missing fields: ' \
                          f'{get_missing_keys(json_udm_sample_header, self.required_fields)}. Data: {json_udm_sample_header}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)

