#######################################################################
#       5748-T15                                                      #
#       Copyright IBM Corp. 2023                                      #
#######################################################################
# ---START OF HISTORY------------------------------------------------ #
#   DATE        DESCRIPTION                                           #
#   20230719    PJ46904 - User-Defined Metrics                        #
# ---END OF HISTORY-------------------------------------------------- #
#######################################################################
#                                                                     #
#  GENERAL INFORMATION:                                               #
#                                                                     #
#  Disclaimer:   This package is provided on an "as is" basis. There  #
#                are no warranties, express or implied, including the #
#                implied warranties of merchantability and fitness    #
#                for a particular purpose. IBM has no obligation to   #
#                provide service, defect correction, or any           #
#                maintenance for the package.  IBM has no obligation  #
#                to supply any updates or enhancements for the        #
#                package to you even if such are or later become      #
#                available.                                           #
#                                                                     #
#######################################################################

import logging
from exceptions.json_err import MissingFieldError
from tpf_user_exit.udm.udm_body import UDMBody
from tpf_udm.tpf_udm_rtmc_header import TPF_UDM_RTMC_Header
from tpf_utils.json_utils import get_missing_keys


logger = logging.getLogger('tpf_zrtmc_analyzer')


class TPF_UDM_Payload(object):

    required_fields = ["header", "body"]

    def __init__(self, json_udm):

        """

        :param json_udm: full UDM JSON structure.This data should be composed of "header" and "body" python
        dictionaries.
        :raise MissingFieldError: If the data is missing a required field.
        """
        
        try:
            self.header = TPF_UDM_RTMC_Header(json_udm["header"])
            self.body = UDMBody(json_udm["body"],self.header.data_type)

        except KeyError:
            log_message = f'Unable to deserialize UDM data. Missing fields: ' \
                          f'{get_missing_keys(json_udm, self.required_fields)}. Data: {json_udm}'
            logger.warning(log_message)
            raise MissingFieldError(log_message)
